IMPLEMENTATION MODULE BmpToRaw;

(******************************************************************)
(*											*)
(*			  BMP To RAW Conversion Module			*)
(*											*)
(*	Programmer :	Matthew Ma (c9315720)				*)
(*											*)
(*	Last edited :	22-10-97						*)
(*											*)
(*	Function : 		This module converts a BMP image to an	*)
(*				un-encoded RAW image.				*)
(*											*)
(*	Status :		Working.						*)
(******************************************************************)


FROM InfoDialogs IMPORT
(* proc *)   CantOpenFile, NotBmpFile, Not8Bit, InvalidSubtype,
             NoPalette, CantCreateFile;

FROM Support IMPORT
(* proc *)   ReadWord, WriteWord, ReadLongWord, WriteLongWord;

FROM Canvas IMPORT
(* var  *)   normal, zoom2x, zoom3x,
(* proc *)   MsgCanvas;

FROM XMS IMPORT
(* proc *)   ObtainXMSMemory, ReleaseXMSMemory;

FROM FIO IMPORT
(* type *)   File;

FROM FileSys IMPORT
(* proc *)   OpenFile, CloseFile, ReadByte, EOF, WriteByte,
             SetPosition, SavePosition, FileSize, ReadRecord, WriteRecord;

FROM GraphicWindow IMPORT
(* var  *)   PutDot, XMAX, YMAX, CharWidth, CharHeight,
(* proc *)   ObtainScreen, ReleaseScreen, PutRectangle, PutLine;

FROM IOErrorCodes IMPORT
(* type *)   ErrorCode;

FROM Str IMPORT
(* proc *)   Compare;

FROM SoundEffects IMPORT
(* proc *)   Beep;


CONST
   block_size = 65534; (* 1024x64-2 *)


VAR
   n_planes, bits_per_pixel : CARDINAL;
   max_colours, unique_colours, important_colours, compression : LONGCARD;
   temp_x, temp_y, x_size, y_size, file_size, bitmap_offset, header_offset : LONGCARD;



(******************************************************************)
(* This procedure converts an 8-bit BMP image to an un-encoded	*)
(* RAW file.									*)
(******************************************************************)
PROCEDURE Bmp256ToRawData(bmp_file, raw_file : ARRAY OF CHAR) : CARDINAL;
VAR
   in_file, out_file : File;
   dummy : BYTE;
   error : ErrorCode;
   red, green, blue : BYTE;
   count : SHORTCARD;
   signature : ARRAY [0..1] OF CHAR;
   ok : BOOLEAN;
   rows, actual, x, y, pixel, block_count, i, blanks, y_count : CARDINAL;
   block : POINTER TO ARRAY [0..block_size-1] OF BYTE;
   x_size2 : LONGCARD;
BEGIN
   IF (OpenFile(in_file, bmp_file, FALSE) <> OK) THEN
	Beep();
	CantOpenFile();
      RETURN (1);
   END;

   IF (OpenFile(out_file, raw_file, TRUE) <> OK) THEN
	Beep();
	CantCreateFile();
	RETURN (2);
   END;

   (****************************)
   (* checks for BMP signature *)
   (****************************)
   error := SetPosition(in_file,0);
   signature[0] := VAL(CHAR,ReadByte(in_file));
   signature[1] := VAL(CHAR,ReadByte(in_file));
   IF Compare(signature, "BM") <> 0 THEN
	Beep();
	NotBmpFile();
      RETURN (3);
   END;

   file_size := FileSize(in_file);

   error := SetPosition(in_file,10);
   bitmap_offset := ReadLongWord(in_file);
   header_offset := ReadLongWord(in_file);

   x_size := ReadLongWord(in_file);
   y_size := ReadLongWord(in_file);

   n_planes := ReadWord(in_file); (* must be set to 1 *)
   IF n_planes <> 1 THEN
	Beep();
	Not8Bit();
      RETURN (4);
   END;

   bits_per_pixel := ReadWord(in_file);
   IF n_planes <> 1 THEN
	Beep();
	Not8Bit();
      RETURN (5);
   END;

   compression := ReadLongWord(in_file);
   IF compression <> 0 THEN (* compression check *)
	Beep();
	InvalidSubtype();
      RETURN (6);
   END;
   

   (****************************************)
   (* BMP scanlines end on a LONG boundary *)
   (****************************************)
   x_size2 := ((x_size + 3) DIV 4) * 4;


   block := ObtainXMSMemory();

   y_count := 1;

   (************************)
   (* scan line conversion *)
   (************************)
   LOOP
   	error := SetPosition(in_file, bitmap_offset+(y_size-LONGCARD(y_count))*x_size2);
   	error := ReadRecord(in_file, block, CARDINAL(x_size), actual);

   	error := WriteRecord(out_file, block, CARDINAL(x_size));

   	INC(y_count);

   	IF LONGCARD(y_count) > y_size THEN
	   EXIT;
   	END;
   END; (* LOOP *)

   ReleaseXMSMemory();

   CloseFile(in_file);
   CloseFile(out_file);

   RETURN (0);

END Bmp256ToRawData;


END BmpToRaw.