IMPLEMENTATION MODULE BmpToTga;

(******************************************************************)
(*											*)
(*			  BMP To TGA Conversion Module			*)
(*											*)
(*	Programmer :	Matthew Ma (c9315720)				*)
(*											*)
(*	Last edited :	22-10-97						*)
(*											*)
(*	Function : 		This module converts a BMP image to a TGA	*)
(*				image.						*)
(*											*)
(*	Status :		Working.						*)
(******************************************************************)


FROM InfoDialogs IMPORT
(* proc *)   CantOpenFile, NotBmpFile, Not8Bit, InvalidSubtype,
             NoPalette, CantCreateFile;

FROM Support IMPORT
(* proc *)   ReadWord, WriteWord, ReadLongWord, WriteLongWord;

FROM Canvas IMPORT
(* var  *)   normal, zoom2x, zoom3x;

FROM XMS IMPORT
(* proc *)   ObtainXMSMemory, ReleaseXMSMemory;

FROM FIO IMPORT
(* type *)   File;

FROM FileSys IMPORT
(* proc *)   OpenFile, CloseFile, ReadByte, EOF, WriteByte,
             SetPosition, SavePosition, FileSize, ReadRecord, WriteRecord;

FROM GraphicWindow IMPORT
(* var  *)   PutDot, XMAX, YMAX, CharWidth, CharHeight,
(* proc *)   ObtainScreen, ReleaseScreen, PutRectangle, PutLine;

FROM IOErrorCodes IMPORT
(* type *)   ErrorCode;

FROM Str IMPORT
(* proc *)   Compare;

FROM SoundEffects IMPORT
(* proc *)   Beep;


CONST
   block_size = 65534;


VAR
   n_planes, bits_per_pixel : CARDINAL;
   max_colours, unique_colours, important_colours, compression : LONGCARD;
   temp_x, temp_y, x_size, y_size, file_size, bitmap_offset, header_offset : LONGCARD;



(******************************************************************)
(* This procedure converts an 8-bit BMP image to an 8-bit TGA	*)
(* image.										*)
(******************************************************************)
PROCEDURE Bmp256ToTga256(bmp_file, tga_file : ARRAY OF CHAR) : CARDINAL;
VAR
   in_file, out_file : File;
   dummy : BYTE;
   error : ErrorCode;
   red, green, blue : BYTE;
   count : SHORTCARD;
   signature : ARRAY [0..1] OF CHAR;
   ok : BOOLEAN;
   rows, actual, x, y, pixel, block_count, i, blanks, y_count : CARDINAL;
   block : POINTER TO ARRAY [0..block_size-1] OF BYTE;
   x_size2 : LONGCARD;
BEGIN
   IF (OpenFile(in_file, bmp_file, FALSE) <> OK) THEN
	Beep();
	CantOpenFile();
      RETURN (1);
   END;

   IF (OpenFile(out_file, tga_file, TRUE) <> OK) THEN
	Beep();
	CantCreateFile();
	RETURN (2);
   END;

   (****************************)
   (* checks for BMP signature *)
   (****************************)
   error := SetPosition(in_file,0);
   signature[0] := VAL(CHAR,ReadByte(in_file));
   signature[1] := VAL(CHAR,ReadByte(in_file));
   IF Compare(signature, "BM") <> 0 THEN
	Beep();
	NotBmpFile();
      RETURN (3);
   END;

   file_size := FileSize(in_file);

   error := SetPosition(in_file,10);
   bitmap_offset := ReadLongWord(in_file);
   header_offset := ReadLongWord(in_file);

   x_size := ReadLongWord(in_file);
   y_size := ReadLongWord(in_file);
   n_planes := ReadWord(in_file); (* must be set to 1 *)
   bits_per_pixel := ReadWord(in_file);

   IF (n_planes > 1) OR (bits_per_pixel <> 8) THEN
	Beep();
	Not8Bit();
	RETURN (4);
   END;

   compression := ReadLongWord(in_file);
   IF compression <> 0 THEN (* compression check *)
	Beep();
	InvalidSubtype();
      RETURN (5);
   END;


   (*************************)
   (* writes the TGA header *)
   (*************************)
   error := WriteByte(out_file, BYTE(0));
   error := WriteByte(out_file, BYTE(1));
   error := WriteByte(out_file, BYTE(1));

   WriteWord(out_file, 0);
   WriteWord(out_file, 256);				(* colourmap length *)
   error := WriteByte(out_file, BYTE(24));	(* colourmap entry *)

   WriteWord(out_file, 0);				(* x origin *)
   WriteWord(out_file, 0);				(* y origin *)

   WriteWord(out_file, CARDINAL(x_size));		(* width *)
   WriteWord(out_file, CARDINAL(y_size));		(* height *)

   error := WriteByte(out_file, 8);			(* bits per pixel *)
   error := WriteByte(out_file, 0);


   error := SetPosition(in_file,14+header_offset);

   (**************************)
   (* writes palette entries *)
   (**************************)
   FOR count := 0 TO 255 DO
	blue := ReadByte(in_file);
	green := ReadByte(in_file);
	red := ReadByte(in_file);
	dummy := ReadByte(in_file);

	error := WriteByte(out_file, blue);
	error := WriteByte(out_file, green);
	error := WriteByte(out_file, red);
   END;


   (****************************************)
   (* BMP scanlines end on a LONG boundary *)
   (****************************************)
   x_size2 := ((x_size + 3) DIV 4) * 4;


   error := SetPosition(in_file, bitmap_offset);

   block := ObtainXMSMemory();

   y_count := 0;

   (************************)
   (* scan line conversion *)
   (************************)
   LOOP
   	error := ReadRecord(in_file, block, CARDINAL(x_size2), actual);

   	error := WriteRecord(out_file, block, CARDINAL(x_size));

   	INC(y_count);

   	IF LONGCARD(y_count) = y_size THEN	(* reached last scan line *)
	   EXIT;					(* terminates conversion process *)
   	END;
   END;

   ReleaseXMSMemory();


   CloseFile(in_file);
   CloseFile(out_file);

   RETURN (0);

END Bmp256ToTga256;


END BmpToTga.