IMPLEMENTATION MODULE BmpToTif;

(******************************************************************)
(*											*)
(*			  BMP To TIF Conversion Module			*)
(*											*)
(*	Programmer :	Matthew Ma (c9315720)				*)
(*											*)
(*	Last edited :	22-10-97						*)
(*											*)
(*	Function : 		This module converts a BMP image to a TIF	*)
(*				image.						*)
(*											*)
(*	Status :		Working.						*)
(******************************************************************)


FROM InfoDialogs IMPORT
(* proc *)   CantOpenFile, NotBmpFile, Not8Bit, InvalidSubtype,
             NoPalette, CantCreateFile;

FROM Support IMPORT
(* proc *)   ReadWord, WriteWord, ReadLongWord, WriteLongWord;

FROM Canvas IMPORT
(* var  *)   normal, zoom2x, zoom3x;

FROM XMS IMPORT
(* proc *)   ObtainXMSMemory, ReleaseXMSMemory;

FROM FIO IMPORT
(* type *)   File;

FROM FileSys IMPORT
(* proc *)   OpenFile, CloseFile, ReadByte, EOF, WriteByte,
             SetPosition, SavePosition, FileSize, ReadRecord, WriteRecord;

FROM GraphicWindow IMPORT
(* var  *)   PutDot, XMAX, YMAX, CharWidth, CharHeight,
(* proc *)   ObtainScreen, ReleaseScreen, PutRectangle, PutLine;

FROM IOErrorCodes IMPORT
(* type *)   ErrorCode;

FROM Str IMPORT
(* proc *)   Compare;

FROM SoundEffects IMPORT
(* proc *)   Beep;


CONST
   block_size = 65534;


VAR
   n_planes, bits_per_pixel : CARDINAL;
   max_colours, unique_colours, important_colours, compression : LONGCARD;
   temp_x, temp_y, x_size, y_size, file_size, bitmap_offset, header_offset : LONGCARD;



(******************************************************************)
(* This procedure converts an 8-bit BMP image to an 8-bit TIF	*)
(* image.										*)
(******************************************************************)
PROCEDURE Bmp256ToTif256(bmp_file, tga_file : ARRAY OF CHAR) : CARDINAL;
VAR
   in_file, out_file : File;
   dummy : BYTE;
   error : ErrorCode;
   red, green, blue : BYTE;
   count : CARDINAL;
   signature : ARRAY [0..1] OF CHAR;
   ok : BOOLEAN;
   rows, actual, x, y, pixel, block_count, i, blanks, y_count : CARDINAL;
   block : POINTER TO ARRAY [0..block_size-1] OF BYTE;
   x_size2 : LONGCARD;
BEGIN
   IF (OpenFile(in_file, bmp_file, FALSE) <> OK) THEN
	Beep();
	CantOpenFile();
      RETURN (1);
   END;

   IF (OpenFile(out_file, tga_file, TRUE) <> OK) THEN
	Beep();
	CantCreateFile();
	RETURN (2);
   END;

   (****************************)
   (* checks for BMP signature *)
   (****************************)
   error := SetPosition(in_file,0);
   signature[0] := VAL(CHAR,ReadByte(in_file));
   signature[1] := VAL(CHAR,ReadByte(in_file));
   IF Compare(signature, "BM") <> 0 THEN
	Beep();
	NotBmpFile();
      RETURN (3);
   END;

   file_size := FileSize(in_file);

   error := SetPosition(in_file,10);
   bitmap_offset := ReadLongWord(in_file);
   header_offset := ReadLongWord(in_file);

   x_size := ReadLongWord(in_file);
   y_size := ReadLongWord(in_file);
   n_planes := ReadWord(in_file); (* must be set to 1 *)
   bits_per_pixel := ReadWord(in_file);

   IF (n_planes > 1) OR (bits_per_pixel <> 8) THEN
	Beep();
	Not8Bit();
	RETURN (4);
   END;

   compression := ReadLongWord(in_file);
   IF compression <> 0 THEN (* compression check *)
	Beep();
	InvalidSubtype();
      RETURN (5);
   END;


   (****************************)
   (* little-endian byte order *)
   (****************************)
   error := WriteByte(out_file, BYTE("I"));
   error := WriteByte(out_file, BYTE("I"));

   WriteWord(out_file, 42);
   WriteLongWord(out_file, 8);	(* first IFD *)

   WriteWord(out_file, 15-3);		(* IFD_entry_count *)

   WriteWord(out_file, 254);		(* NewSubfileType *)
   WriteWord(out_file, 4);
   WriteLongWord(out_file, 1);
   WriteLongWord(out_file, 0);

   WriteWord(out_file, 256);		(* image width *)
   WriteWord(out_file, 3);
   WriteLongWord(out_file, 1);
   WriteLongWord(out_file, x_size);

   WriteWord(out_file, 257);		(* image height *)
   WriteWord(out_file, 3);
   WriteLongWord(out_file, 1);
   WriteLongWord(out_file, y_size);

   WriteWord(out_file, 258);
   WriteWord(out_file, 3);
   WriteLongWord(out_file, 1);
   WriteLongWord(out_file, 8);	(* bits-per-pixel *)

   WriteWord(out_file, 259);
   WriteWord(out_file, 3);
   WriteLongWord(out_file, 1);
   WriteLongWord(out_file, 1);	(* non-compressed *)

   WriteWord(out_file, 262);
   WriteWord(out_file, 3);
   WriteLongWord(out_file, 1);
   WriteLongWord(out_file, 3);	(* palette colour *)

   WriteWord(out_file, 273);
   WriteWord(out_file, 4);
   WriteLongWord(out_file, 1);		(* StripsPerImage *)
   WriteLongWord(out_file, 1730-12*3);	(* strip offset *)

   WriteWord(out_file, 277);
   WriteWord(out_file, 3);
   WriteLongWord(out_file, 1);
   WriteLongWord(out_file, 1);	(* RGB *)

   WriteWord(out_file, 278);
   WriteWord(out_file, 3);
   WriteLongWord(out_file, 1);
   WriteLongWord(out_file, 8192);	(* RowsPerStrip *)

   WriteWord(out_file, 279);
   WriteWord(out_file, 4);
   WriteLongWord(out_file, 1);
   WriteLongWord(out_file, 1);	(* StripByteCounts *)

   WriteWord(out_file, 296);
   WriteWord(out_file, 3);
   WriteLongWord(out_file, 1);
   WriteLongWord(out_file, 1);	(* ResolutionUnit *)

   WriteWord(out_file, 320);
   WriteWord(out_file, 3);
   WriteLongWord(out_file, 768);
   WriteLongWord(out_file, 194-12*3);	(* ColorMap *)

   WriteLongWord(out_file, 0);	(* no more IFDs *)


   (*****************************)
   (* writes red colour entries *)
   (*****************************)
   FOR count := 0 TO 255 DO
	 error := SetPosition(in_file,14+header_offset+LONGCARD(2+4*count));
	 red := ReadByte(in_file);

	 error := WriteByte(out_file, red);
	 error := WriteByte(out_file, red);
   END;

   (*******************************)
   (* writes green colour entries *)
   (*******************************)
   FOR count := 0 TO 255 DO
	 error := SetPosition(in_file,14+header_offset+LONGCARD(1+4*count));
	 green := ReadByte(in_file);

	 error := WriteByte(out_file, green);
	 error := WriteByte(out_file, green);
   END;

   (******************************)
   (* writes blue colour entries *)
   (******************************)
   FOR count := 0 TO 255 DO
	 error := SetPosition(in_file,14+header_offset+LONGCARD(4*count));
	 blue := ReadByte(in_file);

	 error := WriteByte(out_file, blue);
	 error := WriteByte(out_file, blue);
   END;

   (****************************************)
   (* BMP scanlines end on a LONG boundary *)
   (****************************************)
   x_size2 := ((x_size + 3) DIV 4) * 4;


   error := SetPosition(in_file, bitmap_offset);

   block := ObtainXMSMemory();

   y_count := 1;


   (************************)
   (* scan line conversion *)
   (************************)
   LOOP
   	IF LONGCARD(y_count) <= y_size THEN
         error := SetPosition(in_file, bitmap_offset+((y_size-LONGCARD(y_count))*x_size2));
   	END;

   	error := ReadRecord(in_file,block,CARDINAL(x_size),actual);

   	error := WriteRecord(out_file, block, CARDINAL(x_size));

   	INC(y_count);

   	IF LONGCARD(y_count) > y_size THEN	(* reached last scan line *)
	   EXIT;					(* terminates conversion process *)
   	END;
   END;

   ReleaseXMSMemory();


   CloseFile(in_file);
   CloseFile(out_file);

   RETURN (0);

END Bmp256ToTif256;


END BmpToTif.