IMPLEMENTATION MODULE Buttons;

(******************************************************************)
(*											*)
(*			  	   Buttons Module					*)
(*											*)
(*	Originally developed by Chee Hong KOK as BarMenu. It has	*)
(*	been modified, renamed and maintained by Matthew Ma.		*)
(*											*)
(*	Last edited :	22-10-97						*)
(*											*)
(*	Function : 		This module implements 3D buttons to be	*)
(*				used as menu buttons and icon buttons	*)
(*											*)
(*	Modifications :	- more realistic, pressable 3D buttons.	*)
(*				- added ability to display icon button	*)
(*				  descriptions on the status bar when	*)
(*				  placed upon by mouse cursor.		*)
(*											*)
(*	Status :		Working.						*)
(******************************************************************)


(*********** added by Matthew Ma ***********)
FROM Bars IMPORT
(* proc *)   UpdateStatusBar, UpdateTitleBar;

FROM Timer IMPORT
(*proc*)Sleep;

FROM GraphicWindow IMPORT
(*type*)LongStrPtr;

FROM GraphicWindow IMPORT
(*var*)CharHeight, CharWidth, ActualDataOffset,XMAX, YMAX,
(*type*)GraphicalWindow,
(*proc*)CreateGraphicalWindow, DisplayWindowTitle, DisplayWindowData,
	HideWindowData, PutString, ObtainScreen, ReleaseScreen, ClearDataRegion,
	DataRegionDisplayed, PutRectangle, DoNothing, PutChar, PutLine, PutDot,
	WindowTitleUp, WindowTitleDown, MoveWindowABS,
	ReturnDataLeftTop,ActivateCancelWhenMouseNolongerInWindow,
	DeActivateCancelWhenMouseNolongerInWindow;

FROM Str IMPORT
(*proc*)Append, Length, Copy;

FROM Storage1 IMPORT
(*proc*)ALLOCATE, DEALLOCATE;

FROM LinkList IMPORT
(*type*)List, ListElement,
(*proc*)CreateLinkList, CreateListElement, MoveToTop, MoveToBottom, ObtainList,
	ReleaseList, GetFirstElementInList, ElementValid,
	ReadObjectFromElement, GetNextElementInList;

FROM KbInterface IMPORT
(*const*)EXTENDED_AUX, UpArrowMain, DownArrowMain,
         LeftArrowMain, RightArrowMain, ReturnKey, ESCKey, ALTKeyBase, NULL,
(*type*)KeyPressed, Keyboard,
(*proc*)SetupKeyboardLink, ReadFromKeyboard, ConnectToKeyboard, StuffKeyboard;

FROM CancelSequences IMPORT
(*type*)CancelElement, CancelSource,
(*proc*)RegisterCancelSequence, EnableCancelSequence, DisableCancelSequence,
        ActivateCancelSequence;

FROM TaskControl IMPORT
(*type*)Lock,
(*proc*)CreateTask, CreateLock, Obtain, Release;

FROM MouseRoutines IMPORT
(*proc*)   SetArrow, SetCrossHair;


CONST
  OnButtonHelpTreshold = 5;


TYPE
  ButtonHelp = RECORD
                 w:GraphicalWindow;
                 item:BoxMenuItemTYPE;
                 oncount:CARDINAL;
                 displayed:BOOLEAN;
               END;

  BoxMenuItemTYPE = POINTER TO RECORD

                      blIfCloseMenu, blIfExecutedHideAllMenus:BOOLEAN;

		      item_number:INTEGER;

		      prev_item_ptr:BoxMenuItemTYPE;
		      next_item_ptr:BoxMenuItemTYPE;
		
		      item_name:LongStrPtr;
		      IconDraw:BoxMenuIconDrawProcedure;(* procedure to *)
		                                        (*draw the icon on menu*)
		      x, y : INTEGER;  (* the x and y offset of the icon *)
		      (* above 3 integer args are irrelevant if using strings *)
		
                      WhichAlphabetAsKeyPress:INTEGER;
                      AlphabetColour:ColourType;

		      this_sub_menu_ptr:BoxMenuTYPE;
		      next_sub_menu_ptr:BoxMenuTYPE;
                      RespondToSingleMouseClick:BOOLEAN;
		      ProcedureToExecute:BoxMenuProcedure;

		    END;(*RECORD*)

  BoxMenuTYPE = POINTER TO RECORD
		  menu:MenuType;
		  pchSubMenuName:LongStrPtr;
		  isText:BOOLEAN;
		  SubMenuNameColour, MenuNameBkColour:ColourType;

		  WhichAlphabetAsAltKeyControlCharacter:INTEGER;
		  AlphabetColour:ColourType;

		  inSubMenuRow:INTEGER;
		  inSubMenuColumn:INTEGER;
		
      x_item_pos, y_item_pos : INTEGER;
		
      x_size, y_size : INTEGER;

		  inMaxItemLength:INTEGER;
		  x_grid_size : INTEGER;
		  y_grid_size : INTEGER;

		  prev_Level_sub_menu_item_ptr:BoxMenuItemTYPE;
		
		  first_sub_menu_item_ptr:BoxMenuItemTYPE;
		  prev_selected_sub_menu_item_ptr:BoxMenuItemTYPE;
		  current_selected_sub_menu_item_ptr:BoxMenuItemTYPE;
		  last_sub_menu_item_ptr:BoxMenuItemTYPE;
		  inNumberOfItems:INTEGER;

		  caHighlightTextFrColour:ColourType;
		  caHighlightTextBkColour:ColourType;
		  caNormalTextFrColour:ColourType;
		  caNormalTextBkColour:ColourType;

		  TitleWindow:GraphicalWindow;
		  DataWindow:GraphicalWindow;
		
		  WindowNOTCreated:BOOLEAN;

		  LinkListElement:ListElement;
		  Lock:Lock;

		  cursor:CursorProcedure;

		END;(*RECORD*)


VAR
   BoxMenuHelpWindowMaxChars:INTEGER;
   BoxMenuHelpWindowColour:CARDINAL;

  ButtonsHelp:ButtonHelp;
  HelpWindowWidth,HelpWindowHeight:INTEGER;

  AlwaysActiveMenuList, ActiveMenuList, InactiveMenuList:List;

  MenuActivated:BOOLEAN;

  MenuBusy:BOOLEAN;

  MenuCancelElement:CancelElement;

  MenuLock:Lock;

  BoxMenuKeyboard:Keyboard;

  MouseSources:BOOLEAN;
  MouseMenu:BoxMenuTYPE;




PROCEDURE HighLightItem(BoxMenuItem:BoxMenuItemTYPE);
VAR
  xoffset, yoffset:INTEGER;
  endx,endy:INTEGER;
BEGIN
IF BoxMenuItem^.this_sub_menu_ptr^.isText THEN

  (* items are displayed as text *)

  xoffset := ((BoxMenuItem^.item_number) DIV
          BoxMenuItem^.this_sub_menu_ptr^.y_size) *
          (CharWidth * (BoxMenuItem^.this_sub_menu_ptr^.inMaxItemLength + 1));
  yoffset := ((BoxMenuItem^.item_number) MOD
          BoxMenuItem^.this_sub_menu_ptr^.y_size) * CharHeight;

  endx := xoffset + CharWidth * (BoxMenuItem^.this_sub_menu_ptr^.inMaxItemLength)+6;
  endy := yoffset + CharHeight;

  PutLine(BoxMenuItem^.this_sub_menu_ptr^.DataWindow,xoffset,yoffset,endx,yoffset,0);
  PutLine(BoxMenuItem^.this_sub_menu_ptr^.DataWindow,xoffset,yoffset+1,endx-1,yoffset+1,0+8);

  (************************* added by Matthew Ma **************************)
  PutDot(BoxMenuItem^.this_sub_menu_ptr^.DataWindow,endx,yoffset,8);
  (************************************************************************)

  PutLine(BoxMenuItem^.this_sub_menu_ptr^.DataWindow,xoffset,yoffset,xoffset,endy,0);
  PutLine(BoxMenuItem^.this_sub_menu_ptr^.DataWindow,xoffset+1,yoffset,xoffset+1,endy-1,0+8);

  (************************* added by Matthew Ma **************************)
  PutDot(BoxMenuItem^.this_sub_menu_ptr^.DataWindow,xoffset,endy,0);
  (************************************************************************)

  PutLine(BoxMenuItem^.this_sub_menu_ptr^.DataWindow,xoffset+2,endy,endx,endy,15);
  PutLine(BoxMenuItem^.this_sub_menu_ptr^.DataWindow,endx,yoffset+1+0,endx,endy,15);

  PutRectangle(BoxMenuItem^.this_sub_menu_ptr^.DataWindow, xoffset + 3, yoffset+2,
	       endx-1, endy-1,
	       BoxMenuItem^.this_sub_menu_ptr^.caNormalTextBkColour,TRUE);

  PutString(BoxMenuItem^.this_sub_menu_ptr^.DataWindow, BoxMenuItem^.item_name^,
	    xoffset + 5, yoffset + 1,
	    BoxMenuItem^.this_sub_menu_ptr^.caNormalTextFrColour,FALSE);

  IF (BoxMenuItem^.WhichAlphabetAsKeyPress > 0) THEN
    PutChar(BoxMenuItem^.this_sub_menu_ptr^.DataWindow, BoxMenuItem^.item_name^[BoxMenuItem^.WhichAlphabetAsKeyPress - 1],
	        xoffset + 5+CharWidth * (BoxMenuItem^.WhichAlphabetAsKeyPress- 1),
                yoffset+1, BoxMenuItem^.AlphabetColour,FALSE);
  END;(*IF*)

ELSE
  (* items are displayed as icons *)

  xoffset := ((BoxMenuItem^.item_number) DIV
          BoxMenuItem^.this_sub_menu_ptr^.y_size) *
          (BoxMenuItem^.this_sub_menu_ptr^.x_grid_size);
  yoffset := ((BoxMenuItem^.item_number) MOD
          BoxMenuItem^.this_sub_menu_ptr^.y_size) *
          BoxMenuItem^.this_sub_menu_ptr^.y_grid_size;

  endx := xoffset + BoxMenuItem^.this_sub_menu_ptr^.x_grid_size;
  endy := yoffset + BoxMenuItem^.this_sub_menu_ptr^.y_grid_size;

  PutLine(BoxMenuItem^.this_sub_menu_ptr^.DataWindow,xoffset+1+1,yoffset+3-1,endx-1-2,yoffset+3-1,8);
  PutLine(BoxMenuItem^.this_sub_menu_ptr^.DataWindow,xoffset+1+1,yoffset+1+0,endx-1-1,yoffset+1+0,0);

  (************************* added by Matthew Ma **************************)
  PutDot(BoxMenuItem^.this_sub_menu_ptr^.DataWindow,endx-1-1,yoffset+1+0,8);
  (************************************************************************)

  PutLine(BoxMenuItem^.this_sub_menu_ptr^.DataWindow,xoffset+2,yoffset+2,xoffset+2,endy-1-1,0);
  PutLine(BoxMenuItem^.this_sub_menu_ptr^.DataWindow,xoffset+1+2,yoffset+2,xoffset+1+2,endy-1-2,8);

  (************************* added by Matthew Ma **************************)
  PutDot(BoxMenuItem^.this_sub_menu_ptr^.DataWindow,xoffset+2,endy-1-1,0);
  (************************************************************************)

  PutLine(BoxMenuItem^.this_sub_menu_ptr^.DataWindow,xoffset+2+1,endy-2,endx-1-1,endy-2,15);
  PutLine(BoxMenuItem^.this_sub_menu_ptr^.DataWindow,endx-1-1,yoffset+1+1,endx-1-1,endy-2,15);

  PutRectangle(BoxMenuItem^.this_sub_menu_ptr^.DataWindow, xoffset+3+1, yoffset+2+1,
	       endx-1-2, endy-1-2,
	       BoxMenuItem^.this_sub_menu_ptr^.caNormalTextBkColour,TRUE);

  BoxMenuItem^.IconDraw(BoxMenuItem^.this_sub_menu_ptr^.DataWindow,
                        xoffset + 5+BoxMenuItem^.x,
                        yoffset + 1+BoxMenuItem^.y);

END;

END HighLightItem;


PROCEDURE UnHighLightItem(BoxMenuItem:BoxMenuItemTYPE);
VAR
  xoffset, yoffset:INTEGER;
  endx, endy:INTEGER;
BEGIN
IF BoxMenuItem^.this_sub_menu_ptr^.isText THEN
  (* items are displayed as text *)
  xoffset := ((BoxMenuItem^.item_number) DIV
          BoxMenuItem^.this_sub_menu_ptr^.y_size) *
          (CharWidth * (BoxMenuItem^.this_sub_menu_ptr^.inMaxItemLength + 1));
  yoffset := ((BoxMenuItem^.item_number) MOD
          BoxMenuItem^.this_sub_menu_ptr^.y_size) * CharHeight;

  endx := xoffset + CharWidth * (BoxMenuItem^.this_sub_menu_ptr^.inMaxItemLength)+6;
  endy := yoffset + CharHeight;

  PutLine(BoxMenuItem^.this_sub_menu_ptr^.DataWindow,xoffset,yoffset,endx,yoffset,BoxMenuItem^.this_sub_menu_ptr^.caNormalTextBkColour);
  PutLine(BoxMenuItem^.this_sub_menu_ptr^.DataWindow,xoffset,yoffset+1,endx-1,yoffset+1,BoxMenuItem^.this_sub_menu_ptr^.caNormalTextBkColour);

  PutLine(BoxMenuItem^.this_sub_menu_ptr^.DataWindow,xoffset,yoffset,xoffset,endy,BoxMenuItem^.this_sub_menu_ptr^.caNormalTextBkColour);
  PutLine(BoxMenuItem^.this_sub_menu_ptr^.DataWindow,xoffset+1,yoffset,xoffset+1,endy-1,BoxMenuItem^.this_sub_menu_ptr^.caNormalTextBkColour);

  PutLine(BoxMenuItem^.this_sub_menu_ptr^.DataWindow,xoffset+2,endy,endx,endy,BoxMenuItem^.this_sub_menu_ptr^.caNormalTextBkColour);
  PutLine(BoxMenuItem^.this_sub_menu_ptr^.DataWindow,endx,yoffset+1,endx,endy,BoxMenuItem^.this_sub_menu_ptr^.caNormalTextBkColour);

  PutRectangle(BoxMenuItem^.this_sub_menu_ptr^.DataWindow, xoffset + 3, yoffset+2,
	       endx-1, endy-1,
	       BoxMenuItem^.this_sub_menu_ptr^.caNormalTextBkColour,TRUE);

  PutString(BoxMenuItem^.this_sub_menu_ptr^.DataWindow, BoxMenuItem^.item_name^,
	    xoffset + 3, yoffset,
	    BoxMenuItem^.this_sub_menu_ptr^.caNormalTextFrColour,FALSE);

  IF (BoxMenuItem^.WhichAlphabetAsKeyPress > 0) THEN
    PutChar(BoxMenuItem^.this_sub_menu_ptr^.DataWindow, BoxMenuItem^.item_name^[BoxMenuItem^.WhichAlphabetAsKeyPress - 1],
	        xoffset + 3+CharWidth * (BoxMenuItem^.WhichAlphabetAsKeyPress- 1),
                yoffset, BoxMenuItem^.AlphabetColour,FALSE);
  END;(*IF*)
ELSE
  (* items are displayed as icons *)

  xoffset := ((BoxMenuItem^.item_number) DIV
          BoxMenuItem^.this_sub_menu_ptr^.y_size) *
          (BoxMenuItem^.this_sub_menu_ptr^.x_grid_size);
  yoffset := ((BoxMenuItem^.item_number) MOD
          BoxMenuItem^.this_sub_menu_ptr^.y_size) *
          BoxMenuItem^.this_sub_menu_ptr^.y_grid_size;

  endx := xoffset + BoxMenuItem^.this_sub_menu_ptr^.x_grid_size;
  endy := yoffset + BoxMenuItem^.this_sub_menu_ptr^.y_grid_size;

  PutLine(BoxMenuItem^.this_sub_menu_ptr^.DataWindow,xoffset,yoffset,endx,yoffset,BoxMenuItem^.this_sub_menu_ptr^.caNormalTextBkColour);
  PutLine(BoxMenuItem^.this_sub_menu_ptr^.DataWindow,xoffset,yoffset+1,endx-1,yoffset+1,BoxMenuItem^.this_sub_menu_ptr^.caNormalTextBkColour);

  PutLine(BoxMenuItem^.this_sub_menu_ptr^.DataWindow,xoffset+2,yoffset,xoffset+2,endy,BoxMenuItem^.this_sub_menu_ptr^.caNormalTextBkColour);
  PutLine(BoxMenuItem^.this_sub_menu_ptr^.DataWindow,xoffset+1,yoffset,xoffset+1,endy-1,BoxMenuItem^.this_sub_menu_ptr^.caNormalTextBkColour);

  PutLine(BoxMenuItem^.this_sub_menu_ptr^.DataWindow,xoffset+2,endy,endx,endy,BoxMenuItem^.this_sub_menu_ptr^.caNormalTextBkColour);
  PutLine(BoxMenuItem^.this_sub_menu_ptr^.DataWindow,endx,yoffset+1,endx,endy,BoxMenuItem^.this_sub_menu_ptr^.caNormalTextBkColour);

  PutRectangle(BoxMenuItem^.this_sub_menu_ptr^.DataWindow, xoffset + 3, yoffset+2,
	       endx-1, endy-1,
	       BoxMenuItem^.this_sub_menu_ptr^.caNormalTextBkColour,TRUE);


  (****************** unhighlight frame - added by Matthew Ma *******************)
  PutLine(BoxMenuItem^.this_sub_menu_ptr^.DataWindow,
	    xoffset+1+1,yoffset+1+0,endx-1-1,yoffset+1+0,0+15);
  PutLine(BoxMenuItem^.this_sub_menu_ptr^.DataWindow,
	    xoffset+2,yoffset+2,xoffset+2,endy-1-1,0+15);
  PutLine(BoxMenuItem^.this_sub_menu_ptr^.DataWindow,
	    xoffset+2+0,endy-2,endx-1-1,endy-2,15-7);
  PutLine(BoxMenuItem^.this_sub_menu_ptr^.DataWindow,
	    endx-1-1,yoffset+1+0,endx-1-1,endy-2,15-7);
  PutLine(BoxMenuItem^.this_sub_menu_ptr^.DataWindow,
	    xoffset+2+1,endy-2-1,endx-1-2,endy-2-1,15-7);
  PutLine(BoxMenuItem^.this_sub_menu_ptr^.DataWindow,
	    endx-1-2,yoffset+1+1,endx-1-2,endy-2-1,15-7);
  (******************************************************************************)


  BoxMenuItem^.IconDraw(BoxMenuItem^.this_sub_menu_ptr^.DataWindow,
            xoffset + 3+BoxMenuItem^.x, yoffset+BoxMenuItem^.y);
END;

END UnHighLightItem;




(************************* added by Matthew Ma **************************)
(***************************************************)
(* This procedure draws a 3D frame around a button *)
(***************************************************)
PROCEDURE Draw3DFrame(BoxMenuItem : BoxMenuItemTYPE);
VAR
  xoffset, yoffset:INTEGER;
  endx, endy:INTEGER;
BEGIN
   xoffset := ((BoxMenuItem^.item_number) DIV
          	   BoxMenuItem^.this_sub_menu_ptr^.y_size) *
          	   (BoxMenuItem^.this_sub_menu_ptr^.x_grid_size);
   yoffset := ((BoxMenuItem^.item_number) MOD
          	   BoxMenuItem^.this_sub_menu_ptr^.y_size) *
          	   BoxMenuItem^.this_sub_menu_ptr^.y_grid_size;

   endx := xoffset + BoxMenuItem^.this_sub_menu_ptr^.x_grid_size;
   endy := yoffset + BoxMenuItem^.this_sub_menu_ptr^.y_grid_size;


   (*********************** unhighlight frame *************************)
   PutLine(BoxMenuItem^.this_sub_menu_ptr^.DataWindow,
	     xoffset+1+1,yoffset+1+0,endx-1-1,yoffset+1+0,0+15);
   PutLine(BoxMenuItem^.this_sub_menu_ptr^.DataWindow,
	     xoffset+2,yoffset+2,xoffset+2,endy-1-1,0+15);
   PutLine(BoxMenuItem^.this_sub_menu_ptr^.DataWindow,
	     xoffset+2+0,endy-2,endx-1-1,endy-2,15-7);
   PutLine(BoxMenuItem^.this_sub_menu_ptr^.DataWindow,
	     endx-1-1,yoffset+1+0,endx-1-1,endy-2,15-7);
   PutLine(BoxMenuItem^.this_sub_menu_ptr^.DataWindow,
	     xoffset+2+1,endy-2-1,endx-1-2,endy-2-1,15-7);
   PutLine(BoxMenuItem^.this_sub_menu_ptr^.DataWindow,
	     endx-1-2,yoffset+1+1,endx-1-2,endy-2-1,15-7);
   PutLine(BoxMenuItem^.this_sub_menu_ptr^.DataWindow,
	     xoffset+1+2,yoffset+1+1,endx-1-3,yoffset+1+1,0+15-8);
   PutLine(BoxMenuItem^.this_sub_menu_ptr^.DataWindow,
	     xoffset+2+1,yoffset+2+1,xoffset+2+1,endy-1-3,0+15-8);
   (*******************************************************************)
END Draw3DFrame;





(****************************************************************************)
(* HideBoxSubMenuTitle                                                      *)
(****************************************************************************)
(* This procedure will only hide the sub menu passed as rcBoxSubMenu        *)
(****************************************************************************)
PROCEDURE HideBoxSubMenuTitle(VAR (*INOUT*) rcBoxSubMenu:BoxMenuTYPE);
BEGIN
  (*not developed yet*)
END HideBoxSubMenuTitle;

(****************************************************************************)
(* HideBoxSubMenu                                                           *)
(****************************************************************************)
(* This procedure will only hide the sub menu passed as rcBoxSubMenu        *)
(****************************************************************************)
PROCEDURE HideBoxSubMenu(VAR (*INOUT*) rcBoxSubMenu:BoxMenuTYPE);
BEGIN
  IF (rcBoxSubMenu^.menu = Normal) THEN
    HideWindowData(rcBoxSubMenu^.DataWindow);

    WindowTitleUp(rcBoxSubMenu^.TitleWindow);

    MoveToTop(rcBoxSubMenu^.LinkListElement, InactiveMenuList);

    DeActivateCancelWhenMouseNolongerInWindow(rcBoxSubMenu^.DataWindow);
    DeActivateCancelWhenMouseNolongerInWindow(rcBoxSubMenu^.TitleWindow);
  END;(*IF*)
END HideBoxSubMenu;


(****************************************************************************)
(* HideAllBoxSubMenu                                                        *)
(****************************************************************************)
(* This procedure will hide all active menu that activates rcBoxSubMenu     *)
(****************************************************************************)
PROCEDURE HideAllBoxSubMenu(VAR (*INOUT*) rcBoxSubMenu:BoxMenuTYPE);
BEGIN
  IF (rcBoxSubMenu <> NIL) THEN
    HideBoxSubMenu(rcBoxSubMenu);
    IF ((rcBoxSubMenu^.current_selected_sub_menu_item_ptr^.blIfExecutedHideAllMenus) AND
	(rcBoxSubMenu^.prev_Level_sub_menu_item_ptr <> NIL)) THEN
      HideAllBoxSubMenu(rcBoxSubMenu^.prev_Level_sub_menu_item_ptr^.this_sub_menu_ptr);
    END;(*IF*)
  END;(*IF*)
END HideAllBoxSubMenu;

(****************************************************************************)
(* ConditionalHideAllBoxSubMenu                                                        *)
(****************************************************************************)
(* This procedure will hide all active menu that activates rcBoxSubMenu     *)
(****************************************************************************)
PROCEDURE ConditionalHideAllBoxSubMenu(VAR (*INOUT*) rcBoxSubMenu:BoxMenuTYPE);
BEGIN
  IF ((rcBoxSubMenu <> NIL) AND (
      rcBoxSubMenu^.current_selected_sub_menu_item_ptr^.blIfCloseMenu)) THEN
    HideBoxSubMenu(rcBoxSubMenu);
    IF ((rcBoxSubMenu^.current_selected_sub_menu_item_ptr^.blIfExecutedHideAllMenus) AND
	(rcBoxSubMenu^.prev_Level_sub_menu_item_ptr <> NIL)) THEN
      HideAllBoxSubMenu(rcBoxSubMenu^.prev_Level_sub_menu_item_ptr^.this_sub_menu_ptr);
    END;(*IF*)
  END;(*IF*)
END ConditionalHideAllBoxSubMenu;

(****************************************************************************)
(* SetButtonHelp                                                            *)
(****************************************************************************)
(* This procedure will reset help window for each buttons                   *)
(* PARAMETER PASSED:                                                        *)
(*   CurrentItem - current item that is directed in the menu to be given the*)
(*                 help window.                                             *)
(*   FirstTime - basically, set this to TRUE to reset the help timing       *)
(****************************************************************************)
PROCEDURE SetButtonHelp(CurrentItem:BoxMenuItemTYPE; FirstTime:BOOLEAN);
VAR

  (*message : ARRAY [0..80] OF CHAR; (* mine *)*)

  xoffset, yoffset,endx,endy:INTEGER;
BEGIN
  IF FirstTime THEN
    ButtonsHelp.item := NIL;
    ButtonsHelp.oncount := 0;
    IF ButtonsHelp.displayed THEN
      ButtonsHelp.displayed := FALSE;
      IF (ObtainScreen()) THEN
        HideWindowData(ButtonsHelp.w);
        DeActivateCancelWhenMouseNolongerInWindow(ButtonsHelp.w);
        ReleaseScreen();
      END;(*IF*)
    END;(*IF*)
  ELSE
    IF (ButtonsHelp.oncount >= OnButtonHelpTreshold) THEN
      IF NOT(ButtonsHelp.displayed) OR (ButtonsHelp.item <> CurrentItem) THEN
        ButtonsHelp.item := CurrentItem;
        ButtonsHelp.displayed := TRUE;
        ReturnDataLeftTop(ButtonsHelp.item^.this_sub_menu_ptr^.DataWindow,
                          xoffset, yoffset);

        xoffset := 5+xoffset + ((ButtonsHelp.item^.item_number DIV
                    ButtonsHelp.item^.this_sub_menu_ptr^.y_size)+1) *
                    ButtonsHelp.item^.this_sub_menu_ptr^.x_grid_size;
        yoffset := 5+yoffset + ((ButtonsHelp.item^.item_number MOD
                    ButtonsHelp.item^.this_sub_menu_ptr^.y_size)+1) *
                    ButtonsHelp.item^.this_sub_menu_ptr^.y_grid_size;

        IF ((yoffset + HelpWindowHeight) > YMAX) THEN
          yoffset := YMAX - HelpWindowHeight-1;
        END;(*IF*)
        IF ((xoffset + HelpWindowWidth) > XMAX) THEN
          xoffset := XMAX - HelpWindowWidth-1;
        END;(*IF*)


          MoveWindowABS(ButtonsHelp.w,xoffset,yoffset);

          ActivateCancelWhenMouseNolongerInWindow(ButtonsHelp.w);

	    (******************* added by Matthew Ma ********************)
	    UpdateStatusBar(ButtonsHelp.item^.item_name^, 3, 0, 8);
	    (************************************************************)

      END;(*IF*)
    ELSE
      INC(ButtonsHelp.oncount);
      IF ButtonsHelp.item = NIL THEN
        ButtonsHelp.item := CurrentItem;
      END;(*IF*)
    END;(*IF*)
  END;(*IF*)
END SetButtonHelp;

(****************************************************************************)
(* ResetALLMenuProcedure                                                    *)
(****************************************************************************)
(* This procedure will be initiated by keyboard or mouse to cancel all      *)
(*      currently active menus                                              *)
(****************************************************************************)
PROCEDURE ResetALLMenuProcedure(source:CancelSource);
VAR
  TempMenuActivated:BOOLEAN;
  FirstElement, NextElement:ListElement;
  TempBoxMenu:BoxMenuTYPE;
BEGIN

  (*check if any menu is activated*)
  Obtain(MenuLock);
  SetButtonHelp(NIL,TRUE);
  TempMenuActivated := MenuActivated;
  Release(MenuLock);

  IF (TempMenuActivated) THEN
    (*will only cancel if there are active menus*)

    (*will disable menu cancellation process*)
    DisableCancelSequence(MenuCancelElement);

    ObtainList();

    REPEAT
      (*must make sure that NO menu will remain active*)

      (*always read from first element in active list*)
      GetFirstElementInList(FirstElement, ActiveMenuList);
      IF (ElementValid(FirstElement)) THEN
	(*found something, so disable it*)
	ReadObjectFromElement(TempBoxMenu, FirstElement);

	Obtain(TempBoxMenu^.Lock);
	IF (ObtainScreen()) THEN
	  HideAllBoxSubMenu(TempBoxMenu);
	  ReleaseScreen();
	END;(*IF*)
	Release(TempBoxMenu^.Lock);
      END;(*IF*)
    UNTIL (NOT(ElementValid(FirstElement)));

    ReleaseList();

    Obtain(MenuLock);
    MenuActivated := FALSE;
    Release(MenuLock);

  END;(*IF*)
END ResetALLMenuProcedure;


(****************************************************************************)
(* SetCancelSequence                                                        *)
(****************************************************************************)
(* This procedure will be initiated by keyboard or mouse to cancel all      *)
(*      currently active menus                                              *)
(****************************************************************************)
PROCEDURE SetCancelSequence();
VAR
  TempMenuActivated:BOOLEAN;
BEGIN
  Obtain(MenuLock);
  TempMenuActivated := MenuActivated;
  Release(MenuLock);

  IF (NOT(TempMenuActivated)) THEN
    Obtain(MenuLock);
    MenuActivated := TRUE;
    Release(MenuLock);
    EnableCancelSequence(MenuCancelElement);
  END;(*IF*)
END SetCancelSequence;

(****************************************************************************)
(* FindMenuWithWindow                                                       *)
(****************************************************************************)
(* This procedure will find any menu that uses the window given.            *)
(* PARAMETER PASSED:                                                        *)
(*   w - window of interest.                                                *)
(*   BoxMenu - menu that uses this window.                                  *)
(*   element - element in menu link list.                                   *)
(*   list - menu list to check from.                                        *)
(****************************************************************************)
PROCEDURE FindMenuWithWindow(w:GraphicalWindow;
			     VAR (*OUT*) BoxMenu:BoxMenuTYPE;
			     VAR (*OUT*) element:ListElement;
			     MenuList:List):BOOLEAN;
VAR
  blFound:BOOLEAN;
BEGIN
  blFound := FALSE;

  ObtainList();

  GetFirstElementInList(element, MenuList);

  WHILE (ElementValid(element) AND NOT(blFound)) DO
    ReadObjectFromElement(BoxMenu, element);
    IF (w = BoxMenu^.TitleWindow) OR
       (w = BoxMenu^.DataWindow) THEN
      blFound := TRUE;
    ELSE
      GetNextElementInList(element, element);
    END;(*IF*)
  END;(*WHILE*)

  ReleaseList();

  RETURN (blFound);

END FindMenuWithWindow;


(****************************************************************************)
(* BoxSubMenuTitleMouseResponse                                             *)
(****************************************************************************)
(* This procedure will activate rcBoxSubMenuToBeChecked upon mouse         *)
(*      request.                                                            *)
(****************************************************************************)
PROCEDURE BoxSubMenuTitleMouseResponse(w:GraphicalWindow;
				       xOffset, yOffset:INTEGER;
                                       LeftPressed, LeftReleased,
				       DoubleClicked, Drag:BOOLEAN);
VAR
  FirstBoxMenu, TempBoxMenu:BoxMenuTYPE;
  FirstElement, TempElement:ListElement;
  TempMenuActivated:BOOLEAN;
  TempMenuBusy:BOOLEAN;

  (************** mine ************
        TitleDown : BOOLEAN;
  ********************************)

BEGIN

  (*TitleDown := FALSE; mine*)

  (*check if menu is busy executing a menu selection*)
  Obtain(MenuLock);
  TempMenuBusy := MenuBusy;
  Release(MenuLock);

  IF NOT(TempMenuBusy) THEN
    (*menu is not busy executing a procedure*)
    (*so let respond to mouse*)
    IF (FindMenuWithWindow(w, TempBoxMenu, TempElement, InactiveMenuList)) THEN
      (*only find menus from inactive list*)

      ResetALLMenuProcedure(MouseSource);

      IF (ObtainScreen()) THEN

        (*IF (LeftPressed) OR (DoubleClicked) THEN (* by me*)*)

        WindowTitleDown(TempBoxMenu^.TitleWindow);

        (*TitleDown := TRUE; (* mine *)
        END; (* mine *)*)

        (*IF (TitleDown) THEN (* mine *)*)

	DisplayBoxSubMenu(TempBoxMenu);
	
	(*TitleDown := FALSE; (* mine *)
	END; (* mine *)*)
	
	
	ReleaseScreen();			
	SetCancelSequence();
        ActivateCancelWhenMouseNolongerInWindow(TempBoxMenu^.TitleWindow);
      END;(*IF*)
    END;(*IF*)
  END;(*IF*)
END BoxSubMenuTitleMouseResponse;




(****************************************************************************)
(* BoxSubMenuMouseResponse                                                  *)
(****************************************************************************)
(* This procedure will execute the item in a menu upon mouse request        *)
(****************************************************************************)
PROCEDURE BoxSubMenuMouseResponse(w:GraphicalWindow;
				  xOffset, yOffset:INTEGER;
				  LeftPressed, LeftReleased,
				  DoubleClicked, Drag:BOOLEAN);
VAR
  TempBoxMenu:BoxMenuTYPE;
  TempBoxMenuItem:BoxMenuItemTYPE;
  TempElement:ListElement;
  TempXOffset, TempYOffset, inCount, ItemClicked:INTEGER;
  TempKeyPressed:KeyPressed;
  FoundActiveWindow,TempMenuBusy, blFound, ExecuteProcedure:BOOLEAN;
BEGIN
    blFound := FALSE;

    ExecuteProcedure := FALSE;
    FoundActiveWindow := FALSE;

    (*check if menu is busy executing a menu selection*)
    Obtain(MenuLock);
    TempMenuBusy := MenuBusy;
    Release(MenuLock);

    IF (NOT(TempMenuBusy)) THEN
      (*menu is not busy executing a procedure*)

      ObtainList();
      GetFirstElementInList(TempElement, ActiveMenuList);
      ReleaseList();

      IF (ElementValid(TempElement)) THEN
        FoundActiveWindow := TRUE;
        ReadObjectFromElement(TempBoxMenu, TempElement);
        IF (w = TempBoxMenu^.DataWindow) THEN
          blFound := TRUE;
        END;(*IF*)
      END;(*IF*)


      IF NOT(blFound) THEN
        (*found no normal menus, so try always active menus*)
        blFound := FindMenuWithWindow(w,TempBoxMenu, TempElement,AlwaysActiveMenuList);
        IF blFound THEN
          IF FoundActiveWindow THEN
            ResetALLMenuProcedure(MouseSource);
          END;(*IF*)
          SetCancelSequence();
        END;(*IF*)
      END;(*IF*)

      IF (blFound) THEN
        (*found that mouse has actually pointed to an active menu*)
        Obtain(MenuLock);

        Obtain(TempBoxMenu^.Lock);

        IF TempBoxMenu^.isText THEN
          (*modified by chee on 21-8 to remove a bug on selection of last item*)
          TempXOffset := xOffset DIV ((TempBoxMenu^.inMaxItemLength+1) * CharWidth);
          TempYOffset := yOffset DIV CharHeight;
        ELSE
          (*modified by chee on 21-8 to remove a bug on selection of last item*)
          TempXOffset := xOffset DIV (TempBoxMenu^.x_grid_size);
          TempYOffset := yOffset DIV TempBoxMenu^.y_grid_size;
        END; (*IF*)

        IF (TempXOffset >= TempBoxMenu^.x_size) THEN
          TempXOffset := TempBoxMenu^.x_size - 1;
        END;(*IF*)
        IF (TempYOffset >= TempBoxMenu^.y_size) THEN
          TempYOffset := TempBoxMenu^.y_size - 1;
        END;(*IF*)

        ItemClicked := TempXOffset * TempBoxMenu^.y_size + TempYOffset;

        TempBoxMenuItem := TempBoxMenu^.first_sub_menu_item_ptr;

        (*always count from second element because if first element is the one*)
        (*then would not have to go throung this*)
        FOR inCount := 1 TO (ItemClicked) DO
          TempBoxMenuItem := TempBoxMenuItem^.next_item_ptr;
        END;(*FOR*)

        (*execute function only if item responds to single click or when*)
        (*  mouse button is double clicked*)
        ExecuteProcedure := (TempBoxMenuItem^.RespondToSingleMouseClick AND LeftReleased)
                             OR DoubleClicked;

        IF NOT(TempBoxMenu^.isText) AND (TempBoxMenuItem <> NIL) THEN
          SetButtonHelp(TempBoxMenuItem , ExecuteProcedure);
        END;(*IF*)

        IF LeftPressed OR LeftReleased THEN
          TempBoxMenu^.prev_selected_sub_menu_item_ptr :=
                                  TempBoxMenu^.current_selected_sub_menu_item_ptr;
          TempBoxMenu^.current_selected_sub_menu_item_ptr := TempBoxMenuItem;

          IF (TempBoxMenu^.prev_selected_sub_menu_item_ptr <>
             TempBoxMenu^.current_selected_sub_menu_item_ptr) THEN
            IF (ObtainScreen()) THEN
              DisplayBoxSubMenu(TempBoxMenu);
              ReleaseScreen();
            END;(*IF*)
          END;(*IF*)

          IF (ExecuteProcedure) THEN
            (*disable reset from keyboard or mouse*)
            DisableCancelSequence(MenuCancelElement);

            IF (TempBoxMenuItem^.next_sub_menu_ptr <> NIL) THEN
              (*the item clicked it pointing to another bar menu, so display it*)
              (*DisplayBoxSubMenuTitle(TempBoxMenuItem^.next_sub_menu_ptr);*)
              IF (ObtainScreen()) THEN
                DisplayBoxSubMenu(TempBoxMenuItem^.next_sub_menu_ptr);
                ReleaseScreen();
              END;(*IF*)
            ELSE
              TempKeyPressed[0] := TempBoxMenuItem^.item_name^[TempBoxMenuItem^.WhichAlphabetAsKeyPress - 1];
              (*procedure to be executed*)
              MouseSources := TRUE;
              MouseMenu := TempBoxMenu;
              StuffKeyboard(BoxMenuKeyboard, TempKeyPressed);
            END;(*IF*)

            (*if the active menu list has no more elements,*)
            (* then there are no menu activated*)
            ObtainList();
            GetFirstElementInList(TempElement, ActiveMenuList);
            IF (NOT(ElementValid(TempElement))) THEN
              MenuActivated := FALSE;
            END;(*IF*)
            ReleaseList();

          END;(*IF*)
        END;(*IF*)
        Release(TempBoxMenu^.Lock);

        Release(MenuLock);
      END;(*IF*)
    END;(*IF*)
END BoxSubMenuMouseResponse;

PROCEDURE Caps(char:CHAR):CHAR;
BEGIN
  CASE char OF
    "a".."z":
      RETURN (VAL(CHAR, ORD(char) - ORD("a") + ORD("A")));
  |  ELSE
      RETURN(char);
  END;(*CASE*)
END Caps;


(****************************************************************************)
(* BoxSubMenuKeyboardSelection                                              *)
(****************************************************************************)
(* This procedure will execute the item in a menu upon keyboard request     *)
(****************************************************************************)
PROCEDURE BoxSubMenuKeyboardSelection(KeyPressed:KeyPressed);
VAR
  blFound:BOOLEAN;
  TempMenuBusy, TempMenuActivated:BOOLEAN;
  TempBoxMenu:BoxMenuTYPE;
  TempCurrentBoxMenuItem:BoxMenuItemTYPE;
  FirstElement:ListElement;
  ExecuteProcedure:BOOLEAN;
  ProcedureToExecute:BoxMenuProcedure;
  TempChar:CHAR;
  count:CARDINAL;
BEGIN
  ExecuteProcedure := FALSE;
  blFound := FALSE;

  (*check if menu is busy executing a menu selection*)
  Obtain(MenuLock);
  TempMenuBusy := MenuBusy;
  Release(MenuLock);

  IF (NOT(TempMenuBusy)) THEN
    (*menu is not busy executing a procedure*)
    (*so let respond to keyboard*)

    (*check if any menu is currently activated*)
    Obtain(MenuLock);
    TempMenuActivated := MenuActivated;
    Release(MenuLock);

    IF NOT(MouseSources) THEN
      ObtainList();
      (*always read from first element in active list*)
      GetFirstElementInList(FirstElement, ActiveMenuList);
      ReleaseList();
      IF (ElementValid(FirstElement)) THEN
        (*found something, so disable it*)
        ReadObjectFromElement(TempBoxMenu, FirstElement);
        blFound := TRUE;
      END;(*IF*)

      IF NOT(blFound) THEN
        (*found no normal menus, so try always active menus*)
        ObtainList();
        (*always read from first element in active list*)
        GetFirstElementInList(FirstElement, ActiveMenuList);
        ReleaseList();
        IF (ElementValid(FirstElement)) THEN
          (*found something, so disable it*)
          ReadObjectFromElement(TempBoxMenu, FirstElement);
          blFound := TRUE;
        END;(*IF*)
      END;(*IF*)
    ELSE
      TempBoxMenu := MouseMenu;
      blFound := TRUE;
    END;(*IF*)

    IF (MenuActivated) OR MouseSources THEN
      MouseSources := FALSE;

      IF (blFound) THEN
        TempCurrentBoxMenuItem := TempBoxMenu^.current_selected_sub_menu_item_ptr;

        (*some menu is activated, ro respond appropriately*)
        CASE KeyPressed[0] OF
          ESCKey:
            Obtain(MenuLock);
            MenuBusy := TRUE;
            Release(MenuLock);

            ActivateCancelSequence(KeyboardSource);

            (*make free so that response may continue as usual*)
            Obtain(MenuLock);
            MenuBusy := FALSE;
            Release(MenuLock);

        | EXTENDED_AUX:
            IF (KeyPressed[1] = UpArrowMain) OR (KeyPressed[1] = DownArrowMain) THEN

              TempBoxMenu^.prev_selected_sub_menu_item_ptr := TempCurrentBoxMenuItem;

              CASE (KeyPressed[1]) OF
                UpArrowMain:
                  TempBoxMenu^.current_selected_sub_menu_item_ptr :=
                     TempBoxMenu^.current_selected_sub_menu_item_ptr^.prev_item_ptr;
              | DownArrowMain:
                  TempBoxMenu^.current_selected_sub_menu_item_ptr :=
                     TempBoxMenu^.current_selected_sub_menu_item_ptr^.next_item_ptr;
              | ELSE
              END;(*CASE*)

              IF (ObtainScreen()) THEN
                DisplayBoxSubMenu(TempBoxMenu);
                ReleaseScreen();
              END;(*IF*)

            ELSIF (KeyPressed[1] = LeftArrowMain) OR (KeyPressed[1] = RightArrowMain) THEN

              TempBoxMenu^.prev_selected_sub_menu_item_ptr := TempCurrentBoxMenuItem;

              CASE (KeyPressed[1]) OF
                LeftArrowMain:
                  FOR count := 1 TO TempBoxMenu^.y_size DO
                    TempBoxMenu^.current_selected_sub_menu_item_ptr :=
                       TempBoxMenu^.current_selected_sub_menu_item_ptr^.prev_item_ptr;
                  END;(*IF*)
              | RightArrowMain:
                  FOR count := 1 TO TempBoxMenu^.y_size DO
                    TempBoxMenu^.current_selected_sub_menu_item_ptr :=
                       TempBoxMenu^.current_selected_sub_menu_item_ptr^.next_item_ptr;
                  END;(*IF*)
              | ELSE
              END;(*CASE*)

              IF (ObtainScreen()) THEN
                DisplayBoxSubMenu(TempBoxMenu);
                ReleaseScreen();
              END;(*IF*)
            END;(*IF*)

        | ReturnKey:
            IF (TempCurrentBoxMenuItem^.next_sub_menu_ptr <> NIL) THEN
              (*the item clicked it pointing to another bar menu, so display it*)
              IF (ObtainScreen()) THEN
                DisplayBoxSubMenu(TempCurrentBoxMenuItem^.next_sub_menu_ptr);
                ReleaseScreen();
              END;(*IF*)
            ELSE
              ExecuteProcedure := TRUE;
              (*procedure to be executed*)
              ProcedureToExecute := TempCurrentBoxMenuItem^.ProcedureToExecute;
            END;(*IF*)

        | "A".."Z", "a".."z", "0".."9":
            TempChar := Caps(KeyPressed[0]);

            TempCurrentBoxMenuItem := TempBoxMenu^.first_sub_menu_item_ptr;
            IF (TempCurrentBoxMenuItem <> NIL) THEN
              REPEAT
                IF (TempChar = Caps(TempCurrentBoxMenuItem^.item_name^[TempCurrentBoxMenuItem^.WhichAlphabetAsKeyPress - 1])) THEN

                  TempBoxMenu^.prev_selected_sub_menu_item_ptr := TempBoxMenu^.current_selected_sub_menu_item_ptr;
                  TempBoxMenu^.current_selected_sub_menu_item_ptr := TempCurrentBoxMenuItem;

                  IF (ObtainScreen()) THEN
                    DisplayBoxSubMenu(TempBoxMenu);
                    ReleaseScreen();
                  END;(*IF*)

                  IF (TempCurrentBoxMenuItem^.next_sub_menu_ptr <> NIL) THEN
                    (*the item clicked it pointing to another bar menu, so display it*)
                    IF (ObtainScreen()) THEN
                      DisplayBoxSubMenu(TempCurrentBoxMenuItem^.next_sub_menu_ptr);
                      ReleaseScreen();
                    END;(*IF*)
                  ELSE
                    ExecuteProcedure := TRUE;
                    (*procedure to be executed*)
                    ProcedureToExecute := TempCurrentBoxMenuItem^.ProcedureToExecute;
                  END;(*IF*)
        	END;(*IF*)
                TempCurrentBoxMenuItem := TempCurrentBoxMenuItem^.next_item_ptr;
              UNTIL (TempCurrentBoxMenuItem = TempBoxMenu^.first_sub_menu_item_ptr);
            END;(*IF*)

        | ELSE
        END;(*CASE*)
      END;(*IF*)

    ELSE
      (*no menu activated, to respond appropriately*)
      IF (ORD(KeyPressed[0]) >= ALTKeyBase) THEN
        (*respond only to alternate keys*)
        TempChar := VAL(CHAR, ORD(KeyPressed[0]) - 160);

        TempChar := Caps(TempChar);

        ObtainList();

        GetFirstElementInList(FirstElement, InactiveMenuList);

        WHILE (ElementValid(FirstElement) AND NOT(blFound)) DO
          ReadObjectFromElement(TempBoxMenu, FirstElement);
          IF ((TempBoxMenu^.WhichAlphabetAsAltKeyControlCharacter > 0) AND
              (TempChar = Caps(TempBoxMenu^.pchSubMenuName^[TempBoxMenu^.WhichAlphabetAsAltKeyControlCharacter - 1]))) THEN
            blFound := TRUE;
          ELSE
            GetNextElementInList(FirstElement, FirstElement);
          END;(*IF*)
        END;(*WHILE*)

        ReleaseList();

        IF ((blFound) AND (ObtainScreen())) THEN
          WindowTitleDown(TempBoxMenu^.TitleWindow);
          DisplayBoxSubMenu(TempBoxMenu);
          ReleaseScreen();
          SetCancelSequence();
          ActivateCancelWhenMouseNolongerInWindow(TempBoxMenu^.TitleWindow);
        END;(*IF*)
      END;(*IF*)
    END;(*IF*)
  END;(*IF*)

  IF (ExecuteProcedure) THEN
    (*make menu busy so that any respond reguest from keyboard or mouse*)
    (* to menu is ignored*)
    Obtain(MenuLock);
    MenuBusy := TRUE;
    Release(MenuLock);

    ProcedureToExecute();

    IF (ObtainScreen()) THEN
      ConditionalHideAllBoxSubMenu(TempBoxMenu);
      ReleaseScreen();
    END;(*IF*)

    (*if the active menu list has no more elements,*)
    (* then there are no menu activated*)
    ObtainList();
    GetFirstElementInList(FirstElement, ActiveMenuList);
    IF (NOT(ElementValid(FirstElement))) THEN
      Obtain(MenuLock);
      MenuActivated := FALSE;
      Release(MenuLock);
    END;(*IF*)
    ReleaseList();

    (*make free so that response may continue as usual*)
    Obtain(MenuLock);
    MenuBusy := FALSE;
    Release(MenuLock);
  END;(*IF*)
END BoxSubMenuKeyboardSelection;


(****************************************************************************)
(* AllocateWindow                                                           *)
(****************************************************************************)
(* This procedure will only create the window for the submenu               *)
(*      rcBoxSubMenu. As one observes, this will only be access once for    *)
(*      each barmenu                                                        *)
(****************************************************************************)
PROCEDURE AllocateWindow(VAR (*INOUT*) rcBoxSubMenu:BoxMenuTYPE);
VAR
  inCount:INTEGER;
BEGIN
  inCount := rcBoxSubMenu^.inNumberOfItems*CharHeight + 2*ActualDataOffset;

  IF (rcBoxSubMenu^.pchSubMenuName^[0] <> NULL) THEN
    (*there is a menu so allocate one row for it*)
    inCount := inCount + CharHeight + 2*ActualDataOffset;
  END;(*IF*)

  (* Create the appropriate windows for the menu... *)
  (* In this case we will new two separate windows. *)
  (* One will be used only for a title, the other   *)
  (* only for data (the menu items).                *)

  CreateGraphicalWindow(rcBoxSubMenu^.TitleWindow,
	       rcBoxSubMenu^.inSubMenuColumn,
	       rcBoxSubMenu^.inSubMenuRow,
	
         (* xsize data region (just wide enough for menu title) *)
	       rcBoxSubMenu^.inSubMenuColumn +
	                    CharWidth*INTEGER(Length(rcBoxSubMenu^.pchSubMenuName^)) +
	                    2*(ActualDataOffset + 4) - 1,
	
         (* ysize data region (big enough to hold title bar) *)
         rcBoxSubMenu^.inSubMenuRow + CharHeight + 2*ActualDataOffset,

	       rcBoxSubMenu^.caNormalTextBkColour,
	       rcBoxSubMenu^.pchSubMenuName,
	       rcBoxSubMenu^.WhichAlphabetAsAltKeyControlCharacter,
	       rcBoxSubMenu^.AlphabetColour,
	       rcBoxSubMenu^.SubMenuNameColour,
	       rcBoxSubMenu^.MenuNameBkColour,
	       TRUE,TRUE,TRUE, rcBoxSubMenu^.menu = AlwaysVisible, TRUE,TRUE,
	       BoxSubMenuTitleMouseResponse,
         DoNothing,
         rcBoxSubMenu^.cursor);  (* No response for data region *)

(* Create window for menu items.  This will depend on isText flag.  *)

IF rcBoxSubMenu^.isText THEN
  (* We will display menu items as text *)

  CreateGraphicalWindow(rcBoxSubMenu^.DataWindow,
	       rcBoxSubMenu^.inSubMenuColumn + rcBoxSubMenu^.x_item_pos,
	       rcBoxSubMenu^.inSubMenuRow + rcBoxSubMenu^.y_item_pos,
	
         (* right side of window *)
	       rcBoxSubMenu^.inSubMenuColumn + rcBoxSubMenu^.x_item_pos + CharWidth *
	                    (rcBoxSubMenu^.inMaxItemLength + 1) *
	                    (((rcBoxSubMenu^.inNumberOfItems - 1) DIV
	                    rcBoxSubMenu^.y_size) + 1) +
	                    2 * ActualDataOffset,

         (* bottom of window *)
         rcBoxSubMenu^.inSubMenuRow + rcBoxSubMenu^.y_item_pos +
         (((rcBoxSubMenu^.inNumberOfItems - 1) DIV
                      rcBoxSubMenu^.x_size) + 1)
                      * CharHeight +
                      CharHeight + 4 * ActualDataOffset,

	       rcBoxSubMenu^.caNormalTextBkColour,
	       rcBoxSubMenu^.pchSubMenuName,
	       rcBoxSubMenu^.WhichAlphabetAsAltKeyControlCharacter,
	       rcBoxSubMenu^.AlphabetColour,
	       rcBoxSubMenu^.SubMenuNameColour,
	       rcBoxSubMenu^.MenuNameBkColour,
	       TRUE,TRUE,TRUE,rcBoxSubMenu^.menu = AlwaysVisible,TRUE,TRUE,
         DoNothing,  (* No response for title region *)
	       BoxSubMenuMouseResponse,
	       rcBoxSubMenu^.cursor);
	
ELSE

  (* We will display menu items as icons *)

  CreateGraphicalWindow(rcBoxSubMenu^.DataWindow,
	       rcBoxSubMenu^.inSubMenuColumn + rcBoxSubMenu^.x_item_pos,
	       rcBoxSubMenu^.inSubMenuRow + rcBoxSubMenu^.y_item_pos,
	
         (* right side of window *)
	       rcBoxSubMenu^.inSubMenuColumn + rcBoxSubMenu^.x_item_pos +
	                    (rcBoxSubMenu^.x_grid_size) *
	                    (((rcBoxSubMenu^.inNumberOfItems - 1) DIV
	                    rcBoxSubMenu^.y_size) + 1) +
	                    2 * ActualDataOffset,

         (* bottom of window *)
         rcBoxSubMenu^.inSubMenuRow + rcBoxSubMenu^.y_item_pos +
         (((rcBoxSubMenu^.inNumberOfItems - 1) DIV
                      rcBoxSubMenu^.x_size) + 1)
                      * rcBoxSubMenu^.y_grid_size +
                      CharHeight + 4 * ActualDataOffset,

	       rcBoxSubMenu^.caNormalTextBkColour,
	       rcBoxSubMenu^.pchSubMenuName,
	       rcBoxSubMenu^.WhichAlphabetAsAltKeyControlCharacter,
	       rcBoxSubMenu^.AlphabetColour,
	       rcBoxSubMenu^.SubMenuNameColour,
	       rcBoxSubMenu^.MenuNameBkColour,
	       TRUE,TRUE,TRUE,rcBoxSubMenu^.menu = AlwaysVisible,TRUE,TRUE,
         DoNothing,  (* No response for title region *)
	       BoxSubMenuMouseResponse,
	       rcBoxSubMenu^.cursor);
END;
END AllocateWindow;

(****************************************************************************)
(* DisplayBoxSubMenuTitle                                                   *)
(****************************************************************************)
(* This procedure will display the title of the sub menu passed as          *)
(*      rcBoxSubMenu.                                                       *)
(****************************************************************************)
PROCEDURE DisplayBoxSubMenuTitle(VAR (*INOUT*) rcBoxSubMenu:BoxMenuTYPE);
VAR
  chTitleAltKeyChar:ARRAY [0..1] OF CHAR;
BEGIN
  IF (rcBoxSubMenu^.WindowNOTCreated) THEN
    (*window has not been allocated yet, so do so*)
    rcBoxSubMenu^.WindowNOTCreated := FALSE;
    AllocateWindow(rcBoxSubMenu);
  END;

  IF (ObtainScreen()) THEN
    DisplayWindowTitle(rcBoxSubMenu^.TitleWindow);
    ReleaseScreen();
  END;(*IF*)
END DisplayBoxSubMenuTitle;

(****************************************************************************)
(* DisplayBoxSubMenu                                                        *)
(****************************************************************************)
(* This procedure will display the actual menu passed as rcBoxSubMenu       *)
(****************************************************************************)
PROCEDURE DisplayBoxSubMenu(VAR (*INOUT*) rcBoxSubMenu:BoxMenuTYPE);
VAR
  TempBoxSubMenuFirstItem:BoxMenuItemTYPE;
  TempBoxSubMenuItem:BoxMenuItemTYPE;
  count:INTEGER;
BEGIN
  IF (rcBoxSubMenu^.WindowNOTCreated) THEN
    (*window has not been allocated yet, so do so*)
    rcBoxSubMenu^.WindowNOTCreated := FALSE;
    AllocateWindow(rcBoxSubMenu);
  END;

  IF (DataRegionDisplayed(rcBoxSubMenu^.DataWindow)) THEN
    UnHighLightItem(rcBoxSubMenu^.prev_selected_sub_menu_item_ptr);
  ELSE
    IF (rcBoxSubMenu^.menu = Normal) THEN
      ActivateCancelWhenMouseNolongerInWindow(rcBoxSubMenu^.DataWindow);
    END;(*IF*)

    DisplayWindowData(rcBoxSubMenu^.DataWindow);

    TempBoxSubMenuFirstItem := rcBoxSubMenu^.first_sub_menu_item_ptr;
    TempBoxSubMenuItem := TempBoxSubMenuFirstItem;
    IF (TempBoxSubMenuFirstItem <> NIL) THEN
      count := 0;
      IF rcBoxSubMenu^.isText THEN
      (* draw items as strings *)
      REPEAT
        PutString(rcBoxSubMenu^.DataWindow, TempBoxSubMenuItem^.item_name^,

          (count DIV rcBoxSubMenu^.y_size) * CharWidth *
          (rcBoxSubMenu^.inMaxItemLength + 1) + 3,

	        (count MOD (rcBoxSubMenu^.y_size)) * CharHeight,
	
	        rcBoxSubMenu^.caNormalTextFrColour,FALSE);
	
          IF (TempBoxSubMenuItem^.WhichAlphabetAsKeyPress > 0) THEN
            PutChar(TempBoxSubMenuItem^.this_sub_menu_ptr^.DataWindow,
                TempBoxSubMenuItem^.item_name^[TempBoxSubMenuItem^.WhichAlphabetAsKeyPress - 1],

                (count DIV rcBoxSubMenu^.y_size) * CharWidth *
                (rcBoxSubMenu^.inMaxItemLength + 1) + 3 +
                CharWidth * (TempBoxSubMenuItem^.WhichAlphabetAsKeyPress- 1),
	        (count MOD (rcBoxSubMenu^.y_size)) * CharHeight,
                TempBoxSubMenuItem^.AlphabetColour,FALSE);
          END;(*IF*)

          TempBoxSubMenuItem := TempBoxSubMenuItem^.next_item_ptr;
          INC(count);
        UNTIL (TempBoxSubMenuItem = TempBoxSubMenuFirstItem);
      ELSE;
        (* draw items as icons *)
      REPEAT
        TempBoxSubMenuItem^.IconDraw(rcBoxSubMenu^.DataWindow,
          (count DIV rcBoxSubMenu^.y_size) *
          (rcBoxSubMenu^.x_grid_size) + 3 + TempBoxSubMenuItem^.x,

	  (count MOD (rcBoxSubMenu^.y_size)) *
	  rcBoxSubMenu^.y_grid_size + TempBoxSubMenuItem^.y);	

          TempBoxSubMenuItem := TempBoxSubMenuItem^.next_item_ptr;
          INC(count);
        UNTIL (TempBoxSubMenuItem = TempBoxSubMenuFirstItem);


      END;

    END;(*IF*)
  END;(*IF*)

  (*new version of highlighting an item*)
  HighLightItem(rcBoxSubMenu^.current_selected_sub_menu_item_ptr);

  IF (rcBoxSubMenu^.menu = Normal) THEN
    MoveToTop(rcBoxSubMenu^.LinkListElement, ActiveMenuList);
    EnableCancelSequence(MenuCancelElement);
  ELSE
    MoveToTop(rcBoxSubMenu^.LinkListElement, AlwaysActiveMenuList);
  END;(*IF*)
END DisplayBoxSubMenu;

(****************************************************************************)
(* CreateBoxSubMenu                                                         *)
(****************************************************************************)
(* This procedure will create the necessary structure for a bar menu.       *)
(*     All the items to for selection must be setup using                   *)
(*         BoxSubMenuItemSetup.                                             *)
(*                                                                          *)
(*                                                                          *)
(* PARAMETER PASSED:                                                        *)
(*    pchSubMenuName - name of the bar menu                                 *)
(*    isText - TRUE if Text menu FALSE if icons                             *)
(*    WhichAlphabetAsAltKeyControlCharacter - which alphabet in name to     *)
(*              trigger ALT-? bar menu selection                            *)
(*    AlphabetColour - the colour fo this special character                 *)
(*    SubMenuNameColour, MenuNameBkColour - colour of text for of sub menu  *)
(*                                          name and its background         *)
(*    inSubMenuColumn - left coordinate of menu                             *)
(*    inSubMenuRow - top coordinate of the menu                             *)

(*    x_item_pos, y_item_pos - where the items will appear wrt.             *)
(*                             a normal menu's position                     *)

(*    x_size, y_size - number of cols and rows in menu                      *)

(*    x_grid_size, y_grid_size - size of grid that icons are drawn inside   *)
(*                               (only relevant if name is PTR to NULL)     *)

(*    caHighlightTextFrColour - text colour of highlighted item             *)
(*    caHighlightTextBkColour - background clour of item selected           *)
(*    caNormalTextFrColour - normal text colour                             *)
(*    caNormalTextBkColour - normal background colour for items             *)
(*    Menu - this sets the menu to be a normal menu or always visible.      *)
(*    cursor - procedure to check for mouse cursor.                         *)
(****************************************************************************)
PROCEDURE CreateBoxSubMenu(
    pchSubMenuName:LongStr;
    isText:BOOLEAN;
    WhichAlphabetAsAltKeyControlCharacter:INTEGER;
    AlphabetColour:ColourType;
    SubMenuNameColour, MenuNameBkColour:ColourType;
    inSubMenuColumn:INTEGER;
    inSubMenuRow:INTEGER;
    x_item_pos, y_item_pos:INTEGER;
    x_size, y_size:INTEGER;
    x_grid_size, y_grid_size: INTEGER;
    caHighlightTextFrColour:ColourType;
    caHighlightTextBkColour:ColourType;
    caNormalTextFrColour:ColourType;
    caNormalTextBkColour:ColourType;
    Menu:MenuType;
    cursor:CursorProcedure):BoxMenuTYPE;
VAR
  rcTempMenu:BoxMenuTYPE;
BEGIN
  NEW(rcTempMenu);

  (*menu never displayed before*)
  rcTempMenu^.WindowNOTCreated := TRUE;

  (*setup name for this sub menu*)
  ALLOCATE(rcTempMenu^.pchSubMenuName,Length(pchSubMenuName)+1);

  rcTempMenu^.pchSubMenuName^[0] := NULL;

  IF (pchSubMenuName[0] <> NULL) THEN
    Copy(rcTempMenu^.pchSubMenuName^, pchSubMenuName);
  END;(*IF*)

  rcTempMenu^.isText := isText;

  rcTempMenu^.SubMenuNameColour := SubMenuNameColour;
  rcTempMenu^.MenuNameBkColour := MenuNameBkColour;

  rcTempMenu^.WhichAlphabetAsAltKeyControlCharacter:=WhichAlphabetAsAltKeyControlCharacter;
  rcTempMenu^.AlphabetColour := AlphabetColour;

  rcTempMenu^.inSubMenuRow:=inSubMenuRow;
  rcTempMenu^.inSubMenuColumn:=inSubMenuColumn;

  rcTempMenu^.x_item_pos := x_item_pos;
  rcTempMenu^.y_item_pos := y_item_pos;

  rcTempMenu^.x_size := x_size;
  rcTempMenu^.y_size := y_size;

  rcTempMenu^.inMaxItemLength:=0;

  rcTempMenu^.x_grid_size := x_grid_size;
  rcTempMenu^.y_grid_size := y_grid_size;

  rcTempMenu^.prev_Level_sub_menu_item_ptr:=NIL;

  rcTempMenu^.current_selected_sub_menu_item_ptr:=NIL;
  rcTempMenu^.first_sub_menu_item_ptr:=NIL;
  rcTempMenu^.last_sub_menu_item_ptr:=NIL;
  rcTempMenu^.inNumberOfItems:=0;

  (*set colours*)
  rcTempMenu^.caHighlightTextFrColour:=caHighlightTextFrColour;
  rcTempMenu^.caHighlightTextBkColour:=caHighlightTextBkColour;
  rcTempMenu^.caNormalTextFrColour:=caNormalTextFrColour;
  rcTempMenu^.caNormalTextBkColour:=caNormalTextBkColour;


  CreateListElement(InactiveMenuList,rcTempMenu^.LinkListElement,rcTempMenu);

  CreateLock(rcTempMenu^.Lock,"pchSubMenuName");

  rcTempMenu^.menu := Menu;
  rcTempMenu^.cursor := cursor;
  RETURN (rcTempMenu);
END CreateBoxSubMenu;


(****************************************************************************)
(* BoxSubMenuItemSetup                                                      *)
(****************************************************************************)
(* This procedure will setup one of the items to be included into bar menu  *)
(*     passed as sub_menu_ptr.  Note that sub_menu_ptr MUST be created prior*)
(*     to using this procedure.  Items setup by this procedure will follow  *)
(*     the sequence they are setup.                                         *)
(*                                                                          *)
(* PARAMETER PASSED:                                                        *)
(*    sub_menu_ptr - bar menu that will contain this item                   *)
(*    blIfCloseMenu - boolean to set that if this item is executed, TRUE    *)
(*                    will close the menu that contains this item           *)
(*    blIfExecutedHideAllMenus - boolean to set the item selection handler  *)
(*              to clear all menu upon execution of this item, only if      *)
(*              blIfCloseMenu is set TRUE.                                  *)
(*    item_name - name of the item                                          *)
(*    IconDraw - procedure to draw the icons                                *)
(*    x, y -  the x and y offset of the icon (relative to grid and hotspot  *)

(*         above 3 integer args are irrelevant if using strings             *)

(*    WhichAlphabetAsKeyPress - which alphabet in name to execute an item   *)
(*              by pressing this character.                                 *)
(*    AlphabetColour - the colour fo this special character                 *)
(*    next_sub_menu_ptr - if this item calls another menu, then the menu    *)
(*                        will have to be created first and then passed to  *)
(*                        this parameter                                    *)
(*    RespondToSingleMouseClick - this enables an item to respond even to   *)
(*                                single mouse click                        *)
(*    FunctionToExecute - function to execute the action for this item      *)
(****************************************************************************)
PROCEDURE BoxSubMenuItemSetup(
    sub_menu_ptr:BoxMenuTYPE;
    blIfCloseMenu:BOOLEAN;
    blIfExecutedHideAllMenus:BOOLEAN;
    item_name:LongStr;
    IconDraw:BoxMenuIconDrawProcedure;
    x, y : INTEGER;
    WhichAlphabetAsKeyPress:INTEGER;
    AlphabetColour:ColourType;
    next_sub_menu_ptr:BoxMenuTYPE;
    RespondToSingleMouseClick:BOOLEAN;
    ProcedureToExecute:BoxMenuProcedure):BoxMenuItemTYPE;
VAR
  rcTempItem:BoxMenuItemTYPE;
  rcTempBoxMenu:BoxMenuTYPE;
BEGIN
  NEW(rcTempItem);
  (*to register the number for this item to be used in displaying routines*)
  rcTempItem^.item_number := sub_menu_ptr^.inNumberOfItems;
  rcTempItem^.IconDraw := IconDraw;
  rcTempItem^.x := x;
  rcTempItem^.y := y;

  (*IF this item is executed, then will hide the menu provided, that contain*)
  (*    this item*)
  rcTempItem^.blIfCloseMenu := blIfCloseMenu;

  (*IF this item is executed, then will hide all menus provided*)
  (*    blIfExecutedHideAllMenus := TRUE*)
  rcTempItem^.blIfExecutedHideAllMenus := blIfExecutedHideAllMenus;

  (*pointer to sub menu that contains this item*)
  rcTempItem^.this_sub_menu_ptr := sub_menu_ptr;

  (*set the sub menu pointer of this item to next item IF this item will*)
  (*    invoke another sub menu*)
  rcTempItem^.next_sub_menu_ptr := next_sub_menu_ptr;
  IF (next_sub_menu_ptr <> NIL) THEN
    (*there is a sub menu connected to this item*)
    rcTempBoxMenu := next_sub_menu_ptr;
    rcTempBoxMenu^.prev_Level_sub_menu_item_ptr := rcTempItem;
  END;(*IF*)

  (*set this item as first element IF the previous item pointer is NIL*)
  IF (sub_menu_ptr^.first_sub_menu_item_ptr = NIL) THEN
    sub_menu_ptr^.first_sub_menu_item_ptr := rcTempItem;
    sub_menu_ptr^.last_sub_menu_item_ptr := rcTempItem;
    sub_menu_ptr^.current_selected_sub_menu_item_ptr := rcTempItem;
    sub_menu_ptr^.inMaxItemLength := Length(item_name);
  END;(*IF*)

  (*previous item of this item is the last item registered in sub menu*)
  rcTempItem^.prev_item_ptr := sub_menu_ptr^.last_sub_menu_item_ptr;

  (*always set previous item of first item in menu to this item*)
  sub_menu_ptr^.first_sub_menu_item_ptr^.prev_item_ptr := rcTempItem;

  (*next item of previous item is this item*)
  sub_menu_ptr^.last_sub_menu_item_ptr^.next_item_ptr := rcTempItem;

  (*always set this item as last element*)
  sub_menu_ptr^.last_sub_menu_item_ptr := rcTempItem;

  (*next item of this item is first item in the menu*)
  rcTempItem^.next_item_ptr := sub_menu_ptr^.first_sub_menu_item_ptr;

  INC(sub_menu_ptr^.inNumberOfItems);

  (*set name for this item*)
  ALLOCATE(rcTempItem^.item_name, Length(item_name)+2);

  Copy(rcTempItem^.item_name^, item_name);

  IF (next_sub_menu_ptr <> NIL) THEN
    (*there is a sub menu connected to this item, add a [] symbol to*)
    (*  indocate so*)
    Append(rcTempItem^.item_name^, "");
  END;(*IF*)

  rcTempItem^.WhichAlphabetAsKeyPress := WhichAlphabetAsKeyPress;
  rcTempItem^.AlphabetColour := AlphabetColour;

  IF (Length(rcTempItem^.item_name^) > VAL(CARDINAL, sub_menu_ptr^.inMaxItemLength)) THEN
    (*IF the name is longer than any registered, then update the maximum*)
    (*  length*)
    sub_menu_ptr^.inMaxItemLength := VAL(INTEGER, Length(rcTempItem^.item_name^));
  END;(*IF*)

  (*set menu item to respond to even single mouse click*)
  rcTempItem^.RespondToSingleMouseClick := RespondToSingleMouseClick;

  (*set the function to be called for this item*)
  rcTempItem^.ProcedureToExecute := ProcedureToExecute;

  RETURN (rcTempItem);
END BoxSubMenuItemSetup;






(****************************************************************************)
(* DestroyBoxSubMenu                                                        *)
(****************************************************************************)
(* This procedure will free rcBoxSubMenu upon return                       *)
(****************************************************************************)
PROCEDURE DestroyBoxSubMenu(VAR (*IN*) rcBoxSubMenu:BoxMenuTYPE);
VAR
  rcBoxSubMenuItem:BoxMenuItemTYPE;
  rcOldBoxSubMenuItem:BoxMenuItemTYPE;
BEGIN
  rcBoxSubMenuItem := rcBoxSubMenu^.first_sub_menu_item_ptr;

  WHILE (rcBoxSubMenuItem <> NIL) DO
    IF (rcBoxSubMenuItem^.next_sub_menu_ptr <> NIL) THEN
	 (*there is a sub menu connected to this item, so free it*)
      DestroyBoxSubMenu(rcBoxSubMenuItem^.next_sub_menu_ptr);
    END;(*IF*)
    rcOldBoxSubMenuItem := rcBoxSubMenuItem;
    rcBoxSubMenuItem := rcBoxSubMenuItem^.next_item_ptr;
    DISPOSE (rcOldBoxSubMenuItem);
  END;(*while*)


  DISPOSE (rcBoxSubMenu);
END DestroyBoxSubMenu;

(****************************************************************************)
(* MenuKeyboardTask                                                         *)
(****************************************************************************)
(*  This procedure handles menu selection and execute proper procedures     *)
(****************************************************************************)
PROCEDURE MenuKeyboardTask();
VAR
  TempKeyPressed:KeyPressed;
BEGIN
  LOOP (*FOREVER*)
    ReadFromKeyboard(BoxMenuKeyboard, TempKeyPressed);
    BoxSubMenuKeyboardSelection(TempKeyPressed);
  END;(*LOOP*)
END MenuKeyboardTask;


(****************************************************************************)
(* NoIconDraw                                                               *)
(****************************************************************************)
(* This procedure is just a dummy procedure passed for no icon draw         *)
(****************************************************************************)
PROCEDURE NoIconDraw(w:GraphicalWindow; x,y:INTEGER);
BEGIN
END NoIconDraw;



(******************* added by Matthew Ma ********************)
(******************************************************)
(* This procedure hides the menu button 			*)
(******************************************************)
PROCEDURE HideMenu(m:BoxMenuTYPE);
BEGIN
   HideWindowData(m^.DataWindow);
END HideMenu;
(************************************************************)



BEGIN
  CreateLinkList(InactiveMenuList);
  CreateLinkList(ActiveMenuList);
  CreateLinkList(AlwaysActiveMenuList);
  CreateLock(MenuLock,"MenuLock");
  MenuActivated := FALSE;
  MenuBusy := FALSE;
  MouseSources := FALSE;

  (*setup keyboard link*)
  SetupKeyboardLink(BoxMenuKeyboard);
  ConnectToKeyboard(BoxMenuKeyboard);

  CreateTask(MenuKeyboardTask, 1, "Menu Keyboard");

  RegisterCancelSequence(ResetALLMenuProcedure, MenuCancelElement);

   BoxMenuHelpWindowMaxChars := 14;
   BoxMenuHelpWindowColour := 13;

  ButtonsHelp.item := NIL;
  ButtonsHelp.oncount := 0;
  ButtonsHelp.displayed := FALSE;

  HelpWindowWidth := BoxMenuHelpWindowMaxChars*CharWidth+2*ActualDataOffset;
  HelpWindowHeight := CharHeight + 2*ActualDataOffset;

  CreateGraphicalWindow(ButtonsHelp.w,0,0,HelpWindowWidth,HelpWindowHeight,
                        BoxMenuHelpWindowColour, NIL,0,0,0,0,
	                TRUE,TRUE,TRUE, FALSE, TRUE,TRUE,
                        DoNothing, DoNothing, (*SetDefCursor*)SetArrow);


END Buttons.