IMPLEMENTATION MODULE CancelSequences;

(****************************************************************************)
(*      FILE NAME   : CANCELSE.MOD                                          *)
(*                                                                          *)
(*      WRITTEN BY  : Chee Hong, KOK                                        *)
(*      STUD. NO.   : 9328853                                               *)
(*      COURSE      : BACHELOR OF ELECTRICAL ENGINEERING                    *)
(*                                                                          *)
(*      DATE START  : 22 MARCH 1996                                         *)
(*      LAST EDITED : 13 APRIL 1996                                         *)
(*                                                                          *)
(*      DESCRIPTION :                                                       *)
(*                                                                          *)
(*                  This module provides some simple procedures to provide  *)
(*             general abort link for keyboard and mouse.                   *)
(*                                                                          *)
(*                  The main aim is to provide a simple cancellation control*)
(*             when a certain job is to be executed upon cancellation of    *)
(*             jobs.                                                        *)
(*                                                                          *)
(*                  For example, if right button of mouse has the function  *)
(*             of cancelling any job currently activated, such as entering  *)
(*             a file name, then upon pressing right button, a proper       *)
(*             procedure should be activated to cancel the job.  This       *)
(*             modules provides the link between mouse and any other task(s)*)
(*             that wish(es) to be cancelled, enabling mouse to be more     *)
(*             general in coding.                                           *)
(*                                                                          *)
(*                                                                          *)
(*      STATUS      : ok.  working.                                         *)
(*                                                                          *)
(****************************************************************************)

FROM Semaphores IMPORT
(*type*)Semaphore,
(*proc*)CreateSemaphore, Wait, Signal;

FROM Storage1 IMPORT
(*proc*)ALLOCATE, DEALLOCATE;

FROM LinkList IMPORT
(*type*)List, ListElement,
(*proc*)CreateLinkList, CreateListElement, MoveToTop, GetFirstElementInList,
	GetNextElementInList, ElementValid,ReadObjectFromElement, ObtainList,
	ReleaseList;

FROM TaskControl IMPORT
(*var*)Lock,
(*proc*)CreateTask, CreateLock, Obtain, Release;


TYPE
  CancelElement = ListElement;

VAR

  (*list of active reset procedures*)
  ActiveCancelSequenceList:List;

  (*list of active reset procedures*)
  InactiveCancelSequenceList:List;

  MainCancelSource:CancelSource;

  SignalActivateCancelTask:Semaphore;

  CancelLock:Lock;

(****************************************************************************)
(* RegisterCancelSequence                                                   *)
(****************************************************************************)
(* registers a cancel sequence to be called.                                *)
(*  this reset could be initiated by either mouse or keyboard               *)
(* PARAMETER PASSED:                                                        *)
(*   procedure - actual procedure to be called upon reset                   *)
(*   CancelElement - the data structure to hold reset information            *)
(****************************************************************************)
PROCEDURE RegisterCancelSequence(procedure:CancelSequence;
				 VAR (*OUT*) CancelElement:CancelElement);
VAR
  Procedure:POINTER TO CancelSequence;
BEGIN
  (*allocate a new element*)
  NEW(Procedure);

  (*remember the procedure to be called*)
  Procedure^ := procedure;

  (*put element onto inactive list*)
  ObtainList();
  CreateListElement(InactiveCancelSequenceList, CancelElement, Procedure);
  ReleaseList();
END RegisterCancelSequence;

(****************************************************************************)
(* EnableCancelSequence                                                     *)
(****************************************************************************)
(* enables a cancel sequence to be called.                                  *)
(*  this reset could be initiated by either mouse or keyboard               *)
(* IMPORTANT!!                                                              *)
(* will have effect only if the CancelElement is NOT the first in the list   *)
(* PARAMETER PASSED:                                                        *)
(*   CancelElement - the data structure to hold reset information            *)
(****************************************************************************)
PROCEDURE EnableCancelSequence(VAR (*OUT*) CancelElement:CancelElement);
BEGIN
  (*move a cancel element, effectively the procedure to be called*)
  (*to the top of active list*)
  ObtainList();
  MoveToTop(CancelElement, ActiveCancelSequenceList);
  ReleaseList();
END EnableCancelSequence;

(****************************************************************************)
(* DisableCancelSequence                                                    *)
(****************************************************************************)
(* disables a cancel sequence to be called.                                 *)
(*  this reset could be initiated by either mouse or keyboard               *)
(* IMPORTANT!!                                                              *)
(* will have effect only if the CancelElement is NOT the first in the list   *)
(* PARAMETER PASSED:                                                        *)
(*   CancelElement - the data structure to hold reset information            *)
(****************************************************************************)
PROCEDURE DisableCancelSequence(VAR (*OUT*) CancelElement:CancelElement);
BEGIN
  (*move a cancel element, effectively the procedure to be called*)
  (*to the top of inactive list*)
  ObtainList();
  MoveToTop(CancelElement, InactiveCancelSequenceList);
  ReleaseList();
END DisableCancelSequence;

(****************************************************************************)
(* ActivateCancelSequenceTask                                               *)
(****************************************************************************)
(* activates a reset procedure to be called.                                *)
(*  this reset could be initiated by either mouse or keyboard               *)
(****************************************************************************)
PROCEDURE ActivateCancelSequenceTask();
VAR
  FirstElement:ListElement;
  Procedure:POINTER TO CancelSequence;
  TempSource:CancelSource;
BEGIN
  LOOP
    Wait(SignalActivateCancelTask);

    Obtain(CancelLock);
    TempSource := MainCancelSource;
    Release(CancelLock);

    ObtainList();

    (*finds the first sequence in active list to be executed*)
    GetFirstElementInList(FirstElement,ActiveCancelSequenceList);

    ReleaseList();

    IF (ElementValid(FirstElement)) THEN
      (*found an element*)
      ReadObjectFromElement(Procedure,FirstElement);
      Procedure^(TempSource);
    END;(*WHILE*)

  END;(*LOOP*)
END ActivateCancelSequenceTask;


(****************************************************************************)
(* ActivateCancelSequence                                                   *)
(****************************************************************************)
(* activates a reset procedure to be called.                                *)
(*  this reset could be initiated by either mouse or keyboard               *)
(****************************************************************************)
PROCEDURE ActivateCancelSequence(source:CancelSource);
BEGIN
  Obtain(CancelLock);
  MainCancelSource := source;
  Release(CancelLock);
  Signal(SignalActivateCancelTask);
END ActivateCancelSequence;


BEGIN

  (*creating list of active sequence to be called*)
  CreateLinkList(ActiveCancelSequenceList);

  (*creating inactive list to keep any sequence*)
  (*that would not have to be called*)
  CreateLinkList(InactiveCancelSequenceList);

  CreateSemaphore(SignalActivateCancelTask, 0);

  CreateLock(CancelLock,"CancelLock");

  CreateTask(ActivateCancelSequenceTask,1,"Cancel Task");

END CancelSequences.
