IMPLEMENTATION MODULE Canvas;

(******************************************************************)
(*											*)
(*			  	    Canvas Module					*)
(*											*)
(*	Programmer :	Matthew Ma (c9315720)				*)
(*											*)
(*	Last edited :	22-10-97						*)
(*											*)
(*	Function : 		This module draws the canvas window and	*)
(*				implements the canvas task.			*)
(*											*)
(*	Status :		Working.						*)
(******************************************************************)


FROM Toolbox IMPORT
(* var  *)   marquee_x1, marquee_y1, marquee_x2, marquee_y2,
(* proc *)   GetMarqueex1y1, GetMarqueex2y2;

IMPORT ColourRoutines;

IMPORT ImageRoutines;

FROM FIOR IMPORT
(* proc *)   IsExtension;

FROM InfoDialogs IMPORT
(* proc *)   FileExists, UnknownFormat, FileNotFound, UniqueColours;

IMPORT FIO;

FROM Str IMPORT
(* proc *)   CardToStr;

FROM Conversions IMPORT
(* proc *)   CardinalToString;

FROM WriteRAW IMPORT
(* proc *)   WriteRawData;

FROM WriteBMP IMPORT
(* proc *)   Write256BMP;

FROM WritePCX IMPORT
(* proc *)   Write256PCX;

FROM WriteTGA IMPORT
(* proc *)   Write256TGA;

FROM WriteTIF IMPORT
(* proc *)   Write256TIF;

FROM Graphics IMPORT
(* proc *)   Point;

FROM ColourRoutines IMPORT
(* var  *)   ForegroundColour,
(* proc *)   BrightnessControl;

FROM Str IMPORT
(* proc *)   Copy, Length, CharPos, Insert, Append, Caps;

FROM FileRoutines IMPORT
(* var  *)   OpenFileName, SaveAsFileName, CaptName, Actions, FileTypes, new_image, open,
             extension, format, modified, ReplaceFile;

IMPORT ColourRoutines;

IMPORT Graph;

IMPORT Graphics;

FROM Toolbox IMPORT
(* proc *)   FileName;

IMPORT ReadBMP;

IMPORT ReadPCX;

IMPORT ReadTGA;

IMPORT ReadTIF;

FROM Toolbox IMPORT
(* proc *)   Getx1y1, StopRoutine, StartRoutine,
             FreehandRoutine, LineRoutine, CircleRoutine, SolidCircleRoutine, GetPolyx1y1,
	       RectangleRoutine, SolidRectangleRoutine, SquareRoutine, PolygonRoutine, ClosePolygon,
	       SolidSquareRoutine, SmallEraserRoutine, MediumEraserRoutine, LargeEraserRoutine,
	       TextRoutine, ClearTextBuffer, AirBrushRoutine, HVLineRoutine, DropperRoutine,
	       SmallBrushRoutine, MediumBrushRoutine, MarqueeRoutine, LargeBrushRoutine;


FROM Bars IMPORT
(* proc *)   UpdateStatusBar, UpdateTitleBar, UpdateForegroundColour, UpdateTestColour;

FROM TaskControl IMPORT
(* type *)   Lock,
(* proc *)   CreateTask, CreateLock, Obtain, Release;

FROM Storage IMPORT
(* proc *)   ALLOCATE, DEALLOCATE;

FROM GraphicWindow IMPORT
(* var  *)   XMAX, YMAX, MouseParatemer_xoffset, MouseParatemer_yoffset,
(* proc *)   CreateGraphicalWindow,DoNothing,DisplayWindowData,DisplayWindowTitle,
             ReleaseScreen,ObtainScreen,PutChar,PutCircle,PutLine, MoveImage,
          	 PutRectangle, ClearDataRegion, PutString, PutDot,
        	 RespondToMouseButtons, Line, DisableScreen, UnconditionalObtainScreenWithMouseHide,
		 GraphicalWindow, ColourType, Rectangle, GetWindowLeft, GetWindowTop,
	  	 GetWindowRight, GetWindowBottom;

FROM Mailboxes IMPORT
(* type *)   Mailbox,
(* proc *)   CreateMailbox, SendMessage, ReceiveMessage;

FROM MouseRoutines IMPORT
(* proc *)   SetArrow, SetCrossHair, SetFinger,
	   	 SetSmallEraser, SetMediumEraser, SetLargeEraser,
	   	 SetText, SetMagnifier, SetPencil, SetDropper, HideMouse, DisplayMouse,
		 SetAirbrush, SetSmallBrush, SetMediumBrush, SetLargeBrush;

FROM Icons IMPORT
(* const *)  arrow, freehand, line, rectangle, solid_rectangle, circle, solid_circle,
             text, eraser, floodfill, airbrush, dropper, marquee, zoom, crosshair, polygon,
		 small_eraser, medium_eraser, large_eraser, hv_line, square, solid_square,
		 capture1, capture2, brush, small_brush, medium_brush, large_brush;

FROM SoundEffects IMPORT
(* proc *)   Beep;


CONST
   null = CHR(0);
   dummy = 99;


VAR
   CanvasMailbox : Mailbox;

   global_icon_id : CARDINAL;

   CanvasLock : Lock;

   counter (*x1, y1, x2, y2*) : INTEGER;

   IsDropper, IsPolygon : BOOLEAN;

   OldColour : SHORTCARD;
   fill_x1, fill_y1 : INTEGER;
   prev_x, prev_y : INTEGER;



(******************************************************************)
(* This procedure sends a mail message to the canvas task.		*)
(******************************************************************)
PROCEDURE MsgCanvas(command, icon_id : CARDINAL);
VAR
   mail : ARRAY[0..1] OF CHAR;
   ok : BOOLEAN;
BEGIN
   mail[0] := CHR(command);
   mail[1] := CHR(icon_id);

   ok := SendMessage(CanvasMailbox, ADR(mail), 2);
END MsgCanvas;


(******************************************************************)
(* This procedure receives the mail message sent to the canvas	*)
(* task.										*)
(******************************************************************)
PROCEDURE GetMail(VAR command : CARDINAL; VAR icon_id : CARDINAL) : BOOLEAN;
VAR
   length : CARDINAL;
   mail : ARRAY[0..1] OF CHAR;
BEGIN

   IF ReceiveMessage(CanvasMailbox, mail, 0) = 2 THEN
      command := ORD(mail[0]);
      icon_id := ORD(mail[1]);

      RETURN TRUE;
   ELSE
      RETURN FALSE;
   END; (*IF*)

END GetMail;


(******************************************************************)
(* This procedure responds to various mouse operations inside the	*)
(* canvas window.									*)
(******************************************************************)
PROCEDURE CanvasResponse(window : GraphicalWindow; dx,dy : INTEGER;
          			 LeftPressed,LeftReleased,DoubleClicked,Drag : BOOLEAN);
VAR
   ok : BOOLEAN;
   i, x, y, count : CARDINAL;
   idx : SHORTCARD;
   cursor : INTEGER;
   buffer : ARRAY[0..3] OF CHAR;
   string : ARRAY[0..10] OF CHAR;
BEGIN
   cursor := global_icon_id;

   (****************************************************)
   (* display window co-ordinates on the statys bar if *)
   (* the ShowOrdinate variable is set to TRUE		 *)
   (****************************************************)
   IF (ShowOrdinate) AND (cursor >= 0) THEN
	string[0] := "(";
	CardToStr(LONGCARD(ABS(MouseParatemer_xoffset)), buffer, 10, ok);

	IF MouseParatemer_xoffset >= 1000 THEN
	   string[1] := buffer[0];
	   string[2] := buffer[1];
	   string[3] := buffer[2];
	   string[4] := buffer[3];
	   string[5] := ",";
	   i := 6;
	ELSIF (MouseParatemer_xoffset >= 100) AND (MouseParatemer_xoffset < 1000) THEN
	   string[1] := buffer[0];
	   string[2] := buffer[1];
	   string[3] := buffer[2];
	   string[4] := ",";
	   i := 5;
	ELSIF (MouseParatemer_xoffset >= 10) AND (MouseParatemer_xoffset < 100) THEN
	   string[1] := buffer[0];
	   string[2] := buffer[1];
	   string[3] := ",";
	   i := 4;
	ELSIF (MouseParatemer_xoffset >= 0) AND (MouseParatemer_xoffset < 10) THEN
	   string[1] := buffer[0];
	   string[2] := ",";
	   i := 3;
	ELSE
	   FOR count := 0 TO 10 DO
		 string[count] := null;
	   END;
	   i := 0;
	END;

	CardToStr(LONGCARD(ABS(MouseParatemer_yoffset)), buffer, 10, ok);


	IF (MouseParatemer_xoffset >= 0) THEN
	   IF (MouseParatemer_yoffset >= 1000) AND (CARDINAL(ABS(MouseParatemer_yoffset)) < (CanvasBottom-CanvasTop+1)) THEN
	      string[i] := buffer[0];
	      string[i+1] := buffer[1];
	      string[i+2] := buffer[2];
	      string[i+3] := buffer[3];
	      string[i+4] := ")";
	      i := i+4;
	   ELSIF (MouseParatemer_yoffset >= 100) AND (MouseParatemer_yoffset < 1000) AND (CARDINAL(ABS(MouseParatemer_yoffset)) < (CanvasBottom-CanvasTop+1)) THEN
	      string[i] := buffer[0];
	   	string[i+1] := buffer[1];
	   	string[i+2] := buffer[2];
	   	string[i+3] := ")";
	   	i := i+3;
	   ELSIF (MouseParatemer_yoffset >= 10) AND (MouseParatemer_yoffset < 100) AND (CARDINAL(ABS(MouseParatemer_yoffset)) < (CanvasBottom-CanvasTop+1)) THEN
	   	string[i] := buffer[0];
	   	string[i+1] := buffer[1];
	   	string[i+2] := ")";
	   	i := i+2;
	   ELSIF (MouseParatemer_yoffset >= 0) AND (MouseParatemer_yoffset < 10) AND (CARDINAL(ABS(MouseParatemer_yoffset)) < (CanvasBottom-CanvasTop+1)) THEN
	   	string[i] := buffer[0];
	   	string[i+1] := ")";
	   	i := i+1;
	   ELSE
	   	FOR count := 0 TO 10 DO
		    string[count] := null;
	   	END;
		i := 0;
	   END;
	END;

	IF (i < 10) AND (i > 0) THEN
	   FOR count := i+1 TO 10 DO
		 string[count] := null;
	   END;
	END;

	IF (MouseParatemer_xoffset <> prev_x) OR (MouseParatemer_yoffset <> prev_y) THEN
	   UpdateStatusBar(string,1+2,0,8);
	END;

	prev_x := MouseParatemer_xoffset;
	prev_y := MouseParatemer_yoffset;
   END;


   (***************************************************)
   (* checks whether the dropper tool is selected and *)
   (* converts window co-ordnates back to absolute	*)
   (* co-ordinates if IsDropper is set to TRUE		*)
   (***************************************************)
   IF IsDropper THEN
	x := CARDINAL(MouseParatemer_xoffset) + CanvasLeft;
	y := CARDINAL(MouseParatemer_yoffset) + CanvasTop;
	idx := SHORTCARD(Point(x,y));
	UpdateTestColour(idx);
   END;


   (***************************************************)
   (* checks whether the polygon tool is selected and *)
   (* closes the polygon when left mouse button is	*)
   (* double clicked						*)
   (***************************************************)
   IF IsPolygon THEN
	MsgCanvas(25,0);
	IF DoubleClicked THEN
	   MsgCanvas(26,0);	(* close polygon *)
	   MsgCanvas(2,0);
	END;
   END;


   (*****************************************************)
   (* checks whether left mouse button has been clicked *)
   (*****************************************************)
   IF (LeftPressed) AND NOT(Drag) THEN
      MsgCanvas(1, 0);
	IF (global_icon_id <> 0) AND (global_icon_id <> dropper) THEN
	    modified := TRUE;
	END;
   END; (* IF *)


   (*******************************************************)
   (* checks whether left mouse button has been held down *)
   (*******************************************************)
   IF (Drag) THEN
	MsgCanvas(21, 0);
   END;

   (******************************************************)
   (* checks whether left mouse button has been released *)
   (******************************************************)
   IF (LeftReleased) THEN
      MsgCanvas(20, 0);
   END;

END CanvasResponse;


(******************************************************************)
(* This procedure decides which cursor to display when the mouse	*)
(* enters the canvas.								*)
(******************************************************************)
PROCEDURE SelectCursor();
VAR
  temp_id : INTEGER;
  EmptyString : ARRAY [0..1] OF CHAR;
BEGIN

  EmptyString := "";

  Obtain(CanvasLock);
  temp_id := global_icon_id;
  Release(CanvasLock);


  CASE temp_id OF
        0:
		SetArrow();

	| freehand:
		SetPencil();

	| eraser:
		SetSmallEraser();

	| small_eraser:
		SetSmallEraser();

	| medium_eraser:
		SetMediumEraser();

	| large_eraser:
		SetLargeEraser();

	| text:
		SetText();

	| zoom:
		SetMagnifier();

	| dropper:
		SetDropper();

	| airbrush:
		SetAirbrush();

	| capture1:
		SetArrow();
		
      | capture2:
      	SetArrow();

	| brush:
		SetSmallBrush();

	| small_brush:
		SetSmallBrush();

	| medium_brush:
		SetMediumBrush();

	| large_brush:
		SetLargeBrush();

      | ELSE
          	SetCrossHair();
		
   END; (* CASE *)

END SelectCursor;


(******************************************************************)
(* This procedure carries out various canvas related operations	*)
(* according to the mail messages sent to canvas task.		*)
(******************************************************************)
PROCEDURE CanvasTask();
VAR
   canvas, EmptyWindow : GraphicalWindow;
   current_id : INTEGER;
   command, icon_id : CARDINAL;
   i : INTEGER;
   ok : BOOLEAN;
   message,message2,header_string : ARRAY [0..80] OF CHAR;
   test_pic : CARDINAL;
   yes : BOOLEAN;
BEGIN
   (****************)
   (* title header *)
   (****************)
   header_string := "PaintMagic - ";

   (*************************)
   (* creates canvas window *)
   (*************************)
   CreateGraphicalWindow(canvas,67+19+1,22+60-1,XMAX+1,YMAX-45,15+16,NIL,0,0,0,7,
                         TRUE,TRUE,TRUE,TRUE,TRUE,TRUE,DoNothing,CanvasResponse,SelectCursor);

   CanvasLeft := GetWindowLeft(canvas);
   CanvasTop := GetWindowTop(canvas);
   CanvasRight := GetWindowRight(canvas);
   CanvasBottom := GetWindowBottom(canvas);

   ok := ObtainScreen();
   DisplayWindowData(canvas);
   ReleaseScreen();

   Line(XMAX,82,XMAX,YMAX-48,15);
   Line(XMAX-1,82,XMAX-1,YMAX-48,15);
   Line(88,YMAX-47,XMAX,YMAX-47,15);
   Line(88,YMAX-45,XMAX,YMAX-45,8);


   Copy(message2,header_string);
   Append(message2,"untitled");
   UpdateTitleBar(message2,1+2,0,15);


   current_id := 0;


   LOOP
      ok := GetMail(command, icon_id);

      CASE command OF
		  1 : (* Clicked inside canvas window *)

            	IF current_id <> 0 THEN

               	   CASE current_id OF
                    	  freehand:
					StartRoutine();

                  	| line: (* line *)			                         
					Getx1y1();

				| hv_line:
					Getx1y1();
			
               		| text:
					TextRoutine(canvas);                   
					ClearTextBuffer();

               		| circle:                   
				 	Getx1y1();

               		| solid_circle:                     
                         	Getx1y1();

               		| rectangle:                     
					Getx1y1();

               		| solid_rectangle:                     
					Getx1y1();

		   		| square:                     
					Getx1y1();

               		| solid_square:                   			
					Getx1y1();

	       		| eraser:
			 		StartRoutine();          

		 		| small_eraser:
			 		StartRoutine();

		 		| medium_eraser:
			 		StartRoutine();

		 		| large_eraser:
			 		StartRoutine();

	       		| zoom:			

		 		| dropper:
					DropperRoutine();

		 		| marquee:
					GetMarqueex1y1();

		 		| floodfill:
		 		        (*
					fill_x1 := MouseParatemer_xoffset;
					fill_y1 := MouseParatemer_yoffset;
					OldColour := GetOldColour(fill_x1,fill_y1);
					ok := ObtainScreen();
					FloodFill(canvas,fill_x1,fill_y1,OldColour,SHORTCARD(ForegroundColour));
					ReleaseScreen();
					*)

		 		| brush:
					StartRoutine();

		 		| small_brush:
			 		StartRoutine();

		 		| medium_brush:
			 		StartRoutine();

		 		| large_brush:
			 		StartRoutine();

		 		| polygon:
					GetPolyx1y1(canvas);

		 		| capture1:
					(*Write256PCX(0, 0, XMAX, YMAX, "capt0001.pcx",TRUE);*)

		 		| capture2:
					(*Write256PCX(0, 0, XMAX, YMAX, "capt0001.pcx",FALSE);*)

            		ELSE

            	   END; (* CASE *)
         		END; (* IF *)


      	| 2 : (* Sets mouse cursor *)
            	current_id := icon_id;

            	Obtain(CanvasLock);         
            	global_icon_id := icon_id;
            	Release(CanvasLock);

			IF icon_id = dropper THEN
		   	   IsDropper := TRUE;
			ELSE
			   IsDropper := FALSE;
			END;

			IF icon_id = marquee THEN
		   	   IsMarquee := TRUE;
			ELSE
		   	   IsMarquee := FALSE;
			END;

			IF icon_id = polygon THEN
		   	   IsPolygon := TRUE;
			ELSE
		   	   IsPolygon := FALSE;
		   	   StopRoutine();
			END;

      	| 3 : (* Clears canvas *)
            	ok := ObtainScreen();
            	ClearDataRegion(canvas);
            	ReleaseScreen();

	    		Copy(OpenFileName,"untitled");
	    		Copy(message2,header_string);
  	    		Append(message2,OpenFileName);
  	    		UpdateTitleBar(message2,1+2,0,8+7);

	    		ColourRoutines.Initialise();

	    		extension := Untitled;
	    		modified := FALSE;
	    		new_image := FALSE;

      	| 4 :	 

      	| 5 :
	    		ok := ObtainScreen();
          		ClearDataRegion(canvas);
          		ReleaseScreen();
	 
      	| 6 :

	    		IF (IsExtension(OpenFileName, "BMP")) THEN
			   ok := ObtainScreen();
          		   ClearDataRegion(canvas);
          		   ReleaseScreen();

      	 	   extension := Bmp;
		 	   test_pic := ReadBMP.LoadBMP(canvas, OpenFileName);
		 	   Copy(message2,header_string);
  	    	 	   Append(message2,OpenFileName);
  	    	 	   UpdateTitleBar(message2,1+2,0,8+7);

		 	   modified := FALSE;
		 	   open := FALSE;
	    		ELSIF (IsExtension(OpenFileName, "PCX")) THEN
			   ok := ObtainScreen();
          		   ClearDataRegion(canvas);
          		   ReleaseScreen();

      	 	   extension := Pcx;
		 	   test_pic := ReadPCX.LoadPCX(canvas, OpenFileName);
		 	   Copy(message2,header_string);
  	    	 	   Append(message2,OpenFileName);
  	    	 	   UpdateTitleBar(message2,1+2,0,8+7);

		 	   modified := FALSE;
		 	   open := FALSE;
	    		ELSIF (IsExtension(OpenFileName, "TGA")) THEN
			   ok := ObtainScreen();
          		   ClearDataRegion(canvas);
          		   ReleaseScreen();

      	 	   extension := Tga;
		 	   test_pic := ReadTGA.LoadTGA(canvas, OpenFileName);
		 	   Copy(message2,header_string);
  	    	 	   Append(message2,OpenFileName);
  	    	 	   UpdateTitleBar(message2,1+2,0,8+7);

		 	   modified := FALSE;
			   open := FALSE;
	    		ELSIF (IsExtension(OpenFileName, "TIF")) THEN
			   ok := ObtainScreen();
          		   ClearDataRegion(canvas);
          		   ReleaseScreen();

      	 	   extension := Tif;
		 	   test_pic := ReadTIF.LoadTIF(canvas, OpenFileName);
		 	   Copy(message2,header_string);
  	    	 	   Append(message2,OpenFileName);
  	    	 	   UpdateTitleBar(message2,1+2,0,8+7);

		 	   modified := FALSE;
		 	   open := FALSE;
	    		ELSE
			   (*Beep();
		 	   UnknownFormat();*)
		 	   open := FALSE;
	    		END;

		| 7:
			CASE extension OF
			  	  Bmp:
					ReplaceFile(SaveBmp,OpenFileName);

				| Pcx:
					ReplaceFile(SavePcx,OpenFileName);

				| Tga:
					ReplaceFile(SaveTga,OpenFileName);

				| Raw:
					ReplaceFile(SaveRaw,OpenFileName);

				ELSE
			END;

			IF new_image THEN
		   	   ok := ObtainScreen();
          	   	   ClearDataRegion(canvas);
          	   	   ReleaseScreen();
		   	   Copy(OpenFileName,"untitled");
	     	   	   Copy(message2,header_string);
  	    	   	   Append(message2,OpenFileName);
  	    	   	   UpdateTitleBar(message2,1+2,0,8+7);

	    	   	   ColourRoutines.Initialise();

	    	   	   extension := Untitled;
	    	   	   modified := FALSE;
	    	   	   new_image := FALSE;
			END;

		| 8:	(* Flipping *)
			ImageRoutines.FlipImage(CanvasLeft,CanvasTop,CanvasRight,CanvasBottom);
			modified := TRUE;

		| 9:	(* Mirroring *)
			ImageRoutines.MirrorImage(CanvasLeft,CanvasTop,CanvasRight,CanvasBottom);
			modified := TRUE;

		| 10:	(* Cropping *)
			IF (marquee_x1 >= 0) AND (marquee_y1 >= 0) AND
      	         (marquee_x2 >= 0) AND (marquee_y2 >= 0) THEN
			   PutRectangle(canvas,marquee_x1-INTEGER(CanvasLeft), marquee_y1-INTEGER(CanvasTop),
				 	    marquee_x2-INTEGER(CanvasLeft), marquee_y2-INTEGER(CanvasTop), 256, FALSE);
			   ImageRoutines.CropImage(marquee_x1,marquee_y1,marquee_x2,marquee_y2);
			END;

		| 11:	(* Colour count *)
			UniqueColours();

		| 12 :
			CASE extension OF
				  Bmp:
					ok := ObtainScreen();
            			ClearDataRegion(canvas);
            			ReleaseScreen();
	    				ReadBMP.NoZoomBMP(canvas, OpenFileName);
	    				Copy(message2,header_string);
  	    				Append(message2,OpenFileName);
  	    				UpdateTitleBar(message2,1+2,0,8+7);

				| Pcx:
					ok := ObtainScreen();
            			ClearDataRegion(canvas);
            			ReleaseScreen();
	    				ReadPCX.NoZoomPCX(canvas, OpenFileName);
	    				Copy(message2,header_string);
  	    				Append(message2,OpenFileName);
  	    				UpdateTitleBar(message2,1+2,0,8+7);

				| Tga:
					ok := ObtainScreen();
            			ClearDataRegion(canvas);
            			ReleaseScreen();
	    				ReadTGA.NoZoomTGA(canvas, OpenFileName);
	    				Copy(message2,header_string);
  	    				Append(message2,OpenFileName);
  	    				UpdateTitleBar(message2,1+2,0,8+7);

				| Tif:
					ok := ObtainScreen();
            			ClearDataRegion(canvas);
            			ReleaseScreen();
	    				ReadTIF.NoZoomTIF(canvas, OpenFileName);
	    				Copy(message2,header_string);
  	    				Append(message2,OpenFileName);
  	    				UpdateTitleBar(message2,1+2,0,8+7);

				ELSE
			END;
     
      	| 13 :
			CASE extension OF
				  Bmp:
					ok := ObtainScreen();
            			ClearDataRegion(canvas);
            			ReleaseScreen();
	    				ReadBMP.ZoomBMP2x(canvas, OpenFileName);
	    				Copy(message2,header_string);
  	    				Append(message2,OpenFileName);
  	    				UpdateTitleBar(message2,1+2,0,8+7);

				| Pcx:
					ok := ObtainScreen();
            			ClearDataRegion(canvas);
            			ReleaseScreen();
	    				ReadPCX.ZoomPCX2x(canvas, OpenFileName);
	    				Copy(message2,header_string);
  	    				Append(message2,OpenFileName);
  	    				UpdateTitleBar(message2,1+2,0,8+7);

				| Tga:
					ok := ObtainScreen();
            			ClearDataRegion(canvas);
            			ReleaseScreen();
	    				ReadTGA.ZoomTGA2x(canvas, OpenFileName);
	    				Copy(message2,header_string);
  	    				Append(message2,OpenFileName);
  	    				UpdateTitleBar(message2,1+2,0,8+7);

				| Tif:
					ok := ObtainScreen();
            			ClearDataRegion(canvas);
            			ReleaseScreen();
	    				ReadTIF.ZoomTIF2x(canvas, OpenFileName);
	    				Copy(message2,header_string);
  	    				Append(message2,OpenFileName);
  	    				UpdateTitleBar(message2,1+2,0,8+7);

				ELSE
			END;

      	| 14 :
			CASE extension OF
				  Bmp:
					ok := ObtainScreen();
            			ClearDataRegion(canvas);
            			ReleaseScreen();
	    				ReadBMP.ZoomBMP3x(canvas, OpenFileName);
	    				Copy(message2,header_string);
  	    				Append(message2,OpenFileName);
  	    				UpdateTitleBar(message2,1+2,0,8+7);

				| Pcx:
					ok := ObtainScreen();
            			ClearDataRegion(canvas);
            			ReleaseScreen();
	    				ReadPCX.ZoomPCX3x(canvas, OpenFileName);
	    				Copy(message2,header_string);
  	    				Append(message2,OpenFileName);
  	    				UpdateTitleBar(message2,1+2,0,8+7);

				| Tga:
					ok := ObtainScreen();
            			ClearDataRegion(canvas);
            			ReleaseScreen();
	    				ReadTGA.ZoomTGA3x(canvas, OpenFileName);
	    				Copy(message2,header_string);
  	    				Append(message2,OpenFileName);
  	    				UpdateTitleBar(message2,1+2,0,8+7);

				| Tif:
					ok := ObtainScreen();
            			ClearDataRegion(canvas);
            			ReleaseScreen();
	    				ReadTIF.ZoomTIF3x(canvas, OpenFileName);
	    				Copy(message2,header_string);
  	    				Append(message2,OpenFileName);
  	    				UpdateTitleBar(message2,1+2,0,8+7);

				ELSE
			END;

		| 15:	(* cut *)
			IF (marquee_x1 >= 0) AND (marquee_y1 >= 0) AND
      	         (marquee_x2 >= 0) AND (marquee_y2 >= 0) THEN
			   PutRectangle(canvas,marquee_x1-INTEGER(CanvasLeft), marquee_y1-INTEGER(CanvasTop),
				 	    marquee_x2-INTEGER(CanvasLeft), marquee_y2-INTEGER(CanvasTop), 256, FALSE);
			   ImageRoutines.CutImage(canvas,marquee_x1-INTEGER(CanvasLeft), marquee_y1-INTEGER(CanvasTop),
				 	    		  marquee_x2-INTEGER(CanvasLeft), marquee_y2-INTEGER(CanvasTop));
			END;

      	| 20 :
			CASE current_id OF
				  line: (* line *)
                  		LineRoutine(canvas,TRUE);

		  		| hv_line:
					HVLineRoutine(canvas,TRUE);

		  		| rectangle:
					RectangleRoutine(canvas,TRUE);

		  		| solid_rectangle:
					SolidRectangleRoutine(canvas,TRUE);

		  		| square:
					SquareRoutine(canvas,TRUE);

		  		| solid_square:
					SolidSquareRoutine(canvas,TRUE);

		  		| circle:
					CircleRoutine(canvas,TRUE);

		  		| solid_circle:
					SolidCircleRoutine(canvas,TRUE);

		  		| marquee:
					MarqueeRoutine(canvas,TRUE);

				ELSE
			END; (* CASE *)

      	| 21 :
                	CASE current_id OF
			  	  freehand:
                        	FreehandRoutine(canvas);

				| eraser:
					SmallEraserRoutine(canvas);

				| small_eraser:
			  		SmallEraserRoutine(canvas);

				| medium_eraser:
			  		MediumEraserRoutine(canvas);

				| large_eraser:
			  		LargeEraserRoutine(canvas);

				| airbrush:
			  		AirBrushRoutine(canvas);

				| brush:
			  		SmallBrushRoutine(canvas);

				| small_brush:
			  		SmallBrushRoutine(canvas);

				| medium_brush:
			  		MediumBrushRoutine(canvas);

				| large_brush:
			  		LargeBrushRoutine(canvas);

				| line:
			  		LineRoutine(canvas,FALSE);

				| hv_line:
			  		HVLineRoutine(canvas,FALSE);

				| solid_rectangle:
			  		SolidRectangleRoutine(canvas,FALSE);

				| rectangle:
			  		RectangleRoutine(canvas,FALSE);

				| solid_square:
			  		SolidSquareRoutine(canvas,FALSE);

				| square:
			  		SquareRoutine(canvas,FALSE);

				| solid_circle:
			  		SolidCircleRoutine(canvas,FALSE);

				| circle:
			  		CircleRoutine(canvas,FALSE);

				| marquee:
					MarqueeRoutine(canvas,FALSE);

                		ELSE
			END;

      	| 22 :
			CASE format OF
				  (*****************************************)
				  (* saves screen image in BMP file format *)
				  (*****************************************)
				  SaveBmp:
					IF NOT(FIO.Exists(SaveAsFileName)) THEN
				   	   Write256BMP(CanvasLeft,CanvasTop,CanvasRight,CanvasBottom,SaveAsFileName);
				   	   modified := FALSE;
					END;

				  (*****************************************)
				  (* saves screen image in PCX file format *)
				  (*****************************************)
				| SavePcx:
					IF NOT(FIO.Exists(SaveAsFileName)) THEN
				   	   Write256PCX(CanvasLeft,CanvasTop,CanvasRight,CanvasBottom,SaveAsFileName,FALSE);
				   	   modified := FALSE;
					END;

				  (*****************************************)
				  (* saves screen image in Tga file format *)
				  (*****************************************)
				| SaveTga:
					IF NOT(FIO.Exists(SaveAsFileName)) THEN
				   	   Write256TGA(CanvasLeft,CanvasTop,CanvasRight,CanvasBottom,SaveAsFileName);
				  	   modified := FALSE;
					END;

				  (*****************************************)
				  (* saves screen image in TIF file format *)
				  (*****************************************)
				| SaveTif:
					IF NOT(FIO.Exists(SaveAsFileName)) THEN
				   	   Write256TIF(CanvasLeft,CanvasTop,CanvasRight,CanvasBottom,SaveAsFileName);
				   	   modified := FALSE;
					END;

				  (*****************************************)
				  (* saves screen image in RAW file format *)
				  (*****************************************)
				| SaveRaw:
					IF NOT(FIO.Exists(SaveAsFileName)) THEN
				   	   WriteRawData(CanvasLeft,CanvasTop,CanvasRight,CanvasBottom,SaveAsFileName);
				   	   modified := FALSE;
					END;

				| CaptShow:
					(***********************************************************)
					(* captures screen contents in PCX file format with cursor *)
					(***********************************************************)
				 	Write256PCX(0,0,XMAX-1,YMAX-1,CaptName,TRUE);

				| CaptHide:
					(**************************************************************)
					(* captures screen contents in PCX file format without cursor *)
					(**************************************************************)
					Write256PCX(0,0,XMAX-1,YMAX-1,CaptName,FALSE);

				ELSE
			END;

		(*| 23:
			Write256PCX(0,0,XMAX-1,YMAX-1,CaptName,TRUE);

		| 24:
			Write256PCX(0,0,XMAX-1,YMAX-1,CaptName,FALSE);*)

		| 25:
			PolygonRoutine(canvas,FALSE);

		| 26:
	        	ClosePolygon(canvas);

      	ELSE;
	   		FileNotFound();

	END; (*CASE*)

   END; (* LOOP *)

END CanvasTask;



(*******************)
(* initialisations *)
(*******************)
BEGIN
   CanvasMailbox := CreateMailbox(2);

   CreateTask(CanvasTask, 1, "Canvas Task");

   CreateLock(CanvasLock,"Canvas Lock");

   counter := 0;

   normal := TRUE;
   zoom2x := FALSE;
   zoom3x := FALSE;

   IsDropper := FALSE;
   IsMarquee := FALSE;
   ShowOrdinate := TRUE;

   prev_x := 0;
   prev_y := 0;

END Canvas.