IMPLEMENTATION MODULE ColourRoutines;


(******************************************************************)
(*											*)
(*			     Colour Routines Module				*)
(*											*)
(*	Programmer :	Matthew Ma (c9315720)				*)
(*											*)
(*	Last edited :	22-10-97						*)
(*											*)
(*	Function : 		This module handles all colour related	*)
(*				routines including drawing of colour	*)
(*				buttons, colour palette initilaisation,	*)
(*				colour manipulation routines and colour	*)
(*				scaling routines.					*)
(* 											*)
(*	Status :		Working.						*)
(******************************************************************)


FROM Canvas IMPORT
(* var  *)   CanvasTop, CanvasLeft, CanvasBottom, CanvasRight;

FROM Icons IMPORT
(* proc *)   PutIcon;

FROM Bars IMPORT
(* proc *)   UpdateForegroundColour;

FROM FIO IMPORT
(* type *)   File;

FROM FileSys IMPORT
(* proc *)   OpenFile, CloseFile, ReadByte,
             SetPosition, SavePosition, FileSize, ReadRecord;

FROM IOErrorCodes IMPORT
(* type *)   ErrorCode;

FROM Support IMPORT
(* proc *)   RoundOff, RoundOff2, ReadWord, ReadLongWord, AdjustSingleBytePalette;

FROM Graphics IMPORT
(* proc *)   SetPaletteColour, Point;

FROM Buttons IMPORT
(* proc *)   Draw3DFrame;

FROM GraphicWindow IMPORT
(* var  *)   XMAX, YMAX, (*CharWidth, CharHeight, MaxColour,*)
(* type *)   GraphicalWindow,
(* proc *)   Rectangle, ObtainScreen, ReleaseScreen;

FROM Buttons IMPORT
(* var  *)   NoSubMenu,
(* type *)   BoxMenuProcedure, BoxMenuItemTYPE, BoxMenuTYPE, MenuType,
(* proc *)   CreateBoxSubMenu, BoxSubMenuItemSetup, DisplayBoxSubMenuTitle,
             DisplayBoxSubMenu, BoxSubMenuKeyboardSelection, NoIconDraw,
             DestroyBoxSubMenu, HideMenu;

FROM MouseRoutines IMPORT
(* proc *)   SetFinger, SetArrow;

FROM Canvas IMPORT
(* proc *)   MsgCanvas;


VAR
   ok, first_time : BOOLEAN;


(******************************************************************)
(* This procedure draws the black colour icon onto the palette	*)
(* bar.										*)
(******************************************************************)
PROCEDURE DrawBlack(w : GraphicalWindow; x,y : INTEGER);
BEGIN
   PutIcon(x,y,26);
END DrawBlack;


(******************************************************************)
(* This procedure draws the white colour icon onto the palette	*)
(* bar.										*)
(******************************************************************)
PROCEDURE DrawWhite(w : GraphicalWindow; x,y : INTEGER);
BEGIN
   PutIcon(x,y,27);
END DrawWhite;


(******************************************************************)
(* This procedure draws the blue colour icon onto the palette	*)
(* bar.										*)
(******************************************************************)
PROCEDURE DrawBlue(w : GraphicalWindow; x,y : INTEGER);
BEGIN
   PutIcon(x,y,28);
END DrawBlue;


(******************************************************************)
(* This procedure draws the green colour icon onto the palette	*)
(* bar.										*)
(******************************************************************)
PROCEDURE DrawLightGreen(w : GraphicalWindow; x,y : INTEGER);
BEGIN
   PutIcon(x,y,29);
END DrawLightGreen;


(******************************************************************)
(* This procedure draws the cyan colour icon onto the palette	*)
(* bar.										*)
(******************************************************************)
PROCEDURE DrawFaintGreen(w : GraphicalWindow; x,y : INTEGER);
BEGIN
   PutIcon(x,y,30);
END DrawFaintGreen;


(******************************************************************)
(* This procedure draws the red colour icon onto the palette bar.	*)
(******************************************************************)
PROCEDURE DrawRed(w : GraphicalWindow; x,y : INTEGER);
BEGIN
   PutIcon(x,y,31);
END DrawRed;


(******************************************************************)
(* This procedure draws the magenta colour icon onto the palette	*)
(* bar.										*)
(******************************************************************)
PROCEDURE DrawViolet(w : GraphicalWindow; x,y : INTEGER);
BEGIN
   PutIcon(x,y,32);
END DrawViolet;


(******************************************************************)
(* This procedure draws the brown colour icon onto the palette	*)
(* bar.										*)
(******************************************************************)
PROCEDURE DrawFaintRed(w : GraphicalWindow; x,y : INTEGER);
BEGIN
   PutIcon(x,y,33);
END DrawFaintRed;


(******************************************************************)
(* This procedure draws the light grey colour icon onto the		*)
(* palette bar.									*)
(******************************************************************)
PROCEDURE DrawLightGrey(w : GraphicalWindow; x,y : INTEGER);
BEGIN
   PutIcon(x,y,34);
END DrawLightGrey;


(******************************************************************)
(* This procedure draws the dark grey colour icon onto the		*)
(* palette bar.									*)
(******************************************************************)
PROCEDURE DrawDarkGrey(w : GraphicalWindow; x,y : INTEGER);
BEGIN
   PutIcon(x,y,35);
END DrawDarkGrey;


(******************************************************************)
(* This procedure draws the light blue colour icon onto the		*)
(* palette bar.									*)
(******************************************************************)
PROCEDURE DrawNavyBlue(w : GraphicalWindow; x,y : INTEGER);
BEGIN
   PutIcon(x,y,36);
END DrawNavyBlue;


(******************************************************************)
(* This procedure draws the light green colour icon onto the	*)
(* palette bar.									*)
(******************************************************************)
PROCEDURE DrawFluroGreen(w : GraphicalWindow; x,y : INTEGER);
BEGIN
   PutIcon(x,y,37);
END DrawFluroGreen;


(******************************************************************)
(* This procedure draws the light cyan colour icon onto the		*)
(* palette bar.									*)
(******************************************************************)
PROCEDURE DrawFluroBlue(w : GraphicalWindow; x,y : INTEGER);
BEGIN
   PutIcon(x,y,38);
END DrawFluroBlue;


(******************************************************************)
(* This procedure draws the light red colour icon onto the		*)
(* palette bar.									*)
(******************************************************************)
PROCEDURE DrawOrange(w : GraphicalWindow; x,y : INTEGER);
BEGIN
   PutIcon(x,y,39);
END DrawOrange;


(******************************************************************)
(* This procedure draws the light magenta colour icon onto the	*)
(* palette bar.									*)
(******************************************************************)
PROCEDURE DrawLightViolet(w : GraphicalWindow; x,y : INTEGER);
BEGIN
   PutIcon(x,y,40);
END DrawLightViolet;


(******************************************************************)
(* This procedure draws the yellow colour icon onto the palette	*)
(* bar.										*)
(******************************************************************)
PROCEDURE DrawYellow(w : GraphicalWindow; x,y : INTEGER);
BEGIN
   PutIcon(x,y,41);
END DrawYellow;


(******************************************************************)
(* This procedure sets the foreground colour to black.		*)
(******************************************************************)
PROCEDURE SetBlack();
VAR
   idx : SHORTCARD;
BEGIN
   ForegroundColour := 0+16;

   IF first_time THEN
      IF ObtainScreen() THEN
         Rectangle(16,391,49,414-3,0+16,TRUE);
      END;
      ReleaseScreen();
      first_time := FALSE;
   ELSE
      UpdateForegroundColour(16);
   END;
END SetBlack;


(******************************************************************)
(* This procedure sets the foreground colour to blue.			*)
(******************************************************************)
PROCEDURE SetBlue();
VAR
   idx : SHORTCARD;
BEGIN
   ForegroundColour := 1+16;
   UpdateForegroundColour(17);
END SetBlue;


(******************************************************************)
(* This procedure sets the foreground colour to green.		*)
(******************************************************************)
PROCEDURE SetGreen();
VAR
   idx : SHORTCARD;
BEGIN
   ForegroundColour := 2+16;
   UpdateForegroundColour(18);
END SetGreen;


(******************************************************************)
(* This procedure sets the foreground colour to cyan.			*)
(******************************************************************)
PROCEDURE SetCyan();
VAR
   idx : SHORTCARD;
BEGIN
   ForegroundColour := 3+16;
   UpdateForegroundColour(19);
END SetCyan;


(******************************************************************)
(* This procedure sets the foreground colour to red.			*)
(******************************************************************)
PROCEDURE SetRed();
VAR
   idx : SHORTCARD;
BEGIN
   ForegroundColour := 4+16;
   UpdateForegroundColour(20);
END SetRed;


(******************************************************************)
(* This procedure sets the foreground colour to magenta.		*)
(******************************************************************)
PROCEDURE SetMagenta();
VAR
   idx : SHORTCARD;
BEGIN
   ForegroundColour := 5+16;
   UpdateForegroundColour(21);
END SetMagenta;


(******************************************************************)
(* This procedure sets the foreground colour to brown.		*)
(******************************************************************)
PROCEDURE SetBrown();
VAR
   idx : SHORTCARD;
BEGIN
   ForegroundColour := 6+16;
   UpdateForegroundColour(22);
END SetBrown;


(******************************************************************)
(* This procedure sets the foreground colour to light grey.		*)
(******************************************************************)
PROCEDURE SetLightGrey();
VAR
   idx : SHORTCARD;
BEGIN
   ForegroundColour := 7+16;
   UpdateForegroundColour(23);
END SetLightGrey;


(******************************************************************)
(* This procedure sets the foreground colour to dark grey.		*)
(******************************************************************)
PROCEDURE SetDarkGrey();
VAR
   idx : SHORTCARD;
BEGIN
   ForegroundColour := 8+16;
   UpdateForegroundColour(24);
END SetDarkGrey;


(******************************************************************)
(* This procedure sets the foreground colour to light blue.		*)
(******************************************************************)
PROCEDURE SetLightBlue();
VAR
   idx : SHORTCARD;
BEGIN
   ForegroundColour := 9+16;
   UpdateForegroundColour(25);
END SetLightBlue;


(******************************************************************)
(* This procedure sets the foreground colour to light green.	*)
(******************************************************************)
PROCEDURE SetLightGreen();
VAR
   idx : SHORTCARD;
BEGIN
   ForegroundColour := 10+16;
   UpdateForegroundColour(26);
END SetLightGreen;


(******************************************************************)
(* This procedure sets the foreground colour to light cyan.		*)
(******************************************************************)
PROCEDURE SetLightCyan();
VAR
   idx : SHORTCARD;
BEGIN
   ForegroundColour := 11+16;
   UpdateForegroundColour(27);
END SetLightCyan;


(******************************************************************)
(* This procedure sets the foreground colour to light red.		*)
(******************************************************************)
PROCEDURE SetLightRed();
VAR
   idx : SHORTCARD;
BEGIN
   ForegroundColour := 12+16;
   UpdateForegroundColour(28);
END SetLightRed;


(******************************************************************)
(* This procedure sets the foreground colour to light magenta.	*)
(******************************************************************)
PROCEDURE SetLightMagenta();
VAR
   idx : SHORTCARD;
BEGIN
   ForegroundColour := 13+16;
   UpdateForegroundColour(29);
END SetLightMagenta;


(******************************************************************)
(* This procedure sets the foreground colour to yellow.		*)
(******************************************************************)
PROCEDURE SetYellow();
VAR
   idx : SHORTCARD;
BEGIN
   ForegroundColour := 14+16;
   UpdateForegroundColour(30);
END SetYellow;


(******************************************************************)
(* This procedure sets the foreground colour to white.		*)
(******************************************************************)
PROCEDURE SetWhite();
VAR
   idx : SHORTCARD;
BEGIN
   ForegroundColour := 15+16;
   UpdateForegroundColour(31);
END SetWhite;


(******************************************************************)
(* This procedure draws the colour buttons on the palette bar.	*)
(******************************************************************)
PROCEDURE DrawDefaultPaletteBar();
VAR
   PaletteBar : BoxMenuTYPE;
   Black, White, Blue, Green, Cyan, Red, Magenta,
   Brown, LightGrey, DarkGrey, LightBlue, LightGreen,
   LightCyan, LightRed, LightMagenta, Yellow : BoxMenuItemTYPE;
BEGIN
   PaletteBar := CreateBoxSubMenu("Palette",FALSE,3,9,15,4,0,0,86+2+1-2,YMAX-67,2+14,7-6,20+14,20+2,
				  	    0,12,1,7,AlwaysVisible,SetFinger);
   Black := BoxSubMenuItemSetup(PaletteBar,TRUE,TRUE,"Left click to choose a foreground colour",
					  DrawBlack,8-3+89-2,YMAX-38,1,25,NoSubMenu,TRUE,SetBlack);
   White := BoxSubMenuItemSetup(PaletteBar,TRUE,TRUE,"Left click to choose a foreground colour",
					  DrawWhite,8-3+89-2,YMAX-38,2,15+10,NoSubMenu,TRUE,SetWhite);
   Blue := BoxSubMenuItemSetup(PaletteBar,TRUE,TRUE,"Left click to choose a foreground colour",DrawBlue,
				       8-3+89-2,YMAX-38,3,25,NoSubMenu,TRUE,SetBlue);
   Green := BoxSubMenuItemSetup(PaletteBar,TRUE,TRUE,"Left click to choose a foreground colour",DrawLightGreen,
				        8-3+89-2,YMAX-38,4,25,NoSubMenu,TRUE,SetGreen);
   Cyan := BoxSubMenuItemSetup(PaletteBar,TRUE,TRUE,"Left click to choose a foreground colour",DrawFaintGreen,
				       8-3+89-2,YMAX-38,6,25,NoSubMenu,TRUE,SetCyan);
   Red := BoxSubMenuItemSetup(PaletteBar,TRUE,TRUE,"Left click to choose a foreground colour",DrawRed,
				      8-3+89-2,YMAX-38,8,25,NoSubMenu,TRUE,SetRed);
   Magenta := BoxSubMenuItemSetup(PaletteBar,TRUE,TRUE,"Left click to choose a foreground colour",DrawViolet,
				          8-3+89-2,YMAX-38,10,25,NoSubMenu,TRUE,SetMagenta);
   Brown := BoxSubMenuItemSetup(PaletteBar,TRUE,TRUE,"Left click to choose a foreground colour",DrawFaintRed,
				        8-3+89-2,YMAX-38,13,25,NoSubMenu,TRUE,SetBrown);
   LightGrey := BoxSubMenuItemSetup(PaletteBar,TRUE,TRUE,"Left click to choose a foreground colour",DrawLightGrey,
				      	8-3+89-2,YMAX-38,16,25,NoSubMenu,TRUE,SetLightGrey);
   DarkGrey := BoxSubMenuItemSetup(PaletteBar,TRUE,TRUE,"Left click to choose a foreground colour",DrawDarkGrey,
				     	     8-3+89-2,YMAX-38,19,25,NoSubMenu,TRUE,SetDarkGrey);
   LightBlue := BoxSubMenuItemSetup(PaletteBar,TRUE,TRUE,"Left click to choose a foreground colour",DrawNavyBlue,
				     		8-3+89-2,YMAX-38,22,25,NoSubMenu,TRUE,SetLightBlue);
   LightGreen := BoxSubMenuItemSetup(PaletteBar,TRUE,TRUE,"Left click to choose a foreground colour",DrawFluroGreen,
				     		 8-3+89-2,YMAX-38,26,25,NoSubMenu,TRUE,SetLightGreen);
   LightCyan := BoxSubMenuItemSetup(PaletteBar,TRUE,TRUE,"Left click to choose a foreground colour",DrawFluroBlue,
				     		8-3+89-2,YMAX-38,28,26,NoSubMenu,TRUE,SetLightCyan);
   LightRed := BoxSubMenuItemSetup(PaletteBar,TRUE,TRUE,"Left click to choose a foreground colour",DrawOrange,
				     	     8-3+89-2,YMAX-38,31,25,NoSubMenu,TRUE,SetLightRed);
   LightMagenta := BoxSubMenuItemSetup(PaletteBar,TRUE,TRUE,"Left click to choose a foreground colour",DrawLightViolet,
				     		   8-3+89-2,YMAX-38,32,25,NoSubMenu,TRUE,SetLightMagenta);
   Yellow := BoxSubMenuItemSetup(PaletteBar,TRUE,TRUE,"Left click to choose a foreground colour",DrawYellow,
				     	   8-3+89-2,YMAX-38,33,25,NoSubMenu,TRUE,SetYellow);

   DisplayBoxSubMenu(PaletteBar);


   (*****************************************)
   (* draws 3D frames around colour buttons *)
   (*****************************************)
   Draw3DFrame(Black);
   Draw3DFrame(White);
   Draw3DFrame(Blue);
   Draw3DFrame(Green);
   Draw3DFrame(Cyan);
   Draw3DFrame(Red);
   Draw3DFrame(Magenta);
   Draw3DFrame(Brown);
   Draw3DFrame(LightGrey);
   Draw3DFrame(DarkGrey);
   Draw3DFrame(LightBlue);
   Draw3DFrame(LightGreen);
   Draw3DFrame(LightCyan);
   Draw3DFrame(LightRed);
   Draw3DFrame(LightMagenta);
   Draw3DFrame(Yellow);

END DrawDefaultPaletteBar;


(******************************************************************)
(* This procedure assigns the palette array with 8-bit colour	*)
(* values.										*)
(******************************************************************)
PROCEDURE SetPaletteValue(idx : CARDINAL; r,g,b : SHORTCARD);
BEGIN
   palette[idx].red := CARDINAL(r);
   palette[idx].green := CARDINAL(g);
   palette[idx].blue := CARDINAL(b);
END SetPaletteValue;


(******************************************************************)
(* This procedure assigns the vga_pal array with 6-bit colour	*)
(* values.										*)
(******************************************************************)
PROCEDURE SetVgaValue(idx : CARDINAL);
BEGIN
   vga_pal[idx].red := CARDINAL(AdjustSingleBytePalette(SHORTCARD(palette[idx].red)));
   vga_pal[idx].green := CARDINAL(AdjustSingleBytePalette(SHORTCARD(palette[idx].green)));
   vga_pal[idx].blue := CARDINAL(AdjustSingleBytePalette(SHORTCARD(palette[idx].blue)));
END SetVgaValue;


(******************************************************************)
(* This procedure inverts the colour palette by subtracting colur	*)
(* values in palette by 255 and colour values in vga_pal by 63.	*)
(******************************************************************)
PROCEDURE InvertPalette();
VAR
   dummy1, dummy2 : CARDINAL;
   idx : CARDINAL;
BEGIN
   FOR idx := 16 TO 255 DO
	 dummy1 := palette[idx].red;
	 dummy2 := vga_pal[idx].red;
       palette[idx].red := 255 - dummy1;
	 vga_pal[idx].red := 63 - dummy2;

       dummy1 := palette[idx].green;
	 dummy2 := vga_pal[idx].green;
       palette[idx].green := 255 - dummy1;
	 vga_pal[idx].green := 63 - dummy2;

       dummy1 := palette[idx].blue;
	 dummy2 := vga_pal[idx].blue;
       palette[idx].blue := 255 - dummy1;
	 vga_pal[idx].blue := 63 - dummy2;
   END;

   FOR idx := 16 TO 255 DO
       SetPaletteColour(SHORTCARD(idx), SHORTCARD(vga_pal[idx].red), SHORTCARD(vga_pal[idx].green), SHORTCARD(vga_pal[idx].blue));
   END;
END InvertPalette;


(******************************************************************)
(* This procedure adjusts the brightness of an image.			*)
(******************************************************************)
PROCEDURE BrightnessControl(factor : LONGREAL);
VAR
   idx : CARDINAL;
   dummy : LONGREAL;
BEGIN
   FOR idx := 16 TO 255 DO
	 dummy := VAL(LONGREAL,vga_pal[idx].red)*factor;
       IF dummy > 63.0 THEN
	    vga_pal[idx].red := 63;
       ELSE
	    vga_pal[idx].red := CARDINAL(RoundOff2(dummy));
       END;

	 dummy := VAL(LONGREAL,vga_pal[idx].green)*factor;
       IF dummy > 63.0 THEN
	    vga_pal[idx].green := 63;
       ELSE
	    vga_pal[idx].green := CARDINAL(RoundOff2(dummy));
       END;

	 dummy := VAL(LONGREAL,vga_pal[idx].blue)*factor;
       IF dummy > 63.0 THEN
	    vga_pal[idx].blue := 63;
       ELSE
	    vga_pal[idx].blue := CARDINAL(RoundOff2(dummy));
       END;

	 SetPaletteColour(SHORTCARD(idx), SHORTCARD(vga_pal[idx].red), SHORTCARD(vga_pal[idx].green), SHORTCARD(vga_pal[idx].blue));
   END;
END BrightnessControl;


(******************************************************************)
(* This procedure turns an 8-bit colour image into a grey scale	*)
(* image.										*)
(******************************************************************)
PROCEDURE GreyScale();
VAR
   idx : CARDINAL;
   dummy1 : REAL;
   dummy2 : REAL;
BEGIN
   FOR idx := 16 TO 255 DO
       dummy1 := (REAL(palette[idx].red) + REAL(palette[idx].green) + REAL(palette[idx].blue))/3.0;
       dummy2 := (REAL(vga_pal[idx].red) + REAL(vga_pal[idx].green) + REAL(vga_pal[idx].blue))/3.0;
       palette[idx].red := RoundOff(dummy1);
       palette[idx].green := RoundOff(dummy1);
       palette[idx].blue := RoundOff(dummy1);
       vga_pal[idx].red := RoundOff(dummy2);
       vga_pal[idx].green := RoundOff(dummy2);
       vga_pal[idx].blue := RoundOff(dummy2);

       SetPaletteColour(SHORTCARD(idx), SHORTCARD(vga_pal[idx].red), SHORTCARD(vga_pal[idx].green), SHORTCARD(vga_pal[idx].blue));
   END;
END GreyScale;


(******************************************************************)
(* This procedure counts the number of unique colours used by an	*)
(* image.										*)
(******************************************************************)
PROCEDURE CountColours() : CARDINAL;
VAR
   num : SHORTCARD;
   i, j, colour_count, counter : CARDINAL;
   table : ARRAY [0..256] OF SHORTCARD;
BEGIN
   colour_count := 0;

   ok := ObtainScreen();
   FOR j := CanvasTop TO CanvasBottom DO
       FOR i := CanvasLeft TO CanvasRight DO
	   num := SHORTCARD(Point(i,j));
	   IF colour_count = 0 THEN
	      table[colour_count] := num;
	      INC(colour_count);
	   ELSE
	      counter := 0;
	      LOOP
		  IF num = table[counter] THEN
		     EXIT;
		  ELSE
		     INC(counter);
		     IF counter = colour_count THEN
		        EXIT;
		     END;
		  END;
	      END;
	      IF counter = colour_count THEN	(* unique *)
	         table[counter] := num;
	         INC(colour_count);
	      END;
           END;
       END;
   END;
   ReleaseScreen();

   RETURN colour_count;

END CountColours;		


(******************************************************************)
(* This procedure initialises the colour palette with the values	*)
(* stored in the default.pal file.						*)
(******************************************************************)
PROCEDURE Initialise();
VAR
   f : File;
   idx : CARDINAL;
   r, g, b, red, green, blue : CARDINAL;
BEGIN
   IF (OpenFile(f, "default.pal", FALSE) = OK) THEN

   FOR idx := 0 TO 255 DO
	r := CARDINAL(ReadByte(f));
	g := CARDINAL(ReadByte(f));
	b := CARDINAL(ReadByte(f));

	palette[idx].red := r;
	palette[idx].green := g;
	palette[idx].blue := b;

	SetVgaValue(idx);

	SetPaletteColour(SHORTCARD(idx), SHORTCARD(vga_pal[idx].red), SHORTCARD(vga_pal[idx].green), SHORTCARD(vga_pal[idx].blue));
   END; (* FOR *)

   END; (* IF *)

CloseFile(f);

END Initialise;


(*******************)
(* initialisations *)
(*******************)
BEGIN
   first_time := TRUE;
   SetBlack();
   BackgroundColour := 15+16;

   Initialise();
END ColourRoutines.