IMPLEMENTATION MODULE Dialogs;

(******************************************************************)
(*											*)
(*			  	   Dialogs Module					*)
(*											*)
(*	Originally developed by Chee Hong KOK. The present version	*)
(*	is maintained by Matthew Ma.						*)
(*											*)
(*	Last edited :	30-9-97						*)
(*											*)
(*	Function : 		This module implements dialogs.		*)
(*											*)
(*	Comments :		- more realistic 3D dialog windows and	*)
(*				  selection buttons as a result of the 	*)
(*				  modification of the GraphicWindow and	*)
(*				  Buttons modules.				*)
(*											*)
(*	Status :		Working.						*)
(******************************************************************)


FROM MouseRoutines IMPORT
(* proc *)   SetArrow, SetFinger;

FROM GraphicWindow IMPORT
(* const *)  ActualDataOffset,
(* var   *)  CharWidth, CharHeight, NILWindow,
(* type  *)  GraphicalWindow, LongStrPtr,
(* proc  *)  CreateGraphicalWindow, DisplayWindowData, DrawFrame, PutRectangle,
        	 DataRegionDisplayed, HideWindowData, PutString, ObtainScreen,
        	 DisplayWindowTitle, ReleaseScreen, MoveWindowABS, PutChar, PutLine,
        	 HideWindowTitle, DoNothing,
        	 ReturnDataLeftTop,ActivateCancelWhenMouseNolongerInWindow,
        	 DeActivateCancelWhenMouseNolongerInWindow,
	  	 GetWindowLeft, GetWindowTop, GetWindowBottom;

FROM Storage1 IMPORT
(* proc *)   ALLOCATE;

FROM LinkList IMPORT
(* type *)   List, ListElement,
(* proc *)   CreateLinkList, CreateListElement, MoveToTop, ObtainList,
		 ReleaseList, GetFirstElementInList, ElementValid,
		 ReadObjectFromElement,GetNextElementInList;

FROM CancelSequences IMPORT
(* type *)   CancelElement, CancelSource,
(* proc *)   RegisterCancelSequence, EnableCancelSequence, DisableCancelSequence,
        	 ActivateCancelSequence;

FROM TaskControl IMPORT
(* type *)   Lock,
(* proc *)   CreateLock, Obtain, Release;

FROM Str IMPORT
(* proc *)   Length, Copy, Delete, Insert;

FROM KbInterface IMPORT
(* const *)  EXTENDED_AUX, UpArrowMain, DownArrowMain,DeleteKey,BackSpace,TABKey,
             LeftArrowMain, RightArrowMain, ReturnKey, ESCKey, ALTKeyBase, NULL, DummyKey,
(* type  *)  KeyPressed, Keyboard,
(* proc  *)  SetupKeyboardLink, ReadFromKeyboard, ConnectToKeyboard, StuffKeyboard,
        	 DisconnectFromKeyboard, ClearKeyboard;


CONST
  TABCode = -1;
  ESCCode = -2;

TYPE
(*  MultipleString = ARRAY [0..MaxNumberOfStrings-1] OF DialogString;*)

  InputRCD = RECORD
               type:ARRAY [0..MaxNumberOfStrings-1] OF OtherInputType;
               x,y,width,height,
               DisplayNumChars:ARRAY [0..MaxNumberOfStrings-1] OF INTEGER;
               StringLimit:ARRAY [0..MaxNumberOfStrings-1] OF INTEGER;
               colour:ARRAY [0..MaxNumberOfStrings-1] OF CARDINAL;
               true:ARRAY [0..MaxNumberOfStrings-1] OF BOOLEAN;
             END;(*RECORD*)

  DialogBox = POINTER TO RECORD
                number:INTEGER;
                prev, next:DialogBox;
                name:LongStrPtr;
                WhichAlphabetAsKeyPress:INTEGER;
                AlphabetColour:ColourType;
                top, left:INTEGER;
                RespondToSingleClick:BOOLEAN;
              END;(*RECORD*)

  Dialog = POINTER TO RECORD
             name:LongStrPtr;
             t:DialogType;
             inputs:INTEGER;
             w:GraphicalWindow;
             k:Keyboard;
             l:ListElement;
             c:CancelElement;
             top, left:INTEGER;
             BackColour, MessageColour, BoxItemColour:CARDINAL;
             message:LongStrPtr;
             vertical:BOOLEAN;
             boxes:INTEGER;
             FirstTime:BOOLEAN;
             prev_selected_Dialog_Box_ptr, current_selected_Dialog_Box_ptr,
             head, tail:DialogBox;
             MessageWidth, MessageHeight,
             BoxWidth, BoxHeight, BoxXInc, BoxYInc:INTEGER;
             Input:InputRCD;
           END;(*RECORD*)

VAR
  (*list of active dialogs: any window displayed is always put on the top*)
  ActiveDialogList:List;

  (*list of inactive dialogs:TOTALLY not displayed on screen*)
  InactiveDialogList:List;


(****************************************************************************)
(* DummyCancelProcedure                                                     *)
(****************************************************************************)
(* This procedure is used as a dummy cancel procedure so that the right     *)
(*   button will trigger this sequence and not cancel other sequences.      *)
(* THIS DOES NOTHING.                                                       *)
(****************************************************************************)
PROCEDURE DummyCancelProcedure(source:CancelSource);
VAR
  TempElement:ListElement;
  TempDialog:Dialog;
  valid:BOOLEAN;
BEGIN
  GetFirstElementInList(TempElement, ActiveDialogList);
  REPEAT
    valid := ElementValid(TempElement);
    IF valid THEN
      ReadObjectFromElement(TempDialog, TempElement);
      StuffKeyboard(TempDialog^.k, DummyKey);
      StuffKeyboard(TempDialog^.k, ESCKey);
      GetNextElementInList(TempElement, TempElement);
    END;(*IF*)
  UNTIL NOT(valid);
END DummyCancelProcedure;

(****************************************************************************)
(* DrawButton                                                               *)
(****************************************************************************)
(*  This procedure is called to display a buton up or down.                 *)
(*  PARAMETER PASSED:                                                       *)
(*     d - the window that will display the entered string.                 *)
(*     InputNumber - the index of the input.                                *)
(*     down-TRUE will display button down representing button pressed.      *)
(****************************************************************************)
PROCEDURE DrawButton(d:Dialog; InputNumber:CARDINAL; down:BOOLEAN);
BEGIN
  DrawFrame(d^.left + d^.Input.x[InputNumber]-ActualDataOffset,
            d^.top + d^.Input.y[InputNumber]-ActualDataOffset,
            d^.left + d^.Input.x[InputNumber] + d^.Input.width[InputNumber]+ActualDataOffset,
            d^.top + d^.Input.y[InputNumber] + d^.Input.height[InputNumber]+ActualDataOffset,
            d^.Input.colour[InputNumber], NOT(down), TRUE);
END DrawButton;

(****************************************************************************)
(* UpdateDialog                                                             *)
(****************************************************************************)
(* This procedure will update selected boxes on screen.                     *)
(* PARAMETER PASSED:                                                        *)
(*   d - dialog of interest                                                 *)
(****************************************************************************)
PROCEDURE UpdateDialog(d:Dialog);
VAR
  TempCurrentDialogBox:DialogBox;
BEGIN
  IF (d^.prev_selected_Dialog_Box_ptr <> d^.current_selected_Dialog_Box_ptr) THEN
    IF (ObtainScreen()) THEN
      TempCurrentDialogBox := d^.prev_selected_Dialog_Box_ptr;
      IF (TempCurrentDialogBox <> NIL) THEN
        DrawFrame(d^.left + TempCurrentDialogBox^.left-ActualDataOffset,
                  d^.top + TempCurrentDialogBox^.top-ActualDataOffset,
                  d^.left + TempCurrentDialogBox^.left + d^.BoxWidth+ActualDataOffset,
                  d^.top + TempCurrentDialogBox^.top + d^.BoxHeight+ActualDataOffset,
                  d^.BackColour, TRUE, TRUE);
        PutString(d^.w,TempCurrentDialogBox^.name^,
                  TempCurrentDialogBox^.left+1,
                  TempCurrentDialogBox^.top, d^.BoxItemColour,FALSE);
        IF (TempCurrentDialogBox^.WhichAlphabetAsKeyPress > 0) THEN
          PutChar(d^.w,TempCurrentDialogBox^.name^[TempCurrentDialogBox^.WhichAlphabetAsKeyPress-1],
                  TempCurrentDialogBox^.left+(TempCurrentDialogBox^.WhichAlphabetAsKeyPress-1)*CharWidth+1,
                  TempCurrentDialogBox^.top, TempCurrentDialogBox^.AlphabetColour,FALSE);
        END;(*IF*)
      END;(*IF*)
      TempCurrentDialogBox := d^.current_selected_Dialog_Box_ptr;
      IF (TempCurrentDialogBox <> NIL) THEN
        DrawFrame(d^.left + TempCurrentDialogBox^.left-ActualDataOffset,
                  d^.top + TempCurrentDialogBox^.top-ActualDataOffset,
                  d^.left + TempCurrentDialogBox^.left + d^.BoxWidth+ActualDataOffset,
                  d^.top + TempCurrentDialogBox^.top + d^.BoxHeight+ActualDataOffset,
                  d^.BackColour, FALSE, TRUE);
        PutString(d^.w,TempCurrentDialogBox^.name^,
                  TempCurrentDialogBox^.left+ActualDataOffset+1,
                  TempCurrentDialogBox^.top+ActualDataOffset, d^.BoxItemColour,FALSE);
        IF (TempCurrentDialogBox^.WhichAlphabetAsKeyPress > 0) THEN
          PutChar(d^.w,TempCurrentDialogBox^.name^[TempCurrentDialogBox^.WhichAlphabetAsKeyPress-1],
                  TempCurrentDialogBox^.left+ActualDataOffset+(TempCurrentDialogBox^.WhichAlphabetAsKeyPress-1)*CharWidth+1,
                  TempCurrentDialogBox^.top+ActualDataOffset, TempCurrentDialogBox^.AlphabetColour,FALSE);
        END;(*IF*)
      END;(*IF*)
      ReleaseScreen();
    END;(*IF*)
  END;(*IF*)
END UpdateDialog;

(****************************************************************************)
(* DialogMouseResponse                                                      *)
(****************************************************************************)
(* This procedure is used as a procedure to respond to mouse clicks.        *)
(* PARAMETER PASSED:                                                        *)
(*   w - window that represents a particular dialog                         *)
(*   xOffset, yOffset - mouse offset within the window                      *)
(*   LeftPressed - left mouse button pressed                                *)
(*   LeftReleased - left mouse button released                              *)
(*   DoubleClicked - mouse left button double clicked                       *)
(*   Drag - mouse is dragging                                               *)
(****************************************************************************)
PROCEDURE DialogMouseResponse(w:GraphicalWindow;
			      xOffset, yOffset:INTEGER;
                              LeftPressed, LeftReleased,
			      DoubleClicked, Drag:BOOLEAN);
VAR
  TempElement:ListElement;
  TempDialog:Dialog;
  TempCurrentDialogBox:DialogBox;
  ok, blFound:BOOLEAN;
  TempKeyPressed:KeyPressed;
  count:INTEGER;
BEGIN
  TempKeyPressed[1] := NULL;

  blFound := FALSE;

  ObtainList();

  (*find the dialog box in to be activated*)
  GetFirstElementInList(TempElement, ActiveDialogList);

  ReleaseList();

  IF (ElementValid(TempElement)) THEN
    ReadObjectFromElement(TempDialog, TempElement);
    IF (w = TempDialog^.w) THEN
      (*found the dialog box*)
      blFound := TRUE;
    END;(*IF*)
  END;(*IF*)

  IF (blFound) THEN
    TempCurrentDialogBox := TempDialog^.head;
    IF (TempCurrentDialogBox <> NIL) THEN
      blFound := FALSE;
      REPEAT
        IF (((xOffset >= TempCurrentDialogBox^.left) AND (xOffset <= TempCurrentDialogBox^.left + TempDialog^.BoxWidth)) AND
            ((yOffset >= TempCurrentDialogBox^.top) AND (yOffset <= TempCurrentDialogBox^.top + TempDialog^.BoxHeight))) THEN
          blFound := TRUE;
          TempDialog^.prev_selected_Dialog_Box_ptr := TempDialog^.current_selected_Dialog_Box_ptr;
          TempDialog^.current_selected_Dialog_Box_ptr := TempCurrentDialogBox;
          UpdateDialog(TempDialog);
          IF (DoubleClicked OR
             (TempCurrentDialogBox^.RespondToSingleClick AND LeftReleased)) THEN
            TempKeyPressed[0] := TempCurrentDialogBox^.name^[TempCurrentDialogBox^.WhichAlphabetAsKeyPress - 1];
            CASE TempDialog^.t OF
              Standard:
            | WithOtherInput:
                StuffKeyboard(TempDialog^.k, DummyKey);
            END;(*CASE*)
            StuffKeyboard(TempDialog^.k, TempKeyPressed);
          END;(*IF*)
        END;(*IF*)
        TempCurrentDialogBox := TempCurrentDialogBox^.next;
      UNTIL (TempCurrentDialogBox = TempDialog^.head) OR (blFound);

      IF (NOT(blFound) AND (TempDialog^.t = WithOtherInput)) THEN
        count := 0;
        REPEAT
          IF ((TempDialog^.Input.type[count] = Button) AND
              ((xOffset >= TempDialog^.Input.x[count]) AND (xOffset <= TempDialog^.Input.x[count] + TempDialog^.Input.width[count])) AND
              ((yOffset >= TempDialog^.Input.y[count]) AND (yOffset <= TempDialog^.Input.y[count] + TempDialog^.Input.height[count]))
               AND (TempCurrentDialogBox^.RespondToSingleClick AND LeftReleased)) THEN
            blFound := TRUE;

            TempDialog^.Input.true[count] := NOT(TempDialog^.Input.true[count]);

            IF (ObtainScreen()) THEN
              DrawButton(TempDialog,count, TempDialog^.Input.true[count]);

		  (*************************************
		  IF (MouseParameter_LeftPressed) THEN
		     DrawButton(TempDialog,count, TURE);
		  ELSE
		     DrawButton(TempDialog,count, FALSE);
		  END;
		  *************************************)

              ReleaseScreen();
            END;(*IF*)
          END;(*IF*)
          count := count + 1;
        UNTIL (count >= TempDialog^.inputs) OR (blFound);
      END;(*IF*)
    END;(*IF*)
  END;(*IF*)
END DialogMouseResponse;

(****************************************************************************)
(* Caps                                                                     *)
(****************************************************************************)
(* This procedure will convert a lower case alphabet to upper case.         *)
(* PARAMETER PASSED:                                                        *)
(*   char - character of interest                                           *)
(* RETURNS:                                                                 *)
(*   upper cased of the character                                           *)
(****************************************************************************)
PROCEDURE Caps(char:CHAR):CHAR;
BEGIN
  CASE char OF
    "a".."z":
      RETURN (VAL(CHAR, ORD(char) - ORD("a") + ORD("A")));
  |  ELSE
      RETURN(char);
  END;(*CASE*)
END Caps;

(****************************************************************************)
(* DialogKeyboardSelection                                                  *)
(****************************************************************************)
(* This procedure will execute the item in dialog upon keyboard request     *)
(* PARAMETER PASSED:                                                        *)
(*   KeyPressed - keyboard entry for the menus                              *)
(****************************************************************************)
PROCEDURE DialogKeyboardSelection(d:Dialog; KeyPressed:KeyPressed):INTEGER;
VAR
  TempCurrentDialogBox:DialogBox;
  TempChar:CHAR;
BEGIN
  TempCurrentDialogBox := d^.current_selected_Dialog_Box_ptr;

  CASE KeyPressed[0] OF
    ESCKey:
      RETURN (ESCCode);

  | TABKey:
      RETURN (TABCode);

  | EXTENDED_AUX:
      IF d^.vertical THEN
        d^.prev_selected_Dialog_Box_ptr := TempCurrentDialogBox;
        CASE (KeyPressed[1]) OF
          UpArrowMain:
            d^.current_selected_Dialog_Box_ptr :=
                 TempCurrentDialogBox^.prev;
        | DownArrowMain:
            d^.current_selected_Dialog_Box_ptr :=
                 TempCurrentDialogBox^.next;
        | LeftArrowMain, RightArrowMain:
            RETURN (TABCode);

        | ELSE
            RETURN (0);
        END;(*CASE*)

      ELSIF NOT(d^.vertical) THEN
        d^.prev_selected_Dialog_Box_ptr := TempCurrentDialogBox;
        CASE (KeyPressed[1]) OF
          LeftArrowMain:
            d^.current_selected_Dialog_Box_ptr :=
               TempCurrentDialogBox^.prev;

        | RightArrowMain:
            d^.current_selected_Dialog_Box_ptr :=
               TempCurrentDialogBox^.next;

        | UpArrowMain, DownArrowMain:
            RETURN (TABCode);

        | ELSE
            RETURN (0);
        END;(*CASE*)
      END;(*IF*)
      UpdateDialog(d);
      RETURN (0);

  | ReturnKey:
      RETURN (d^.current_selected_Dialog_Box_ptr^.number);

  | "A".."Z", "a".."z":
      TempChar := Caps(KeyPressed[0]);

      TempCurrentDialogBox := d^.head;
      IF (TempCurrentDialogBox <> NIL) THEN
        REPEAT
          IF (TempChar = Caps(TempCurrentDialogBox^.name^[TempCurrentDialogBox^.WhichAlphabetAsKeyPress - 1])) THEN
            d^.prev_selected_Dialog_Box_ptr := d^.current_selected_Dialog_Box_ptr;
            d^.current_selected_Dialog_Box_ptr := TempCurrentDialogBox;

            UpdateDialog(d);
            RETURN (d^.current_selected_Dialog_Box_ptr^.number);
          END;(*IF*)
          TempCurrentDialogBox := TempCurrentDialogBox^.next;
        UNTIL (TempCurrentDialogBox = d^.head);
      END;(*IF*)
      RETURN (0);

  | ELSE
      RETURN (0);
  END;(*CASE*)
END DialogKeyboardSelection;


(****************************************************************************)
(* SelectButton                                                             *)
(****************************************************************************)
(*  This procedure selects a button to be modified.                         *)
(*  PARAMETER PASSED:                                                       *)
(*     d - the window that will display the entered string.                 *)
(*     InputNumber - the index of the input.                                *)
(****************************************************************************)
PROCEDURE SelectButton(d:Dialog; InputNumber:CARDINAL);
BEGIN
  PutLine(d^.w,
          d^.Input.x[InputNumber]-ActualDataOffset-2,
          d^.Input.y[InputNumber]-ActualDataOffset-2,
          d^.Input.x[InputNumber]-ActualDataOffset-2,
          d^.Input.y[InputNumber]+d^.Input.height[InputNumber]+ActualDataOffset+2,
          15-d^.BackColour);

  PutLine(d^.w,
          d^.Input.x[InputNumber]-ActualDataOffset-2,
          d^.Input.y[InputNumber]-ActualDataOffset-2,
          d^.Input.x[InputNumber]+d^.Input.width[InputNumber]+ActualDataOffset+2,
          d^.Input.y[InputNumber]-ActualDataOffset-2,
          15-d^.BackColour);

  PutLine(d^.w,
          d^.Input.x[InputNumber]+d^.Input.width[InputNumber]+ActualDataOffset+2,
          d^.Input.y[InputNumber]-ActualDataOffset-2,
          d^.Input.x[InputNumber]+d^.Input.width[InputNumber]+ActualDataOffset+2,
          d^.Input.y[InputNumber]+d^.Input.height[InputNumber]+ActualDataOffset+2,
          15-d^.BackColour);

  PutLine(d^.w,
          d^.Input.x[InputNumber]+d^.Input.width[InputNumber]+ActualDataOffset+2,
          d^.Input.y[InputNumber]+d^.Input.height[InputNumber]+ActualDataOffset+2,
          d^.Input.x[InputNumber]-ActualDataOffset-2,
          d^.Input.y[InputNumber]+d^.Input.height[InputNumber]+ActualDataOffset+2,
          15-d^.BackColour);

END SelectButton;

(****************************************************************************)
(* UnSelectButton                                                           *)
(****************************************************************************)
(*  This procedure un-select a button to be modified.                       *)
(*  PARAMETER PASSED:                                                       *)
(*     d - the window that will display the entered string.                 *)
(*     InputNumber - the index of the input.                                *)
(****************************************************************************)
PROCEDURE UnSelectButton(d:Dialog; InputNumber:CARDINAL);
BEGIN
  PutLine(d^.w,
          d^.Input.x[InputNumber]-ActualDataOffset-2,
          d^.Input.y[InputNumber]-ActualDataOffset-2,
          d^.Input.x[InputNumber]-ActualDataOffset-2,
          d^.Input.y[InputNumber]+d^.Input.height[InputNumber]+ActualDataOffset+2,
          d^.BackColour);

  PutLine(d^.w,
          d^.Input.x[InputNumber]-ActualDataOffset-2,
          d^.Input.y[InputNumber]-ActualDataOffset-2,
          d^.Input.x[InputNumber]+d^.Input.width[InputNumber]+ActualDataOffset+2,
          d^.Input.y[InputNumber]-ActualDataOffset-2,
          d^.BackColour);

  PutLine(d^.w,
          d^.Input.x[InputNumber]+d^.Input.width[InputNumber]+ActualDataOffset+2,
          d^.Input.y[InputNumber]-ActualDataOffset-2,
          d^.Input.x[InputNumber]+d^.Input.width[InputNumber]+ActualDataOffset+2,
          d^.Input.y[InputNumber]+d^.Input.height[InputNumber]+ActualDataOffset+2,
          d^.BackColour);

  PutLine(d^.w,
          d^.Input.x[InputNumber]+d^.Input.width[InputNumber]+ActualDataOffset+2,
          d^.Input.y[InputNumber]+d^.Input.height[InputNumber]+ActualDataOffset+2,
          d^.Input.x[InputNumber]-ActualDataOffset-2,
          d^.Input.y[InputNumber]+d^.Input.height[InputNumber]+ActualDataOffset+2,
          d^.BackColour);

END UnSelectButton;


(****************************************************************************)
(* Display                                                                  *)
(****************************************************************************)
(*  this procedure will display a string onto screen                        *)
(* PARAMETER PASSED:                                                        *)
(*   d - dialog of interest                                                 *)
(*   string - string edited                                                 *)
(*   CurrentIndex - current input edited                                    *)
(*   DisplayFrom - starting character in string to be displayed             *)
(*   MAXCHARS - length of the string                                        *)
(****************************************************************************)
PROCEDURE Display(d:Dialog; string:ARRAY OF CHAR; CurrentIndex, DisplayFrom, MAXCHARS:INTEGER);
VAR
  TempCharDisplay:INTEGER;
BEGIN
  TempCharDisplay := 0;
  WHILE (TempCharDisplay + DisplayFrom < MAXCHARS) AND (TempCharDisplay < d^.Input.DisplayNumChars[CurrentIndex]) DO
    PutChar(d^.w,"",d^.Input.x[CurrentIndex]+TempCharDisplay*CharWidth,d^.Input.y[CurrentIndex],d^.BackColour,FALSE);
    PutChar(d^.w,string[TempCharDisplay+DisplayFrom],d^.Input.x[CurrentIndex]+TempCharDisplay*CharWidth,d^.Input.y[CurrentIndex],d^.Input.colour[CurrentIndex],FALSE);
    INC(TempCharDisplay);
  END;(*WHILE*)
  PutChar(d^.w,"",d^.Input.x[CurrentIndex]+TempCharDisplay*CharWidth,d^.Input.y[CurrentIndex],d^.BackColour,FALSE);
END Display;



(****************************************************************************)
(* ReadStringFromKeyboard2                                                  *)
(****************************************************************************)
(*  This procedure is called by other tasks to read keys pressed.           *)
(*  PARAMETER PASSED:                                                       *)
(*     w - the window that will display the entered string.                 *)
(*     KeyBoard - a data structure to hold information about the keyboard   *)
(*                link.                                                     *)
(*     string - string to be read.                                          *)
(*     x, y - character offset of the string in the window, starting from   *)
(*            (0,0).                                                        *)
(*     DisplayNumChars - allow this number of characters of the string to be*)
(*                       displayed                                          *)
(*     colour - colour of the string to be displayed.                       *)
(*     backcolour - background colour of the string.                        *)
(****************************************************************************)
PROCEDURE ReadStringFromKeyboard2(d:Dialog;
                                  strings:DialogStringPointer):INTEGER;
VAR
  Selection, count, CurrentIndex:INTEGER;
  MAXCHARS, CharCount,
  CursorAt,DisplayFrom:ARRAY [0..MaxNumberOfStrings-1] OF INTEGER;
  TempKeyPressed:KeyPressed;
  display:BOOLEAN;
BEGIN

  CurrentIndex := 0;

  Selection := 0;

  IF (ObtainScreen()) THEN
    FOR count := 0 TO d^.inputs - 1 DO
      MAXCHARS[count] := Length(strings^[count]);
      CharCount[count] := 0;
      CursorAt[count] := 0;
      DisplayFrom[count] := 0;
      IF (MAXCHARS[count] > 0) THEN
        Display(d,strings^[count],count, 0,MAXCHARS[count]);
      END;(*IF*)
      IF (d^.Input.type[count] = Button) THEN
        IF (strings^[count][0] = "T") THEN
          d^.Input.true[count] := TRUE;
          DrawButton(d,count,TRUE);
        ELSE
          strings^[count][0] := "F";
          d^.Input.true[count] := FALSE;
          DrawButton(d,count,FALSE);
        END;(*IF*)
      END;(*IF*)
    END;(*FOR*)
    ReleaseScreen();
  END;(*IF*)

  REPEAT

    display := FALSE;

    IF (ObtainScreen()) THEN
      IF (d^.Input.type[CurrentIndex] = String) THEN
        PutLine(d^.w,d^.Input.x[CurrentIndex]+CursorAt[CurrentIndex]*CharWidth,d^.Input.y[CurrentIndex] + CharHeight-1,d^.Input.x[CurrentIndex]+(CursorAt[CurrentIndex]+1)*CharWidth,d^.Input.y[CurrentIndex]+ CharHeight-1,d^.Input.colour[CurrentIndex]);
        PutLine(d^.w,d^.Input.x[CurrentIndex]+CursorAt[CurrentIndex]*CharWidth,d^.Input.y[CurrentIndex]+ CharHeight,d^.Input.x[CurrentIndex]+(CursorAt[CurrentIndex]+1)*CharWidth,d^.Input.y[CurrentIndex]+ CharHeight,d^.Input.colour[CurrentIndex]);
      ELSE
        SelectButton(d,CurrentIndex);
      END;(*IF*)
      ReleaseScreen();
    END;(*IF*)

    ReadFromKeyboard(d^.k, TempKeyPressed);

    IF (ObtainScreen()) THEN
      IF (d^.Input.type[CurrentIndex] = String) THEN
        PutLine(d^.w,d^.Input.x[CurrentIndex]+CursorAt[CurrentIndex]*CharWidth,d^.Input.y[CurrentIndex] + CharHeight-1,d^.Input.x[CurrentIndex]+(CursorAt[CurrentIndex]+1)*CharWidth,d^.Input.y[CurrentIndex]+ CharHeight-1,d^.BackColour);
        PutLine(d^.w,d^.Input.x[CurrentIndex]+CursorAt[CurrentIndex]*CharWidth,d^.Input.y[CurrentIndex]+ CharHeight,d^.Input.x[CurrentIndex]+(CursorAt[CurrentIndex]+1)*CharWidth,d^.Input.y[CurrentIndex]+ CharHeight,d^.BackColour);
      ELSE
        UnSelectButton(d,CurrentIndex);
      END;(*IF*)
      ReleaseScreen();
    END;(*IF*)

    CASE TempKeyPressed[0] OF
      TABKey:
        INC(CurrentIndex);
        IF (CurrentIndex >= d^.inputs) THEN
          CurrentIndex := 0;
          REPEAT
            ReadFromKeyboard(d^.k, TempKeyPressed);
            Selection := DialogKeyboardSelection(d,TempKeyPressed);
          UNTIL (TempKeyPressed[0] = ESCKey) OR (Selection <> 0);
        END;(*IF*)

    | DummyKey:
        (*Used for Mouse Input Only*)
        ReadFromKeyboard(d^.k, TempKeyPressed);
        Selection := DialogKeyboardSelection(d,TempKeyPressed);

    | EXTENDED_AUX:
        IF (d^.Input.type[CurrentIndex] <> Button) THEN
          (*respond to extended keys only if current input is not button*)

          CASE TempKeyPressed[1] OF
            LeftArrowMain:
              (*left arrow*)
              IF (CharCount[CurrentIndex] > 0) THEN
                DEC(CharCount[CurrentIndex]);
                IF (CursorAt[CurrentIndex] > 0) THEN
                  DEC(CursorAt[CurrentIndex]);
                ELSE
                  DEC(DisplayFrom[CurrentIndex]);
                  display := TRUE;
                END;(*IF*)
              END;(*IF*)

          | RightArrowMain:
              (*right arrow*)
              IF (CharCount[CurrentIndex] < MAXCHARS[CurrentIndex]) THEN
                INC(CharCount[CurrentIndex]);
                IF (CursorAt[CurrentIndex] < d^.Input.DisplayNumChars[CurrentIndex]) THEN
                  INC(CursorAt[CurrentIndex]);
                ELSE
                  INC(DisplayFrom[CurrentIndex]);
                  display := TRUE;
                END;(*IF*)
              END;(*IF*)

          | DeleteKey:
            (*delete key*)
            IF (CharCount[CurrentIndex] < MAXCHARS[CurrentIndex]) THEN
              DEC(MAXCHARS[CurrentIndex]);
              Delete(strings^[CurrentIndex],CharCount[CurrentIndex],1);
              display := TRUE;
            END;(*IF*)

          |ELSE
          END;(*IF*)
        END;(*IF*)

        CASE TempKeyPressed[1] OF
          UpArrowMain:
            CurrentIndex := (CurrentIndex + (d^.inputs - 1)) MOD d^.inputs;
            IF (CurrentIndex = d^.inputs - 1) THEN
              REPEAT
                ReadFromKeyboard(d^.k, TempKeyPressed);
                Selection := DialogKeyboardSelection(d,TempKeyPressed);
              UNTIL (TempKeyPressed[0] = ESCKey) OR (Selection <> 0);
            END;(*IF*)

        | DownArrowMain:
            INC(CurrentIndex);
            IF (CurrentIndex >= d^.inputs) THEN
              CurrentIndex := 0;
              REPEAT
                ReadFromKeyboard(d^.k, TempKeyPressed);
                Selection := DialogKeyboardSelection(d,TempKeyPressed);
              UNTIL (TempKeyPressed[0] = ESCKey) OR (Selection <> 0);
            END;(*IF*)
        END;(*CASE*)

    | BackSpace:
        (*backspace key*)

        IF ((CharCount[CurrentIndex] > 0) AND (d^.Input.type[CurrentIndex] <> Button)) THEN
          DEC(CharCount[CurrentIndex]);
          DEC(MAXCHARS[CurrentIndex]);
          Delete(strings^[CurrentIndex],CharCount[CurrentIndex],1);
          IF (CursorAt[CurrentIndex] > 0) THEN
            DEC(CursorAt[CurrentIndex]);
          ELSE
            DEC(DisplayFrom[CurrentIndex]);
          END;(*IF*)
          display := TRUE;
        END;(*IF*)

    | ReturnKey:
        IF (d^.Input.type[CurrentIndex] = Button) THEN
          d^.Input.true[CurrentIndex] := NOT(d^.Input.true[CurrentIndex]);
            DrawButton(d,CurrentIndex,d^.Input.true[CurrentIndex]);
        END;(*IF*)

    | ESCKey:

    | " ".."~":
      IF ((CharCount[CurrentIndex] < d^.Input.StringLimit[CurrentIndex])
           AND (MAXCHARS[CurrentIndex] < d^.Input.StringLimit[CurrentIndex])
           AND (d^.Input.type[CurrentIndex] <> Button)) THEN
        TempKeyPressed[1] := NULL;
        Insert(strings^[CurrentIndex], TempKeyPressed,CharCount[CurrentIndex]);
        INC(MAXCHARS[CurrentIndex]);
        INC(CharCount[CurrentIndex]);
        IF (CursorAt[CurrentIndex] < d^.Input.DisplayNumChars[CurrentIndex]) THEN
          INC(CursorAt[CurrentIndex]);
        ELSE
          INC(DisplayFrom[CurrentIndex]);
        END;(*IF*)
        display := TRUE;
      END;(*IF*)

    | ELSE
    END;(*CASE*)

    IF (display) THEN
      IF (ObtainScreen()) THEN
        Display(d,strings^[CurrentIndex],CurrentIndex, DisplayFrom[CurrentIndex], MAXCHARS[CurrentIndex]);
        ReleaseScreen();
      END;(*IF*)

    END;(*IF*)

  UNTIL (Selection > 0) OR (Selection = ESCCode);


  FOR CurrentIndex := 0 TO d^.inputs - 1 DO
    IF (d^.Input.type[CurrentIndex] = Button) THEN
      IF (d^.Input.true[CurrentIndex]) THEN
        strings^[CurrentIndex] := "T";
      ELSE
        strings^[CurrentIndex] := "F";
      END;(*IF*)
    END;(*IF*)
  END;(*FOR*)

  RETURN (Selection);
END ReadStringFromKeyboard2;

(****************************************************************************)
(* DisplayDialogData                                                        *)
(****************************************************************************)
(* This procedure displays the window of a dialog for once only.            *)
(* PARAMETER PASSED:                                                        *)
(*   d - dialog of interest.                                                *)
(****************************************************************************)
PROCEDURE DisplayDialogData(d:Dialog);
VAR
  TempCurrentDialogBox:DialogBox;
BEGIN
  IF (ObtainScreen()) THEN
    DisplayWindowTitle(d^.w);
    DisplayWindowData(d^.w);
    IF (d^.t = Standard) THEN
      PutString(d^.w,d^.message^,5,((d^.MessageHeight-CharHeight) DIV 2),d^.MessageColour,FALSE);
    END;(*IF*)
    TempCurrentDialogBox := d^.head;

    IF (TempCurrentDialogBox <> NIL) THEN
      REPEAT
        DrawFrame(d^.left + TempCurrentDialogBox^.left - ActualDataOffset,
                  d^.top + TempCurrentDialogBox^.top-ActualDataOffset,
                  d^.left + TempCurrentDialogBox^.left + d^.BoxWidth+ActualDataOffset,
                  d^.top + TempCurrentDialogBox^.top + d^.BoxHeight+ActualDataOffset,
                  d^.BackColour, TRUE, TRUE);
        PutString(d^.w,TempCurrentDialogBox^.name^,
                  TempCurrentDialogBox^.left+1,
                  TempCurrentDialogBox^.top, d^.BoxItemColour,FALSE);
        IF (TempCurrentDialogBox^.WhichAlphabetAsKeyPress > 0) THEN
          PutChar(d^.w,TempCurrentDialogBox^.name^[TempCurrentDialogBox^.WhichAlphabetAsKeyPress-1],
                  TempCurrentDialogBox^.left+(TempCurrentDialogBox^.WhichAlphabetAsKeyPress-1)*CharWidth+1,
                  TempCurrentDialogBox^.top, TempCurrentDialogBox^.AlphabetColour,FALSE);
        END;(*IF*)
        TempCurrentDialogBox := TempCurrentDialogBox^.next;
      UNTIL (TempCurrentDialogBox = d^.head);
    END;(*IF*)
    ReleaseScreen();
  END;(*IF*)
  UpdateDialog(d);
END DisplayDialogData;


(****************************************************************************)
(* DisplayDialog                                                            *)
(****************************************************************************)
(* This procedure displays the dialog on screen.                            *)
(* PARAMETER PASSED:                                                        *)
(*   d - dialog of interest                                                 *)
(*   ClearMessageRegion - set to true if want to clear region for displaying*)
(*                        messages.                                         *)
(****************************************************************************)
PROCEDURE DisplayDialog (d:Dialog; ClearMessageRegion:BOOLEAN);
VAR
   x1, y1, y2 : INTEGER;
BEGIN
  IF d^.FirstTime THEN
    d^.FirstTime := FALSE;
    DisplayDialogData(d);
  ELSE
    IF (ObtainScreen()) THEN
      DisplayWindowTitle(d^.w);
      DisplayWindowData(d^.w);

      IF ClearMessageRegion THEN
        PutRectangle(d^.w,0,0,d^.MessageWidth-1,d^.MessageHeight-1,d^.BackColour,TRUE);
      END;(*IF*)

      ReleaseScreen();
    END;(*IF*)
  END;(*IF*)
END DisplayDialog;

(****************************************************************************)
(* HideDialog                                                               *)
(****************************************************************************)
(* This procedure hides a dialog from screen.                               *)
(* PARAMETER PASSED:                                                        *)
(*   d - dialog of interest                                                 *)
(****************************************************************************)
PROCEDURE HideDialog (d:Dialog);
BEGIN
  IF (ObtainScreen()) THEN
    HideWindowData(d^.w);
    HideWindowTitle(d^.w);
    ReleaseScreen();
  END;(*IF*)
END HideDialog;

(****************************************************************************)
(* DialogSelection                                                          *)
(****************************************************************************)
(* This procedure gets selection from a dialog.                             *)
(* MUST USE DisplayDialog before using this procedure.                      *)
(* PARAMETER PASSED:                                                        *)
(*   d - dialog of interest                                                 *)
(*   strings - all the strings entered.                                     *)
(*             IGNORE THIS PARAMETER IF DIALOG TYPE IS Standard             *)
(*        if input type is String then the corresponding string is the      *)
(*           string entered;                                                *)
(*        otherwise, the string has only one character, that is 'T' or 'F'  *)
(*           corresponding to TRUE or FALSE: TRUE=button pressed.           *)
(* RETUENS:                                                                 *)
(*   an integer number corresponding to the selection made.                 *)
(*   0 - indicates cancelled.                                               *)
(*   1 - first selection.                                                   *)
(*   2 - second selection.                                                  *)
(*      etc.                                                                *)
(****************************************************************************)
PROCEDURE DialogSelection (d:Dialog; strings:DialogStringPointer):INTEGER;
VAR
  TempKeyPressed:KeyPressed;
  Selection:INTEGER;
BEGIN
  ConnectToKeyboard(d^.k);
  EnableCancelSequence(d^.c);

  IF NOT(DataRegionDisplayed(d^.w)) THEN
    RETURN (0);
  END;(*IF*)

  ObtainList();
  MoveToTop(d^.l, ActiveDialogList);
  ReleaseList();

  ClearKeyboard(d^.k);

  ActivateCancelWhenMouseNolongerInWindow(d^.w);

  CASE d^.t OF
    Standard:
      REPEAT
        ReadFromKeyboard(d^.k, TempKeyPressed);
        Selection := DialogKeyboardSelection(d,TempKeyPressed);
      UNTIL (TempKeyPressed[0] = ESCKey) OR (Selection > 0) ;
  | WithOtherInput:
      Selection := ReadStringFromKeyboard2(d,strings);
  END;(*CASE*)

  DeActivateCancelWhenMouseNolongerInWindow(d^.w);

  ObtainList();
  MoveToTop(d^.l, InactiveDialogList);
  ReleaseList();

  DisableCancelSequence(d^.c);
  DisconnectFromKeyboard(d^.k);

  IF (Selection < 1) THEN
    Selection := 0;
  END;(*IF*)

  RETURN (Selection);
END DialogSelection;

(****************************************************************************)
(* DialogWindow                                                             *)
(****************************************************************************)
(* This procedure returns the window structure for the given dialog.        *)
(* PARAMETER PASSED:                                                        *)
(*   d - dialog of interest.                                                *)
(* RETUENS:                                                                 *)
(*   window for the dialog.                                                 *)
(****************************************************************************)
PROCEDURE DialogWindow (d:Dialog):GraphicalWindow;
BEGIN
  RETURN(d^.w);
END DialogWindow;

(****************************************************************************)
(* StuffSelection                                                           *)
(****************************************************************************)
(* This procedure stuff a dummy or useful selection to a dialog window.     *)
(* PARAMETER PASSED:                                                        *)
(*   d - dialog of interest.                                                *)
(*   selection - the intended selection.                                    *)
(****************************************************************************)
PROCEDURE StuffSelection (d:Dialog; selection:KeyPressed);
BEGIN
  StuffKeyboard(d^.k, selection);
END StuffSelection;

(****************************************************************************)
(* MoveDialogABS                                                            *)
(****************************************************************************)
(* This procedure moves a dialog to a location on screen.                   *)
(* PARAMETER PASSED:                                                        *)
(*   d - dialog of interest.                                                *)
(*   x,y - absolute position on screen.                                     *)
(****************************************************************************)
PROCEDURE MoveDialogABS (d:Dialog; x,y:INTEGER);
BEGIN
  MoveWindowABS(d^.w,x,y);
  ReturnDataLeftTop(d^.w,d^.left, d^.top);
END MoveDialogABS;

(****************************************************************************)
(* CreateDialog                                                             *)
(****************************************************************************)
(* This procedure creates a dialog window; without displaying it.           *)
(* PARAMETER PASSED:                                                        *)
(*   d - the dialog window to be created                                    *)
(*   name - name of the dialog                                              *)
(*   type - type of dialog to be used.                                      *)
(*   NumberOfOtherInputs - number of other inputs.                          *)
(*   left, top,    - top left corner of the dialog box                      *)
(*   DialogBackColour - background colour of the dialog window              *)
(*   vertical - 2 types of button arrangement                               *)
(*   message - message to be displayed on the dialog window                 *)
(*   MessageWidth - width of box allocated for displaying a message.        *)
(*   MessageHeight - height of box allocated for displaying a message.      *)
(*   BoxWidth - width of buttons.                                           *)
(*   BoxHeight - height of buttons.                                         *)
(*   BoxXInc - number of pixels between 2 buttons.                          *)
(*   BoxYInc - number of pixels between 2 buttons.                          *)
(****************************************************************************)
PROCEDURE CreateDialog (VAR (*OUT*) d:Dialog;
                        name:LongStrPtr;
                        type:DialogType;
                        NumberOfOtherInputs:INTEGER;
			left, top: INTEGER;
			DialogBackColour, MessageColour, BoxItemColour:ColourType;
			vertical:BOOLEAN;
			message:LongStrPtr;
                        MessageWidth, MessageHeight,
			BoxWidth, BoxHeight, BoxXInc, BoxYInc:INTEGER);

BEGIN
  (*make new dialog window*)
  NEW(d);

  d^.name := name;

  d^.t := type;

  IF (d^.t = WithOtherInput) THEN
    d^.inputs := NumberOfOtherInputs;
  ELSE
    d^.inputs := 0;
  END;(*IF*)

  d^.w := NILWindow;

  (*setup a keyboard link for this dialog*)
  SetupKeyboardLink(d^.k);

  (*add this dialog to the list*)
  CreateListElement(InactiveDialogList, d^.l, d);

  (*create dummy cancel sequence for bypassing the right button cancel ability*)
  RegisterCancelSequence(DummyCancelProcedure,d^.c);

  (*are boxes arranged in vertival alignment?*)
  d^.vertical := vertical;

  (*number of boxes available to this dialog*)
  d^.boxes := 0;

  d^.FirstTime := TRUE;

  d^.head := NIL;
  d^.tail := NIL;

  IF message <> NIL THEN
    d^.message := message;
  ELSE
    ALLOCATE(d^.message,1);
    Copy(d^.message^, " ");
  END;(*IF*)

  d^.left := left;
  d^.top := top;
  d^.BackColour := DialogBackColour;
  d^.MessageColour := MessageColour;
  d^.BoxItemColour := BoxItemColour;

  d^.MessageHeight := MessageHeight;
  d^.MessageWidth := MessageWidth;
  d^.BoxWidth := BoxWidth;
  d^.BoxHeight := BoxHeight;
  d^.BoxXInc := BoxXInc;
  d^.BoxYInc := BoxYInc;

  d^.prev_selected_Dialog_Box_ptr := NIL;
  d^.current_selected_Dialog_Box_ptr := NIL;

END CreateDialog;

(****************************************************************************)
(* AddABox                                                                  *)
(****************************************************************************)
(* This procedure creates a box selection within a dialog window.           *)
(* PARAMETER PASSED:                                                        *)
(*   d - the dialog to be added with this button                            *)
(*   string - name for this button.                                         *)
(*   WhichAlphabetAsKeyPress - which alphabet in name to execute an item    *)
(*              by pressing this character.                                 *)
(*   AlphabetColour - the colour fo this special character                  *)
(*   RespondToSingleClick - set TRUE if waht this button to respond to a    *)
(*                          single mouse button click.                      *)
(****************************************************************************)
PROCEDURE AddABox (d:Dialog; string:LongStrPtr;
                   WhichAlphabetAsKeyPress:INTEGER;
                   AlphabetColour:ColourType;
                   RespondToSingleClick:BOOLEAN);
VAR
  db:DialogBox;
BEGIN
  NEW(db);

  INC(d^.boxes);
  db^.number := d^.boxes;
  db^.RespondToSingleClick := RespondToSingleClick;

  IF (d^.head = NIL) THEN
    (*first box in the list*)
    d^.head := db;
    d^.tail := db;
    d^.current_selected_Dialog_Box_ptr := db;
  END;(*IF*)

  (*previous box is the last box registered in the dialog*)
  db^.prev := d^.tail;
  (*next box is the firxt box registered in the dialog*)
  db^.next := d^.head;

  (*this box is now the next of last box registered in the dialog*)
  d^.tail^.next := db;

  (*this box is now the previous of first box registered in the dialog*)
  d^.head^.prev := db;

  (*this box is now the last box registered in the dialog*)
  d^.tail := db;

  db^.WhichAlphabetAsKeyPress := WhichAlphabetAsKeyPress;
  db^.AlphabetColour := AlphabetColour;

  IF string <> NIL THEN
    db^.name := string;
  ELSE
    ALLOCATE(db^.name, 1);
    Copy(db^.name^, " ");
  END;(*IF*)

  IF (d^.vertical) THEN
    (*Vertically arranged*)
    db^.top := (d^.BoxHeight + d^.BoxYInc+2*ActualDataOffset)*db^.number - d^.BoxHeight;
    db^.left := d^.MessageWidth + d^.BoxXInc;
  ELSE
    (*Horizontally arranged*)
    db^.left := (d^.BoxWidth + d^.BoxXInc+2*ActualDataOffset)*db^.number - d^.BoxWidth;
    db^.top := d^.MessageHeight + d^.BoxYInc;
  END;(*IF*)
END AddABox;

(****************************************************************************)
(* EndCreateDialog                                                          *)
(****************************************************************************)
(* This procedure displays the window of a dialog for once only.            *)
(* PARAMETER PASSED:                                                        *)
(*   d - dialog of interest.                                                *)
(****************************************************************************)
PROCEDURE EndCreateDialog(d:Dialog);
VAR
  right, bottom:INTEGER;
BEGIN
  IF (d^.vertical) THEN
    right := d^.MessageWidth + 2*d^.BoxXInc + d^.BoxWidth + 2*ActualDataOffset;
    bottom := d^.tail^.top + d^.BoxHeight+d^.BoxYInc + 2*ActualDataOffset;
    IF (bottom < d^.MessageHeight) THEN
      bottom := d^.MessageHeight;
    END;(*IF*)
  ELSE
    bottom := d^.MessageHeight + 2*d^.BoxYInc + d^.BoxHeight + 2*ActualDataOffset;
    right := d^.tail^.left + d^.BoxWidth+d^.BoxXInc + 2*ActualDataOffset;
    IF (right < d^.MessageWidth) THEN
      right := d^.MessageWidth;
    END;(*IF*)
  END;(*IF*)

  IF d^.name <> NIL THEN
    bottom := bottom + CharHeight + 2*ActualDataOffset;
  END;(*IF*)

  (*open a window for this dialog*)
  CreateGraphicalWindow(d^.w, d^.left, d^.top, d^.left + right, d^.top + bottom,
                        d^.BackColour,d^.name,0,0,d^.BackColour,d^.MessageColour,
                        TRUE,TRUE,FALSE,FALSE,FALSE,TRUE,
                        DoNothing, DialogMouseResponse,
                        (*SetDefCursor(*SetFinger*)*)SetArrow);
  ReturnDataLeftTop(d^.w,d^.left, d^.top);

END EndCreateDialog;

(****************************************************************************)
(* SetOtherInput                                                            *)
(****************************************************************************)
(* This procedure sets information about each string to be input in a dialog*)
(* PARAMETER PASSED:                                                        *)
(*   d - dialog of interest                                                 *)
(*   InputNumber - the input index in the dialog. from 1, 2, 3...           *)
(*   input - type of input, String or Button.                               *)
(*   x,y - offset into the display dialog                                   *)
(*   width, height - button dimension                                       *)
(*   DiaplayNumChars - the number of characters to be displayed.            *)
(*   StringLimit - maximum length of this string.                           *)
(*   colour - colour of the string                                          *)
(****************************************************************************)
PROCEDURE SetOtherInput (d:Dialog; InputNumber:CARDINAL;
                         input:OtherInputType;
                         x,y,width,height,DisplayNumChars,StringLimit:INTEGER; colour:CARDINAL);
BEGIN
  d^.Input.type[InputNumber-1] := input;
  d^.Input.x[InputNumber-1] := x;
  d^.Input.y[InputNumber-1] := y;
  d^.Input.width[InputNumber-1] := width;
  d^.Input.height[InputNumber-1] := height;
  d^.Input.DisplayNumChars[InputNumber-1] := DisplayNumChars;
  d^.Input.StringLimit[InputNumber-1] := StringLimit;
  d^.Input.colour[InputNumber-1] := colour;
END SetOtherInput;

BEGIN
  CreateLinkList(ActiveDialogList);

  CreateLinkList(InactiveDialogList);
END Dialogs.