IMPLEMENTATION MODULE IconButtons;

(******************************************************************)
(*											*)
(*				Icon Buttons Module				*)
(*											*)
(*	Programmer :	Matthew Ma (c9315720)				*)
(*											*)
(*	Last edited :	22-10-97						*)
(*											*)
(*	Function : 		This module implements the toolbox icon	*)
(*				buttons and menu short-cut icon buttons.	*)
(* 											*)
(*	Status :		Working.						*)
(******************************************************************)


IMPORT FileRoutines;

FROM FileRoutines IMPORT
(* type *)   Actions, modified, new_image, open,
(* var  *)   extension, NewImage,
(* proc *)   OpenFile, SaveFile, SaveFileAs, DeleteFile;

FROM WriteBMP IMPORT
(* proc *)   Write256BMP;

FROM Canvas IMPORT
(* var  *)   normal, CanvasLeft, CanvasTop, CanvasRight, CanvasBottom;

FROM Bars IMPORT
(* proc *)   EraserOptions, RectangleOptions, SolidRectangleOptions,
		 ClearOptionsWindow, SetOption, LineOptions, BrushOptions;

FROM InfoDialogs IMPORT
(* proc *)   ShowImageInfo, Restart, UniqueColours, AboutPaintMagic, Exit, FileModified,
		 NoPrint;

FROM PopUpWindows IMPORT
(* proc *)   Palette256, Index;

IMPORT Graphics;

IMPORT Lib;

FROM Icons IMPORT
(* const *)  arrow, freehand, line, rectangle, solid_rectangle, circle, solid_circle,
	       text, eraser, dropper, floodfill, airbrush, marquee, zoom, crosshair,
	       palette, small_eraser, brush, polygon,
(* proc  *)  PutIcon;	

FROM Semaphores IMPORT
(* type *)   Semaphore,
(* proc *)   CreateSemaphore, Wait, Signal, DestroySemaphore;

FROM MouseRoutines IMPORT
(* proc *)   SetFinger, SetMagnifier, SetArrow;

FROM Buttons IMPORT
(* var  *)   NoSubMenu,
(* type *)	 BoxMenuProcedure, BoxMenuItemTYPE, BoxMenuTYPE, MenuType,
(* proc *)	 CreateBoxSubMenu, BoxSubMenuItemSetup, DisplayBoxSubMenuTitle,
        	 DisplayBoxSubMenu, BoxSubMenuKeyboardSelection, NoIconDraw, Draw3DFrame;

FROM PopUpWindows IMPORT
(* proc *)   Index;

FROM Canvas IMPORT
(* proc *)   MsgCanvas;

FROM GraphicWindow IMPORT
(* var  *)   XMAX, YMAX, CharWidth, CharHeight, MaxColour,
(* type *)   GraphicalWindow, CreateGraphicalWindow, ObtainScreen, ReleaseScreen,
(* proc *)	 PutChar, PutLine, DoNothing,
		 Plot, Line, Rectangle, MoveWindowABS, DisplayWindowData, HideWindowData,
        	 UnconditionalObtainScreenWithMouseHide;

FROM SetModes IMPORT
(* var  *)   video_mode, ScreenBackgroundColour;

FROM KBdriver IMPORT
(* proc *)   KeyboardInterruptTask;

FROM Toolbox IMPORT
(* proc *)   InputText, StopRoutine;

FROM SoundEffects IMPORT
(* proc *)   Beep;


CONST
   dummy = 99;



(******************************************************************)
(* A dummy procedure.								*)
(******************************************************************)
PROCEDURE Dummy;
BEGIN
END Dummy;


(******************************************************************)
(* This procedure notifies the canvas task that the New menu	*)
(* short-cut button has been clicked by the user.			*)
(******************************************************************)
PROCEDURE ClickNew();
BEGIN
   MsgCanvas(3,0);
END ClickNew;


(******************************************************************)
(* This procedure notifies the canvas task that the Open menu	*)
(* short-cut button has been clicked by the user.			*)
(******************************************************************)
PROCEDURE ClickOpen();
BEGIN
   open := TRUE;
   IF NOT(modified) THEN
	OpenFile();
   ELSE
	new_image := FALSE;
	Beep();
	FileModified();
   END;
END ClickOpen;


(******************************************************************)
(* This procedure notifies the canvas task that the Save menu	*)
(* short-cut button has been clicked by the user.			*)
(******************************************************************)
PROCEDURE ClickSave();
BEGIN
   (*MsgCanvas(2,3-3);*)
END ClickSave;


(*
PROCEDURE ClickCut();
BEGIN MsgCanvas(2,4-4);
END ClickCut;
*)


(******************************************************************)
(* This procedure notifies the canvas task that the Arrow tool	*)
(* button has been clicked by the user.					*)
(******************************************************************)
PROCEDURE ClickArrow();
BEGIN
   SetOption(0);
   ClearOptionsWindow();
   MsgCanvas(2, arrow);
END ClickArrow;


(******************************************************************)
(* This procedure notifies the canvas task that the About menu	*)
(* short-cut button has been clicked by the user.			*)
(******************************************************************)
PROCEDURE About();
BEGIN
   AboutPaintMagic();
END About;


(******************************************************************)
(* This procedure notifies the canvas task that the ImageInfo	*)
(* menu short-cut button has been clicked by the user.		*)
(******************************************************************)
PROCEDURE ImageInfo();
BEGIN
   ShowImageInfo();
END ImageInfo;


(******************************************************************)
(* This procedure notifies the canvas task that the Print menu	*)
(* short-cut button has been clicked by the user.			*)
(******************************************************************)
PROCEDURE NoPrinting();
BEGIN
   Beep();
   NoPrint();
END NoPrinting;


(******************************************************************)
(* This procedure notifies the canvas task that the Freehand tool	*)
(* button has been clicked by the user.					*)
(******************************************************************)
PROCEDURE ClickFreehand();
BEGIN
   SetOption(0);
   ClearOptionsWindow();
   MsgCanvas(2, freehand);
END ClickFreehand;


(******************************************************************)
(* This procedure notifies the canvas task that the Line tool	*)
(* button has been clicked by the user.					*)
(******************************************************************)
PROCEDURE ClickLine();
BEGIN
   SetOption(5);
   LineOptions(1);
   MsgCanvas(2, line);
END ClickLine;


(******************************************************************)
(* This procedure notifies the canvas task that the transparent	*)
(* Rectangle/Square tool button has been clicked by the user.	*)
(******************************************************************)
PROCEDURE ClickSquare();
BEGIN
   SetOption(3);
   RectangleOptions(1);
   MsgCanvas(2, rectangle);
END ClickSquare;


(******************************************************************)
(* This procedure notifies the canvas task that the solid		*)
(* Rectangle/Square tool button has been clicked by the user.	*)
(******************************************************************)
PROCEDURE ClickFilledSquare();
BEGIN
   SetOption(4);
   SolidRectangleOptions(1);
   MsgCanvas(2, solid_rectangle);
END ClickFilledSquare;


(******************************************************************)
(* This procedure notifies the canvas task that the transparent	*)
(* Circle tool button has been clicked by the user.			*)
(******************************************************************)
PROCEDURE ClickCircle();
BEGIN
   SetOption(0);
   ClearOptionsWindow();
   MsgCanvas(2, circle);
END ClickCircle;


(******************************************************************)
(* This procedure notifies the canvas task that the solid circle	*)
(* button has been clicked by the user.					*)
(******************************************************************)
PROCEDURE ClickFilledCircle();
BEGIN
   SetOption(0);
   ClearOptionsWindow();
   MsgCanvas(2, solid_circle);
END ClickFilledCircle;


(******************************************************************)
(* This procedure notifies the canvas task that the Text tool	*)
(* button has been clicked by the user.					*)
(******************************************************************)
PROCEDURE ClickText();
BEGIN
   SetOption(0);
   ClearOptionsWindow();
   MsgCanvas(2, text);
   InputText();
END ClickText;


(******************************************************************)
(* This procedure notifies the canvas task that the Eraser tool	*)
(* button has been clicked by the user.					*)
(******************************************************************)
PROCEDURE ClickEraser();
BEGIN
   SetOption(1);
   EraserOptions(1);
   MsgCanvas(2, small_eraser);
END ClickEraser;


(******************************************************************)
(* This procedure notifies the canvas task that the Dropper tool	*)
(* button has been clicked by the user.					*)
(******************************************************************)
PROCEDURE ClickDropper();
BEGIN
   SetOption(0);
   ClearOptionsWindow();
   MsgCanvas(2, dropper);
END ClickDropper;


(******************************************************************)
(* This procedure notifies the canvas task that the FloodFill	*)
(* tool button has been clicked by the user.				*)
(******************************************************************)
PROCEDURE ClickFloodFill();
BEGIN
   (*SetOption(0);
   ClearOptionsWindow();*)
END ClickFloodFill;


(******************************************************************)
(* This procedure notifies the canvas task that the Marquee tool	*)
(* button has been clicked by the user.					*)
(******************************************************************)
PROCEDURE ClickMarquee();
BEGIN
   SetOption(0);
   ClearOptionsWindow();
   MsgCanvas(2, marquee);
END ClickMarquee;


(******************************************************************)
(* This procedure notifies the canvas task that the Airbrush tool	*)
(* button has been clicked by the user.					*)
(******************************************************************)
PROCEDURE ClickAirbrush();
BEGIN
   SetOption(0);
   ClearOptionsWindow();
   MsgCanvas(2, airbrush);
END ClickAirbrush;


(******************************************************************)
(* This procedure notifies the canvas task that the Magnifier	*)
(* tool button has been clicked by the user.				*)
(******************************************************************)
PROCEDURE ClickPolygon();
BEGIN
   SetOption(0);
   ClearOptionsWindow();
   StopRoutine();
   MsgCanvas(2,polygon);
END ClickPolygon;


(******************************************************************)
(* This procedure notifies the canvas task that the Brush tool	*)
(* button has been clicked by the user.					*)
(******************************************************************)
PROCEDURE ClickBrush();
BEGIN
   SetOption(2);
   BrushOptions(1);
   MsgCanvas(2, brush);
END ClickBrush;


(******************************************************************)
(* This procedure notifies the canvas task that the Normal View	*)
(* menu short-cut button has been clicked by the user.		*)
(******************************************************************)
PROCEDURE ClickNoZoom();
BEGIN
   IF NOT(normal) THEN
   	MsgCanvas(12, dummy);
   END;
END ClickNoZoom;


(******************************************************************)
(* This procedure notifies the canvas task that the Cut menu	*)
(* short-cut button has been clicked by the user.			*)
(******************************************************************)
PROCEDURE ClickCut();
BEGIN
   MsgCanvas(15, dummy);
END ClickCut;


(******************************************************************)
(* A dummy procedure.								*)
(******************************************************************)
PROCEDURE DrawNothing(w : GraphicalWindow; x,y : INTEGER);
BEGIN
END DrawNothing;


(******************************************************************)
(* This procedure places the New icon onto the button.		*)
(******************************************************************)
PROCEDURE DrawNew(w : GraphicalWindow; x,y : INTEGER);
BEGIN
   PutIcon(x,y,1);
END DrawNew;


(******************************************************************)
(* This procedure places the Open icon onto the button.		*)
(******************************************************************)
PROCEDURE DrawOpen(w : GraphicalWindow; x,y : INTEGER);
BEGIN
   PutIcon(x,y,2);
END DrawOpen;


(******************************************************************)
(* This procedure places the Save icon onto the button.		*)
(******************************************************************)
PROCEDURE DrawSave(w : GraphicalWindow; x,y : INTEGER);
BEGIN
   PutIcon(x,y,3);
END DrawSave;


(******************************************************************)
(* This procedure places the Cut icon onto the button.		*)
(******************************************************************)
PROCEDURE DrawCut(w : GraphicalWindow; x,y : INTEGER);
BEGIN
   PutIcon(x,y,4);
END DrawCut;


(******************************************************************)
(* This procedure places the Copy icon onto the button.		*)
(******************************************************************)
PROCEDURE DrawCopy(w : GraphicalWindow; x,y : INTEGER);
BEGIN
   PutIcon(x,y,5);
END DrawCopy;


(******************************************************************)
(* This procedure places the Paste icon onto the button.		*)
(******************************************************************)
PROCEDURE DrawPaste(w : GraphicalWindow; x,y : INTEGER);
BEGIN
   PutIcon(x,y,6);
END DrawPaste;


(******************************************************************)
(* This procedure places the Undo icon onto the button.		*)
(******************************************************************)
PROCEDURE DrawUndo(w : GraphicalWindow; x,y : INTEGER);
BEGIN
   PutIcon(x,y,7);
END DrawUndo;


(******************************************************************)
(* This procedure places the Redo icon onto the button.		*)
(******************************************************************)
PROCEDURE DrawRedo(w : GraphicalWindow; x,y : INTEGER);
BEGIN
   PutIcon(x,y,8);
END DrawRedo;


(******************************************************************)
(* This procedure places the Print icon onto the button.		*)
(******************************************************************)
PROCEDURE DrawPrint(w : GraphicalWindow; x,y : INTEGER);
BEGIN
   PutIcon(x,y,0);
END DrawPrint;


(******************************************************************)
(* This procedure places the Arrow icon onto the button.		*)
(******************************************************************)
PROCEDURE DrawArrow(w : GraphicalWindow; x,y : INTEGER);
BEGIN
   PutIcon(x,y,9);
END DrawArrow;


(******************************************************************)
(* This procedure places the Pencil icon onto the button.		*)
(******************************************************************)
PROCEDURE DrawPencil(w : GraphicalWindow; x,y : INTEGER);
BEGIN
   PutIcon(x,y,10);
END DrawPencil;


PROCEDURE DrawFill(w : GraphicalWindow; x,y : INTEGER);
BEGIN
   PutIcon(x,y,11);
END DrawFill;


(******************************************************************)
(* This procedure places the transparent Circle icon onto the	*)
(* button.										*)
(******************************************************************)
PROCEDURE DrawCircle(w : GraphicalWindow; x,y : INTEGER);
BEGIN
   PutIcon(x,y,12);
END DrawCircle;


(******************************************************************)
(* This procedure places the transparent Rectangle/Square icon	*)
(* onto the button.								*)
(******************************************************************)
PROCEDURE DrawSquare(w : GraphicalWindow; x,y : INTEGER);
BEGIN
   PutIcon(x,y,13);
END DrawSquare;


(******************************************************************)
(* This procedure places the solid Rectangle/Square icon onto the	*)
(* button.										*)
(******************************************************************)
PROCEDURE DrawFilledSquare(w : GraphicalWindow; x,y : INTEGER);
BEGIN
   PutIcon(x,y,14);
END DrawFilledSquare;


(******************************************************************)
(* This procedure places the Line icon onto the button.		*)
(******************************************************************)
PROCEDURE DrawStraightLine(w : GraphicalWindow; x,y : INTEGER);
BEGIN
   PutIcon(x,y,15);
END DrawStraightLine;


(******************************************************************)
(* This procedure places the Text icon onto the button.		*)
(******************************************************************)
PROCEDURE DrawText(w : GraphicalWindow; x,y : INTEGER);
BEGIN
   PutIcon(x,y,16);
END DrawText;


(******************************************************************)
(* This procedure places the Dropper icon onto the button.		*)
(******************************************************************)
PROCEDURE DrawDropper(w : GraphicalWindow; x,y : INTEGER);
BEGIN
   PutIcon(x,y,17);
END DrawDropper;


(******************************************************************)
(* This procedure places the Eraser icon onto the button.		*)
(******************************************************************)
PROCEDURE DrawEraser(w : GraphicalWindow; x,y : INTEGER);
BEGIN
   PutIcon(x,y,18);
END DrawEraser;


(******************************************************************)
(* This procedure places the Airbrush icon onto the button.		*)
(******************************************************************)
PROCEDURE DrawAirBrush(w : GraphicalWindow; x,y : INTEGER);
BEGIN
   PutIcon(x,y,19);
END DrawAirBrush;


(******************************************************************)
(* This procedure places the solid Circle icon onto the button.	*)
(******************************************************************)
PROCEDURE DrawFilledCircle(w : GraphicalWindow; x,y : INTEGER);
BEGIN
   PutIcon(x,y,20);
END DrawFilledCircle;


(******************************************************************)
(* This procedure places the Marquee icon onto the button.		*)
(******************************************************************)
PROCEDURE DrawMarquee(w : GraphicalWindow; x,y : INTEGER);
BEGIN
   PutIcon(x,y,21);
END DrawMarquee;


(******************************************************************)
(* This procedure places the NoZoom icon onto the button.		*)
(******************************************************************)
PROCEDURE DrawZoom(w : GraphicalWindow; x,y : INTEGER);
BEGIN
   PutIcon(x,y,22);
END DrawZoom;


(******************************************************************)
(* This procedure places the Palette icon onto the button.		*)
(******************************************************************)
PROCEDURE DrawPalette(w : GraphicalWindow; x,y : INTEGER);
BEGIN
   PutIcon(x,y,42);
END DrawPalette;


(******************************************************************)
(* This procedure places the Polygon icon onto the button.		*)
(******************************************************************)
PROCEDURE DrawPolygon(w : GraphicalWindow; x,y : INTEGER);
BEGIN
   PutIcon(x,y,47);
END DrawPolygon;


(******************************************************************)
(* This procedure places the Help icon onto the button.		*)
(******************************************************************)
PROCEDURE DrawHelp(w : GraphicalWindow; x,y : INTEGER);
BEGIN
   PutIcon(x,y,43);
END DrawHelp;


(******************************************************************)
(* This procedure places the Magnifier icon onto the button.	*)
(******************************************************************)
PROCEDURE DrawNoZoom(w : GraphicalWindow; x,y : INTEGER);
BEGIN
   PutIcon(x,y,44);
END DrawNoZoom;


(******************************************************************)
(* This procedure places the Info icon onto the button.		*)
(******************************************************************)
PROCEDURE DrawInfo(w : GraphicalWindow; x,y : INTEGER);
BEGIN
   PutIcon(x,y,45);
END DrawInfo;


(******************************************************************)
(* This procedure places the Brush icon onto the button.		*)
(******************************************************************)
PROCEDURE DrawBrush(w : GraphicalWindow; x,y : INTEGER);
BEGIN
   PutIcon(x,y,46);
END DrawBrush;


(******************************************************************)
(* This procedure draws the first set of menu short-cut icon	*)
(* buttons.										*)
(******************************************************************)
PROCEDURE DrawMenuIconWindow();
VAR
   MenuIconWindow : BoxMenuTYPE;
   New, Open, Save, Cut, Copy, Paste, Undo, Redo, Print : BoxMenuItemTYPE;
BEGIN
   MenuIconWindow := CreateBoxSubMenu("MenuIcon",FALSE,3,9,15,4,0,0,0+2,21+6,2+7,7-6,20+5,20+3,
				      	  0,12,1,7,AlwaysVisible,SetFinger);

   New := BoxSubMenuItemSetup(MenuIconWindow,TRUE,TRUE,"Creates a new drawing",DrawNew,
                   		8-3+4,22-12+46+1,1+10,25,NoSubMenu,TRUE,FileRoutines.NewImage);

   Open := BoxSubMenuItemSetup(MenuIconWindow,TRUE,TRUE,"Opens an existing drawing",DrawOpen,
				   	 8-3+1,22-12+46,1,25,NoSubMenu,TRUE,ClickOpen);
   Save := BoxSubMenuItemSetup(MenuIconWindow,TRUE,TRUE,"Saves current drawing",DrawSave,
				   	 8-3+2,22-12+46,1,25,NoSubMenu,TRUE,FileRoutines.SaveFile);

   Cut := BoxSubMenuItemSetup(MenuIconWindow,TRUE,TRUE,"Cut",DrawCut,
				   	8-3+5,22-12+46,1,25,NoSubMenu,TRUE,ClickCut);

   Copy := BoxSubMenuItemSetup(MenuIconWindow,TRUE,TRUE,"Copy",DrawCopy,
                   		 8-3+2,22-12+46,1+3,25,NoSubMenu,TRUE,Dummy);

   Paste := BoxSubMenuItemSetup(MenuIconWindow,TRUE,TRUE,"Paste",DrawPaste,
                   		  8-3+1,22-12+46,1,25,NoSubMenu,TRUE,Dummy);

   Undo := BoxSubMenuItemSetup(MenuIconWindow,TRUE,TRUE,"Undo",DrawUndo,
			   	   	 8-1+1,22-12+50,1,25,NoSubMenu,TRUE,Dummy);

   Redo := BoxSubMenuItemSetup(MenuIconWindow,TRUE,TRUE,"Redo",DrawRedo,
				   	 8-1+1,22-12+50,1,25,NoSubMenu,TRUE,Dummy);

   Print := BoxSubMenuItemSetup(MenuIconWindow,TRUE,TRUE,"Prints current drawing",DrawPrint,
				   	  8-3+1,22-12+46,1+2,25,NoSubMenu,TRUE,NoPrinting);

   DisplayBoxSubMenu(MenuIconWindow);

   Draw3DFrame(New);
   Draw3DFrame(Open);
   Draw3DFrame(Save);
   Draw3DFrame(Cut);
   Draw3DFrame(Copy);
   Draw3DFrame(Paste);
   Draw3DFrame(Undo);
   Draw3DFrame(Redo);
   Draw3DFrame(Print);

END DrawMenuIconWindow;


(******************************************************************)
(* This procedure draws the second set of menu short-cut icon	*)
(* buttons.										*)
(******************************************************************)
PROCEDURE DrawMenuIcon2();
VAR
   MenuIcon2 : BoxMenuTYPE;
   no_zoom, info, help : BoxMenuItemTYPE;
BEGIN
   MenuIcon2 := CreateBoxSubMenu("MenuIcon2",FALSE,3,9,15,4,0,0,240,21+6,2+7,7-6,20+5,20+3,
				         0,12,1,7,AlwaysVisible,SetFinger);

   no_zoom := BoxSubMenuItemSetup(MenuIcon2,TRUE,TRUE,"Normal viewing",DrawNoZoom,
                   		    244,22-12+46-0,1+10,25,NoSubMenu,TRUE,ClickNoZoom);

   info := BoxSubMenuItemSetup(MenuIcon2,TRUE,TRUE,"Displays image information",DrawInfo,
				   	 246-2,22-12+46+1-1,1,25,NoSubMenu,TRUE,ImageInfo);

   help := BoxSubMenuItemSetup(MenuIcon2,TRUE,TRUE,"Displays information about PaintMagic",DrawHelp,
				   	 248,22-12+46,1+1,25,NoSubMenu,TRUE,About);

   DisplayBoxSubMenu(MenuIcon2);

   Draw3DFrame(no_zoom);
   Draw3DFrame(info);
   Draw3DFrame(help);

END DrawMenuIcon2;


(******************************************************************)
(* This procedure draws the toolbox icon buttons.			*)
(******************************************************************)
PROCEDURE DrawToolIconWindow();
VAR
   ToolIconWindow : BoxMenuTYPE;
   Arrow, AirBrush, Dropper, Pencil, Circle, Rectangle, Zoom, Brush, Polygon,
   Eraser, Text, FloodFill, Line, SolidCircle, SolidRectangle, Marquee, Palette : BoxMenuItemTYPE;
BEGIN
   ToolIconWindow := CreateBoxSubMenu("Toolbox",FALSE,3,9,15,4,0-3+1,0-1,0,21+36+3,2,7,40+2,40+2,
				      	  0,12,1,7,AlwaysVisible,SetFinger);

   Arrow := BoxSubMenuItemSetup(ToolIconWindow,TRUE,TRUE,"Arrow tool",DrawArrow,
				   	  8+3+3,5+7+82,1,15,NoSubMenu,TRUE,ClickArrow);

   AirBrush := BoxSubMenuItemSetup(ToolIconWindow,TRUE,TRUE,"Draws sprays of dots",DrawAirBrush,
				   	     8-1+2,22+71,1,15,NoSubMenu,TRUE,ClickAirbrush);

   Marquee := BoxSubMenuItemSetup(ToolIconWindow,TRUE,TRUE,"Selects a rectangular area",
				   	    DrawMarquee,8-2+2,22+73,1,25,NoSubMenu,TRUE,ClickMarquee);

   Pencil := BoxSubMenuItemSetup(ToolIconWindow,TRUE,TRUE,"Freehand sketch",DrawPencil,
				   	   8-2+1+1,22+71,1,25,NoSubMenu,TRUE,ClickFreehand);

   Circle := BoxSubMenuItemSetup(ToolIconWindow,TRUE,TRUE,"Draws a transparent circle",DrawCircle,
				   	   8-1+1,22+72,1+1,25,NoSubMenu,TRUE,ClickCircle);

   Rectangle := BoxSubMenuItemSetup(ToolIconWindow,TRUE,TRUE,"Draws a transparent rectangle or square",DrawSquare,
				   		8-1+1,22+72,1+3,25,NoSubMenu,TRUE,ClickSquare);

   (*Zoom := BoxSubMenuItemSetup(ToolIconWindow,TRUE,TRUE,"Draws a transparent polygon",DrawZoom,
				   	 8-1+1,22+72,1+20,15,NoSubMenu,TRUE,ClickZoom);*)

   Polygon := BoxSubMenuItemSetup(ToolIconWindow,TRUE,TRUE,"Draws a transparent polygon",DrawPolygon,
				   	 8-1+3,22+75,1+20,15,NoSubMenu,TRUE,ClickPolygon);


   Eraser := BoxSubMenuItemSetup(ToolIconWindow,TRUE,TRUE,"Erases unwanted parts of current drawing",DrawEraser,
				   	   13-7+2,27+68,1,15,NoSubMenu,TRUE,ClickEraser);

   Text := BoxSubMenuItemSetup(ToolIconWindow,TRUE,TRUE,"Adds text to current drawing",DrawText,
                   		 8-1+1,22+71,1+5,25,NoSubMenu,TRUE,ClickText);

   Dropper := BoxSubMenuItemSetup(ToolIconWindow,TRUE,TRUE,"Picks a colour from current drawing",
				   	    DrawDropper,8-2+1+1,22+72,1+3,25,NoSubMenu,TRUE,ClickDropper);

   Line := BoxSubMenuItemSetup(ToolIconWindow,TRUE,TRUE,"Draws a free-angle or fixed-angle straight line",
				   	 DrawStraightLine,8-1+1,22+71,1+12+3,25,NoSubMenu,TRUE,ClickLine);

   SolidCircle := BoxSubMenuItemSetup(ToolIconWindow,TRUE,TRUE,"Draws a solid circle",
				   		  DrawFilledCircle,8-2+2,22+72,1+9,15,NoSubMenu,TRUE,ClickFilledCircle);

   SolidRectangle := BoxSubMenuItemSetup(ToolIconWindow,TRUE,TRUE,"Draws a solid rectangle or square",
				   		     DrawFilledSquare,8-1+1,22+72,1+28,25,NoSubMenu,TRUE,ClickFilledSquare);

   Brush := BoxSubMenuItemSetup(ToolIconWindow,TRUE,TRUE,"Brush tool",DrawBrush,
                   		  8-2+2,22+72,1+2,15,NoSubMenu,TRUE,ClickBrush);

   DisplayBoxSubMenu(ToolIconWindow);


   Draw3DFrame(Arrow);
   Draw3DFrame(AirBrush);
   Draw3DFrame(Marquee);
   Draw3DFrame(Pencil);
   Draw3DFrame(Circle);
   Draw3DFrame(Rectangle);
   (*Draw3DFrame(Zoom);*)
   Draw3DFrame(Polygon);
   Draw3DFrame(Eraser);
   Draw3DFrame(Text);
   Draw3DFrame(Dropper);
   Draw3DFrame(Line);
   Draw3DFrame(SolidCircle);
   Draw3DFrame(SolidRectangle);
   Draw3DFrame(Brush);

END DrawToolIconWindow;



END IconButtons.