IMPLEMENTATION MODULE ImageRoutines;

(******************************************************************)
(*											*)
(*		     Image Manipulation Routines Module			*)
(*											*)
(*	Programmer :	Matthew Ma (c9315720)				*)
(*											*)
(*	Last edited :	22-10-97						*)
(*											*)
(*	Function : 		This module contains image manipulation	*)
(*				routines.						*)
(* 											*)
(*	Status :		Working.						*)
(******************************************************************)


FROM ColourRoutines IMPORT
(* var  *)   ForegroundColour, BackgroundColour;

FROM Graphics IMPORT
(* proc *)   Fill, Rectangle;

FROM Canvas IMPORT
(* var *)    CanvasLeft, CanvasTop, CanvasRight, CanvasBottom;

FROM Toolbox IMPORT
(* var *)    marquee_x1, marquee_y1, marquee_x2, marquee_y2;

FROM Lib IMPORT
(* proc *)   AddAddr, IncAddr;

FROM Support IMPORT
(* proc *)   ReadWord, WriteWord, ReadLongWord, WriteLongWord;

FROM XMS IMPORT
(* proc *)   ObtainXMSMemory, ReleaseXMSMemory;

FROM FIO IMPORT
(* type *)   File, Exists;

FROM FileSys IMPORT
(* proc *)   OpenFile, CloseFile, ReadByte, WriteByte,
             SetPosition, SavePosition, FileSize, ReadRecord, WriteRecord;

FROM IOErrorCodes IMPORT
(* type *)   ErrorCode;

FROM Graphics IMPORT
(* proc *)   Point;

FROM Str IMPORT
(* proc *)   Compare;

FROM GraphicWindow IMPORT
(* const *)  _GPSET,
(* var   *)  XMAX, YMAX, MaxColour,
(* proc  *)  PutRectangle, GetImage, PutImage, ObtainScreen, ReleaseScreen;

FROM WritePCX IMPORT
(* proc *)   Write256PCX;


CONST
   block_size = 65534;



(******************************************************************)
(* This procedure rotates an screen image vertically about the 	*)
(* horizontal symmetric axis.							*)
(******************************************************************)
PROCEDURE FlipImage(x1,y1,x2,y2 : CARDINAL);
VAR
   ok : BOOLEAN;
   i, j, swap_count, height : CARDINAL;
   row1, row2 : POINTER TO ARRAY [0..block_size-1] OF BYTE;
BEGIN
   
   height := y2-y1+1;
   IF ODD(height) THEN
      swap_count := (height-1) DIV 2;
   ELSE
      swap_count := height DIV 2;
   END;

   row1 := ObtainXMSMemory();
   row2 := row1;
   IncAddr(row2,(block_size DIV 2));


   ok := ObtainScreen();
   FOR i := 0 TO swap_count-1 DO

      GetImage(x1,y2-i,x2,y2-i,row1);		(* scanning up *)
      GetImage(x1,y1+i,x2,y1+i,row2);		(* scanning down *)

      PutImage(x1,y1+i,row1,_GPSET);		(* swapping lines *)
      PutImage(x1,y2-i,row2,_GPSET);		(* swapping lines *)
   END;
   ReleaseScreen();

   ReleaseXMSMemory();
END FlipImage;


(******************************************************************)
(* This procedure rotates an screen image horizontally about the 	*)
(* vertical symmetric axis.							*)
(******************************************************************)
PROCEDURE MirrorImage(x1,y1,x2,y2 : CARDINAL);
VAR
   ok : BOOLEAN;
   i, j, swap_count, width : CARDINAL;
   col1, col2 : POINTER TO ARRAY [0..block_size-1] OF BYTE;
BEGIN

   width := x2-x1+1;
   IF ODD(width) THEN
      swap_count := (width-1) DIV 2;
   ELSE
      swap_count := width DIV 2;
   END;

   col1 := ObtainXMSMemory();
   col2 := col1;
   IncAddr(col2,(block_size DIV 2));


   ok := ObtainScreen();
   FOR i := 0 TO swap_count-1 DO
      GetImage(x2-i,y1,x2-i,y2,col1);	(* scanning from left *)
      GetImage(x1+i,y1,x1+i,y2,col2);	(* scanning from right *)

      PutImage(x1+i,y1,col1,_GPSET);	(* swapping lines *)
      PutImage(x2-i,y1,col2,_GPSET);	(* swapping lines *)
   END;
   ReleaseScreen();

   ReleaseXMSMemory();
END MirrorImage;


(******************************************************************)
(* This procedure selects a rectangular area of an image and	*)
(* discards the area outside the rectangle.				*)
(******************************************************************)
PROCEDURE CropImage(x1,y1,x2,y2 : CARDINAL);
VAR
   ok : BOOLEAN;
   i, width, height : CARDINAL;
   row : POINTER TO ARRAY [0..block_size-1] OF BYTE;
BEGIN

   width := x2-x1+1;
   height := y2-y1+1;

   row := ObtainXMSMemory();

   ok := ObtainScreen();

   FOR i := 0 TO height-1 DO
	 (**************************)
	 (* cuts the selected area *)
	 (**************************)
       GetImage(x1,y1+i,x2,y1+i,row);

	 (****************************************)
	 (* places the cropped part of the image *)
	 (* onto the upper-left corner of canvas *)
	 (****************************************)
       PutImage(CanvasLeft,CanvasTop+i,row,_GPSET);
   END;

   (***************************************************************)
   (* fills the area outside the rectangle with background colour *)
   (***************************************************************)
   Fill(CanvasLeft+width,y2,CanvasRight,CanvasTop,BackgroundColour);
   Fill(CanvasLeft,CanvasBottom,CanvasRight,CanvasTop+height,BackgroundColour);

   ReleaseScreen();

   marquee_x1 := -1;
   marquee_y1 := -1;
   marquee_x2 := -1;
   marquee_y2 := -1;

   ReleaseXMSMemory();

END CropImage;


(******************************************************************)
(* This procedure dumps the contents of the entire screen to an	*)
(* 8-bit PCX file.								*)
(******************************************************************)
PROCEDURE CaptureScreen(x1,y1,x2,y2 : CARDINAL; file_name : ARRAY OF CHAR; cursor : BOOLEAN);
BEGIN
   Write256PCX(x1,y1,x2,y2,file_name,cursor);
END CaptureScreen;


(******************************************************************)
(* This procedure selects a rectangular area of an image and	*)
(* fills the area inside the rectangle with background colour.	*)
(******************************************************************)
PROCEDURE CutImage(window : GraphicalWindow; x1,y1,x2,y2 : CARDINAL);
VAR
   ok : BOOLEAN;
   i, width, height : CARDINAL;
   row : POINTER TO ARRAY [0..block_size-1] OF BYTE;
BEGIN

   ok := ObtainScreen();
   PutRectangle(window,x1,y1,x2,y2,BackgroundColour,TRUE);
   ReleaseScreen();

   marquee_x1 := -1;
   marquee_y1 := -1;
   marquee_x2 := -1;
   marquee_y2 := -1;

END CutImage;



END ImageRoutines.