IMPLEMENTATION MODULE InfoDialogs;

(******************************************************************)
(*											*)
(*		     		Info Dialogs Module				*)
(*											*)
(*	Programmer :	Matthew Ma (c9315720)				*)
(*											*)
(*	Last edited :	22-10-97						*)
(*											*)
(*	Function : 		This module contains all of the info	*)
(*				dialogs.						*)
(* 											*)
(*	Status :		Working.						*)
(******************************************************************)


FROM ColourRoutines IMPORT
(* proc *)   CountColours;

IMPORT FileRoutines;

FROM WriteBMP IMPORT
(* proc *)   Write256BMP;

FROM WritePCX IMPORT
(* proc *)   Write256PCX;

FROM WriteTGA IMPORT
(* proc *)   Write256TGA;

FROM WriteRAW IMPORT
(* proc *)   WriteRawData;

FROM Canvas IMPORT
(* var  *)   CanvasLeft, CanvasTop, CanvasRight, CanvasBottom,
(* proc *)   MsgCanvas;

IMPORT FIO;

FROM Icons IMPORT
(* proc *)   Palette, Error;

FROM TerminationControl IMPORT
(* proc *)   Crash;

IMPORT ReadBMP;

IMPORT ReadPCX;

IMPORT ReadTGA;

IMPORT ReadTIF;

FROM FIO IMPORT
(* type *)   File, FileStamp,
(* proc *)   GetFileStamp;

FROM FileSys IMPORT
(* proc *)   OpenFile, CloseFile, FileSize;

FROM Str IMPORT
(* proc *)   Copy, Length, Append, CardToStr, Compare;

FROM IOErrorCodes IMPORT
(* type *)   ErrorCode;

FROM Conversions IMPORT
(* proc *)   ShortCardToString, CardinalToString, LongCardToString;

FROM FileRoutines IMPORT
(*  var  *)   extension, OpenFileName, FileTypes, modified, new_image, open,
(* proc  *)   ReplaceFile, SaveFileAs;

FROM Dialogs IMPORT
(* type *)   Dialog, DialogType, OtherInputType,
(* proc *)   CreateDialog, AddABox, DisplayDialog, DialogWindow,
           	 MoveDialogABS, EndCreateDialog, HideDialog, DialogSelection;

FROM GraphicWindow IMPORT
(* var  *)   CharWidth, CharHeight, XMAX, YMAX, GraphicalWindow, DoNothing,
(* proc *)	 PutRectangle, DataRegionDisplayed, PutLine,
        	 DisplayWindowData, HideWindowData, PutString, PutRectangle, RespondToMouseButtons,
        	 ObtainScreen, ReleaseScreen, PutChar, ReturnDataLeftTop, ClearDataRegion, PutTitleStrip,
        	 MouseParatemer_xoffset, MouseParatemer_yoffset;

FROM TaskControl IMPORT
(* type *)   Lock,
(* proc *)	 CreateLock, Obtain, Release;

FROM Canvas IMPORT
(* proc *)   MsgCanvas;


CONST
   x_offset = 5;
   y_offset = 10+5+5+2;
   dummy = 99;

TYPE
   LongStr = ARRAY [0..255] OF CHAR;

VAR
  image_info, palette16, palette256, new_mode, changed : Dialog;


PROCEDURE FileModified(); FORWARD;


(******************************************************************)
(* This procedure displays a dialog reminding the user to restart	*)
(* the program after a new video mode has been selected.		*)
(******************************************************************)
PROCEDURE Restart();
VAR
  FileName:ARRAY [0..0] OF LongStr;
BEGIN
   DisplayDialog(new_mode,TRUE);	(* clears dialog window *)

   PutTitleStrip(DialogWindow(new_mode),"PaintMagic");	(* places title strip *)
   PutString(DialogWindow(new_mode),"You will need to restart the",3+2,25+0*CharHeight,8,FALSE);
   PutString(DialogWindow(new_mode),"program in order for the new",3+2,25+1*CharHeight,8,FALSE);
   PutString(DialogWindow(new_mode),"settings to take effect!",3+2,25+2*CharHeight,8,FALSE);

   CASE DialogSelection(new_mode,NIL) OF
    	  1:
  	| ELSE
   END;

   HideDialog(new_mode);	(* hides dialog *)
END Restart;


(******************************************************************)
(* This procedure displays a dialog showing the number of unique	*)
(* colours used by an image.							*)
(******************************************************************)
PROCEDURE UniqueColours();
VAR
   ok : BOOLEAN;
   FileName:ARRAY [0..0] OF LongStr;
   temp_string, temp_string2 : ARRAY [0..9] OF CHAR;
   temp_long_card, day, month, year, hour, min, sec : LONGCARD;
BEGIN
   temp_long_card := LONGCARD(CountColours());
   DisplayDialog(new_mode,TRUE);	(* clears dialog *)

   PutTitleStrip(DialogWindow(new_mode),"Colours Used");
   PutString(DialogWindow(new_mode),"Unique Colours: ",x_offset,28+1*CharHeight,8,FALSE);

   CardToStr(temp_long_card,temp_string,10,ok);
   PutString(DialogWindow(new_mode),temp_string,x_offset+16*CharWidth,28+1*CharHeight,8,FALSE);

   CASE DialogSelection(new_mode,NIL) OF
    	  1:
  	| ELSE
   END;(* CASE *)

   HideDialog(new_mode);
END UniqueColours;


(******************************************************************)
(* This procedure displays a dialog asking the user whether or	*)
(* not the modified file should be saved before exiting program.	*)
(******************************************************************)
PROCEDURE Exit();
VAR
  FileName:ARRAY [0..0] OF LongStr;
BEGIN
   DisplayDialog(changed,TRUE);

   PutTitleStrip(DialogWindow(changed),"PaintMagic");
   PutString(DialogWindow(changed),"File has been modified!",50+3*CharWidth,19+1*CharHeight,8,FALSE);
   PutString(DialogWindow(changed),"Save the changes?",50+3*CharWidth,19+2*CharHeight,8,FALSE);
   Error(DialogWindow(changed),25-10,40-3);

   CASE DialogSelection(changed,NIL) OF
     	  1:
	 	HideDialog(changed);
	 	SaveFileAs(SaveBmp,FALSE);

   	| 2:
	 	Crash("PaintMagic - Version 1.0 Copyright 1997 Matt Ma");

   	| 3:

   	ELSE

   END;

   HideDialog(changed);
END Exit;


(******************************************************************)
(* This procedure displays a dialog indicating that the specified	*)
(* file does not exist.								*)
(******************************************************************)
PROCEDURE FileNotFound();
BEGIN
   DisplayDialog(new_mode,TRUE);

   PutTitleStrip(DialogWindow(new_mode),"PaintMagic");
   PutString(DialogWindow(new_mode),"File not found!",50+2*CharWidth,28+1*CharHeight,8,FALSE);
   Error(DialogWindow(new_mode),25-10,40-3);

   CASE DialogSelection(new_mode,NIL) OF
    	  1:
  	| 2:
  	ELSE
   END;

   HideDialog(new_mode);
END FileNotFound;


(******************************************************************)
(* This procedure displays a dialog indicating that the specified	*)
(* path does not exist.								*)
(******************************************************************)
PROCEDURE PathNotFound();
BEGIN
   DisplayDialog(new_mode,TRUE);

   PutTitleStrip(DialogWindow(new_mode),"PaintMagic");
   PutString(DialogWindow(new_mode),"Path not found!",50+2*CharWidth,28+1*CharHeight,8,FALSE);
   Error(DialogWindow(new_mode),25-10,40-3);

   CASE DialogSelection(new_mode,NIL) OF
    	  1:
  	| 2:
  	ELSE
   END;

   HideDialog(new_mode);
END PathNotFound;


(******************************************************************)
(* This procedure displays a dialog indicating that the numeric	*)
(* input is invalid.								*)
(******************************************************************)
PROCEDURE InvalidInput();
BEGIN
   DisplayDialog(new_mode,TRUE);

   PutTitleStrip(DialogWindow(new_mode),"PaintMagic");
   PutString(DialogWindow(new_mode),"Invalid input!",50+2*CharWidth,28+1*CharHeight,8,FALSE);
   Error(DialogWindow(new_mode),25-10,40-3);

   CASE DialogSelection(new_mode,NIL) OF
    	  1:
  	| 2:
  	ELSE
   END;

   HideDialog(new_mode);
END InvalidInput;


(******************************************************************)
(* This procedure displays a dialog indicating that the specified	*)
(* file could not be openned.							*)
(******************************************************************)
PROCEDURE CantOpenFile();
BEGIN
   DisplayDialog(new_mode,TRUE);

   PutTitleStrip(DialogWindow(new_mode),"PaintMagic");
   PutString(DialogWindow(new_mode),"Can't open file!",50+CharWidth,28+1*CharHeight,8,FALSE);
   Error(DialogWindow(new_mode),25-10,40-2);

   CASE DialogSelection(new_mode,NIL) OF
    	  1:
  	| 2:
  	ELSE
   END;

   HideDialog(new_mode);
END CantOpenFile;


(******************************************************************)
(* This procedure displays a dialog indicating that the specified	*)
(* file could not be created.							*)
(******************************************************************)
PROCEDURE CantCreateFile();
BEGIN
   DisplayDialog(new_mode,TRUE);

   PutTitleStrip(DialogWindow(new_mode),"PaintMagic");
   PutString(DialogWindow(new_mode),"Can't create file!",50,28+1*CharHeight,8,FALSE);
   Error(DialogWindow(new_mode),25-15,40-3);

   CASE DialogSelection(new_mode,NIL) OF
    	  1:
  	| 2:
  	ELSE
   END;

   HideDialog(new_mode);
END CantCreateFile;


(******************************************************************)
(* This procedure displays a dialog indicating that the specified	*)
(* file is not a valid BMP file.						*)
(******************************************************************)
PROCEDURE NotBmpFile();
BEGIN
   DisplayDialog(new_mode,TRUE);

   PutTitleStrip(DialogWindow(new_mode),"PaintMagic");
   PutString(DialogWindow(new_mode),"Invalid BMP file!",50+CharWidth,28+1*CharHeight,8,FALSE);
   Error(DialogWindow(new_mode),25-10,40-3);

   CASE DialogSelection(new_mode,NIL) OF
    	  1:
  	| 2:
  	ELSE
   END;

   HideDialog(new_mode);
END NotBmpFile;


(******************************************************************)
(* This procedure displays a dialog indicating that the specified	*)
(* file is a compressed BMP file.						*)
(******************************************************************)
PROCEDURE CompressedBmp();
BEGIN
   DisplayDialog(new_mode,TRUE);

   PutTitleStrip(DialogWindow(new_mode),"PaintMagic");
   PutString(DialogWindow(new_mode),"Compressed BMP file!",50-CharWidth,28+1*CharHeight,8,FALSE);
   Error(DialogWindow(new_mode),25-10,40-3);

   CASE DialogSelection(new_mode,NIL) OF
    	  1:
  	| 2:
  	ELSE
   END;

   HideDialog(new_mode);
END CompressedBmp;


(******************************************************************)
(* This procedure displays a dialog indicating that the specified	*)
(* file is not a valid PCX file.						*)
(******************************************************************)
PROCEDURE NotPcxFile();
BEGIN
   DisplayDialog(new_mode,TRUE);

   PutTitleStrip(DialogWindow(new_mode),"PaintMagic");
   PutString(DialogWindow(new_mode),"Invalid PCX file!",50+CharWidth,28+1*CharHeight,8,FALSE);
   Error(DialogWindow(new_mode),25-10,40-3);

   CASE DialogSelection(new_mode,NIL) OF
    	  1:
  	| 2:
  	ELSE
   END;

   HideDialog(new_mode);
END NotPcxFile;


(******************************************************************)
(* This procedure displays a dialog indicating that the specified	*)
(* file does not contain colour palette entries.			*)
(******************************************************************)
PROCEDURE NoPalette();
BEGIN
   DisplayDialog(new_mode,TRUE);

   PutTitleStrip(DialogWindow(new_mode),"PaintMagic");
   PutString(DialogWindow(new_mode),"Palette not found!",50,28+1*CharHeight,8,FALSE);
   Error(DialogWindow(new_mode),25-10,40-3);

   CASE DialogSelection(new_mode,NIL) OF
    	  1:
  	| 2:
  	| ELSE
   END;

   HideDialog(new_mode);
END NoPalette;


(******************************************************************)
(* This procedure displays a dialog indicating that the specified	*)
(* file is not a valid Targa file.						*)
(******************************************************************)
PROCEDURE NotTgaFile();
BEGIN
   DisplayDialog(new_mode,TRUE);

   PutTitleStrip(DialogWindow(new_mode),"PaintMagic");
   PutString(DialogWindow(new_mode),"Invalid TGA file!",50+CharWidth,28+1*CharHeight,8,FALSE);
   Error(DialogWindow(new_mode),25-15,40-3);

   CASE DialogSelection(new_mode,NIL) OF
    	  1:
  	| 2:
  	ELSE
   END;

   HideDialog(new_mode);
END NotTgaFile;


(******************************************************************)
(* This procedure displays a dialog indicating that the specified	*)
(* file is a compressed TGA file.						*)
(******************************************************************)
PROCEDURE CompressedTga();
BEGIN
   DisplayDialog(new_mode,TRUE);

   PutTitleStrip(DialogWindow(new_mode),"PaintMagic");
   PutString(DialogWindow(new_mode),"Compressed TGA file!",50-CharWidth,28+1*CharHeight,8,FALSE);
   Error(DialogWindow(new_mode),25-10,40-3);

   CASE DialogSelection(new_mode,NIL) OF
    	  1:
  	| 2:
  	| ELSE
   END;

   HideDialog(new_mode);
END CompressedTga;


(******************************************************************)
(* This procedure displays a dialog indicating that the specified	*)
(* file is not a valid TIFF file.						*)
(******************************************************************)
PROCEDURE NotTifFile();
BEGIN
   DisplayDialog(new_mode,TRUE);

   PutTitleStrip(DialogWindow(new_mode),"PaintMagic");
   PutString(DialogWindow(new_mode),"Invalid TIF file!",50+CharWidth,28+1*CharHeight,8,FALSE);
   Error(DialogWindow(new_mode),25-10,40-3);

   CASE DialogSelection(new_mode,NIL) OF
    	  1:
  	| 2:
  	ELSE
   END;

   HideDialog(new_mode);
END NotTifFile;


(******************************************************************)
(* This procedure displays a dialog indicating that the specified	*)
(* file is a compressed TIFF file.						*)
(******************************************************************)
PROCEDURE CompressedTif();
BEGIN
   DisplayDialog(new_mode,TRUE);

   PutTitleStrip(DialogWindow(new_mode),"PaintMagic");
   PutString(DialogWindow(new_mode),"Compressed TIF file!",50-CharWidth,28+1*CharHeight,8,FALSE);
   Error(DialogWindow(new_mode),25-10,40-3);

   CASE DialogSelection(new_mode,NIL) OF
    	  1:
  	| 2:
  	| ELSE
   END;

   HideDialog(new_mode);
END CompressedTif;


(******************************************************************)
(* This procedure displays a dialog indicating that the specified	*)
(* image file is an unsupported subtype.					*)
(******************************************************************)
PROCEDURE InvalidSubtype();
BEGIN
   DisplayDialog(new_mode,TRUE);

   PutTitleStrip(DialogWindow(new_mode),"PaintMagic");
   PutString(DialogWindow(new_mode),"Unsupported subtype!",50-CharWidth,28+1*CharHeight,8,FALSE);
   Error(DialogWindow(new_mode),25-10,40-3);

   CASE DialogSelection(new_mode,NIL) OF
    	  1:
  	| 2:
  	| ELSE
   END;

   HideDialog(new_mode);
END InvalidSubtype;


(******************************************************************)
(* This procedure displays a dialog indicating that the specified	*)
(* file is not an 256-colour image.						*)
(******************************************************************)
PROCEDURE Not8Bit();
BEGIN
   DisplayDialog(new_mode,TRUE);

   PutTitleStrip(DialogWindow(new_mode),"PaintMagic");
   PutString(DialogWindow(new_mode),"Not an 8-bit image!",50,28+1*CharHeight,8,FALSE);
   Error(DialogWindow(new_mode),25-10,40-3);

   CASE DialogSelection(new_mode,NIL) OF
    	  1:
  	| 2:
  	ELSE
   END;

   HideDialog(new_mode);
END Not8Bit;


(******************************************************************)
(* This procedure displays a dialog indicating that the file to	*)
(* be loaded is of an unknown format.					*)
(******************************************************************)
PROCEDURE UnknownFormat();
BEGIN
   DisplayDialog(new_mode,TRUE);

   PutTitleStrip(DialogWindow(new_mode),"PaintMagic");
   PutString(DialogWindow(new_mode),"Unknown format!",50+2*CharWidth,28+1*CharHeight,8,FALSE);
   Error(DialogWindow(new_mode),25-10,40-3);

   CASE DialogSelection(new_mode,NIL) OF
    	1:
  	| 2:
  	ELSE
   END;

   HideDialog(new_mode);
END UnknownFormat;


(******************************************************************)
(* This procedure displays a dialog indicating that printing is	*)
(* not supported in this version of PaintMagic.				*)
(******************************************************************)
PROCEDURE NoPrint();
BEGIN
   DisplayDialog(new_mode,TRUE);

   PutTitleStrip(DialogWindow(new_mode),"PaintMagic");
   PutString(DialogWindow(new_mode),"Printing is not supported in",50-5*CharWidth-5,20+1*CharHeight,8,FALSE);
   PutString(DialogWindow(new_mode),"this version of PaintMagic:)",50-5*CharWidth-5,20+2*CharHeight,8,FALSE);

   CASE DialogSelection(new_mode,NIL) OF
    	  1:
  	| 2:
  	ELSE
   END;

   HideDialog(new_mode);
END NoPrint;


(******************************************************************)
(* This procedure displays a dialog indicating that the TIFF file *)
(* to be loaded is not a single-strip-per-image file.			*)
(******************************************************************)
PROCEDURE NotSingleStrip();
BEGIN
   DisplayDialog(new_mode,TRUE);

   PutTitleStrip(DialogWindow(new_mode),"PaintMagic");
   PutString(DialogWindow(new_mode),"Not a single-strip",50,28+1*CharHeight,8,FALSE);
   PutString(DialogWindow(new_mode),"TIFF image!",50,28+2*CharHeight,8,FALSE);
   Error(DialogWindow(new_mode),25-10,40-3);

   CASE DialogSelection(new_mode,NIL) OF
    	  1:
  	| 2:
  	ELSE
   END;

   HideDialog(new_mode);
END NotSingleStrip;


(******************************************************************)
(* This procedure displays a dialog indicating that the file to	*)
(* be loaded is not in Intel byte format.					*)
(******************************************************************)
PROCEDURE NotIntel();
BEGIN
   DisplayDialog(new_mode,TRUE);

   PutTitleStrip(DialogWindow(new_mode),"PaintMagic");
   PutString(DialogWindow(new_mode),"Not an Intel byte",50+CharWidth,28+1*CharHeight,8,FALSE);
   PutString(DialogWindow(new_mode),"order TIFF image!",50+CharWidth,28+2*CharHeight,8,FALSE);
   Error(DialogWindow(new_mode),25-10,40-3);

   CASE DialogSelection(new_mode,NIL) OF
    	  1:
  	| 2:
  	ELSE
   END;

   HideDialog(new_mode);
END NotIntel;


(******************************************************************)
(* This procedure displays a dialog indicating that the current	*)
(* image has been modified.							*)
(******************************************************************)
PROCEDURE FileModified();
BEGIN
   DisplayDialog(changed,TRUE);

   PutTitleStrip(DialogWindow(changed),"PaintMagic");
   PutString(DialogWindow(changed),"File has been modified!",50+3*CharWidth,19+1*CharHeight,8,FALSE);
   PutString(DialogWindow(changed),"Save the changes?",50+3*CharWidth,19+2*CharHeight,8,FALSE);
   Error(DialogWindow(changed),25-10,40-3);

   CASE DialogSelection(changed,NIL) OF
	  1:
		HideDialog(changed);
		IF extension <> Untitled THEN
		   MsgCanvas(7,99);
		ELSE
		   SaveFileAs(SaveBmp,FALSE);
		   IF new_image THEN
		   	MsgCanvas(3,0);
		   END;
		END;

  	| 2:
		HideDialog(changed);
		IF new_image THEN
		   MsgCanvas(3,0);
		ELSIF open THEN
		   FileRoutines.OpenFile();
		END;

	| 3:
		HideDialog(changed);

  	ELSE
   END; (* CASE *)
END FileModified;


(******************************************************************)
(* This procedure displays a dialog indicating that a file with	*)
(* the same name already exists.						*)
(******************************************************************)
PROCEDURE FileExists(type : Actions; file_name : ARRAY OF CHAR);
BEGIN
   DisplayDialog(new_mode,TRUE);

   PutTitleStrip(DialogWindow(new_mode),"PaintMagic");
   PutString(DialogWindow(new_mode),"File exists!",50+3*CharWidth,19+1*CharHeight,8,FALSE);
   PutString(DialogWindow(new_mode),"Replace it?",50+3*CharWidth,19+2*CharHeight,8,FALSE);
   Error(DialogWindow(new_mode),25-10,40-3);

   CASE DialogSelection(new_mode,NIL) OF
    	  1:
		HideDialog(new_mode);
		CASE type OF
		  	  SaveBmp:
				FIO.Erase(file_name);
				MsgCanvas(22,dummy);
				(*Write256BMP(CanvasLeft,CanvasTop,CanvasRight,CanvasBottom,file_name);*)
				modified := FALSE;

			| SavePcx:
				FIO.Erase(file_name);
				MsgCanvas(22,dummy);
				(*Write256PCX(CanvasLeft,CanvasTop,CanvasRight,CanvasBottom,file_name,FALSE);*)
				modified := FALSE;

			| SaveTga:
				FIO.Erase(file_name);
				MsgCanvas(22,dummy);
				(*Write256TGA(CanvasLeft,CanvasTop,CanvasRight,CanvasBottom,file_name);*)
				modified := FALSE;

			| SaveRaw:
				FIO.Erase(file_name);
				MsgCanvas(22,dummy);
				(*WriteRawData(CanvasLeft,CanvasTop,CanvasRight,CanvasBottom,file_name);*)
				modified := FALSE;

			| CaptShow:
				FIO.Erase(file_name);
				MsgCanvas(23-1,dummy);
				modified := FALSE;

			| CaptHide:
				FIO.Erase(file_name);
				MsgCanvas(24-2,dummy);
				modified := FALSE;
			ELSE
		   END;

  	| 2:

  	ELSE
   END; (* CASE *)

   HideDialog(new_mode);
END FileExists;


(******************************************************************)
(* This procedure asks the user whether he/she really wants to	*)
(* delete the specified file.							*)
(******************************************************************)
PROCEDURE ConfirmDelete(file_name : ARRAY OF CHAR);
BEGIN
   DisplayDialog(new_mode,TRUE);

   PutTitleStrip(DialogWindow(new_mode),"PaintMagic");
   PutString(DialogWindow(new_mode),"Delete",50,28+1*CharHeight,8,FALSE);
   PutString(DialogWindow(new_mode),file_name,50+7*CharWidth,28+1*CharHeight,8,FALSE);
   PutString(DialogWindow(new_mode),"?",50+(7+INTEGER(Length(file_name)))*CharWidth,28+1*CharHeight,8,FALSE);
   Error(DialogWindow(new_mode),25-10,40-3);

   CASE DialogSelection(new_mode,NIL) OF
    	  1:
		FIO.Erase(file_name);

  	| 2:

  	ELSE
   END;

   HideDialog(new_mode);
END ConfirmDelete;


(************************************************************************)
(* This procedure displays the file name of the current drawing on the	*)
(* Image Information dialog.								*)
(************************************************************************)
PROCEDURE ShowFileName();
VAR
   ok : BOOLEAN;
   FileName:ARRAY [0..0] OF LongStr;
   temp_string, temp_string2 : ARRAY [0..9] OF CHAR;
   temp_card, str_leng, stamp_offset : CARDINAL;
   temp_long_card, day, month, year, hour, min, sec : LONGCARD;
BEGIN
   PutString(DialogWindow(image_info),"File Name: ",x_offset,y_offset,1+7,FALSE);
   PutString(DialogWindow(image_info),OpenFileName,CARDINAL(40*CharWidth)-3-Length(OpenFileName)*CARDINAL(CharWidth),y_offset,1+7,FALSE);
END ShowFileName;


(************************************************************************)
(* This procedure displays the file type of the current drawing on the	*)
(* Image Information dialog.								*)
(************************************************************************)
PROCEDURE ShowFileType();
VAR
   ok : BOOLEAN;
   FileName:ARRAY [0..0] OF LongStr;
   temp_string, temp_string2 : ARRAY [0..9] OF CHAR;
   temp_card, str_leng, stamp_offset : CARDINAL;
   temp_long_card, day, month, year, hour, min, sec : LONGCARD;
BEGIN
   PutString(DialogWindow(image_info),"File Type: ",x_offset,y_offset+CharHeight,1+7,FALSE);
   CASE extension OF
	    Bmp:
		PutString(DialogWindow(image_info),"OS/2 or Windows Bitmap",40*CharWidth-3-22*CharWidth,y_offset+CharHeight,1+7,FALSE);

	| Pcx:
		PutString(DialogWindow(image_info),"ZSoft Paintbrush",40*CharWidth-3-16*CharWidth,y_offset+CharHeight,1+7,FALSE);

	| Tga:
		PutString(DialogWindow(image_info),"Truevision Targa",40*CharWidth-3-16*CharWidth,y_offset+CharHeight,1+7,FALSE);

	| Tif:
		PutString(DialogWindow(image_info),"Tagged Image File Format",40*CharWidth-3-24*CharWidth,y_offset+CharHeight,1+7,FALSE);

	| Gif:
	| Jpg:

	ELSE
		PutString(DialogWindow(image_info),"N/A",40*CharWidth-3-3*CharWidth,y_offset+CharHeight,1+7,FALSE);
      END;
END ShowFileType;


(************************************************************************)
(* This procedure displays the file sub-type of the current drawing  - 	*)
(* version number, compressed or uncompressed, on the Image Information	*)
(* dialog.											*)
(************************************************************************)
PROCEDURE ShowSubType();
VAR
   ok : BOOLEAN;
   FileName:ARRAY [0..0] OF LongStr;
   temp_string, temp_string2 : ARRAY [0..9] OF CHAR;
   temp_card, str_leng, stamp_offset : CARDINAL;
   temp_long_card, day, month, year, hour, min, sec : LONGCARD;
BEGIN
   PutString(DialogWindow(image_info),"Sub Type: ",x_offset,y_offset+2*CharHeight,1+7,FALSE);
   CASE extension OF
	  Bmp:
		IF ReadBMP.GetSubType() = 0 THEN
		   PutString(DialogWindow(image_info),"Windows RGB Encoded",
			     40*CharWidth-3-19*CharWidth,y_offset+2*CharHeight,1+7,FALSE);
		ELSIF ReadBMP.GetSubType() = 1 THEN
		   PutString(DialogWindow(image_info),"Windows RLE Encoded",
			     40*CharWidth-3-19*CharWidth,y_offset+2*CharHeight,1+7,FALSE);
		ELSE
		   PutString(DialogWindow(image_info),"Mode 2",
			     40*CharWidth-3-6*CharWidth,y_offset+2*CharHeight,1+7,FALSE);
		END;

	| Pcx:
		IF ReadPCX.GetSubType() = 5 THEN
		   PutString(DialogWindow(image_info),"run-length encoded",
			     40*CharWidth-3-18*CharWidth,y_offset+2*CharHeight,1+7,FALSE);
		ELSE
		END;

	| Tga:
		IF (ReadTGA.GetSubType() = 1) OR (ReadTGA.GetSubType() = 9) THEN
		   PutString(DialogWindow(image_info),"uncompressed",
			     40*CharWidth-3-12*CharWidth,y_offset+2*CharHeight,1+7,FALSE);
		ELSE
		END;

	| Tif:
		IF ReadTIF.GetSubType() = 1 THEN
		   PutString(DialogWindow(image_info),"uncompressed",
			     40*CharWidth-3-12*CharWidth,y_offset+2*CharHeight,1+7,FALSE);
		ELSE
		   PutString(DialogWindow(image_info),"compressed",
			     40*CharWidth-3-10*CharWidth,y_offset+2*CharHeight,1+7,FALSE);
		END;

	| Gif:
	| Jpg:

	ELSE
		PutString(DialogWindow(image_info),"N/A",40*CharWidth-3-3*CharWidth,y_offset+2*CharHeight,1+7,FALSE);
      END;
END ShowSubType;


(************************************************************************)
(* This procedure displays the file size of the current drawing on the	*)
(* Image Information dialog.								*)
(************************************************************************)
PROCEDURE ShowFileSize();
VAR
   ok : BOOLEAN;
   FileName:ARRAY [0..0] OF LongStr;
   temp_string, temp_string2 : ARRAY [0..9] OF CHAR;
   temp_card, str_leng, stamp_offset : CARDINAL;
   temp_long_card, day, month, year, hour, min, sec : LONGCARD;
BEGIN
   PutString(DialogWindow(image_info),"File Size: ",x_offset,y_offset+3*CharHeight,1+7,FALSE);
   CASE extension OF
	 Bmp:
		temp_long_card := ReadBMP.GetFileSize();
	 	LongCardToString(temp_long_card,temp_string,10);
	 	PutString(DialogWindow(image_info),temp_string,CARDINAL(40*CharWidth)-3-Length(temp_string)*CARDINAL(CharWidth),y_offset+3*CharHeight,1+7,FALSE);

	| Pcx:
		temp_long_card := ReadPCX.GetFileSize();
	 	LongCardToString(temp_long_card,temp_string,10);
	 	PutString(DialogWindow(image_info),temp_string,CARDINAL(40*CharWidth)-3-Length(temp_string)*CARDINAL(CharWidth),y_offset+3*CharHeight,1+7,FALSE);

	| Tga:
		temp_long_card := ReadTGA.GetFileSize();
	 	LongCardToString(temp_long_card,temp_string,10);
	 	PutString(DialogWindow(image_info),temp_string,CARDINAL(40*CharWidth)-3-Length(temp_string)*CARDINAL(CharWidth),y_offset+3*CharHeight,1+7,FALSE);

	| Tif:
		temp_long_card := ReadTIF.GetFileSize();
	 	LongCardToString(temp_long_card,temp_string,10);
	 	PutString(DialogWindow(image_info),temp_string,CARDINAL(40*CharWidth)-3-Length(temp_string)*CARDINAL(CharWidth),y_offset+3*CharHeight,1+7,FALSE);

	| Gif:
	| Jpg:

	ELSE
		PutString(DialogWindow(image_info),"N/A",40*CharWidth-3-3*CharWidth,y_offset+3*CharHeight,1+7,FALSE);

      END;
END ShowFileSize;


(******************************************************************)
(* This procedure displays the file date and time of the current	*)
(* drawing on the Image Information dialog.				*)
(******************************************************************)
PROCEDURE ShowFileStamp();
VAR
   f : File;
   stamp : FileStamp;
   i : CARDINAL;
   ok : BOOLEAN;
   FileName:ARRAY [0..0] OF LongStr;
   day_str, month_str, year_str, hour_str (*, min_str*) : ARRAY [0..2] OF CHAR;
   sec_str, am_pm, min_str : ARRAY [0..1] OF CHAR;
   temp_string, temp_string2 : ARRAY [0..9] OF CHAR;
   temp_card, str_leng, stamp_offset : CARDINAL;
   temp_long_card, day, month, year, hour, min, sec : LONGCARD;
BEGIN
   PutString(DialogWindow(image_info),"File Date: ",x_offset,y_offset+4*CharHeight,1+7,FALSE);
   IF (OpenFile(f, OpenFileName, FALSE) = OK) THEN
      IF GetFileStamp(f, stamp) THEN
	   CloseFile(f);
         IF LONGCARD(stamp.Day) < 10 THEN
	      day_str[0] := "0";
           	day_str[1] := VAL(CHAR,stamp.Day+48);
	   ELSE
           	CardToStr(LONGCARD(stamp.Day),day_str,10,ok);
	   END;
	   day_str[2] := "-";

         IF LONGCARD(stamp.Month) < 10 THEN
	      month_str[0] := "0";
           	month_str[1] := VAL(CHAR,stamp.Month+48);
	   ELSE
           	CardToStr(LONGCARD(stamp.Month),month_str,10,ok);
	   END;
	   month_str[2] := "-";

         IF LONGCARD(stamp.Year) < 10 THEN
	      year_str[0] := "0";
           	year_str[1] := VAL(CHAR,stamp.Year+48);
	   ELSE
           	CardToStr(LONGCARD(stamp.Year),year_str,10,ok);
	   END;
	   year_str[2] := " ";

         IF LONGCARD(stamp.Hour) < 10 THEN
           	IF LONGCARD(stamp.Hour) = 0 THEN
		   hour_str[0] := "1";
	         hour_str[1] := "2";
	      ELSE
	         hour_str[0] := "0";
               hour_str[1] := VAL(CHAR,stamp.Hour+48);
	      END;
         ELSIF (LONGCARD(stamp.Hour) >= 10) AND (LONGCARD(stamp.Hour) <= 12) THEN
           	CardToStr(LONGCARD(stamp.Hour),hour_str,10,ok);
         ELSIF LONGCARD(stamp.Hour) > 12 THEN
           	CardToStr(LONGCARD(stamp.Hour-12),hour_str,10,ok);
           	IF LONGCARD(stamp.Hour-12) < 10 THEN
	         hour_str[0] := "0";
               hour_str[1] := VAL(CHAR,stamp.Hour-12+48);
	      END;
	   END;

         IF LONGCARD(stamp.Hour) < 12 THEN
	      am_pm[0] := "a";
	      am_pm[1] := "m";
	   ELSE
	      am_pm[0] := "p";
	      am_pm[1] := "m";
	   END;
	   hour_str[2] := ":";

         IF LONGCARD(stamp.Min) < 10 THEN
	      min_str[0] := "0";
           	min_str[1] := VAL(CHAR,stamp.Min+48);
	   ELSE
           	CardToStr(LONGCARD(stamp.Min),min_str,10,ok);
	   END;

	   PutString(DialogWindow(image_info),day_str,5+23*CharWidth,y_offset+4*CharHeight,1+7,FALSE);
	   PutString(DialogWindow(image_info),month_str,5+26*CharWidth,y_offset+4*CharHeight,1+7,FALSE);
	   PutString(DialogWindow(image_info),year_str,5+29*CharWidth,y_offset+4*CharHeight,1+7,FALSE);
	   PutString(DialogWindow(image_info),hour_str,5+32*CharWidth,y_offset+4*CharHeight,1+7,FALSE);
	   PutString(DialogWindow(image_info),min_str,5+35*CharWidth,y_offset+4*CharHeight,1+7,FALSE);
	   PutString(DialogWindow(image_info),am_pm,5+37*CharWidth,y_offset+4*CharHeight,1+7,FALSE);

	END; (* IF *)

   ELSE
	PutString(DialogWindow(image_info),"N/A",40*CharWidth-3-3*CharWidth,y_offset+4*CharHeight,1+7,FALSE);
   END; (* IF *)
END ShowFileStamp;


(************************************************************************)
(* This procedure displays the width of the current drawing(in pixels)	*)
(* on the Image Information dialog.							*)
(************************************************************************)
PROCEDURE ShowImageWidth();
VAR
   ok : BOOLEAN;
   FileName:ARRAY [0..0] OF LongStr;
   temp_string, temp_string2 : ARRAY [0..9] OF CHAR;
   temp_card, str_leng, stamp_offset : CARDINAL;
   temp_long_card, day, month, year, hour, min, sec : LONGCARD;
BEGIN
   PutString(DialogWindow(image_info),"Image Width: ",x_offset,y_offset+6*CharHeight,1+7,FALSE);
      CASE extension OF
	  Bmp:
		temp_long_card := ReadBMP.GetWidth();
		LongCardToString(temp_long_card,temp_string,10);
		PutString(DialogWindow(image_info),temp_string,CARDINAL(40*CharWidth)-3-Length(temp_string)*CARDINAL(CharWidth),y_offset+6*CharHeight,1+7,FALSE);

	| Pcx:
		temp_card := ReadPCX.GetWidth();
		CardinalToString(temp_card,temp_string,10);
		PutString(DialogWindow(image_info),temp_string,CARDINAL(40*CharWidth)-3-Length(temp_string)*CARDINAL(CharWidth),y_offset+6*CharHeight,1+7,FALSE);

	| Tga:
		temp_card := ReadTGA.GetWidth();
		CardinalToString(temp_card,temp_string,10);
		PutString(DialogWindow(image_info),temp_string,CARDINAL(40*CharWidth)-3-Length(temp_string)*CARDINAL(CharWidth),y_offset+6*CharHeight,1+7,FALSE);

	| Tif:
		temp_long_card := ReadTIF.GetWidth();
		LongCardToString(temp_long_card,temp_string,10);
		PutString(DialogWindow(image_info),temp_string,CARDINAL(40*CharWidth)-3-Length(temp_string)*CARDINAL(CharWidth),y_offset+6*CharHeight,1+7,FALSE);

	| Gif:
	| Jpg:

	ELSE
		temp_long_card := LONGCARD(CanvasRight-CanvasLeft+1);
		LongCardToString(temp_long_card,temp_string,10);
		PutString(DialogWindow(image_info),temp_string,CARDINAL(40*CharWidth)-3-Length(temp_string)*CARDINAL(CharWidth),y_offset+6*CharHeight,1+7,FALSE);
      END;
END ShowImageWidth;


(************************************************************************)
(* This procedure displays the height of the current drawing(in pixels)	*)
(* on the Image Information dialog.							*)
(************************************************************************)
PROCEDURE ShowImageHeight();
VAR
   ok : BOOLEAN;
   FileName:ARRAY [0..0] OF LongStr;
   temp_string, temp_string2 : ARRAY [0..9] OF CHAR;
   temp_card, str_leng, stamp_offset : CARDINAL;
   temp_long_card, day, month, year, hour, min, sec : LONGCARD;
BEGIN
   PutString(DialogWindow(image_info),"Image Height: ",x_offset,y_offset+7*CharHeight,1+7,FALSE);
   CASE extension OF
	  Bmp:
		temp_long_card := ReadBMP.GetHeight();
		LongCardToString(temp_long_card,temp_string,10);
		PutString(DialogWindow(image_info),temp_string,CARDINAL(40*CharWidth)-3-Length(temp_string)*CARDINAL(CharWidth),y_offset+7*CharHeight,1+7,FALSE);

	| Pcx:
		temp_card := ReadPCX.GetHeight();
		CardinalToString(temp_card,temp_string,10);
		PutString(DialogWindow(image_info),temp_string,CARDINAL(40*CharWidth)-3-Length(temp_string)*CARDINAL(CharWidth),y_offset+7*CharHeight,1+7,FALSE);

	| Tga:
		temp_card := ReadTGA.GetHeight();
		CardinalToString(temp_card,temp_string,10);
		PutString(DialogWindow(image_info),temp_string,CARDINAL(40*CharWidth)-3-Length(temp_string)*CARDINAL(CharWidth),y_offset+7*CharHeight,1+7,FALSE);

	| Tif:
		temp_long_card := ReadTIF.GetHeight();
		LongCardToString(temp_long_card,temp_string,10);
		PutString(DialogWindow(image_info),temp_string,CARDINAL(40*CharWidth)-3-Length(temp_string)*CARDINAL(CharWidth),y_offset+7*CharHeight,1+7,FALSE);

	| Gif:
	| Jpg:

	ELSE
		temp_long_card := LONGCARD(CanvasBottom-CanvasTop+1);
		LongCardToString(temp_long_card,temp_string,10);
		PutString(DialogWindow(image_info),temp_string,CARDINAL(40*CharWidth)-3-Length(temp_string)*CARDINAL(CharWidth),y_offset+7*CharHeight,1+7,FALSE);
      END;
END ShowImageHeight;


(************************************************************************)
(* This procedure displays the number of bits used to represent one	*)
(* pixel of the current drawing on the Image Information dialog.		*)
(************************************************************************)
PROCEDURE ShowBitsPerPixel();
VAR
   ok : BOOLEAN;
   FileName:ARRAY [0..0] OF LongStr;
   temp_string, temp_string2 : ARRAY [0..9] OF CHAR;
   temp_card, str_leng, stamp_offset : CARDINAL;
   temp_long_card, day, month, year, hour, min, sec : LONGCARD;
BEGIN
   PutString(DialogWindow(image_info),"Bits Per Pixel: ",x_offset,y_offset+8*CharHeight,1+7,FALSE);
   CASE extension OF
	  Bmp:
		temp_card := ReadBMP.GetBitsPerPixel();
		CardinalToString(temp_card,temp_string,10);
		PutString(DialogWindow(image_info),temp_string,CARDINAL(40*CharWidth)-3-Length(temp_string)*CARDINAL(CharWidth),y_offset+8*CharHeight,1+7,FALSE);

	| Pcx:
		temp_card := ReadPCX.GetBitsPerPixel();
		CardinalToString(temp_card,temp_string,10);
		PutString(DialogWindow(image_info),temp_string,CARDINAL(40*CharWidth)-3-Length(temp_string)*CARDINAL(CharWidth),y_offset+8*CharHeight,1+7,FALSE);

	| Tga:
		temp_card := ReadTGA.GetBitsPerPixel();
		CardinalToString(temp_card,temp_string,10);
		PutString(DialogWindow(image_info),temp_string,CARDINAL(40*CharWidth)-3-Length(temp_string)*CARDINAL(CharWidth),y_offset+8*CharHeight,1+7,FALSE);

	| Tif:
		temp_long_card := ReadTIF.GetBitsPerPixel();
		LongCardToString(temp_long_card,temp_string,10);
		PutString(DialogWindow(image_info),temp_string,CARDINAL(40*CharWidth)-3-Length(temp_string)*CARDINAL(CharWidth),y_offset+8*CharHeight,1+7,FALSE);

	| Gif:
	| Jpg:

	ELSE
		temp_long_card := 8;
		LongCardToString(temp_long_card,temp_string,10);
		PutString(DialogWindow(image_info),temp_string,CARDINAL(40*CharWidth)-3-Length(temp_string)*CARDINAL(CharWidth),y_offset+8*CharHeight,1+7,FALSE);
      END;
END ShowBitsPerPixel;


(************************************************************************)
(* This procedure displays the number of colour planes used for the	*)
(* current drawing on the Information Dialog.					*)
(************************************************************************)
PROCEDURE ShowColourPlanes();
VAR
   ok : BOOLEAN;
   FileName:ARRAY [0..0] OF LongStr;
   temp_string, temp_string2 : ARRAY [0..9] OF CHAR;
   temp_card, str_leng, stamp_offset : CARDINAL;
   temp_long_card, day, month, year, hour, min, sec : LONGCARD;
BEGIN
   PutString(DialogWindow(image_info),"Colour Planes: ",x_offset,y_offset+9*CharHeight,1+7,FALSE);
   CASE extension OF
	  Bmp:
		temp_card := ReadBMP.GetColourPlanes();
		CardinalToString(temp_card,temp_string,10);
		PutString(DialogWindow(image_info),temp_string,CARDINAL(40*CharWidth)-3-Length(temp_string)*CARDINAL(CharWidth),y_offset+9*CharHeight,1+7,FALSE);

	| Pcx:
		temp_card := ReadPCX.GetColourPlanes();
		CardinalToString(temp_card,temp_string,10);
		PutString(DialogWindow(image_info),temp_string,CARDINAL(40*CharWidth)-3-Length(temp_string)*CARDINAL(CharWidth),y_offset+9*CharHeight,1+7,FALSE);

	| Tga:
		temp_card := ReadTGA.GetColourPlanes();
		CardinalToString(temp_card,temp_string,10);
		PutString(DialogWindow(image_info),temp_string,CARDINAL(40*CharWidth)-3-Length(temp_string)*CARDINAL(CharWidth),y_offset+9*CharHeight,1+7,FALSE);

	| Tif:
		temp_card := ReadTIF.GetColourPlanes();
		CardinalToString(temp_card,temp_string,10);
		PutString(DialogWindow(image_info),temp_string,CARDINAL(40*CharWidth)-3-Length(temp_string)*CARDINAL(CharWidth),y_offset+9*CharHeight,1+7,FALSE);

	| Gif:
	| Jpg:

	ELSE
		temp_card := 1;
		CardinalToString(temp_card,temp_string,10);
		PutString(DialogWindow(image_info),temp_string,CARDINAL(40*CharWidth)-3-Length(temp_string)*CARDINAL(CharWidth),y_offset+9*CharHeight,1+7,FALSE);
      END;
END ShowColourPlanes;


(************************************************************************)
(* This procedure displays the maximum number of colours the current	*)
(* can contain										*)
(************************************************************************)
PROCEDURE ShowMaxColours();
VAR
   ok : BOOLEAN;
   FileName:ARRAY [0..0] OF LongStr;
   temp_string, temp_string2 : ARRAY [0..9] OF CHAR;
   temp_card, str_leng, stamp_offset : CARDINAL;
   temp_long_card, day, month, year, hour, min, sec : LONGCARD;
BEGIN
   PutString(DialogWindow(image_info),"Max # of Colours: ",x_offset,y_offset+10*CharHeight,1+7,FALSE);
   CASE extension OF
	  Bmp:
		temp_long_card := ReadBMP.GetMaxColours();
		LongCardToString(temp_long_card,temp_string,10);
		PutString(DialogWindow(image_info),temp_string,CARDINAL(40*CharWidth)-3-Length(temp_string)*CARDINAL(CharWidth),y_offset+10*CharHeight,1+7,FALSE);

	| Pcx:
		temp_long_card := ReadPCX.GetMaxColours();
		LongCardToString(temp_long_card,temp_string,10);
		PutString(DialogWindow(image_info),temp_string,CARDINAL(40*CharWidth)-3-Length(temp_string)*CARDINAL(CharWidth),y_offset+10*CharHeight,1+7,FALSE);

	| Tga:
		temp_long_card := ReadTGA.GetMaxColours();
		LongCardToString(temp_long_card,temp_string,10);
		PutString(DialogWindow(image_info),temp_string,CARDINAL(40*CharWidth)-3-Length(temp_string)*CARDINAL(CharWidth),y_offset+10*CharHeight,1+7,FALSE);

	| Tif:
		temp_long_card := ReadTIF.GetMaxColours();
		LongCardToString(temp_long_card,temp_string,10);
		PutString(DialogWindow(image_info),temp_string,CARDINAL(40*CharWidth)-3-Length(temp_string)*CARDINAL(CharWidth),y_offset+10*CharHeight,1+7,FALSE);

	| Gif:
	| Jpg:

	ELSE
		temp_long_card := 256;
		LongCardToString(temp_long_card,temp_string,10);
		PutString(DialogWindow(image_info),temp_string,CARDINAL(40*CharWidth)-3-Length(temp_string)*CARDINAL(CharWidth),y_offset+10*CharHeight,1+7,FALSE);
      END;
END ShowMaxColours;


(************************************************************************)
(* This procedure displays unpacked size of the current drawing on the	*)
(* Image Information dialog.								*)
(************************************************************************)
PROCEDURE ShowUnpackedSize();
VAR
   i : CARDINAL;
   ok : BOOLEAN;
   FileName:ARRAY [0..0] OF LongStr;
   temp_string, temp_string2 : ARRAY [0..9] OF CHAR;
   temp_card, str_leng, stamp_offset : CARDINAL;
   temp_long_card, day, month, year, hour, min, sec : LONGCARD;
BEGIN
   PutString(DialogWindow(image_info),"Unpacked Size: ",x_offset,y_offset+11*CharHeight,1+7,FALSE);
   CASE extension OF
	  Bmp:
		temp_long_card := ReadBMP.GetUnpackedSize();
		CardToStr(temp_long_card,temp_string,10,ok);
		PutString(DialogWindow(image_info),temp_string,CARDINAL(40*CharWidth)-3-Length(temp_string)*CARDINAL(CharWidth),y_offset+11*CharHeight,1+7,FALSE);

	| Pcx:
		temp_long_card := ReadPCX.GetUnpackedSize();
		CardToStr(temp_long_card,temp_string,10,ok);
		PutString(DialogWindow(image_info),temp_string,CARDINAL(40*CharWidth)-3-Length(temp_string)*CARDINAL(CharWidth),y_offset+11*CharHeight,1+7,FALSE);

	| Tga:
		temp_long_card := ReadTGA.GetUnpackedSize();
		CardToStr(temp_long_card,temp_string,10,ok);
		PutString(DialogWindow(image_info),temp_string,CARDINAL(40*CharWidth)-3-Length(temp_string)*CARDINAL(CharWidth),y_offset+11*CharHeight,1+7,FALSE);

	| Tif:
		temp_long_card := ReadTIF.GetUnpackedSize();
		CardToStr(temp_long_card,temp_string,10,ok);
		PutString(DialogWindow(image_info),temp_string,CARDINAL(40*CharWidth)-3-Length(temp_string)*CARDINAL(CharWidth),y_offset+11*CharHeight,1+7,FALSE);

	| Gif:
	| Jpg:

	ELSE
		temp_long_card := LONGCARD(CanvasRight-CanvasLeft+1)*LONGCARD(CanvasBottom-CanvasTop+1);
		CardToStr(temp_long_card,temp_string,10,ok);
		PutString(DialogWindow(image_info),temp_string,CARDINAL(40*CharWidth)-3-Length(temp_string)*CARDINAL(CharWidth),y_offset+11*CharHeight,1+7,FALSE);
      END;
END ShowUnpackedSize;


(************************************************************************)
(* This procedure brings up the Image Information dialog and displays	*)
(* all the image information.								*)
(************************************************************************)
PROCEDURE ShowImageInfo();
BEGIN

   DisplayDialog(image_info,(*FALSE*)TRUE);

   IF (ObtainScreen()) THEN
   	PutTitleStrip(DialogWindow(image_info),"Image Information");

   	ShowFileName();
   	ShowFileType();
   	ShowSubType();
   	ShowFileSize();
   	ShowFileStamp();
   	ShowImageWidth();
   	ShowImageHeight();
   	ShowBitsPerPixel();
   	ShowColourPlanes();
   	ShowMaxColours();
   	ShowUnpackedSize();

   	ReleaseScreen();
   END; (* IF *)

   CASE DialogSelection(image_info,NIL) OF
    	  1:
  	| ELSE
   END;

   HideDialog(image_info);
END ShowImageInfo;


(************************************************************************)
(* This procedure brings up the About dialog and displays the		*)
(* information about PaintMagic.							*)
(************************************************************************)
PROCEDURE AboutPaintMagic();
BEGIN
   DisplayDialog(image_info,(*FALSE*)TRUE);

   IF (ObtainScreen()) THEN
  	PutTitleStrip(DialogWindow(image_info),"About PaintMagic");

      Palette(DialogWindow(image_info),25+2,40+2);
	PutString(DialogWindow(image_info),"PaintMagic",x_offset+14*CharWidth+3,y_offset+2*CharHeight-8,1+7,FALSE);
	PutString(DialogWindow(image_info),"Version 1.0.0",x_offset+13*CharWidth+4-4-4+2,y_offset+4*CharHeight-8-5,1+7,FALSE);
	PutString(DialogWindow(image_info),"Copyright 1997 Matt Ma",x_offset+8*CharWidth+3,y_offset+6*CharHeight-8-10,1+7,FALSE);
	PutString(DialogWindow(image_info),"Computer Engineering Final Year Project",x_offset+1,y_offset+8*CharHeight-8-15,1+7,FALSE);
	PutString(DialogWindow(image_info),"Email: c9315720@ee.newcastle.edu.au",x_offset+CharWidth+8,y_offset+10*CharHeight+5+2,1+7,FALSE);
	PutLine(DialogWindow(image_info),5,y_offset+9*CharHeight-1,316,y_offset+9*CharHeight-1,1+7);
	PutLine(DialogWindow(image_info),317,y_offset+9*CharHeight-1,317,y_offset+9*CharHeight-1,15);
	PutLine(DialogWindow(image_info),5,y_offset+9*CharHeight,317,y_offset+9*CharHeight,15);

   	ReleaseScreen();
   END; (* IF *)


   CASE DialogSelection(image_info,NIL) OF
    	  1:
  	| ELSE
   END;

   HideDialog(image_info);
END AboutPaintMagic;



BEGIN

   CreateDialog(image_info, NIL,Standard,0,(XMAX-40*CharWidth-5-3) DIV 2,(YMAX-240+20-5+6) DIV 2,7,1+7,1+7,
		FALSE,NIL,40*CharWidth+5+3,240-20+5-6,8*CharWidth+0,CharHeight,5+53,5);
   AddABox(image_info,ADR("   Ok"),1+3,15-7,TRUE);
   AddABox(image_info,ADR(" Cancel"),1+1,4+4,TRUE);
   EndCreateDialog(image_info);

   CreateDialog(new_mode, NIL,Standard,0,(XMAX-182-54-4) DIV 2,(YMAX-60-20) DIV 2,7,1+7,1+7,
		FALSE,NIL,182+54+4,60+20,6*CharWidth+1,CharHeight,5+33+1,5);
   AddABox(new_mode,ADR("  Ok"),1+2,15-7,TRUE);
   AddABox(new_mode,ADR("Cancel"),1,4+4,TRUE);
   EndCreateDialog(new_mode);

   CreateDialog(changed, NIL,Standard,0,(XMAX-182-54-30) DIV 2,(YMAX-60-20) DIV 2,7,1+7,1+7,
		FALSE,NIL,182+54+30,60+20,6*CharWidth+1,CharHeight,5+33+1,5);
   AddABox(changed,ADR(" Yes"),1+1,15-7,TRUE);
   AddABox(changed,ADR("  No"),1+2,15-7,TRUE);
   AddABox(changed,ADR("Cancel"),1,4+4,TRUE);
   EndCreateDialog(changed);

END InfoDialogs.