IMPLEMENTATION MODULE KbInterface;

(****************************************************************************)
(*      FILE NAME   : KBINTERF.DEF                                          *)
(*                                                                          *)
(*      WRITTEN BY  : Chee Hong, KOK AND Tung Pek, LEAN                     *)
(*      STUD. NO.   : 9328853 AND 9241506                                   *)
(*      COURSE      : BACHELOR OF ELECTRICAL ENGINEERING                    *)
(*                                                                          *)
(*      DATE START  : 22 MARCH 1996                                         *)
(*      LAST EDITED : 13 APRIL 1996                                         *)
(*                                                                          *)
(*      DESCRIPTION :                                                       *)
(*                                                                          *)
(*                                                                          *)
(*      STATUS      : ok.  working.                                         *)
(*                                                                          *)
(****************************************************************************)


FROM CancelSequences IMPORT
(*proc*)ActivateCancelSequence;

FROM LinkList IMPORT
(*type*)List, ListElement,
(*proc*)ObtainList, ReleaseList, CreateLinkList, CreateListElement, ElementValid,
        MoveToTop, GetFirstElementInList, ReadObjectFromElement;

FROM GraphicWindow IMPORT
(*var*)CharWidth, CharHeight,
(*proc*)ObtainScreen, ReleaseScreen, PutChar,PutLine;

FROM CircularBuffers IMPORT
(*type*)CircularBuffer,
(*proc*)CreateBuffer, PutBuffer, GetBuffer, BufferEmpty;

FROM Storage1 IMPORT
(*proc*)ALLOCATE, DEALLOCATE;

FROM Str IMPORT
(*proc*)Delete, Insert;

FROM Keyboard IMPORT
(*proc*)InKey;

FROM TaskControl IMPORT
(*type*)Lock,
(*proc*)CreateTask, CreateLock, Obtain, Release;

TYPE
  Keyboard = ListElement;

  KeyboardRecord = POINTER TO RECORD
                     buffer:CircularBuffer;
                   END;(*RECORD*)

VAR
  ActiveKeyboardList, InactiveKeyboardList:List;
  CurrentKeyboardRecord:KeyboardRecord;
  KeyboardLock:Lock;


PROCEDURE SetupKeyboardLink(VAR (*OUT*) KeyBoard:Keyboard);
VAR
  TempKeyboardRecord:KeyboardRecord;
BEGIN
  NEW(TempKeyboardRecord);

  CreateBuffer(TempKeyboardRecord^.buffer, 64);

  CreateListElement(InactiveKeyboardList, KeyBoard, TempKeyboardRecord);
END SetupKeyboardLink;

PROCEDURE ReadFromKeyboard(KeyBoard:Keyboard;
                           VAR (*OUT*) KeyPressed:KeyPressed);
VAR
  TempKeyboardRecord:KeyboardRecord;
BEGIN
  ReadObjectFromElement(TempKeyboardRecord, KeyBoard);

  KeyPressed[0] := GetBuffer(TempKeyboardRecord^.buffer);
  IF (KeyPressed[0] = NULL) THEN
    KeyPressed[1] := GetBuffer(TempKeyboardRecord^.buffer);
  ELSE
    KeyPressed[1] := NULL;
  END;(*IF*)
END ReadFromKeyboard;

PROCEDURE StuffKeyboard(KeyBoard:Keyboard;
                        KeyPressed:KeyPressed);
BEGIN
  Obtain(KeyboardLock);
  IF (CurrentKeyboardRecord <> NIL) THEN
    PutBuffer(CurrentKeyboardRecord^.buffer, KeyPressed[0]);
    IF (KeyPressed[0] = NULL) THEN
      PutBuffer(CurrentKeyboardRecord^.buffer, KeyPressed[1]);
    END;(*IF*)
  END;(*IF*)
  Release(KeyboardLock);
END StuffKeyboard;

PROCEDURE ConnectToKeyboard(KeyBoard:Keyboard);
VAR
  TempKeyboardRecord:KeyboardRecord;
BEGIN
  ObtainList();
  MoveToTop(KeyBoard, ActiveKeyboardList);
  ReleaseList();
  ReadObjectFromElement(TempKeyboardRecord, KeyBoard);
  Obtain(KeyboardLock);
  CurrentKeyboardRecord := TempKeyboardRecord;
  Release(KeyboardLock);
END ConnectToKeyboard;

PROCEDURE DisconnectFromKeyboard(KeyBoard:Keyboard);
VAR
  TempKeyboard:Keyboard;
  TempKeyboardRecord:KeyboardRecord;
BEGIN
  ObtainList();
  MoveToTop(KeyBoard, InactiveKeyboardList);
  GetFirstElementInList(TempKeyboard, ActiveKeyboardList);
  ReleaseList();
  ReadObjectFromElement(TempKeyboardRecord, TempKeyboard);
  Obtain(KeyboardLock);
  CurrentKeyboardRecord := TempKeyboardRecord;
  Release(KeyboardLock);
END DisconnectFromKeyboard;

(****************************************************************************)
(* ClearKeyboard                                                            *)
(****************************************************************************)
(*  This procedure is called by other tasks to clear all keys pressed.      *)
(*  PARAMETER PASSED:                                                       *)
(*     KeyBoard - a data structure to hold information about the keyboard   *)
(*                link.                                                     *)
(****************************************************************************)
PROCEDURE ClearKeyboard(KeyBoard:Keyboard);
VAR
  TempKeyboardRecord:KeyboardRecord;
  ch:CHAR;
BEGIN
  ReadObjectFromElement(TempKeyboardRecord, KeyBoard);
  WHILE (NOT(BufferEmpty(TempKeyboardRecord^.buffer))) DO
    ch := GetBuffer(TempKeyboardRecord^.buffer);
  END;(*WHILE*)
END ClearKeyboard;

(****************************************************************************)
(* ReadStringFromKeyboard                                                   *)
(****************************************************************************)
(*  This procedure is called by other tasks to read keys pressed.           *)
(*  PARAMETER PASSED:                                                       *)
(*     w - the window that will display the entered string.                 *)
(*     KeyBoard - a data structure to hold information about the keyboard   *)
(*                link.                                                     *)
(*     string - string to be read.                                          *)
(*     x, y - character offset of the string in the window, starting from   *)
(*            (0,0).                                                        *)
(*     DisplayNumChars - allow this number of characters of the string to be*)
(*                       displayed                                          *)
(*     colour - colour of the string to be displayed.                       *)
(*     backcolour - background colour of the string.                        *)
(****************************************************************************)
PROCEDURE ReadStringFromKeyboard(w:GraphicalWindow; KeyBoard:Keyboard;
                           VAR (*OUT*) string:ARRAY OF CHAR; x,y:INTEGER;
                           DisplayNumChars:INTEGER;
                           colour, backcolour:CARDINAL):BOOLEAN;
VAR
  MAXCHARS,MaxCharsAllowed, CharCount,
  TempCharDisplay,CursorAt,DisplayFrom:INTEGER;
  TempKeyPressed:KeyPressed;
  display:BOOLEAN;
BEGIN
  MAXCHARS := 0;
  x := x * CharWidth;
  y := y * CharHeight;
  string[0] := NULL;
  MaxCharsAllowed := HIGH(string)-1;
  CharCount := 0;
  CursorAt := 0;
  DisplayFrom := 0;

  REPEAT

    display := FALSE;

    IF (ObtainScreen()) THEN
      PutLine(w,x+CursorAt*CharWidth,y + CharHeight-1,x+(CursorAt+1)*CharWidth,y + CharHeight-1,colour);
      PutLine(w,x+CursorAt*CharWidth,y + CharHeight,x+(CursorAt+1)*CharWidth,y + CharHeight,colour);
      ReleaseScreen();
    END;(*IF*)

    ReadFromKeyboard(KeyBoard, TempKeyPressed);

    IF (ObtainScreen()) THEN
      PutLine(w,x+CursorAt*CharWidth,y + CharHeight-1,x+(CursorAt+1)*CharWidth,y + CharHeight-1,backcolour);
      PutLine(w,x+CursorAt*CharWidth,y + CharHeight,x+(CursorAt+1)*CharWidth,y + CharHeight,backcolour);
      ReleaseScreen();
    END;(*IF*)

    CASE TempKeyPressed[0] OF
      EXTENDED_AUX:
        CASE TempKeyPressed[1] OF
          LeftArrowMain:
            IF (CharCount > 0) THEN
              DEC(CharCount);
              IF (CursorAt > 0) THEN
                DEC(CursorAt);
              ELSE
                DEC(DisplayFrom);
                display := TRUE;
              END;(*IF*)
            END;(*IF*)
        | RightArrowMain:
            IF (CharCount < MAXCHARS) THEN
              INC(CharCount);
              IF (CursorAt < DisplayNumChars) THEN
                INC(CursorAt);
              ELSE
                INC(DisplayFrom);
                display := TRUE;
              END;(*IF*)
            END;(*IF*)
        | DeleteKey:
          IF (CharCount < MAXCHARS) THEN
            DEC(MAXCHARS);
            Delete(string,CharCount,1);
            display := TRUE;
          END;(*IF*)

        |ELSE
        END;(*IF*)

    | BackSpace:
        IF (CharCount > 0) THEN
          DEC(CharCount);
          DEC(MAXCHARS);
          Delete(string,CharCount,1);
          IF (CursorAt > 0) THEN
            DEC(CursorAt);
          ELSE
            DEC(DisplayFrom);
          END;(*IF*)
          display := TRUE;
        END;(*IF*)

    | ReturnKey, ESCKey:

    | " ".."~":
      IF (CharCount < MaxCharsAllowed) AND (MAXCHARS < MaxCharsAllowed) THEN
        TempKeyPressed[1] := NULL;
        Insert(string, TempKeyPressed,CharCount);
        INC(MAXCHARS);
        INC(CharCount);
        IF (CursorAt < DisplayNumChars) THEN
          INC(CursorAt);
        ELSE
          INC(DisplayFrom);
        END;(*IF*)
        display := TRUE;
      END;(*IF*)

    | ELSE
    END;(*CASE*)

    IF (display) AND (ObtainScreen()) THEN
      TempCharDisplay := 0;
      WHILE (TempCharDisplay + DisplayFrom < MAXCHARS) AND (TempCharDisplay < DisplayNumChars) DO
        PutChar(w,"",x+TempCharDisplay*CharWidth,y,backcolour,FALSE);
        PutChar(w,string[TempCharDisplay+DisplayFrom],x+TempCharDisplay*CharWidth,y,colour,FALSE);
        INC(TempCharDisplay);
      END;(*WHILE*)
      PutChar(w,"",x+TempCharDisplay*CharWidth,y,backcolour,FALSE);
      ReleaseScreen();
    END;(*IF*)

  UNTIL (TempKeyPressed[0] = ReturnKey) OR (TempKeyPressed[0] = ESCKey);

  RETURN (TempKeyPressed[0] = ReturnKey);
END ReadStringFromKeyboard;

PROCEDURE KeyboardMonitor();
VAR
  char:CHAR;
BEGIN
  LOOP
    char := InKey();
    Obtain(KeyboardLock);
    IF (CurrentKeyboardRecord <> NIL) THEN
      PutBuffer(CurrentKeyboardRecord^.buffer, char);
    END;(*IF*)
    Release(KeyboardLock);
  END;(*IF*)
END KeyboardMonitor;

BEGIN
  CreateLinkList(ActiveKeyboardList);

  CreateLinkList(InactiveKeyboardList);

  CreateLock(KeyboardLock,"KeyboardLock");

  CreateTask(KeyboardMonitor, 1, "KB Monitor");

  CurrentKeyboardRecord := NIL;

END KbInterface.
