IMPLEMENTATION MODULE MouseRoutines;

(******************************************************************)
(*											*)
(*		     		Mouse Routines Module				*)
(*											*)
(*	Programmer :	Matthew Ma (c9315720)				*)
(*				Based on the code originally developed	*)
(*				by Lin Fong WEE and Wai Kwong IP.		*)
(*											*)
(*	Last edited :	22-10-97						*)
(*											*)
(*	Function : 		This module contains all of the mouse	*)
(*				cursors used in PaintMagic as well as	*)
(*				mouse	routines including displaying &	*)
(*				hiding of mouse cursors.			*)
(*				of mouser cursors.				*)
(* 											*)
(*	Status :		Working.						*)
(******************************************************************)


FROM Mouse0 IMPORT
(* type *)   Buttons,ButtonSet;

FROM Mouse33 IMPORT
(* proc *)   InitialiseMouseDriver, Reset, SetHorizontalLimits,
		 SetVerticalLimits, GetPosBut, SetCursorPos;

FROM Storage1 IMPORT
(* proc *)   ALLOCATE;

FROM GraphicWindow IMPORT
(* type *)   _GPSET, CharWidth, CharHeight, XMAX, YMAX,
(* proc *)   RespondToMouseButtons, UnconditionalObtainScreen, FastReleaseScreen,
		 ImageSize, GetImage, PutImage, Line, Rectangle, Plot, TrueDisc, TrueCircle,
		 PutString,ObtainScreen, ReleaseScreen;

FROM TaskControl IMPORT
(* type *)   Lock,
(* proc *)   CreateTask, CreateLock, Obtain, Release;

FROM Semaphores IMPORT
(* type *)   Semaphore,
(* proc *)   CreateSemaphore, Wait, Signal, DestroySemaphore;

FROM Timer IMPORT
(* proc *)   TimedWait;

FROM Str IMPORT
(* proc *)   Copy, StrToReal, StrToCard, CardToStr;


CONST
   XMax = 23;		(* maximum size for mouse cursor *)
   YMax = 23;
   DCThresh = 10;		(* treshold speed for double click *)

VAR 
   temp_lock, cursor_lock : Lock;
   addr : ADDRESS;
   cursor_pointer : CursorPtr;
   x, y, hot_x, hot_y : INTEGER;
   MouseEnabled : BOOLEAN;



(******************************************************************)
(* This is a near procedure for drawing lines.				*)
(******************************************************************)
PROCEDURE DrawLine(x_offset, y_offset, x1, y1, x2, y2, colour : INTEGER);
BEGIN
   Line(x_offset + x1, y_offset + y1, x_offset + x2, y_offset + y2, colour);
END DrawLine;


(******************************************************************)
(* This procedure displays the mouse cursor.				*)
(******************************************************************)
PROCEDURE DisplayMouse();
BEGIN
   Obtain(temp_lock);
   IF MouseEnabled THEN

	(**********************************)
	(* saves area of screen where the *)
	(* mouse cursor is to be placed   *)
	(**********************************)
	GetImage(x-hot_x,y-hot_y,x-hot_x+XMax,y-hot_y+YMax, addr);
	cursor_pointer(x-hot_x, y-hot_y);
   END;
   Release(temp_lock);
END DisplayMouse;


(******************************************************************)
(* This procedure refreshes the mouse cursor.				*)
(******************************************************************)
PROCEDURE HideMouse();
BEGIN
   Obtain(temp_lock);
   IF MouseEnabled THEN

	(**************************************)
	(* restores area of screen where the  *)
	(* mouse cursor was previously placed *)
	(**************************************)
	PutImage(x-hot_x,y-hot_y,addr,_GPSET);
   END;
   Release(temp_lock);
END HideMouse;


(******************************************************************)
(* This procedure draws the "Arrow" mouse cursor.			*)
(******************************************************************)
PROCEDURE ArrowCursor(x_offset, y_offset : INTEGER);
BEGIN
   Obtain(cursor_lock);

   (******** Frame *********)
   DrawLine(x_offset,y_offset,0,0,0,14,0);
   DrawLine(x_offset,y_offset,0,0,10,10,0);
   DrawLine(x_offset,y_offset,0,14,3,11,0);
   DrawLine(x_offset,y_offset,10,10,6,10,0);
   DrawLine(x_offset,y_offset,4,12,4,13,0);
   DrawLine(x_offset,y_offset,5,14,5,15,0);
   DrawLine(x_offset,y_offset,6,16,6,17,0);
   DrawLine(x_offset,y_offset,7,18,8,18,0);

   DrawLine(x_offset,y_offset,6,10,6,11,0);
   DrawLine(x_offset,y_offset,7,12,7,13,0);
   DrawLine(x_offset,y_offset,8,14,8,15,0);
   DrawLine(x_offset,y_offset,9,16,9,17,0);

   (******** handle ********)
   DrawLine(x_offset,y_offset,1,2,1,12,15);
   DrawLine(x_offset,y_offset,2,3,2,11,15);
   DrawLine(x_offset,y_offset,3,4,3,10,15);
   DrawLine(x_offset,y_offset,4,5,4,11,15);
   DrawLine(x_offset,y_offset,5,6,5,13,15);
   DrawLine(x_offset,y_offset,6,7,6,9,15);
   DrawLine(x_offset,y_offset,7,8,7,9,15);
   Plot(x+8,y+9,15);
   DrawLine(x_offset,y_offset,6,12,6,15,15);
   DrawLine(x_offset,y_offset,7,14,7,17,15);
   DrawLine(x_offset,y_offset,8,16,8,17,15);

   Release(cursor_lock);
END ArrowCursor;


(******************************************************************)
(* This procedure draws the "CrossHair" mouse cursor.			*)
(******************************************************************)
PROCEDURE CrossHair(x_offset,y_offset : INTEGER);
BEGIN

   Obtain(cursor_lock);

   DrawLine(x_offset,y_offset,0,9,7,9,0);
   DrawLine(x_offset,y_offset,9,0,9,7,0);
   DrawLine(x_offset,y_offset,11,9,18,9,0);
   DrawLine(x_offset,y_offset,9,11,9,18,0);

   Release(cursor_lock);

END CrossHair;


(******************************************************************)
(* This procedure draws the small eraser mouse cursor.		*)
(******************************************************************)
PROCEDURE SmallEraserCursor(x_offset,y_offset : INTEGER);
BEGIN

   Obtain(cursor_lock);

   DrawLine(x_offset,y_offset,1,1,2,1,15);
   DrawLine(x_offset,y_offset,1,2,2,2,15);
   DrawLine(x_offset,y_offset,0,0,3,0,0);
   DrawLine(x_offset,y_offset,0,3,3,3,0);
   DrawLine(x_offset,y_offset,0,0,0,3,0);
   DrawLine(x_offset,y_offset,3,0,3,3,0);

   Release(cursor_lock);

END SmallEraserCursor;


(******************************************************************)
(* This procedure draws the medium eraser mouse cursor.		*)
(******************************************************************)
PROCEDURE MediumEraserCursor(x_offset,y_offset : INTEGER);
BEGIN

   Obtain(cursor_lock);

   DrawLine(x_offset,y_offset,1,1,4,1,15);
   DrawLine(x_offset,y_offset,1,2,4,2,15);
   DrawLine(x_offset,y_offset,1,3,4,3,15);
   DrawLine(x_offset,y_offset,1,4,4,4,15);
   DrawLine(x_offset,y_offset,0,0,5,0,0);
   DrawLine(x_offset,y_offset,0,5,5,5,0);
   DrawLine(x_offset,y_offset,0,0,0,5,0);
   DrawLine(x_offset,y_offset,5,0,5,5,0);

   Release(cursor_lock);

END MediumEraserCursor;


(******************************************************************)
(* This procedure draws the large eraser mouse cursor.		*)
(******************************************************************)
PROCEDURE LargeEraserCursor(x_offset,y_offset : INTEGER);
BEGIN

   Obtain(cursor_lock);

   DrawLine(x_offset,y_offset,1,1,6,1,15);
   DrawLine(x_offset,y_offset,1,2,6,2,15);
   DrawLine(x_offset,y_offset,1,3,6,3,15);
   DrawLine(x_offset,y_offset,1,4,6,4,15);
   DrawLine(x_offset,y_offset,1,5,6,5,15);
   DrawLine(x_offset,y_offset,1,6,6,6,15);
   DrawLine(x_offset,y_offset,0,0,7,0,0);
   DrawLine(x_offset,y_offset,0,7,7,7,0);
   DrawLine(x_offset,y_offset,0,0,0,7,0);
   DrawLine(x_offset,y_offset,7,0,7,7,0);

   Release(cursor_lock);

END LargeEraserCursor;


(******************************************************************)
(* This procedure draws the text tool mouse cursor.			*)
(******************************************************************)
PROCEDURE TextCursor(x_offset,y_offset : INTEGER);
BEGIN

   Obtain(cursor_lock);

   DrawLine(x_offset,y_offset,2,0,2,15,0);
   DrawLine(x_offset,y_offset,0,0,4,0,0);
   DrawLine(x_offset,y_offset,0,15,4,15,0);

   Release(cursor_lock);

END TextCursor;


(******************************************************************)
(* This procedure draws the fingermouse cursor.				*)
(******************************************************************)
PROCEDURE FingerCursor(x_offset, y_offset : INTEGER);
BEGIN

   Obtain(cursor_lock);

   DrawLine(x_offset,y_offset,0,9,0,11,0);
   Plot(x_offset+1,y_offset+12,0);
   DrawLine(x_offset,y_offset,2,13,2,14,0);
   DrawLine(x_offset,y_offset,3,15,3,16,0);
   DrawLine(x_offset,y_offset,4,17,4,18,0);
   DrawLine(x_offset,y_offset,5,19,5,21,0);
   DrawLine(x_offset,y_offset,5,21,14,21,0);
   DrawLine(x_offset,y_offset,14,21,14,19,0);
   DrawLine(x_offset,y_offset,15,18,15,16,0);
   DrawLine(x_offset,y_offset,16,15,16,9,0);
   Plot(x_offset+15,y_offset+8,0);
   Plot(x_offset+14,y_offset+7,0);
   DrawLine(x_offset,y_offset,13,7,13,10,0);
   DrawLine(x_offset,y_offset,12,6,10,6,0);
   DrawLine(x_offset,y_offset,10,6,10,9,0);
   DrawLine(x_offset,y_offset,9,5,8,5,0);
   DrawLine(x_offset,y_offset,7,1,7,9,0);
   DrawLine(x_offset,y_offset,6,0,5,0,0);
   DrawLine(x_offset,y_offset,4,1,4,13,0);
   Plot(x_offset+3,y_offset+10,0);
   DrawLine(x_offset,y_offset,2,9,0,9,0);

   DrawLine(x_offset,y_offset,5,11,14,11,15);
   DrawLine(x_offset,y_offset,5,12,14,12,15);
   DrawLine(x_offset,y_offset,5,13,14,13,15);
   DrawLine(x_offset,y_offset,5,14,14,14,15);
   DrawLine(x_offset,y_offset,5,15,14,15,15);
   DrawLine(x_offset,y_offset,5,16,14,16,15);
   DrawLine(x_offset,y_offset,5,17,14,17,15);
   DrawLine(x_offset,y_offset,5,18,14,18,15);

   DrawLine(x_offset,y_offset,6,19,13,19,15);
   DrawLine(x_offset,y_offset,6,20,13,20,15);

   DrawLine(x_offset,y_offset,5,1,5,9,15);
   DrawLine(x_offset,y_offset,6,1,6,9,15);

   DrawLine(x_offset,y_offset,8,6,8,9,15);
   DrawLine(x_offset,y_offset,9,6,9,9,15);

   DrawLine(x_offset,y_offset,11,7,11,9,15);
   DrawLine(x_offset,y_offset,12,7,12,9,15);

   DrawLine(x_offset,y_offset,5,10,12,10,15);
   DrawLine(x_offset,y_offset,14,8,14,10,15);
   DrawLine(x_offset,y_offset,15,9,15,15,15);
   DrawLine(x_offset,y_offset,1,10,1,11,15);
   DrawLine(x_offset,y_offset,2,10,2,12,15);
   DrawLine(x_offset,y_offset,3,11,3,14,15);
   DrawLine(x_offset,y_offset,4,14,4,16,15);

   Release(cursor_lock);

END FingerCursor;


(******************************************************************)
(* This procedure draws the "Magnifier" mouse cursor.			*)
(******************************************************************)
PROCEDURE Magnifier(x_offset, y_offset: INTEGER);
BEGIN
   Obtain(cursor_lock);

   (* outer frame *)
   DrawLine(x_offset,y_offset,0,7,0,8,0);
   DrawLine(x_offset,y_offset,1,6,1,5,0);
   DrawLine(x_offset,y_offset,1,5,5,1,0);
   DrawLine(x_offset,y_offset,5,1,6,1,0);
   DrawLine(x_offset,y_offset,7,0,8,0,0);
   DrawLine(x_offset,y_offset,9,1,10,1,0);
   DrawLine(x_offset,y_offset,10,1,14,5,0);
   DrawLine(x_offset,y_offset,14,5,14,6,0);
   DrawLine(x_offset,y_offset,15,7,15,8,0);
   DrawLine(x_offset,y_offset,14,9,14,10,0);
   DrawLine(x_offset,y_offset,14,10,20,16,0);
   DrawLine(x_offset,y_offset,20,16,17,19,0);
   DrawLine(x_offset,y_offset,16,19,10,13,0);
   DrawLine(x_offset,y_offset,10,13,9,13,0);
   DrawLine(x_offset,y_offset,8,14,7,14,0);
   DrawLine(x_offset,y_offset,6,13,5,13,0);
   DrawLine(x_offset,y_offset,5,13,1,9,0);
   
   (* inner frame *)
   DrawLine(x_offset,y_offset,3,8,3,6,0);
   DrawLine(x_offset,y_offset,3,6,6,3,0);
   DrawLine(x_offset,y_offset,6,3,9,3,0);
   DrawLine(x_offset,y_offset,9,3,12,6,0);
   DrawLine(x_offset,y_offset,12,6,12,9,0);
   DrawLine(x_offset,y_offset,12,9,10,11,0);
   DrawLine(x_offset,y_offset,10,11,6,11,0);
   DrawLine(x_offset,y_offset,6,11,4,9,0);

   DrawLine(x_offset,y_offset,1,8,5,12,15);
   DrawLine(x_offset,y_offset,1,7,7,13,15);
   DrawLine(x_offset,y_offset,1,7,2,7,15);
   DrawLine(x_offset,y_offset,2,6,7,1,15);
   DrawLine(x_offset,y_offset,2,5,5,2,15);
   DrawLine(x_offset,y_offset,7,2,8,2,15);
   DrawLine(x_offset,y_offset,8,1,14,7,15);
   DrawLine(x_offset,y_offset,10,2,13,5,15);
   DrawLine(x_offset,y_offset,14,7,13,7,15);
   DrawLine(x_offset,y_offset,13,7,13,8,15);
   DrawLine(x_offset,y_offset,14,8,10,12,15);
   DrawLine(x_offset,y_offset,10,12,7,12,15);
   DrawLine(x_offset,y_offset,8,12,8,13,15);
   DrawLine(x_offset,y_offset,11,13,16,18,15);
   DrawLine(x_offset,y_offset,11,12,17,18,15);
   DrawLine(x_offset,y_offset,12,12,17,17,15);
   DrawLine(x_offset,y_offset,12,11,18,17,15);
   DrawLine(x_offset,y_offset,13,11,18,16,15);
   DrawLine(x_offset,y_offset,13,10,19,16,15);

   Release(cursor_lock);
END Magnifier;


(******************************************************************)
(* This procedure draws the "Pencil" mouse cursor.			*)
(******************************************************************)
PROCEDURE Pencil(x_offset, y_offset: INTEGER);
BEGIN
   Obtain(cursor_lock);

   DrawLine(x_offset,y_offset,0,14,0,9,0);
   DrawLine(x_offset,y_offset,1,13,1,10,0);
   DrawLine(x_offset,y_offset,2,12,2,11,0);
   DrawLine(x_offset,y_offset,1,8,1,7,0);
   DrawLine(x_offset,y_offset,2,6,2,5,0);
   DrawLine(x_offset,y_offset,3,4,3,3,0);
   DrawLine(x_offset,y_offset,4,3,4,1,0);
   DrawLine(x_offset,y_offset,5,0,7,0,0);
   DrawLine(x_offset,y_offset,8,1,8,2,0);
   DrawLine(x_offset,y_offset,7,3,7,4,0);
   DrawLine(x_offset,y_offset,7,4,5,4,0);
   DrawLine(x_offset,y_offset,6,5,6,6,0);
   DrawLine(x_offset,y_offset,5,6,5,8,0);
   DrawLine(x_offset,y_offset,4,8,4,10,0);
   DrawLine(x_offset,y_offset,3,10,3,11,0);

   DrawLine(x_offset,y_offset,2,10,2,7,15);
   DrawLine(x_offset,y_offset,3,9,3,5,15);
   DrawLine(x_offset,y_offset,4,7,4,4,15);
   DrawLine(x_offset,y_offset,5,5,5,5,15);
   DrawLine(x_offset,y_offset,1,9,1,9,15);
   DrawLine(x_offset,y_offset,5,3,5,1,15);
   DrawLine(x_offset,y_offset,6,3,6,1,15);
   DrawLine(x_offset,y_offset,7,2,7,1,15);

   Release(cursor_lock);
END Pencil;


(******************************************************************)
(* This procedure draws the "Dropper" mouse cursor.			*)
(******************************************************************)
PROCEDURE DropperCursor(x_offset, y_offset : INTEGER);
BEGIN
   Obtain(cursor_lock);

   (* tip *)
   DrawLine(x_offset,y_offset,12,0,14,0,0);
   DrawLine(x_offset,y_offset,11,1,15,1,0);
   DrawLine(x_offset,y_offset,11,2,15,2,0);
   DrawLine(x_offset,y_offset,8,3,15,3,0);
   DrawLine(x_offset,y_offset,9,4,14,4,0);
   DrawLine(x_offset,y_offset,10,5,12,5,0);
   DrawLine(x_offset,y_offset,11,6,12,6,0);
   DrawLine(x_offset,y_offset,12,7,12,7,0);

   DrawLine(x_offset,y_offset,8,5,1,12,0);
   DrawLine(x_offset,y_offset,10,7,3,14,0);
   DrawLine(x_offset,y_offset,1,13,0,14,0);
   DrawLine(x_offset,y_offset,1,15,2,14,0);

   DrawLine(x_offset,y_offset,1,14,1,14,15);
   DrawLine(x_offset,y_offset,2,12,9,5,15);
   DrawLine(x_offset,y_offset,2,13,9,6,15);
   DrawLine(x_offset,y_offset,3,13,10,6,15);

   Release(cursor_lock);
END DropperCursor;


(******************************************************************)
(* This procedure draws the "AirBrush" mouse cursor.			*)
(******************************************************************)
PROCEDURE AirbrushCursor(x_offset, y_offset : INTEGER);
BEGIN
   Obtain(cursor_lock);

   DrawLine(x_offset,y_offset,0,0,3,3,0);
   DrawLine(x_offset,y_offset,0,2,2,4,0);
   DrawLine(x_offset,y_offset,0,4,1,5,0);
   DrawLine(x_offset,y_offset,0,6,1,5,0);

   DrawLine(x_offset,y_offset,3,3,8,3,0);
   DrawLine(x_offset,y_offset,4,2,4,7,0);
   DrawLine(x_offset,y_offset,4,3,5,4,0);
   DrawLine(x_offset,y_offset,4,7,9,12,0);
   DrawLine(x_offset,y_offset,5,7,10,12,0);
   DrawLine(x_offset,y_offset,6,7,10,11,0);
   DrawLine(x_offset,y_offset,6,7,6,6,0);
   DrawLine(x_offset,y_offset,6,6,7,6,0);
   DrawLine(x_offset,y_offset,7,6,7,5,0);
   DrawLine(x_offset,y_offset,7,6,11,10,0);
   DrawLine(x_offset,y_offset,11,11,12,10,0);
   DrawLine(x_offset,y_offset,8,3,8,4,0);
   DrawLine(x_offset,y_offset,8,3,13,8,0);
   DrawLine(x_offset,y_offset,13,8,13,9,0);

   Release(cursor_lock);
END AirbrushCursor;


(******************************************************************)
(* This procedure draws the small "Brush" mouse cursor.		*)
(******************************************************************)
PROCEDURE SmallBrushCursor(x_offset, y_offset : INTEGER);
BEGIN
   Obtain(cursor_lock);

   DrawLine(x_offset,y_offset,0,1,3,1,0);
   DrawLine(x_offset,y_offset,0,2,3,2,0);
   DrawLine(x_offset,y_offset,1,0,1,3,0);
   DrawLine(x_offset,y_offset,2,0,2,3,0);

   Release(cursor_lock);
END SmallBrushCursor;


(******************************************************************)
(* This procedure draws the medium "Brush" mouse cursor.		*)
(******************************************************************)
PROCEDURE MediumBrushCursor(x_offset, y_offset : INTEGER);
BEGIN
   Obtain(cursor_lock);

   DrawLine(x_offset,y_offset,0,2,0,4,0);
   DrawLine(x_offset,y_offset,1,1,1,5,0);
   DrawLine(x_offset,y_offset,2,0,2,6,0);
   DrawLine(x_offset,y_offset,3,0,3,6,0);
   DrawLine(x_offset,y_offset,4,0,4,6,0);
   DrawLine(x_offset,y_offset,5,1,5,5,0);
   DrawLine(x_offset,y_offset,6,2,6,4,0);

   Release(cursor_lock);
END MediumBrushCursor;


PROCEDURE LargeBrushCursor(x_offset, y_offset : INTEGER);
BEGIN
   Obtain(cursor_lock);

   DrawLine(x_offset,y_offset,0,2,0,5,0);
   DrawLine(x_offset,y_offset,1,1,1,6,0);
   DrawLine(x_offset,y_offset,2,0,2,7,0);
   DrawLine(x_offset,y_offset,3,0,3,7,0);
   DrawLine(x_offset,y_offset,4,0,4,7,0);
   DrawLine(x_offset,y_offset,5,0,5,7,0);
   DrawLine(x_offset,y_offset,6,1,6,6,0);
   DrawLine(x_offset,y_offset,7,2,7,5,0);

   Release(cursor_lock);
END LargeBrushCursor;


(******************************************************************)
(* This procedure draws the large "Brush" mouse cursor.		*)
(******************************************************************)
PROCEDURE SetArrow();
BEGIN
    UnconditionalObtainScreen();
    HideMouse();
    Obtain(temp_lock);
    cursor_pointer := ArrowCursor;
    hot_x := 0;
    hot_y := 0;
    Release(temp_lock);
    DisplayMouse();
    FastReleaseScreen();
END SetArrow;


(******************************************************************)
(* This procedure sets the mouse cursor to the arrow.			*)
(******************************************************************)
PROCEDURE SetCrossHair();
BEGIN
   UnconditionalObtainScreen();
   HideMouse();
   Obtain(temp_lock);
   cursor_pointer := CrossHair;
   hot_x := 9;
   hot_y := 9;
   Release(temp_lock);
   DisplayMouse();
   FastReleaseScreen();
END SetCrossHair;


(******************************************************************)
(* This procedure sets the mouse cursor to the small eraser.	*)
(******************************************************************)
PROCEDURE SetSmallEraser();
BEGIN
   UnconditionalObtainScreen();
   HideMouse();
   Obtain(temp_lock);
   cursor_pointer := SmallEraserCursor;
   hot_x := 2;
   hot_y := 1;
   Release(temp_lock);
   DisplayMouse();
   FastReleaseScreen();
END SetSmallEraser;


(******************************************************************)
(* This procedure sets the mouse cursor to medium eraser.		*)
(******************************************************************)
PROCEDURE SetMediumEraser();
BEGIN
   UnconditionalObtainScreen();
   HideMouse();
   Obtain(temp_lock);
   cursor_pointer := MediumEraserCursor;
   hot_x := 3;
   hot_y := 2;
   Release(temp_lock);
   DisplayMouse();
   FastReleaseScreen();
END SetMediumEraser;


(******************************************************************)
(* This procedure sets the mouse cursor to the large eraser.	*)
(******************************************************************)
PROCEDURE SetLargeEraser();
BEGIN
   UnconditionalObtainScreen();
   HideMouse();
   Obtain(temp_lock);
   cursor_pointer := LargeEraserCursor;
   hot_x := 4;
   hot_y := 3;
   Release(temp_lock);
   DisplayMouse();
   FastReleaseScreen();
END SetLargeEraser;


(******************************************************************)
(* This procedure sets the mouse cursor to the text beam.		*)
(******************************************************************)
PROCEDURE SetText();
BEGIN
   UnconditionalObtainScreen();
   HideMouse();
   Obtain(temp_lock);
   cursor_pointer := TextCursor;
   hot_x := 2;
   hot_y := 0;
   Release(temp_lock);
   DisplayMouse();
   FastReleaseScreen();
END SetText;


(******************************************************************)
(* This procedure sets the mouse cursor to the finger.		*)
(******************************************************************)
PROCEDURE SetFinger();
BEGIN
   UnconditionalObtainScreen();
   HideMouse();
   Obtain(temp_lock);
   cursor_pointer := FingerCursor;
   hot_x := 5;
   hot_y := 0;
   Release(temp_lock);
   DisplayMouse();
   FastReleaseScreen();
END SetFinger;


(******************************************************************)
(* This procedure sets the mouse cursor to the magnifier.		*)
(******************************************************************)
PROCEDURE SetMagnifier();
BEGIN
   UnconditionalObtainScreen();
   HideMouse();
   Obtain(temp_lock);
   cursor_pointer := Magnifier;
   hot_x := 8;
   hot_y := 7;
   Release(temp_lock);
   DisplayMouse();
   FastReleaseScreen();
END SetMagnifier;


(******************************************************************)
(* This procedure sets the mouse cursor to the pencil.		*)
(******************************************************************)
PROCEDURE SetPencil();
BEGIN
   UnconditionalObtainScreen();
   HideMouse();
   Obtain(temp_lock);
   cursor_pointer := Pencil;
   hot_x := 0;
   hot_y := 14;
   Release(temp_lock);
   DisplayMouse();
   FastReleaseScreen();
END SetPencil;


(******************************************************************)
(* This procedure sets the mouse cursor to colour dropper.		*)
(******************************************************************)
PROCEDURE SetDropper();
BEGIN
   UnconditionalObtainScreen();
   HideMouse();
   Obtain(temp_lock);
   cursor_pointer := DropperCursor;
   hot_x := 0;
   hot_y := 15;
   Release(temp_lock);
   DisplayMouse();
   FastReleaseScreen();
END SetDropper;


(******************************************************************)
(* This procedure sets the mouse cursor to the airbrush.		*)
(******************************************************************)
PROCEDURE SetAirbrush();
BEGIN
   UnconditionalObtainScreen();
   HideMouse();
   Obtain(temp_lock);
   cursor_pointer := AirbrushCursor;
   hot_x := 0;
   hot_y := 0;
   Release(temp_lock);
   DisplayMouse();
   FastReleaseScreen();
END SetAirbrush;


(******************************************************************)
(* This procedure sets the mouse cursor to the small brush.		*)
(******************************************************************)
PROCEDURE SetSmallBrush();
BEGIN
   UnconditionalObtainScreen();
   HideMouse();
   Obtain(temp_lock);
   cursor_pointer := SmallBrushCursor;
   hot_x := 2;
   hot_y := 1;
   Release(temp_lock);
   DisplayMouse();
   FastReleaseScreen();
END SetSmallBrush;


(******************************************************************)
(* This procedure sets the mouse cursor to the medium brush.	*)
(******************************************************************)
PROCEDURE SetMediumBrush();
BEGIN
   UnconditionalObtainScreen();
   HideMouse();
   Obtain(temp_lock);
   cursor_pointer := MediumBrushCursor;
   hot_x := 3;
   hot_y := 3;
   Release(temp_lock);
   DisplayMouse();
   FastReleaseScreen();
END SetMediumBrush;


(******************************************************************)
(* This procedure sets the mouse cursor to the large brush.		*)
(******************************************************************)
PROCEDURE SetLargeBrush();
BEGIN
   UnconditionalObtainScreen();
   HideMouse();
   Obtain(temp_lock);
   cursor_pointer := LargeBrushCursor;
   hot_x := 4;
   hot_y := 3;
   Release(temp_lock);
   DisplayMouse();
   FastReleaseScreen();
END SetLargeBrush;


(******************************************************************)
(* This procedure is the mouse interface task.				*)
(******************************************************************)
PROCEDURE MouseInterfaceTask();
VAR
   time : Semaphore;
   Butt : Buttons;
   BStat, Stat : ButtonSet;
   current_x, current_y : INTEGER;
   temp_x, temp_y : CARDINAL;
   DoubleClickCount, butt_num: CARDINAL;
   MiddleBut, LeftBut, RightBut, DoubleClick,Drag, ButPress, PreButPress : BOOLEAN;
   ok, Called : BOOLEAN;
BEGIN
   CreateSemaphore(time,0);

   (**********************************)
   (* waiting for mouse to be set up *)
   (**********************************)
   LOOP
   	IF InitialiseMouseDriver() THEN;
	   Reset(ok,butt_num);
	   SetHorizontalLimits(0, XMAX+1);
	   SetVerticalLimits(0, YMAX+1);

	   GetPosBut(Stat,temp_x,temp_y);	(* gets current mouse position *)
	   current_x := VAL(INTEGER,temp_x);
	   current_y := VAL(INTEGER,temp_y);
	   Obtain(temp_lock);
	
	   x := current_x;	(* previous position of mouse cursor *)
	   y := current_y;

	   MouseEnabled := TRUE;	(* enables mouse *)

	   Release(temp_lock);
	
	   SetArrow();	(* sets arrow as default cursor *)

	   EXIT;	(* sets arrow as default cursor *)

	END; (* IF *)
   END; (* LOOP *)

   DoubleClick := FALSE;
   Drag := FALSE;

   LOOP
	Called := FALSE;	(* used to check whether RespondToMouseButtons *)
				(* is called in one loop   			     *)

	TimedWait(time,10,ok);

	IF MouseEnabled THEN
	   GetPosBut(Stat,temp_x,temp_y);
	   current_x := VAL(INTEGER,temp_x);
	   current_y := VAL(INTEGER,temp_y);

	   IF DoubleClickCount <= DCThresh THEN
		INC(DoubleClickCount);
	   END;

	   (**********************************************)
	   (* checks which mouse button has been pressed *)
	   (**********************************************)
	   BStat := ButtonSet{};
	   IF (LeftButton IN Stat) THEN
		   BStat := BStat + ButtonSet{LeftButton};
	   END;


	   IF (RightButton IN Stat) THEN
		   BStat := BStat + ButtonSet{RightButton};
	   END;

	   IF (MiddleButton IN Stat) THEN
		BStat := BStat + ButtonSet{MiddleButton}
	   END;

	   LeftBut := LeftButton IN BStat;
	   RightBut := RightButton IN BStat;
	   MiddleBut := MiddleButton IN BStat;

	   IF (LeftBut) OR (RightBut) OR (MiddleBut) THEN

		IF (NOT(ButPress)) THEN

		   IF (RightButton IN BStat) OR (MiddleButton IN BStat) THEN
			DoubleClickCount := DCThresh;
		   END;

		   (* checks if second click is within threshold *)
		   IF (DoubleClickCount < DCThresh) OR (MiddleButton IN BStat) THEN
			DoubleClick := TRUE;
		   END;

		   DoubleClickCount := 0;
		   ButPress := TRUE;	(* button has been pressed *)

		ELSE

		   Drag := TRUE;	(* button is held down *)

		END; (*IF buttonhold*)

		PreButPress := TRUE;

		RespondToMouseButtons(current_x,current_y,LeftBut,RightBut,DoubleClick,Drag);
		Called := TRUE;
		DoubleClick := FALSE;

	   ELSE

		(********************)
		(* resets variables *)
		(********************)
		ButPress := FALSE;
		Drag := FALSE;

		IF PreButPress THEN
		   RespondToMouseButtons(current_x,current_y,LeftBut,RightBut,DoubleClick,Drag);
		   Called := TRUE;
		END;
		PreButPress := FALSE;

	   END;

	   (****************************************************)
	   (* compares current position with previous position *)
	   (****************************************************)
	   IF (current_x <> x) OR (current_y <> y) THEN
		UnconditionalObtainScreen();

		HideMouse();	(* restores screen area previously *)
					(* hidden by the mouse cursor      *)
		
		Obtain(temp_lock);
		x := current_x;
		y := current_y;
		Release (temp_lock);
		
		DisplayMouse();  (* displays mouse cursor at new position *)
		FastReleaseScreen();

		DoubleClickCount := DCThresh;
		IF NOT(Called) THEN
		   RespondToMouseButtons(current_x,current_y,LeftBut,RightBut,DoubleClick,Drag);
		END; (* IF *)
	   END; (* IF *)

	END; (* IF *)
   END; (* LOOP *)

END MouseInterfaceTask;


(******************************************************************)
(* This procedure initialises the mouse.					*)
(******************************************************************)
PROCEDURE InitialiseMouse();
BEGIN
  ALLOCATE(addr,VAL(CARDINAL,ImageSize(0,0,XMax,YMax)));

  CreateLock(temp_lock, "Temp Lock");
  CreateLock(cursor_lock, "Cursor Lock");

  CreateTask(MouseInterfaceTask, 1, "MouseInt Task");

  MouseEnabled := FALSE;
END InitialiseMouse;


END MouseRoutines.