IMPLEMENTATION MODULE PcxToRaw;

(******************************************************************)
(*											*)
(*			  PCX To RAW Conversion Module			*)
(*											*)
(*	Programmer :	Matthew Ma (c9315720)				*)
(*											*)
(*	Last edited :	22-10-97						*)
(*											*)
(*	Function : 		This module converts a PCX image to an	*)
(*				un-encoded RAW image.				*)
(*											*)
(*	Status :		Working.						*)
(******************************************************************)


FROM InfoDialogs IMPORT
(* proc *)   CantOpenFile, NotPcxFile, Not8Bit, InvalidSubtype,
             NoPalette, CantCreateFile;

FROM ColourRoutines IMPORT
(* proc *)   SetPaletteValue, SetVgaValue;

FROM Lib IMPORT
(* proc *)   IncAddr;

FROM XMS IMPORT
(* proc *)   ObtainXMSMemory, ReleaseXMSMemory;

FROM FIO IMPORT
(* type *)   File;

FROM FileSys IMPORT
(* proc *)   OpenFile, CloseFile, ReadByte, WriteByte,
             SetPosition, SavePosition, FileSize, ReadRecord;

FROM Graphics IMPORT
(* proc *)   SetPaletteColour, SetMode, PlotDot, PlotLine;

FROM GraphicWindow IMPORT
(* const *)   _GPSET, _GPRESET,
(* proc  *)   Plot, PutDot, PutLine, PutImage, ObtainScreen, ReleaseScreen,
              GetWindowLeft, GetWindowTop, GetDownOffset;

FROM IOErrorCodes IMPORT
(* type *)   ErrorCode;

FROM Support IMPORT
(* proc *)   ReadWord, ReadLongWord, AdjustSingleBytePalette, RoundOff;

FROM SoundEffects IMPORT
(* proc *)   Beep;


CONST
   block_size = 65534;


VAR
   file_size : LONGCARD;
   data, buffer : ARRAY [0..block_size-1] OF BYTE;
   x_size, y_size, x_offset, y_offset, version,
   bits_per_pixel, n_planes, bytes_per_line : CARDINAL;
   max_colours : LONGCARD;



(******************************************************************)
(* This procedure converts an 8-bit PCX image to an un-encoded	*)
(* RAW file.									*)
(******************************************************************)
PROCEDURE Pcx256ToRawData(pcx_file, raw_file : ARRAY OF CHAR) : CARDINAL;
VAR
   ok, repeater, first_byte : BOOLEAN;
   i, j, k, pixel : CARDINAL;
   in_file, out_file : File;
   total_bytes : LONGCARD;
   actual, dummy : CARDINAL;
   count, r, g, b, red, green, blue : SHORTCARD;
   block_count, remain, rows, image_counter, counter, x, y, x_min, y_min, x_max, y_max, duplicate : CARDINAL;
   error : ErrorCode;
   block : POINTER TO ARRAY [0..block_size] OF BYTE;
BEGIN

   IF (OpenFile(in_file, pcx_file, FALSE) <> OK) THEN
	Beep();
	CantOpenFile();
      RETURN (1);
   END;

   IF (OpenFile(out_file, raw_file, TRUE) <> OK) THEN
	Beep();
	CantCreateFile();
	RETURN (2);
   END;

   error := SetPosition(in_file,0);

   (* check if it's a valid PCX file *)
   IF (VAL(CARDINAL, ReadByte(in_file)) <> 10) THEN
      (* NOT a valid PCX file! *)
	Beep();
	NotPcxFile();
      RETURN (3);
   END;

   version := VAL(CARDINAL, ReadByte(in_file));
   IF version <> 5 THEN
      (* wrong version! *)
	Beep();
	InvalidSubtype();
      RETURN (4);
   END;

   error := SetPosition(in_file,3);
   bits_per_pixel := VAL(CARDINAL, ReadByte(in_file));
   IF bits_per_pixel <> 8 THEN
	Beep();
	Not8Bit();
	RETURN (5);
   END;

   CASE bits_per_pixel OF
         4:
		(*max_colours := 16;*)
        |8:
		max_colours := 256;
   END;

   error := SetPosition(in_file,65);
   n_planes := VAL(CARDINAL, ReadByte(in_file));

   IF ((bits_per_pixel = 3) AND (n_planes = 8)) THEN
      (* 24-bit not supported *)
	Beep();
	Not8Bit();
      RETURN (6);
   END;

   IF ((bits_per_pixel = 1) AND (n_planes = 4)) THEN
      (* 4-bit not supported *)
	Beep();
	Not8Bit();
      RETURN (7);
   END;

   IF ((bits_per_pixel = 1) AND (n_planes = 1)) THEN
      (* 2-bit not supported *)
	Beep();
	Not8Bit();
      RETURN (8);
   END;

   error := SetPosition(in_file,4);
   x_min := ReadWord(in_file);
   y_min := ReadWord(in_file);
   x_max := ReadWord(in_file);
   y_max := ReadWord(in_file);

   x_size := x_max - x_min + 1;
   y_size := y_max - y_min + 1;

   error := SetPosition(in_file,66);
   bytes_per_line := ReadWord(in_file);

   file_size := FileSize(in_file);


   x := 0;
   y := 0;

   error := SetPosition(in_file, 128);

   rows := block_size DIV CARDINAL(x_size+1);
   total_bytes := (file_size-770)-128+1;
   block_count := 0;

   
   repeater := FALSE;
   remain := 0;

   block := ObtainXMSMemory();

   (*ok := ObtainScreen();*)

   LOOP (* 1 *)
   	IF y > y_size-1 THEN
         EXIT;
   	END;

  	error := ReadRecord(in_file,block,block_size-1,actual);

   	INC(block_count);
   	i := 0;

   	IF actual>0 THEN
	   LOOP
         	pixel := VAL(CARDINAL,block^[i]);

      	IF (pixel = 12) AND (y > y_size-1) THEN EXIT END;

      	   IF repeater THEN
	 	   	FOR j := 0 TO duplicate-1 DO
	     		    error := WriteByte(out_file,BYTE(pixel));
	     		    INC(x);
	     		    IF x > x_size-1 THEN
				 IF ODD(x_size) THEN
		   		    INC(i);
			    	 END;
			    	 x := 0;
			    	 INC(y);
	     		    END;
         	   	END;
         	   	repeater := FALSE;
	 	      INC(i);
	 	   	IF i > actual-1 THEN
	    		   EXIT;
	 	   	END;
      	   ELSE
         	   	IF pixel >= 192 THEN
	    		   duplicate := pixel - 192;
	    		   INC(i);
	    		   IF i > actual-1 THEN
	       	   	repeater := TRUE;
	       	   	EXIT;
	    	   	   ELSE
	       		pixel := VAL(CARDINAL,block^[i]);
	    	   	   END;
         		ELSE
	    		   duplicate := 1;
         		END;

         		FOR j := 0 TO duplicate-1 DO
	     		    error := WriteByte(out_file,BYTE(pixel));
	     		    INC(x);
	     		    IF x > x_size-1 THEN
				 IF ODD(x_size) THEN
		   		 INC(i);
			    END;
			    x := 0;
			   INC(y);
	     		END;
         	   END;
	 	   INC(i);
	 	   IF i > actual-1 THEN
	    		EXIT;
	 	   END;
      	END;
	   END; (* LOOP 2 *)
      ELSE
         EXIT;
   	END;
   END; (* LOOP 1 *)

   (*ReleaseScreen();*)

   ReleaseXMSMemory();

   CloseFile(in_file);
   CloseFile(out_file);

   RETURN (0);

END Pcx256ToRawData;


END PcxToRaw.