IMPLEMENTATION MODULE PcxToTif;

(******************************************************************)
(*											*)
(*			  PCX To TIF Conversion Module			*)
(*											*)
(*	Programmer :	Matthew Ma (c9315720)				*)
(*											*)
(*	Last edited :	30-9-97						*)
(*											*)
(*	Function : 		This module converts a PCX image to a TIF	*)
(*				image.						*)
(*											*)
(*	Status :		Working.						*)
(******************************************************************)


FROM InfoDialogs IMPORT
(* proc *)   CantOpenFile, NotPcxFile, Not8Bit, InvalidSubtype,
             NoPalette, CantCreateFile;

FROM ColourRoutines IMPORT
(* proc *)   SetPaletteValue, SetVgaValue;

FROM Lib IMPORT
(* proc *)   IncAddr;

FROM XMS IMPORT
(* proc *)   ObtainXMSMemory, ReleaseXMSMemory;

FROM FIO IMPORT
(* type *)   File;

FROM FileSys IMPORT
(* proc *)   OpenFile, CloseFile, ReadByte, WriteByte,
             SetPosition, SavePosition, FileSize, ReadRecord;

FROM Graphics IMPORT
(* proc *)   SetPaletteColour, SetMode, PlotDot, PlotLine;

FROM GraphicWindow IMPORT
(* const *)   _GPSET, _GPRESET,
(* proc  *)   Plot, PutDot, PutLine, PutImage, ObtainScreen, ReleaseScreen,
              GetWindowLeft, GetWindowTop, GetDownOffset;

FROM IOErrorCodes IMPORT
(* type *)   ErrorCode;

FROM Support IMPORT
(* proc *)   ReadWord, ReadLongWord, WriteWord, WriteLongWord;

FROM SoundEffects IMPORT
(* proc *)   Beep;


CONST
   block_size = 65534;


VAR
   file_size : LONGCARD;
   data, buffer : ARRAY [0..block_size-1] OF BYTE;
   x_size, y_size, x_offset, y_offset, version,
   bits_per_pixel, n_planes, bytes_per_line : CARDINAL;
   max_colours : LONGCARD;



(******************************************************************)
(* This procedure converts an 8-bit PCX image to an 8-bit TIF	*)
(* image.										*)
(******************************************************************)
PROCEDURE Pcx256ToTif256(pcx_file, tif_file : ARRAY OF CHAR) : CARDINAL;
VAR
   ok, repeater, first_byte : BOOLEAN;
   i, j, k, pixel : CARDINAL;
   in_file, out_file : File;
   total_bytes : LONGCARD;
   count, actual, dummy : CARDINAL;
   r, g, b, red, green, blue : SHORTCARD;
   block_count, remain, rows, image_counter, counter, x, y, x_min, y_min, x_max, y_max, duplicate : CARDINAL;
   error : ErrorCode;
   block : POINTER TO ARRAY [0..block_size] OF BYTE;
BEGIN

   IF (OpenFile(in_file, pcx_file, FALSE) <> OK) THEN
	Beep();
	CantOpenFile();
      RETURN (1);
   END;

   IF (OpenFile(out_file, tif_file, TRUE) <> OK) THEN
	Beep();
	CantCreateFile();
	RETURN (2);
   END;

   error := SetPosition(in_file,0);

   (* check if it's a valid PCX file *)
   IF (VAL(CARDINAL, ReadByte(in_file)) <> 10) THEN
      (* NOT a valid PCX file! *)
	Beep();
	NotPcxFile();
      RETURN (3);
   END;

   version := VAL(CARDINAL, ReadByte(in_file));
   (*IF (VAL(CARDINAL, ReadByte(f)) <> 5) THEN*)
   IF version <> 5 THEN
      (* wrong version! *)
	Beep();
	InvalidSubtype();
      RETURN (4);
   END;

   error := SetPosition(in_file,3);
   bits_per_pixel := VAL(CARDINAL, ReadByte(in_file));
   IF bits_per_pixel <> 8 THEN
	Beep();
	Not8Bit();
	RETURN (5);
   END;

   CASE bits_per_pixel OF
         4:
		(*max_colours := 16;*)
        |8:
		max_colours := 256;
   END;

   error := SetPosition(in_file,65);
   n_planes := VAL(CARDINAL, ReadByte(in_file));

   (*bytes_per_line := VAL(CARDINAL, ReadByte(f));
   total_bytes := n_planes * bytes_per_line;*)

   IF ((bits_per_pixel = 3) AND (n_planes = 8)) THEN
      (* 24-bit not supported *)
	Beep();
	Not8Bit();
      RETURN (6);
   END;

   IF ((bits_per_pixel = 1) AND (n_planes = 4)) THEN
      (* 4-bit not supported *)
	Beep();
	Not8Bit();
      RETURN (7);
   END;

   IF ((bits_per_pixel = 1) AND (n_planes = 1)) THEN
      (* 2-bit not supported *)
	Beep();
	Not8Bit();
      RETURN (8);
   END;

   error := SetPosition(in_file,4);
   x_min := ReadWord(in_file);
   y_min := ReadWord(in_file);
   x_max := ReadWord(in_file);
   y_max := ReadWord(in_file);

   x_size := x_max - x_min + 1;
   y_size := y_max - y_min + 1;

   error := SetPosition(in_file,66);
   bytes_per_line := ReadWord(in_file);

   file_size := FileSize(in_file);


   (****************************)
   (* little-endian byte order *)
   (****************************)
   error := WriteByte(out_file, BYTE("I"));
   error := WriteByte(out_file, BYTE("I"));

   WriteWord(out_file, 42);
   WriteLongWord(out_file, 8);	(* first IFD *)

   WriteWord(out_file, 15-3);		(* IFD_entry_count *)

   WriteWord(out_file, 254);		(* NewSubfileType *)
   WriteWord(out_file, 4);
   WriteLongWord(out_file, 1);
   WriteLongWord(out_file, 0);

   WriteWord(out_file, 256);		(* image width *)
   WriteWord(out_file, 3);
   WriteLongWord(out_file, 1);
   WriteLongWord(out_file, LONGCARD(x_size));

   WriteWord(out_file, 257);		(* image height *)
   WriteWord(out_file, 3);
   WriteLongWord(out_file, 1);
   WriteLongWord(out_file, LONGCARD(y_size));

   WriteWord(out_file, 258);
   WriteWord(out_file, 3);
   WriteLongWord(out_file, 1);
   WriteLongWord(out_file, 8);	(* bits-per-pixel *)

   WriteWord(out_file, 259);
   WriteWord(out_file, 3);
   WriteLongWord(out_file, 1);
   WriteLongWord(out_file, 1);	(* non-compressed *)

   WriteWord(out_file, 262);
   WriteWord(out_file, 3);
   WriteLongWord(out_file, 1);
   WriteLongWord(out_file, 3);	(* palette colour *)

   WriteWord(out_file, 273);
   WriteWord(out_file, 4);
   WriteLongWord(out_file, 1);		(* StripsPerImage *)
   WriteLongWord(out_file, 1730-12*3);	(* strip offset *)

   WriteWord(out_file, 277);
   WriteWord(out_file, 3);
   WriteLongWord(out_file, 1);
   WriteLongWord(out_file, 1);	(* RGB *)

   WriteWord(out_file, 278);
   WriteWord(out_file, 3);
   WriteLongWord(out_file, 1);
   WriteLongWord(out_file, 8192);	(* RowsPerStrip *)

   WriteWord(out_file, 279);
   WriteWord(out_file, 4);
   WriteLongWord(out_file, 1);
   WriteLongWord(out_file, 1);	(* StripByteCounts *)

   WriteWord(out_file, 296);
   WriteWord(out_file, 3);
   WriteLongWord(out_file, 1);
   WriteLongWord(out_file, 1);	(* ResolutionUnit *)

   WriteWord(out_file, 320);
   WriteWord(out_file, 3);
   WriteLongWord(out_file, 768);
   WriteLongWord(out_file, 194-12*3);	(* ColorMap *)

   WriteLongWord(out_file, 0);	(* no more IFDs *)


   
   error := SetPosition(in_file, file_size-768-1);

   IF (VAL(CARDINAL, ReadByte(in_file)) <> 12) THEN
	Beep();
	NoPalette();
	RETURN (8);
   END;

   (*****************************)
   (* writes red colour entries *)
   (*****************************)
   FOR count := 0 TO 255 DO
	error := SetPosition(in_file,file_size-768+LONGCARD(3*count));
	red := ReadByte(in_file);

	error := WriteByte(out_file, red);
	error := WriteByte(out_file, red);
   END;

   (*******************************)
   (* writes green colour entries *)
   (*******************************)
   FOR count := 0 TO 255 DO
	error := SetPosition(in_file,file_size-768+1+LONGCARD(3*count));
	green := ReadByte(in_file);

	error := WriteByte(out_file, green);
	error := WriteByte(out_file, green);
   END;

   (******************************)
   (* writes blue colour entries *)
   (******************************)
   FOR count := 0 TO 255 DO
	error := SetPosition(in_file,file_size-768+2+LONGCARD(3*count));
	blue := ReadByte(in_file);

	error := WriteByte(out_file, blue);
	error := WriteByte(out_file, blue);
   END;


   x := 0;
   y := 0;

   error := SetPosition(in_file, 128);

   rows := block_size DIV CARDINAL(x_size+1);
   total_bytes := (file_size-770)-128+1;
   block_count := 0;

   
   repeater := FALSE;
   remain := 0;

   block := ObtainXMSMemory();

   LOOP (* 1 *)
   	IF y > y_size-1 THEN
         EXIT;
   	END;

   	error := ReadRecord(in_file,block,block_size-1,actual);

   	INC(block_count);
   	i := 0;

   	IF actual>0 THEN
   	   LOOP
      	pixel := VAL(CARDINAL,block^[i]);

      	   IF (pixel = 12) AND (y > y_size-1) THEN EXIT END;

      		IF repeater THEN
	 		   FOR j := 0 TO duplicate-1 DO
	     		   error := WriteByte(out_file,BYTE(pixel));
	     		   INC(x);
	     		   IF x > x_size-1 THEN
				IF ODD(x_size) THEN
		   		   INC(i);
			   	END;
			   	x := 0;
			   	INC(y);
	     		   END;
         	    	END;
         	   	repeater := FALSE;
	 	   	INC(i);
	 	   	IF i > actual-1 THEN
	    		   EXIT;
	 	   	END;
      	   ELSE
         		IF pixel >= 192 THEN
	    		   duplicate := pixel - 192;
	    		   INC(i);
	    		   IF i > actual-1 THEN
	       		repeater := TRUE;
	       		EXIT;
	    		   ELSE
	       	   	pixel := VAL(CARDINAL,block^[i]);
	    		   END;
         		ELSE
	    		   duplicate := 1;
         		END;

         		FOR j := 0 TO duplicate-1 DO
	     		    error := WriteByte(out_file,BYTE(pixel));
	     		    INC(x);
	     		    IF x > x_size-1 THEN
				 IF ODD(x_size) THEN
		   		 INC(i);
			    END;
			    x := 0;
			    INC(y);
	     		END;
         	   END;
	 	   INC(i);
	 	   IF i > actual-1 THEN
	    		EXIT;
	 	   END;
      	END;
   	   END; (* LOOP 2 *)
      ELSE
         EXIT;
   	END;
   END; (* LOOP 1 *)

   ReleaseXMSMemory();

   CloseFile(in_file);
   CloseFile(out_file);

RETURN (0);


END Pcx256ToTif256;


END PcxToTif.