IMPLEMENTATION MODULE PopUpWindows;

(******************************************************************)
(*											*)
(*			  	Pop-Up Windows Module				*)
(*											*)
(*	Programmer :	Matthew Ma (c9315720)				*)
(*											*)
(*	Last edited :	22-10-97						*)
(*											*)
(*	Function : 		This module contains pop-up windows used	*)
(*				for displaying the 240-colour palette and *)
(*				on-line help.					*)
(*											*)
(*	Status :		Working.						*)
(******************************************************************)


FROM Storage1 IMPORT
(* proc *)   ALLOCATE, DEALLOCATE;

FROM Bars IMPORT
(* proc *)   UpdateForegroundColour, UpdateBackgroundColour, UpdateTestColour,
		 ClearOptionsWindow, UpdateStatusBar;

FROM KbInterface IMPORT
(* const *)  NULL,
(* type  *)  KeyPressed, Keyboard,
(* proc  *)  SetupKeyboardLink, ReadFromKeyboard, ConnectToKeyboard, StuffKeyboard,
        	 DisconnectFromKeyboard, ClearKeyboard;

FROM CancelSequences IMPORT
(* type *)   CancelElement, CancelSource,
(* proc *)   RegisterCancelSequence, EnableCancelSequence, DisableCancelSequence;

FROM GraphicWindow IMPORT
(* type *)   GraphicalWindow, _GPSET,
(* var  *)   CharHeight, CharWidth, XMAX, YMAX, MouseParatemer_xoffset, MouseParatemer_yoffset,
(* proc *)   EnableScreen, UnconditionalObtainScreenWithMouseHide, ReleaseScreen, Rectangle,
        	 DisableScreen,DisplayWindowTitle, DisplayWindowData, DoNothing, ObtainScreen,
        	 CreateGraphicalWindow, PutString, HideWindowTitle, HideWindowData, PutRectangle,
        	 PutString, ClearDataRegion, GetWindowLeft, GetWindowTop,
	  	 PutTitleStrip, Line, ImageSize, GetImage, PutImage;

FROM TaskControl IMPORT
(* type *)   Lock,
(* proc *)   CreateTask, CreateLock, Obtain, Release;

FROM Semaphores IMPORT
(* type *)   Semaphore,
(* proc *)   CreateSemaphore, Wait, Signal;

FROM MouseRoutines IMPORT
(* proc *)   SetArrow, SetFinger;

FROM Conversions IMPORT
(* proc *)   ShortCardToString, CardinalToString, LongCardToString;

FROM ColourRoutines IMPORT
(* var  *)   ForegroundColour;

FROM Graph IMPORT
(* proc *)   Point;


TYPE
   PopupTYPE = (PopupHelp, s1, s2, PopupPalette);

   PageTYPE = (Front, Page1a, Page1b, Page1c, Page1d, Page1e,
		   Page1f, Page1g, Page1h, Page1i);

   PaletteTYPE = RECORD
		    		window : GraphicalWindow;
		    		idx : CARDINAL;
		     END;

CONST
   x_offset = 20;
   y_offset = 20;
   width = 17;
   height = 17;

VAR
   window : PopupTYPE;
   page : PageTYPE;
   help, FullPalette : GraphicalWindow;

   help_lock, page_lock : Lock;
   cancel_element:CancelElement;
   PopupSemaphore:Semaphore;

   pressed : KeyPressed;
   keyboard : Keyboard;

   col, row, idx, left, top, temp_card : CARDINAL;
   temp_string : ARRAY[0..9] OF CHAR;
   addr : ADDRESS;



PROCEDURE Page1A();
BEGIN
   Obtain(help_lock);
   window := s1;
   Release(help_lock);

   Obtain(page_lock);
   page := Page1a;
   Release(page_lock);

   DisplayWindowTitle(help);
   DisplayWindowData(help);

   PutTitleStrip(help,"Help");

   PutString(help,"Page1a",x_offset-15,1*CharHeight+y_offset,8,FALSE);

   Signal(PopupSemaphore);
   Wait(PopupSemaphore);
END Page1A;


(******************************************************************)
(* This is a cancel routine used as a parameter for the		*)		
(* RegisterCancelSequence routine.						*)
(******************************************************************)
PROCEDURE CancelRoutine(source:CancelSource);
BEGIN
  StuffKeyboard(keyboard,pressed);
END CancelRoutine;


(******************************************************************)
(* This procedure hides the pop-up windows when either the left	*)
(* or right mouse button is clicked inside the window.		*)
(******************************************************************)
PROCEDURE HidePopupTask();
VAR
   ok : BOOLEAN;
   temp_window : PopupTYPE;
   temp_pressed : KeyPressed;
BEGIN
   LOOP
     	Wait(PopupSemaphore);

    	ConnectToKeyboard(keyboard);
    	EnableCancelSequence(cancel_element);
    	ClearKeyboard(keyboard);
    	ReadFromKeyboard(keyboard, temp_pressed);
    	DisconnectFromKeyboard(keyboard);

    	Obtain(help_lock);
    	temp_window := window;
    	Release(help_lock);

    	DisableCancelSequence(cancel_element);
    	UnconditionalObtainScreenWithMouseHide();

    	CASE (temp_window) OF
        	  PopupHelp:

			(****************************************)
   			(* hides the on-line help pop-up window *)
   			(****************************************)
        		HideWindowData(help);

    		| PopupPalette:

			(******************************************)
   			(* hides the colour palette pop-up window *)
   			(******************************************)
        		HideWindowData(FullPalette);

    		| s1:
			
        		HideWindowData(help);

    		ELSE
    	END; (* CASE *)

    	ReleaseScreen();
    	EnableScreen();
    	Signal(PopupSemaphore);

    	ok := ObtainScreen();

 	(************************************)
   	(* restores the options window with *)
	(* previously saved contents		*)
   	(************************************)
	PutImage(10,431,addr,_GPSET);

    	ReleaseScreen();

   END; (*LOOP*)
END HidePopupTask;


(******************************************************************)
(* This procedure keeps track of the mouse position inside the	*)
(* on-line help pop-up window as well as carrying out routines	*)
(* corresponding to different mouse clicks.				*)
(******************************************************************)
PROCEDURE IndexResponse(w:GraphicalWindow; dx,dy:INTEGER;
          	       LeftPressed,RightPressed,DoubleClicked,Drag:BOOLEAN);
VAR
   temp_int : INTEGER;
   row1, row2 : CARDINAL;
BEGIN
   CASE page OF
	  Front:
		IF (MouseParatemer_yoffset >= y_offset+3*CharHeight) AND
		   (MouseParatemer_yoffset <= y_offset+20*CharHeight) THEN
		   row := (MouseParatemer_yoffset+CharHeight) DIV CharHeight;
		   IF (ODD(row)) THEN
			SetFinger();
		   END;
		   IF (LeftPressed) AND (MouseParatemer_yoffset >= y_offset+3*CharHeight) AND
		   (MouseParatemer_yoffset <= y_offset+20*CharHeight) AND ODD(row) THEN
		   (*temp_int := MouseParatemer_yoffset+CharHeight;
      	   row2 := (temp_int) DIV CharHeight;
		   IF (row2)=1 THEN*)

			(*Page1A();*)

		   (*END;*)
   		END;
   		END;
	| Page1a:
		SetArrow();
	| Page1b:
	| Page1c:
	| Page1d:
	| Page1e:
	| Page1f:
	| Page1g:
	| Page1h:
	| Page1i:
	ELSE
   END;

END IndexResponse;
   

(******************************************************************)
(* This procedure brings up the on-line help pop-up window.		*)
(******************************************************************)
PROCEDURE Index();
BEGIN
   Obtain(help_lock);
   GetImage(10,431,77,451,addr);
   window := PopupHelp;
   Release(help_lock);

   Obtain(page_lock);
   page := Front;
   Release(page_lock);

   DisableScreen();
   UnconditionalObtainScreenWithMouseHide();
   DisplayWindowTitle(help);
   DisplayWindowData(help);

   PutTitleStrip(help,"Help");

   PutString(help,"How To...",x_offset-15,1*CharHeight+y_offset,8,FALSE);
   PutString(help,"Use drawing tools",x_offset,3*CharHeight+y_offset,8,FALSE);
   PutString(help,"Use image routines",x_offset,5*CharHeight+y_offset,8,FALSE);
   PutString(help,"Use colour routines",x_offset,7*CharHeight+y_offset,8,FALSE);
   PutString(help,"Access full colour palette",x_offset,9*CharHeight+y_offset,8,FALSE);
   PutString(help,"Convert a file format",x_offset,11*CharHeight+y_offset,8,FALSE);
   PutString(help,"Set screen modes",x_offset,13*CharHeight+y_offset,8,FALSE);
   PutString(help,"Capture the screen",x_offset,15*CharHeight+y_offset,8,FALSE);
   PutString(help,"Magnify an image",x_offset,17*CharHeight+y_offset,8,FALSE);
   PutString(help,"Use other program features",x_offset,19*CharHeight+y_offset,8,FALSE);

   (*PutString(help,"About PaintMagic...",x_offset-15,15*CharHeight+8*(CharHeight DIV 2)+y_offset-10,8,FALSE);*)
   
   ReleaseScreen();

   Signal(PopupSemaphore);
   Wait(PopupSemaphore);
END Index;


(******************************************************************)
(* This procedure brings up the 240-colour palette pop-up window.	*)
(******************************************************************)
PROCEDURE Palette256();
VAR
   i,j,n, x_left, y_top : CARDINAL;
BEGIN

   (********************************************)
   (* saves the contents of the options window *)
   (********************************************)
   Obtain(help_lock);
   GetImage(10,431,77,451,addr);
   window := PopupPalette;
   Release(help_lock);

   left := GetWindowLeft(FullPalette);
   top := GetWindowTop(FullPalette);

   x_left := (*GetWindowLeft(FullPalette)+*)1;
   y_top := (*GetWindowTop(FullPalette)+*)1+20;

   (**********************************)
   (* displays message on status bar *)
   (**********************************)
   UpdateStatusBar("Left click to pick a colour or right click to cancel", 3, 0, 8);

   DisableScreen();
   UnconditionalObtainScreenWithMouseHide();

   DisplayWindowData(FullPalette);

   PutTitleStrip(FullPalette,"Colour Palette");

   (****************************************)
   (* draws a 15-row x 16-column maxtrix   *)
   (* of 20x20-pixel small squares showing *)
   (* the accessible 240 colours		 *)
   (****************************************)
   FOR j := 0 TO 15-1 DO
       FOR i := 0 TO 15 DO
	     PutRectangle(FullPalette,x_left+i*x_offset,y_top+j*y_offset,x_left+i*x_offset+width,y_top+j*y_offset+height,i+j*16+16,TRUE);
       END;
   END;

   ReleaseScreen();

   Signal(PopupSemaphore);
   Wait(PopupSemaphore);
END Palette256;


(******************************************************************)
(* This procedure selects a colour from the 240-colour palette.	*)
(******************************************************************)
PROCEDURE SelectColour(win : GraphicalWindow; dx,dy : INTEGER;
          	           LeftPressed,RightPressed,DoubleClicked,Drag : BOOLEAN);
VAR
   temp_card : CARDINAL;
   ok : BOOLEAN;
BEGIN

   (****************************************)
   (* show arrow cursor title strip region *)
   (****************************************)
   IF (MouseParatemer_yoffset < 20) THEN
      SetArrow();
   END;

   (*******************************************)
   (* calculates the column and row number by *)
   (* dividing the x co-ordiantes and the	    *)
   (* y co-ordinates by 20			    *)
   (*******************************************)
   IF (MouseParatemer_yoffset >= 20) THEN
   	col := (MouseParatemer_xoffset) DIV 20;
   	row := (MouseParatemer_yoffset-20) DIV 20;
   END;

   (**************************************)
   (* calculates the actual colour index *)
   (**************************************)
   idx := col+row*16+16;

   (**************************************)
   (* updates the colour being probed on *)
   (* the option windows		     *)
   (**************************************)
   IF (idx >= 0+16 ) AND (idx <= 255) AND (MouseParatemer_yoffset >= 20) THEN
      Rectangle(10,431,77,451,idx,TRUE);
   ELSE
      Rectangle(10,431,77,451,7,TRUE);
   END;

   (**************************************************)
   (* updates the selected colour on the foreground  *)
   (* colour rectangle, sets the ForegroundColour    *)
   (* variable to the selected colour number and     *)
   (* restores options window with previous contents *)
   (**************************************************)
   IF (LeftPressed) AND (MouseParatemer_yoffset >= 20) THEN
      ForegroundColour := idx;
	UpdateForegroundColour(SHORTCARD(idx));

	ok := ObtainScreen();
      PutImage(10,431,addr,_GPSET);
	ReleaseScreen();
   END;

END SelectColour;


(****************************************************************************)
(* Initialise                                                               *)
(****************************************************************************)
(* this procedure is called to initialise this module.  Main reason for     *)
(*      having this is to have less computation done in getting the About   *)
(*      Window setup at the middle of the screen using temp_x and temp_y without*)
(*      adding two global variables by havinh them as local.                *)
(****************************************************************************)
PROCEDURE Initialise();
VAR
   temp_x, temp_y : INTEGER;
BEGIN

   temp_x := XMAX DIV 2;
   temp_y := YMAX DIV 2;

   (***********************************)
   (* Creates the on-line help window *)
   (***********************************)
   CreateGraphicalWindow(help,temp_x-200+25,temp_y-100-115,temp_x+200-25,temp_y+100+115,5+2,NIL,0,0,11,12,
                        TRUE,TRUE,FALSE,FALSE,TRUE,TRUE,DoNothing,IndexResponse,SetArrow);

   (*************************************)
   (* Creates the colour palette window *)
   (*************************************)
   CreateGraphicalWindow(FullPalette,temp_x-163,temp_y-163,temp_x+163,temp_y+163,5+2,NIL,0,0,11,12,
                         TRUE,TRUE,FALSE,FALSE,TRUE,TRUE,DoNothing,SelectColour,SetFinger);

   pressed[0] := "A";
   pressed[1] := NULL;

   SetupKeyboardLink(keyboard);

   CreateLock(help_lock, "Help Lock");

   CreateLock(page_lock, "Page Lock");

   CreateSemaphore(PopupSemaphore,0);

   CreateTask(HidePopupTask,1,"Hide Popup Task");

   RegisterCancelSequence(CancelRoutine,cancel_element);
END Initialise;


(*******************)
(* Initialisations *)
(*******************)
BEGIN
  ALLOCATE(addr,VAL(CARDINAL,ImageSize(0,0,67,20)));
  Initialise();
END PopUpWindows.