IMPLEMENTATION MODULE ReadBMP;

(******************************************************************)
(*											*)
(*			  	 BMP Decoding Module				*)
(*											*)
(*	Programmer :	Matthew Ma (c9315720)				*)
(*											*)
(*	Last edited :	22-10-97						*)
(*											*)
(*	Function : 		This module reads and plots a BMP image	*)
(*				from a file.					*)
(*											*)
(*	Status :		Working.						*)
(******************************************************************)


FROM InfoDialogs IMPORT
(* proc *)   CantOpenFile, NotBmpFile, Not8Bit, InvalidSubtype;

FROM ColourRoutines IMPORT
(* type *)   palette,
(* proc *)   SetPaletteValue, SetVgaValue;

FROM Support IMPORT
(* proc *)   ReadWord, ReadLongWord, AdjustSingleBytePalette, RoundOff;

FROM MATHLIB IMPORT
(* proc *)   Pow;

FROM Canvas IMPORT
(* var  *)   normal, zoom2x, zoom3x;

FROM Lib IMPORT
(* proc *)   IncAddr;

FROM XMS IMPORT
(* proc *)   ObtainXMSMemory, ReleaseXMSMemory;

FROM FIO IMPORT
(* type *)   File;

FROM FileSys IMPORT
(* proc *)   OpenFile, CloseFile, ReadByte, EOF,
             SetPosition, SavePosition, FileSize, ReadRecord;

FROM Graphics IMPORT
(* proc *)   SetPaletteColour;

FROM GraphicWindow IMPORT
(* var *)    PutDot, XMAX, YMAX, CharWidth, CharHeight,
(* proc *)   ObtainScreen, ReleaseScreen, PutRectangle, PutLine;

FROM IOErrorCodes IMPORT
(* type *)   ErrorCode;

FROM Str IMPORT
(* proc *)   Compare;

FROM SoundEffects IMPORT
(* proc *)   Beep;


CONST
   block_size = 65534;


VAR
   n_planes, bits_per_pixel, blanks : CARDINAL;
   max_colours, unique_colours, important_colours, compression : LONGCARD;
   temp_x, temp_y, x_size, y_size, file_size, bitmap_offset, header_offset : LONGCARD;



(******************************************************************)
(* This procedure displays an BMP image in normal ratio.		*)
(******************************************************************)
PROCEDURE NoZoomBMP(w : GraphicalWindow; file_name : ARRAY OF CHAR);
VAR
   ok : BOOLEAN;
   f : File;
   dummy : BYTE;
   error : ErrorCode;
   r, g, b, red, green, blue, count : SHORTCARD;
   rows, actual, x, y, pixel,block_count, i : CARDINAL;
   block : POINTER TO ARRAY [0..block_size-1] OF BYTE;
   x_size2 : LONGCARD;
BEGIN
   IF (OpenFile(f, file_name, FALSE) = OK) THEN

   	x_size2 := ((x_size + 3) DIV 4) * 4;
   	blanks := CARDINAL(x_size2 - x_size);

   	x := 0;
   	y := CARDINAL(y_size);


   	error := SetPosition(f,bitmap_offset);

   	rows := block_size DIV CARDINAL(x_size+LONGCARD(blanks));


   	block := ObtainXMSMemory();
   	block_count := 0;


	ok := ObtainScreen();

	LOOP (* 1 *)
   	   error := ReadRecord(f,block,rows*CARDINAL(x_size+LONGCARD(blanks)),actual);

   	   INC(block_count);
   	   i := 0;

   	   IF actual > 0 THEN
   		LOOP (* 2 *)
      	   pixel := VAL(CARDINAL,block^[i]);

      	   IF pixel > 239 THEN
	 		PutDot(w,x,y-1,pixel);
      	   ELSE
	 		PutDot(w,x,y-1,pixel+16);
      	   END;

      	   INC(x);

      	   IF LONGCARD(x) > x_size-1 THEN
	 		INC(i,blanks);
         		x := 0;
         		DEC(y);

	 		IF (y = 0) THEN
      	    	   EXIT; (* LOOP 2 *)
   	 		END; (* IF *)
      	   END;

      	   INC(i);
      	   IF i > actual-1 THEN
	 		EXIT; (* LOOP 2 *)
      	   END;
   		END; (* LOOP 2 *)
   	   END; (* IF *)

   	   IF (y = 0) THEN      
      	EXIT; (* LOOP 1 *)
   	   END; (* IF *)
	END; (* LOOP 1*)

	ReleaseScreen();

	ReleaseXMSMemory();
   END;

   CloseFile(f);

   normal := TRUE;
   zoom2x := FALSE;
   zoom3x := FALSE;

END NoZoomBMP;


(******************************************************************)
(* This procedure magnifies an BMP image 2 times.			*)
(******************************************************************)
PROCEDURE ZoomBMP2x(w : GraphicalWindow; file_name : ARRAY OF CHAR);
VAR
   f : File;
   ok : BOOLEAN;
   dummy : BYTE;
   error : ErrorCode;
   r, g, b, red, green, blue, count : SHORTCARD;
   rows, actual, x, y, pixel,block_count, i : CARDINAL;
   x_size2 : LONGCARD;
   block : POINTER TO ARRAY [0..block_size-1] OF BYTE;
BEGIN
   IF (OpenFile(f, file_name, FALSE) = OK) THEN

	x_size2 := ((x_size + 3) DIV 4) * 4;
   	blanks := CARDINAL(x_size2 - x_size);

   	x := 0;
   	y := CARDINAL(y_size*2);

   	error := SetPosition(f,bitmap_offset);

	rows := block_size DIV CARDINAL(x_size+LONGCARD(blanks));

   	block := ObtainXMSMemory();

	ok := ObtainScreen();

	LOOP (* 1 *)
   	   error := ReadRecord(f,block,rows*CARDINAL(x_size+LONGCARD(blanks)),actual);

   	   i := 0;

   	   IF actual > 0 THEN
   		LOOP (* 2 *)
      	   pixel := VAL(CARDINAL,block^[i]);

      	   IF pixel > 239 THEN
	 		PutLine(w,x,y-1,x,y-2,pixel);
	 		PutLine(w,x+1,y-1,x+1,y-2,pixel);
      	   ELSE
	 		PutLine(w,x,y-1,x,y-2,pixel+16);
	 		PutLine(w,x+1,y-1,x+1,y-2,pixel+16);
      	   END;

      	   INC(x,2);

      	   IF LONGCARD(x) > (x_size*2-1) THEN
	    		INC(i,blanks);
         		x := 0;
         		DEC(y,2);

	   		IF (y = 0) THEN
      	    	   EXIT; (* LOOP 2 *)
   	 		END; (* IF *)
      	   END;

      	   INC(i);

      	   IF i > actual-1 THEN
	 		EXIT; (* LOOP 2 *)
      	   END;
   		END; (* LOOP 2 *)
   	   END; (* IF *)

   	   IF (y = 0) THEN
      	EXIT; (* LOOP 1 *)
   	   END; (* IF *)
	END; (* LOOP 1*)

	ReleaseScreen();

	ReleaseXMSMemory();
   END;

   CloseFile(f);

   zoom2x := TRUE;
   normal := FALSE;
   zoom3x := FALSE;

END ZoomBMP2x;


(******************************************************************)
(* This procedure magnifies an BMP image 3 times.			*)
(******************************************************************)
PROCEDURE ZoomBMP3x(w : GraphicalWindow; file_name : ARRAY OF CHAR);
VAR
   f : File;
   ok : BOOLEAN;
   dummy : BYTE;
   error : ErrorCode;
   r, g, b, red, green, blue, count : SHORTCARD;
   rows, actual, x, y, pixel,block_count, i : CARDINAL;
   x_size2 : LONGCARD;
   block : POINTER TO ARRAY [0..block_size-1] OF BYTE;
BEGIN
   IF (OpenFile(f, file_name, FALSE) = OK) THEN

   	x_size2 := ((x_size + 3) DIV 4) * 4;
   	blanks := CARDINAL(x_size2 - x_size);

   	x := 0;
   	y := CARDINAL(y_size*3);

   	error := SetPosition(f,bitmap_offset);

	rows := block_size DIV CARDINAL(x_size+LONGCARD(blanks));

   	block := ObtainXMSMemory();

	ok := ObtainScreen();

	LOOP (* 1 *)
   	   error := ReadRecord(f,block,rows*CARDINAL(x_size+LONGCARD(blanks)),actual);

   	   i := 0;

   	   IF actual > 0 THEN
   		LOOP (* 2 *)
      	   pixel := VAL(CARDINAL,block^[i]);

      	   IF pixel > 239 THEN
	 		PutLine(w,x,y-1,x,y-3,pixel);
	 		PutLine(w,x+1,y-1,x+1,y-3,pixel);
	 		PutLine(w,x+2,y-1,x+2,y-3,pixel);
      	   ELSE
	 	   	PutLine(w,x,y-1,x,y-3,pixel+16);
	 		PutLine(w,x+1,y-1,x+1,y-3,pixel+16);
	 		PutLine(w,x+2,y-1,x+2,y-3,pixel+16);
      	   END;

      	   INC(x,3);

      	   IF LONGCARD(x) > (x_size*3-1) THEN
	    		INC(i,blanks);
         		x := 0;
         		DEC(y,3);

	   		IF (y = 0) THEN
      	    	   EXIT; (* LOOP 2 *)
   	 		END; (* IF *)
      	   END;

      	   INC(i);
      	   IF i > actual-1 THEN
	 		EXIT; (* LOOP 2 *)
      	   END;
   		END; (* LOOP 2 *)
   	   END; (* IF *)

   	   IF (y = 0) THEN
      	EXIT; (* LOOP 1 *)
   	   END; (* IF *)
	END; (* LOOP 1*)

      ReleaseScreen();

   	ReleaseXMSMemory();
   END;

   CloseFile(f);

   zoom3x := TRUE;
   normal := FALSE;
   zoom2x := FALSE;

END ZoomBMP3x;


(******************************************************************)
(* This procedure loads and plots a 256-colour BMP image on the 	*)
(* specified window.								*)
(******************************************************************)
PROCEDURE Load256BMP(w : GraphicalWindow; f : File);
VAR
   ok : BOOLEAN;
   dummy : BYTE;
   error : ErrorCode;
   r, g, b, red, green, blue, count : SHORTCARD;
   rows, actual, x, y, pixel,block_count, i : CARDINAL;
   block : POINTER TO ARRAY [0..block_size-1] OF BYTE;
   x_size2 : LONGCARD;
BEGIN
   error := SetPosition(f,46);
   unique_colours := ReadLongWord(f);
   IF unique_colours = 0 THEN
      unique_colours := max_colours;
   END;

   important_colours := ReadLongWord(f);

   error := SetPosition(f,14+header_offset);	(* jumps to start os palette *)

   (*********************************************************)
   (* sets up palettes by storing original 8-bit RGB values *)
   (* and adjusted 6-bit RGB values			    		*)
   (*********************************************************)
   FOR count := 0 TO 239 DO

	b := SHORTCARD(ReadByte(f));
	g := SHORTCARD(ReadByte(f));
	r := SHORTCARD(ReadByte(f));

	SetPaletteValue(CARDINAL(count+16), r, g, b);
	SetVgaValue(CARDINAL(count+16));

	   blue := AdjustSingleBytePalette(b);
	   green := AdjustSingleBytePalette(g);
	   red := AdjustSingleBytePalette(r);

	dummy := ReadByte(f);	(* reserved byte *)

	SetPaletteColour(count+16, red, green, blue);

   END;

   x_size2 := ((x_size + 3) DIV 4) * 4;
   blanks := CARDINAL(x_size2 - x_size);

   x := 0;
   y := CARDINAL(y_size);

   error := SetPosition(f,bitmap_offset);

   rows := block_size DIV CARDINAL(x_size+LONGCARD(blanks));

   block := ObtainXMSMemory();
   block_count := 0;

   ok := ObtainScreen();

   (*******************************)
   (* plots BMP image onto screen *)
   (*******************************)
   LOOP (* 1 *)
   	error := ReadRecord(f,block,rows*CARDINAL(x_size+LONGCARD(blanks)),actual);

   	INC(block_count);
   	i := 0;

   	IF actual > 0 THEN
   	   LOOP (* 2 *)
      	pixel := VAL(CARDINAL,block^[i]);

      	IF pixel > 239 THEN
	 	   PutDot(w,x,y-1,pixel);
      	ELSE
	 	   PutDot(w,x,y-1,pixel+16);
      	END;

      	INC(x);

      	IF LONGCARD(x) > x_size-1 THEN
	 	   INC(i,blanks);
         	   x := 0;
         	   DEC(y);

	 	   IF (y = 0) THEN
      	    	EXIT; (* LOOP 2 *)
   	 	   END; (* IF *)
      	END;

      	INC(i);

      	IF i > actual-1 THEN
	 	   EXIT; (* LOOP 2 *)
      	END;
   	   END; (* LOOP 2 *)
   	END; (* IF *)

   	IF (y = 0) THEN      
         EXIT; (* LOOP 1 *)
   	END; (* IF *)
   END; (* LOOP 1*)

   ReleaseScreen();

   ReleaseXMSMemory();

   normal := TRUE;
   zoom2x := FALSE;
   zoom3x := FALSE;

END Load256BMP;


(******************************************************************)
(* This procedure reads and decodes the BMP headers, determines 	*)
(* their types and calles the corresponding decoding routines.	*)
(******************************************************************)
PROCEDURE LoadBMP(w : GraphicalWindow; bmp_file : ARRAY OF CHAR) : CARDINAL;
VAR
   f : File;
   dummy : BYTE;
   error : ErrorCode;
   r, g, b, red, green, blue, count : SHORTCARD;
   x, y, pixel (*, n_planes, bits_per_pixel*) : CARDINAL;
   signature : ARRAY [0..1] OF CHAR;
BEGIN
   IF (OpenFile(f, bmp_file, FALSE) <> OK) THEN
	Beep();
	CantOpenFile();
      RETURN (1);
   END;

   error := SetPosition(f,0);

   (****************************)
   (* checks for BMP signature *)
   (****************************)
   signature[0] := VAL(CHAR,ReadByte(f));
   signature[1] := VAL(CHAR,ReadByte(f));
   IF Compare(signature, "BM") <> 0 THEN
	Beep();
      NotBmpFile();
	RETURN (2);
   END;

   file_size := FileSize(f);

   error := SetPosition(f,10);
   bitmap_offset := ReadLongWord(f);
   header_offset := ReadLongWord(f);

   x_size := ReadLongWord(f);
   y_size := ReadLongWord(f);

   n_planes := ReadWord(f); (* must be set to 1 *)
   bits_per_pixel := ReadWord(f);
   IF (n_planes > 1) OR (bits_per_pixel <> 8) THEN
	Beep();
	Not8Bit();
	RETURN (3);
   END;

   compression := ReadLongWord(f);
   IF compression <> 0 THEN (* compression check *)
	Beep();
	InvalidSubtype();
      RETURN (4);
   END;

   CASE bits_per_pixel OF
	 1:
	     (*max_colours := 2;*)
	     (*LoadMonoBMP(f);*)
		RETURN (5);
      |  4:
	     (*max_colours := 16;*)
	     (*Load16BMP(f);*)
		RETURN (6);
      |  8:
	     	max_colours := 256;
		Load256BMP(w,f);
      | 24:
		Beep();
		Not8Bit();
	     	RETURN(7);
      ELSE
   END;

   CloseFile(f);

   RETURN (0);

END LoadBMP;


(******************************************************************)
(* This function returns the width of the image.			*)
(******************************************************************)
PROCEDURE GetWidth() : LONGCARD;
BEGIN
   RETURN x_size;
END GetWidth;


(******************************************************************)
(* This function returns the height of the image.			*)
(******************************************************************)
PROCEDURE GetHeight() : LONGCARD;
BEGIN
   RETURN y_size;
END GetHeight;


(******************************************************************)
(* This function returns the number of colour planes.			*)
(******************************************************************)
PROCEDURE GetColourPlanes() : CARDINAL;
BEGIN
   RETURN n_planes;
END GetColourPlanes;


(******************************************************************)
(* This function returns the bits per pixel.				*)
(******************************************************************)
PROCEDURE GetBitsPerPixel() : CARDINAL;
BEGIN
   RETURN bits_per_pixel;
END GetBitsPerPixel;


(******************************************************************)
(* This function returns the maximum number of colours.		*)
(******************************************************************)
PROCEDURE GetMaxColours() : LONGCARD;
BEGIN
   RETURN max_colours;
END GetMaxColours;


(******************************************************************)
(* This function returns the number of unique colours.		*)
(******************************************************************)
PROCEDURE GetColoursUsed() : LONGCARD;
BEGIN
   RETURN unique_colours;
END GetColoursUsed;


(******************************************************************)
(* This function returns the image sub-type.				*)
(******************************************************************)
PROCEDURE GetSubType() : LONGCARD;
BEGIN
   RETURN compression;
END GetSubType;


(******************************************************************)
(* This function returns the file size.					*)
(******************************************************************)
PROCEDURE GetFileSize() : LONGCARD;
BEGIN
   RETURN file_size;
END GetFileSize;


(******************************************************************)
(* This function returns the unpacked image size.			*)
(******************************************************************)
PROCEDURE GetUnpackedSize() : LONGCARD;
BEGIN
   RETURN (x_size)*(y_size)*LONGCARD(bits_per_pixel DIV 8);
END GetUnpackedSize;



END ReadBMP.