IMPLEMENTATION MODULE GIF;

(******************************************************************)
(*											*)
(*			  	 GIF Decoding Module				*)
(*											*)
(*	Programmer :	Matthew Ma (c9315720)				*)
(*											*)
(*	Last edited :	22-10-97						*)
(*											*)
(*	Function : 		This module reads and plots a GIF image	*)
(*				from a file.					*)
(*											*)
(*	Status :		Still working on it.				*)
(******************************************************************)


FROM Lib IMPORT
(* proc *)   IncAddr;

FROM XMS IMPORT
(* proc *)   ObtainXMSMemory, ReleaseXMSMemory;

FROM FIO IMPORT
(* type *)   File;

FROM FileSys IMPORT
(* proc *)   OpenFile, CloseFile, ReadByte,
             SetPosition, SavePosition, FileSize, ReadRecord;

FROM Graphics IMPORT
(* proc *)   SetPaletteColour, SetMode, PlotDot, PlotLine;

FROM GraphicWindow IMPORT
(* const *)   _GPSET, _GPRESET,
(* proc  *)   Plot, PutDot, PutLine, PutImage, GetWindowLeft, GetWindowTop, GetDownOffset;

FROM IOErrorCodes IMPORT
(* type *)   ErrorCode;

FROM Str IMPORT
(* proc *)   Compare;

FROM LowLevel IMPORT
(* proc *)   LSB;



ReadCode(size : CARDINAL) : LONGCARD;
VAR
   i, code : CARDINAL;
BEGIN
   code := 0;
   FOR i := 0 TO size-1 DO
       code := code + Pow(2,LONGREAL(i));
   END;
END ReadCode;


PROCEDURE LoadGIF87a(w : GraphicalWindow; gif_file : ARRAY OF CHAR) : CARDINAL;
VAR
   f : File;
   separator : CHAR;
   global_map : BOOLEAN;
   temp_byte, temp_byte2, dummy : BYTE;
   error : ErrorCode;
   r, g, b, red, green, blue, count : SHORTCARD;
   x_start, y_start, background, colour_resolution, x_size, y_size, i, x, y, pixel, n_planes, bits_per_pixel : CARDINAL;
   temp_x, temp_y, file_size, bitmap_offset, header_offset : LONGCARD;
   signature : ARRAY [0..5] OF CHAR;
BEGIN
   IF (OpenFile(f, gif_file, FALSE) <> OK) THEN
      RETURN (1);
   END;

   error := SetPosition(f,0);
   FOR i := 0 TO 5 DO
       signature[i] := VAL(CHAR,ReadByte(f));
   END;
   IF Compare(signature, "GIF87a") <> 0 THEN
      RETURN (2);
   END;

   x_size := ReadWord(f);
   y_size := ReadWord(f);

   temp_byte := ReadByte(f);
   
   IF VAL(CARDINAL,temp_byte) >= 128 THEN
      global_map := TRUE;
   END;

   (*IF temp_card-128 < 16 THEN
      bits_per_pixel := 1;
   ELSIF temp_card-128 >= 112
      bits_per_pixel := 8;
   ELSE
      bits_per_pixel := 4;
   END;*)

   temp_byte2 := temp_byte;
   IF VAL(CARDINAL,LSB(temp_byte2,1)) < 32 THEN
      colour_resolution := 1;
   ELSIF VAL(CARDINAL,LSB(temp_byte2,1)) > 224 THEN
      colour_resolution := 8;
   ELSE
      colour_resolution := 4;
   END;

   temp_byte2 := temp_byte;
   IF VAL(CARDINAL,LSB(temp_byte2,5)) < 32 THEN
      bits_per_pixel := 1;
   ELSIF VAL(CARDINAL,LSB(temp_byte2,5)) > 224 THEN
      bits_per_pixel := 8;
   ELSE
      bits_per_pixel := 4;
   END;

   background := VAL(CARDINAL, ReadByte(f));

   dummy := ReadByte(f);

   FOR count := 0 TO 239 DO
	r := SHORTCARD(ReadByte(f));
	g := SHORTCARD(ReadByte(f));
	b := SHORTCARD(ReadByte(f));
	red := ReScaleColour(r);
	green := ReScaleColour(g);
	blue := ReScaleColour(b);
	dummy := ReadByte(f);

	SetPaletteColour(count+16, red, green, blue);

   END;

   separator := VAL(CHAR, ReadByte(f));
   IF separator <> "," THEN
      RETURN (3);
   END;

   x_start := ReadWord(f);
   y_start := ReadWord(f);
   x_size := ReadWord(f);
   y_size := ReadWord(f);

   temp_byte := ReadByte(f);
   IF VAL(CARDINAL(temp_byte) >= 128) THEN
	local_map := TRUE;
   END;

   IF VAL(CARDINAL, LSB(temp_byte,1)) >= 128 THEN
      RETURN (4); (* interlaced!!! *)
   END;

   code_size := VAL(CARDINAL, ReadByte(f));
   clear_code := VAL(LONGCARD,Pow(2.0,8.0));
   EOI_code := clear_code + 1;
   compression_code := clear_code + 2;

   output_code := VAL(CRADINAL, ReadByte(f)) + 1;

   x := 0;
   y := 0;

LOOP (* 1 *)
   pixel := VAL(CARDINAL, ReadByte(f));

   IF pixel > 239 THEN
      PutDot(w,x,y,pixel);
   ELSE
      PutDot(w,x,y,pixel+16);
   END;

   INC(x);

   IF x > x_size-1 THEN
      IF ODD(x_size) THEN dummy := ReadByte(f); END;
      x := 0;
      DEC(y);

      IF (y = 0) THEN
	 LOOP
            pixel := VAL(CARDINAL, ReadByte(f));
            IF pixel > 239 THEN
	       PutDot(w,x,y,pixel);
            ELSE
               PutDot(w,x,y,pixel+16);
            END;
	    INC(x);
	    IF x > x_size-1 THEN
	       IF ODD(x_size) THEN dummy := ReadByte(f); END;
	       EXIT;
	    END; (* IF *)
         END; (* LOOP *)
         EXIT;
      END; (* IF *)
   END; (* IF *)
END; (* LOOP *)

CloseFile(f);

   RETURN (0);

END LoadGIF87a;

END GIF.