IMPLEMENTATION MODULE ReadPCX;

(******************************************************************)
(*											*)
(*			  	 PCX Decoding Module				*)
(*											*)
(*	Programmer :	Matthew Ma (c9315720)				*)
(*											*)
(*	Last edited :	22-10-97						*)
(*											*)
(*	Function : 		This module reads and plots a PCX image	*)
(*				from a file.					*)
(*											*)
(*	Status :		Working						*)
(******************************************************************)


FROM InfoDialogs IMPORT
(* proc *)   CantOpenFile, NotPcxFile, Not8Bit, InvalidSubtype, NoPalette;

FROM ColourRoutines IMPORT
(* proc *)   SetPaletteValue, SetVgaValue;

FROM Lib IMPORT
(* proc *)   IncAddr;

FROM XMS IMPORT
(* proc *)   ObtainXMSMemory, ReleaseXMSMemory;

FROM FIO IMPORT
(* type *)   File;

FROM FileSys IMPORT
(* proc *)   OpenFile, CloseFile, ReadByte,
             SetPosition, SavePosition, FileSize, ReadRecord;

FROM Graphics IMPORT
(* proc *)   SetPaletteColour, SetMode, PlotDot, PlotLine;

FROM GraphicWindow IMPORT
(* const *)   _GPSET, _GPRESET,
(* proc  *)   Plot, PutDot, PutLine, PutImage, ObtainScreen, ReleaseScreen,
              GetWindowLeft, GetWindowTop, GetDownOffset;

FROM IOErrorCodes IMPORT
(* type *)   ErrorCode;

FROM Support IMPORT
(* proc *)   ReadWord, ReadLongWord, AdjustSingleBytePalette, RoundOff;

FROM Canvas IMPORT
(* var  *)   normal, zoom2x, zoom3x;

FROM SoundEffects IMPORT
(* proc *)   Beep;


CONST
   block_size = 65534;


VAR
   file_size : LONGCARD;
   data, buffer : ARRAY [0..block_size-1] OF BYTE;
   x_size, y_size, x_offset, y_offset, version,
   bits_per_pixel, n_planes, bytes_per_line : CARDINAL;
   max_colours : LONGCARD;



(******************************************************************)
(* This procedure displays an PCX image in normal ratio.		*)
(******************************************************************)
PROCEDURE NoZoomPCX(w : GraphicalWindow; file_name : ARRAY OF CHAR);
VAR
   ok, repeater, first_byte : BOOLEAN;
   i, j, k, pixel : CARDINAL;
   f : File;
   total_bytes : LONGCARD;
   actual, dummy : CARDINAL;
   count, r, g, b, red, green, blue : SHORTCARD;
   block_count, remain, rows, image_counter, counter, x, y, x_min, y_min, x_max, y_max, duplicate : CARDINAL;
   error : ErrorCode;
   block : POINTER TO ARRAY [0..block_size] OF BYTE;
BEGIN

   IF (OpenFile(f, file_name, FALSE) = OK) THEN

   error := SetPosition(f,4);
   x_min := ReadWord(f);
   y_min := ReadWord(f);
   x_max := ReadWord(f);
   y_max := ReadWord(f);

   x_size := x_max - x_min + 1;
   y_size := y_max - y_min + 1;


   x := 0;
   y := 0;

   error := SetPosition(f, 128);

   rows := block_size DIV CARDINAL(x_size+1);
   total_bytes := (file_size-770)-128+1;
   block_count := 0;
   
   repeater := FALSE;
   remain := 0;

   block := ObtainXMSMemory();

   ok := ObtainScreen();

   (*******************************)
   (* plots PCX image onto screen *)
   (*******************************)
   LOOP (* 1 *)
   	IF y > y_size-1 THEN
         EXIT;
   	END;

   	error := ReadRecord(f,block,block_size-1,actual);

   	INC(block_count);
   	i := 0;

   	IF actual>0 THEN
   	   LOOP
      	pixel := VAL(CARDINAL,block^[i]);

      	IF (pixel = 12) AND (y > y_size-1) THEN EXIT END;	(* reached end   *)
											(* of image data *)
      	   IF repeater THEN
	 		(*FOR j := 0 TO duplicate-1 DO*)
			FOR j := 1 TO duplicate DO
	     		IF pixel > 239 THEN		(* last 16 colours *)
	        	   PutDot(w,x,y,pixel);
	     		ELSE
	        	   PutDot(w,x,y,pixel+16);	(* first 240 colours *)
	     		END;
	     		INC(x);
	     		IF x > x_size-1 THEN		(* reached end of a scan line *)
			   IF ODD(x_size) THEN
		   		INC(i);			(* skip padding byte *)
			   END;
			   x := 0;
			   INC(y);
	     		END;
         	   END;
         	   repeater := FALSE;
	 	   INC(i);
	 	   IF i > actual-1 THEN		(* reached end of array *)
	    		EXIT;
	 	   END;
      	ELSE
         	   IF pixel >= 192 THEN		(* a repeater *)
	    		duplicate := pixel - 192;
	    		INC(i);
	    		IF i > actual-1 THEN	
	       	   repeater := TRUE;	(* last byte in array is a repeater *)
	       	   EXIT;
	    		ELSE
	       	   pixel := VAL(CARDINAL,block^[i]);
	    		END;
         	   ELSE
	    		duplicate := 1;
         	   END;

         	   (*FOR j := 0 TO duplicate-1 DO*)
		   FOR j := 1 TO duplicate DO
	     		 IF pixel > 239 THEN		(* last 16 colours *)
	        	    PutDot(w,x,y,pixel);
	     		 ELSE
	        	    PutDot(w,x,y,pixel+16);	(* first 240 colours *)
	     		 END;
	     		 INC(x);
	     		 IF x > x_size-1 THEN		(* reached end of a scan line *)
			    IF ODD(x_size) THEN
		   	    	 INC(i);			(* skip padding byte *)
			    END;
			    x := 0;
			    INC(y);				(* next scan line *)
	     		 END;
         	   END;
	 	   INC(i);
	 	   IF i > actual-1 THEN
	    		EXIT;
	 	   END;
      	END;
   	   END; (* LOOP 2 *)
      ELSE
         EXIT;		(* reached end of array *)
   	END;
   END; (* LOOP 1 *)

   ReleaseScreen();

   ReleaseXMSMemory();

   CloseFile(f);

   normal := TRUE;
   zoom2x := FALSE;
   zoom3x := FALSE;

   END;

END NoZoomPCX;


(******************************************************************)
(* This procedure magnifies an PCX image 2 times.			*)
(******************************************************************)
PROCEDURE ZoomPCX2x(w : GraphicalWindow; file_name : ARRAY OF CHAR);
VAR
   ok, repeater, first_byte : BOOLEAN;
   i, j, k, pixel : CARDINAL;
   f : File;
   total_bytes : LONGCARD;
   actual, dummy : CARDINAL;
   count, r, g, b, red, green, blue : SHORTCARD;
   block_count, remain, rows, image_counter, counter,
   x, y, x_min, y_min, x_max, y_max, duplicate : CARDINAL;
   error : ErrorCode;
   block : POINTER TO ARRAY [0..block_size] OF BYTE;
BEGIN

   IF (OpenFile(f, file_name, FALSE) = OK) THEN

   error := SetPosition(f,4);
   x_min := ReadWord(f);
   y_min := ReadWord(f);
   x_max := ReadWord(f);
   y_max := ReadWord(f);

   x_size := x_max - x_min + 1;
   y_size := y_max - y_min + 1;


   x := 0;
   y := 0;

   error := SetPosition(f, 128);

   rows := block_size DIV CARDINAL(x_size+1);
   total_bytes := (file_size-770)-128+1;
   block_count := 0;
   
   repeater := FALSE;
   remain := 0;

   block := ObtainXMSMemory();

   ok := ObtainScreen();

   (*******************************)
   (* plots PCX image onto screen *)
   (*******************************)
   LOOP (* 1 *)
   	IF y > y_size*2-1 THEN
         EXIT;
   	END;

   	error := ReadRecord(f,block,block_size-1,actual);

   	INC(block_count);
   	i := 0;

   	IF actual>0 THEN
   	   LOOP
      	pixel := VAL(CARDINAL,block^[i]);

      	IF (pixel = 12) AND (y > y_size*2-1) THEN EXIT END;	(* reached end   *)
											(* of image data *)
      	   IF repeater THEN
	 		(*FOR j := 0 TO duplicate-1 DO*)
			FOR j := 1 TO duplicate DO
	     		IF pixel > 239 THEN			(* last 16 colours *)
	        	   PutLine(w,x,y,x,y+1,pixel);
	 		   PutLine(w,x+1,y,x+1,y+1,pixel);
	     		ELSE						(* first 240 colours *)
	        	   PutLine(w,x,y,x,y+1,pixel+16);
	 		   PutLine(w,x+1,y,x+1,y+1,pixel+16);	
	     		END;
	     		INC(x,2);
	     		IF x > x_size-1 THEN		(* reached end of a scan line *)
			   IF ODD(x_size) THEN
		   		INC(i);			(* skip padding byte *)
			   END;
			   x := 0;
			   INC(y,2);
	     		END;
         	   END;
         	   repeater := FALSE;
	 	   INC(i);
	 	   IF i > actual-1 THEN		(* reached end of array *)
	    		EXIT;
	 	   END;
      	ELSE
         	   IF pixel >= 192 THEN		(* a repeater *)
	    		duplicate := pixel - 192;
	    		INC(i);
	    		IF i > actual-1 THEN	
	       	   repeater := TRUE;	(* last byte in array is a repeater *)
	       	   EXIT;
	    		ELSE
	       	   pixel := VAL(CARDINAL,block^[i]);
	    		END;
         	   ELSE
	    		duplicate := 1;
         	   END;

         	   (*FOR j := 0 TO duplicate-1 DO*)
		   FOR j := 1 TO duplicate DO
	     		 IF pixel > 239 THEN		(* last 16 colours *)
	        	    PutLine(w,x,y,x,y+1,pixel);
	 		    PutLine(w,x+1,y,x+1,y+1,pixel);
	     		 ELSE					(* first 240 colours *)
	        	    PutLine(w,x,y,x,y+1,pixel+16);
	 		    PutLine(w,x+1,y,x+1,y+1,pixel+16);	
	     		 END;
	     		 INC(x,2);
	     		 IF x > x_size*2-1 THEN		(* reached end of a scan line *)
			    IF ODD(x_size) THEN
		   	    	 INC(i);			(* skip padding byte *)
			    END;
			    x := 0;
			    INC(y,2);			(* next scan line *)
	     		 END;
         	   END;
	 	   INC(i);
	 	   IF i > actual-1 THEN
	    		EXIT;
	 	   END;
      	END;
   	   END; (* LOOP 2 *)
      ELSE
         EXIT;		(* reached end of array *)
   	END;
   END; (* LOOP 1 *)

   ReleaseScreen();

   ReleaseXMSMemory();

   CloseFile(f);

   zoom2x := TRUE;
   normal := FALSE;
   zoom3x := FALSE;

   END;

END ZoomPCX2x;


(******************************************************************)
(* This procedure magnifies an PCX image 3 times.			*)
(******************************************************************)
PROCEDURE ZoomPCX3x(w : GraphicalWindow; file_name : ARRAY OF CHAR);
VAR
   ok, repeater, first_byte : BOOLEAN;
   i, j, k, pixel : CARDINAL;
   f : File;
   total_bytes : LONGCARD;
   actual, dummy : CARDINAL;
   count, r, g, b, red, green, blue : SHORTCARD;
   block_count, remain, rows, image_counter, counter,
   x, y, x_min, y_min, x_max, y_max, duplicate : CARDINAL;
   error : ErrorCode;
   block : POINTER TO ARRAY [0..block_size] OF BYTE;
BEGIN

   IF (OpenFile(f, file_name, FALSE) = OK) THEN

   error := SetPosition(f,4);
   x_min := ReadWord(f);
   y_min := ReadWord(f);
   x_max := ReadWord(f);
   y_max := ReadWord(f);

   x_size := x_max - x_min + 1;
   y_size := y_max - y_min + 1;


   x := 0;
   y := 0;

   error := SetPosition(f, 128);

   rows := block_size DIV CARDINAL(x_size+1);
   total_bytes := (file_size-770)-128+1;
   block_count := 0;
   
   repeater := FALSE;
   remain := 0;

   block := ObtainXMSMemory();

   ok := ObtainScreen();

   (*******************************)
   (* plots PCX image onto screen *)
   (*******************************)
   LOOP (* 1 *)
   	IF y > y_size*3-1 THEN
         EXIT;
   	END;

   	error := ReadRecord(f,block,block_size-1,actual);

   	INC(block_count);
   	i := 0;

   	IF actual>0 THEN
   	   LOOP
      	pixel := VAL(CARDINAL,block^[i]);

      	IF (pixel = 12) AND (y > y_size*3-1) THEN EXIT END;	(* reached end   *)
											(* of image data *)
      	   IF repeater THEN
	 		(*FOR j := 0 TO duplicate-1 DO*)
			FOR j := 1 TO duplicate DO
	     		IF pixel > 239 THEN			(* last 16 colours *)
	        	   PutLine(w,x,y,x,y+2,pixel);
	 		   PutLine(w,x+1,y,x+1,y+2,pixel);
	 		   PutLine(w,x+2,y,x+2,y+2,pixel);
	     		ELSE						(* first 240 colours *)
	        	   PutLine(w,x,y,x,y+2,pixel+16);
	 		   PutLine(w,x+1,y,x+1,y+2,pixel+16);
	 		   PutLine(w,x+2,y,x+2,y+2,pixel+16);	
	     		END;
	     		INC(x,3);
	     		IF x > x_size-1 THEN		(* reached end of a scan line *)
			   IF ODD(x_size) THEN
		   		INC(i);			(* skip padding byte *)
			   END;
			   x := 0;
			   INC(y,3);
	     		END;
         	   END;
         	   repeater := FALSE;
	 	   INC(i);
	 	   IF i > actual-1 THEN		(* reached end of array *)
	    		EXIT;
	 	   END;
      	ELSE
         	   IF pixel >= 192 THEN		(* a repeater *)
	    		duplicate := pixel - 192;
	    		INC(i);
	    		IF i > actual-1 THEN	
	       	   repeater := TRUE;	(* last byte in array is a repeater *)
	       	   EXIT;
	    		ELSE
	       	   pixel := VAL(CARDINAL,block^[i]);
	    		END;
         	   ELSE
	    		duplicate := 1;
         	   END;

         	   (*FOR j := 0 TO duplicate-1 DO*)
		   FOR j := 1 TO duplicate DO
	     		 IF pixel > 239 THEN		(* last 16 colours *)
	        	    PutLine(w,x,y,x,y+2,pixel);
	 		    PutLine(w,x+1,y,x+1,y+2,pixel);
	 		    PutLine(w,x+2,y,x+2,y+2,pixel);
	     		 ELSE					(* first 240 colours *)
	        	    PutLine(w,x,y,x,y+2,pixel+16);
	 		    PutLine(w,x+1,y,x+1,y+2,pixel+16);
	 		    PutLine(w,x+2,y,x+2,y+2,pixel+16);	
	     		 END;
	     		 INC(x,3);
	     		 IF x > x_size*3-1 THEN		(* reached end of a scan line *)
			    IF ODD(x_size) THEN
		   	    	 INC(i);			(* skip padding byte *)
			    END;
			    x := 0;
			    INC(y,3);				(* next scan line *)
	     		 END;
         	   END;
	 	   INC(i);
	 	   IF i > actual-1 THEN
	    		EXIT;
	 	   END;
      	END;
   	   END; (* LOOP 2 *)
      ELSE
         EXIT;		(* reached end of array *)
   	END;
   END; (* LOOP 1 *)

   ReleaseScreen();

   ReleaseXMSMemory();

   CloseFile(f);

   zoom3x := TRUE;
   normal := FALSE;
   zoom2x := FALSE;

   END;

END ZoomPCX3x;




(******************************************************************)
(* This procedure reads and decodes the PCX image file and plots	*)
(* image onto the specified window.						*)
(******************************************************************)
PROCEDURE LoadPCX(w : GraphicalWindow; pcx_file : ARRAY OF CHAR) : CARDINAL;
VAR
   ok, repeater, first_byte : BOOLEAN;
   i, j, k, pixel : CARDINAL;
   f : File;
   total_bytes : LONGCARD;
   actual, dummy : CARDINAL;
   count, r, g, b, red, green, blue : SHORTCARD;
   block_count, remain, rows, image_counter, counter, x, y, x_min, y_min, x_max, y_max, duplicate : CARDINAL;
   error : ErrorCode;
   block : POINTER TO ARRAY [0..block_size] OF BYTE;
BEGIN

   IF (OpenFile(f, pcx_file, FALSE) <> OK) THEN
	Beep();
	CantOpenFile();
      RETURN (1);
   END;

   error := SetPosition(f,0);

   (**********************************)
   (* check if it's a valid PCX file *)
   (**********************************)
   IF (VAL(CARDINAL, ReadByte(f)) <> 10) THEN
      (* NOT a valid PCX file! *)
	Beep();
	NotPcxFile();
      RETURN (2);
   END;

   version := VAL(CARDINAL, ReadByte(f));
   IF version <> 5 THEN
	Beep();
      InvalidSubtype();
      RETURN (3);
   END;

   error := SetPosition(f,3);
   bits_per_pixel := VAL(CARDINAL, ReadByte(f));
   IF bits_per_pixel <> 8 THEN
	Beep();
	Not8Bit();
	RETURN (4);
   END;

   CASE bits_per_pixel OF
         4:
		(*max_colours := 16;*)
        |8:
		max_colours := 256;
   END;

   error := SetPosition(f,65);
   n_planes := VAL(CARDINAL, ReadByte(f));

   IF ((bits_per_pixel = 3) AND (n_planes = 8)) THEN
	Beep();
      Not8Bit();
      RETURN (5);
   END;

   IF ((bits_per_pixel = 1) AND (n_planes = 4)) THEN
	Beep();
      Not8Bit();
      RETURN (6);
   END;

   IF ((bits_per_pixel = 1) AND (n_planes = 1)) THEN
	Beep();
      Not8Bit();
      RETURN (7);
   END;

   error := SetPosition(f,4);
   x_min := ReadWord(f);
   y_min := ReadWord(f);
   x_max := ReadWord(f);
   y_max := ReadWord(f);

   x_size := x_max - x_min + 1;
   y_size := y_max - y_min + 1;

   error := SetPosition(f,66);
   bytes_per_line := ReadWord(f);

   file_size := FileSize(f);

   
   error := SetPosition(f, file_size-768-1);


   (*****************************)
   (* checks for colour palette *)
   (*****************************)
   IF (VAL(CARDINAL, ReadByte(f)) <> 12) THEN
	Beep();
	NoPalette();
	RETURN (8);
   END;



   (*********************************************************)
   (* sets up palettes by storing original 8-bit RGB values *)
   (* and adjusted 6-bit RGB values			    		*)
   (*********************************************************)
   FOR count := 0 TO 239 DO
	r := SHORTCARD(ReadByte(f));
	g := SHORTCARD(ReadByte(f));
	b := SHORTCARD(ReadByte(f));

	SetPaletteValue(CARDINAL(count+16), r, g, b);
	SetVgaValue(CARDINAL(count+16));

	red := AdjustSingleBytePalette(r);
	green := AdjustSingleBytePalette(g);
	blue := AdjustSingleBytePalette(b);

	SetPaletteColour(count+16, red, green, blue);

   END;

   x := 0;
   y := 0;

   error := SetPosition(f, 128);

   rows := block_size DIV CARDINAL(x_size+1);
   total_bytes := (file_size-770)-128+1;
   block_count := 0;
   
   repeater := FALSE;
   remain := 0;

   block := ObtainXMSMemory();

   ok := ObtainScreen();

   (*******************************)
   (* plots PCX image onto screen *)
   (*******************************)
   LOOP (* 1 *)
   	IF y > y_size-1 THEN
         EXIT;
   	END;

   	error := ReadRecord(f,block,block_size-1,actual);

   	INC(block_count);
   	i := 0;

   	IF actual>0 THEN
   	   LOOP
      	pixel := VAL(CARDINAL,block^[i]);

      	IF (pixel = 12) AND (y > y_size-1) THEN EXIT END;	(* reached end   *)
											(* of image data *)
      	   IF repeater THEN
	 		(*FOR j := 0 TO duplicate-1 DO*)
			FOR j := 1 TO duplicate DO
	     		IF pixel > 239 THEN		(* last 16 colours *)
	        	   PutDot(w,x,y,pixel);
	     		ELSE
	        	   PutDot(w,x,y,pixel+16);	(* first 240 colours *)
	     		END;
	     		INC(x);
	     		IF x > x_size-1 THEN		(* reached end of a scan line *)
			   IF ODD(x_size) THEN
		   		INC(i);			(* skip padding byte *)
			   END;
			   x := 0;
			   INC(y);
	     		END;
         	   END;
         	   repeater := FALSE;
	 	   INC(i);
	 	   IF i > actual-1 THEN		(* reached end of array *)
	    		EXIT;
	 	   END;
      	ELSE
         	   IF pixel >= 192 THEN		(* a repeater *)
	    		duplicate := pixel - 192;
	    		INC(i);
	    		IF i > actual-1 THEN	
	       	   repeater := TRUE;	(* last byte in array is a repeater *)
	       	   EXIT;
	    		ELSE
	       	   pixel := VAL(CARDINAL,block^[i]);
	    		END;
         	   ELSE
	    		duplicate := 1;
         	   END;

         	   (*FOR j := 0 TO duplicate-1 DO*)
		   FOR j := 1 TO duplicate DO
	     		 IF pixel > 239 THEN		(* last 16 colours *)
	        	    PutDot(w,x,y,pixel);
	     		 ELSE
	        	    PutDot(w,x,y,pixel+16);	(* first 240 colours *)
	     		 END;
	     		 INC(x);
	     		 IF x > x_size-1 THEN		(* reached end of a scan line *)
			    IF ODD(x_size) THEN
		   	    	 INC(i);			(* skip padding byte *)
			    END;
			    x := 0;
			    INC(y);				(* next scan line *)
	     		 END;
         	   END;
	 	   INC(i);
	 	   IF i > actual-1 THEN
	    		EXIT;
	 	   END;
      	END;
   	   END; (* LOOP 2 *)
      ELSE
         EXIT;		(* reached end of array *)
   	END;
   END; (* LOOP 1 *)

   ReleaseScreen();

   ReleaseXMSMemory();

   CloseFile(f);

   normal := TRUE;
   zoom2x := FALSE;
   zoom3x := FALSE;

   RETURN (0);

END LoadPCX;


(******************************************************************)
(* This function returns the width of the image.			*)
(******************************************************************)
PROCEDURE GetWidth() : CARDINAL;
VAR
   f : File;
   error : ErrorCode;
BEGIN
      RETURN (x_size);
END GetWidth;


(******************************************************************)
(* This function returns the height of the image.			*)
(******************************************************************)
PROCEDURE GetHeight() : CARDINAL;
VAR
   f : File;
   error : ErrorCode;
BEGIN
      RETURN (y_size);
END GetHeight;


(******************************************************************)
(* This function returns the number of colour planes.			*)
(******************************************************************)
PROCEDURE GetColourPlanes() : CARDINAL;
VAR
   f : File;
   error : ErrorCode;
BEGIN
      RETURN n_planes;
END GetColourPlanes;


(******************************************************************)
(* This function returns the bits per pixel.				*)
(******************************************************************)
PROCEDURE GetBitsPerPixel() : CARDINAL;
VAR
   f : File;
   error : ErrorCode;
BEGIN
      RETURN bits_per_pixel;
END GetBitsPerPixel;


(******************************************************************)
(* This function returns the maximum number of colours.		*)
(******************************************************************)
PROCEDURE GetMaxColours() : LONGCARD;
VAR
   f : File;
   error : ErrorCode;
BEGIN
      RETURN max_colours;
END GetMaxColours;


(******************************************************************)
(* This function returns the number of unique colours.		*)
(******************************************************************)
PROCEDURE GetColoursUsed() : LONGCARD;
VAR
   f : File;
   error : ErrorCode;
BEGIN
      (*RETURN unique_colours;*)
   RETURN 256;
END GetColoursUsed;


(******************************************************************)
(* This function returns the image sub-type.				*)
(******************************************************************)
PROCEDURE GetSubType() : CARDINAL;
VAR
   f : File;
   error : ErrorCode;
BEGIN
      RETURN version;
END GetSubType;


(******************************************************************)
(* This function returns the file size.					*)
(******************************************************************)
PROCEDURE GetFileSize() : LONGCARD;
BEGIN
      RETURN file_size;
END GetFileSize;


(******************************************************************)
(* This function returns the unpacked image size.			*)
(******************************************************************)
PROCEDURE GetUnpackedSize() : LONGCARD;
VAR
   size : LONGCARD;
BEGIN
   RETURN LONGCARD(x_size)*LONGCARD(y_size)*LONGCARD(bits_per_pixel DIV 8);
END GetUnpackedSize;



END ReadPCX.