IMPLEMENTATION MODULE ReadTGA;

(******************************************************************)
(*											*)
(*			  	Targa Decoding Module				*)
(*											*)
(*	Programmer :	Matthew Ma (c9315720)				*)
(*											*)
(*	Last edited :	22-10-97						*)
(*											*)
(*	Function : 		This module reads and plots a TGA image	*)
(*				from a file.					*)
(*											*)
(*	Status :		Working.						*)
(******************************************************************)


FROM InfoDialogs IMPORT
(* proc *)   CantOpenFile, NotTgaFile, Not8Bit, InvalidSubtype;

FROM Support IMPORT
(* proc *)   ReadWord, ReadLongWord, AdjustSingleBytePalette, RoundOff;

FROM ColourRoutines IMPORT
(* proc *)   SetPaletteValue, SetVgaValue;

FROM Lib IMPORT
(* proc *)   IncAddr;

FROM XMS IMPORT
(* proc *)   ObtainXMSMemory, ReleaseXMSMemory;

FROM FIO IMPORT
(* type *)   File;

FROM FileSys IMPORT
(* proc *)   OpenFile, CloseFile, ReadByte,
             SetPosition, ReadRecord, FileSize;

FROM Graphics IMPORT
(* proc *)   SetPaletteColour;

FROM GraphicWindow IMPORT
(* var  *)   XMAX, YMAX, CharWidth, CharHeight,
(* proc *)   PutDot, PutLine, ObtainScreen, ReleaseScreen;

FROM IOErrorCodes IMPORT
(* type *)   ErrorCode;

FROM Canvas IMPORT
(* var  *)   normal, zoom2x, zoom3x;

FROM SoundEffects IMPORT
(* proc *)   Beep;


CONST
   block_size = 65534;


VAR
   file_size, max_colours : LONGCARD;
   x_size, y_size, x_start, y_start, map_entry_size,
   bits_per_pixel, image_type_code : CARDINAL;



(******************************************************************)
(* This procedure displays an TGA image in normal ratio.		*)
(******************************************************************)
PROCEDURE NoZoomTGA(w : GraphicalWindow; file_name : ARRAY OF CHAR);
VAR
   ok : BOOLEAN;
   dummy : BYTE;
   f : File;
   error : ErrorCode;
   r, g, b, red, green, blue, count : SHORTCARD;
   rows, actual, x, y, pixel,block_count, i : CARDINAL;
   block : POINTER TO ARRAY [0..block_size] OF BYTE;
BEGIN
   IF (OpenFile(f, file_name, FALSE) = OK) THEN

   x := 0;
   y := y_size;

   error := SetPosition(f,18+768);

   rows := block_size DIV x_size;

   block := ObtainXMSMemory();
   block_count := 0;

   ok := ObtainScreen();

   (*******************************)
   (* plots TGA image onto screen *)
   (*******************************)
   LOOP (* 1 *)
   	error := ReadRecord(f,block,x_size*rows,actual);
   
   	INC(block_count);
   	i := 0;

   	IF actual > 0 THEN
   	   LOOP (* 2 *)
      	pixel := VAL(CARDINAL,block^[i]);

      	IF pixel > 239 THEN		(* last 16 colours *)
         	   PutDot(w,x,y-1,pixel);
      	ELSE					(* first 240 colours *)
         	   PutDot(w,x,y-1,pixel+16);
      	END;

      	INC(x);

      	IF x > x_size-1 THEN		(* reached end of a scan line *)
         	   x := 0;
         	   DEC(y);

	 	   IF (y = 0) THEN		(* reached last scan line *)
      	    	EXIT; (* LOOP 2 *)
   	 	   END; (* IF *)
      	END;

      	INC(i);
      	IF i > actual-1 THEN		(* reached end of array *)
	 	   EXIT; (* LOOP 2 *)
      	END;
   	   END; (* LOOP 2 *)
   	END; (* IF *)

   	IF (y = 0) THEN			(* reached last sacn line *)
         EXIT; (* LOOP 1 *)		(* terminates plotting process *)
   	END; (* IF *)

   END; (* LOOP 1 *)

   ReleaseScreen();

   ReleaseXMSMemory();
   END;

   CloseFile(f);

   normal := TRUE;
   zoom2x := FALSE;
   zoom3x := FALSE;

END NoZoomTGA;


(******************************************************************)
(* This procedure magnifies an TGA image 2 times.			*)
(******************************************************************)
PROCEDURE ZoomTGA2x(w : GraphicalWindow; file_name : ARRAY OF CHAR);
VAR
   ok : BOOLEAN;
   dummy : BYTE;
   f : File;
   error : ErrorCode;
   r, g, b, red, green, blue, count : SHORTCARD;
   rows, actual, x, y, pixel,block_count, i : CARDINAL;
   block : POINTER TO ARRAY [0..block_size] OF BYTE;
BEGIN
   IF (OpenFile(f, file_name, FALSE) = OK) THEN

   x := 0;
   y := y_size*2;

   error := SetPosition(f,18+768);

   rows := block_size DIV x_size;

   block := ObtainXMSMemory();
   block_count := 0;

   ok := ObtainScreen();

   (*******************************)
   (* plots TGA image onto screen *)
   (*******************************)
   LOOP (* 1 *)
   	error := ReadRecord(f,block,x_size*rows,actual);
   
   	INC(block_count);
   	i := 0;

   	IF actual > 0 THEN
   	   LOOP (* 2 *)
      	pixel := VAL(CARDINAL,block^[i]);

      	IF pixel > 239 THEN		(* last 16 colours *)
         	   PutLine(w,x,y-1,x,y-2,pixel);
	 	   PutLine(w,x+1,y-1,x+1,y-2,pixel);
      	ELSE					(* first 240 colours *)
         	   PutLine(w,x,y-1,x,y-2,pixel+16);
	 	   PutLine(w,x+1,y-1,x+1,y-2,pixel+16);
      	END;

      	INC(x,2);

      	IF x > x_size*2-1 THEN		(* reached end of a scan line *)
         	   x := 0;
         	   DEC(y,2);

	 	   IF (y = 0) THEN		(* reached last scan line *)
      	    	EXIT; (* LOOP 2 *)
   	 	   END; (* IF *)
      	END;

      	INC(i);
      	IF i > actual-1 THEN		(* reached end of array *)
	 	   EXIT; (* LOOP 2 *)
      	END;
   	   END; (* LOOP 2 *)
   	END; (* IF *)

   	IF (y = 0) THEN			(* reached last sacn line *)
         EXIT; (* LOOP 1 *)		(* terminates plotting process *)
   	END; (* IF *)

   END; (* LOOP 1 *)

   ReleaseScreen();

   ReleaseXMSMemory();

   END;

   CloseFile(f);

   zoom2x := TRUE;
   normal := FALSE;
   zoom3x := FALSE;

END ZoomTGA2x;


(******************************************************************)
(* This procedure magnifies an TGA image 3 times.			*)
(******************************************************************)
PROCEDURE ZoomTGA3x(w : GraphicalWindow; file_name : ARRAY OF CHAR);
VAR
   ok : BOOLEAN;
   dummy : BYTE;
   f : File;
   error : ErrorCode;
   r, g, b, red, green, blue, count : SHORTCARD;
   rows, actual, x, y, pixel,block_count, i : CARDINAL;
   block : POINTER TO ARRAY [0..block_size] OF BYTE;
BEGIN
   IF (OpenFile(f, file_name, FALSE) = OK) THEN

   x := 0;
   y := y_size*3;

   error := SetPosition(f,18+768);

   rows := block_size DIV x_size;

   block := ObtainXMSMemory();
   block_count := 0;

   ok := ObtainScreen();

   (*******************************)
   (* plots TGA image onto screen *)
   (*******************************)
   LOOP (* 1 *)
   	error := ReadRecord(f,block,x_size*rows,actual);
   
   	INC(block_count);
   	i := 0;

   	IF actual > 0 THEN
   	   LOOP (* 2 *)
      	pixel := VAL(CARDINAL,block^[i]);

      	IF pixel > 239 THEN		(* last 16 colours *)
         	   PutLine(w,x,y-1,x,y-3,pixel);
	 	   PutLine(w,x+1,y-1,x+1,y-3,pixel);
	 	   PutLine(w,x+2,y-1,x+2,y-3,pixel);
      	ELSE					(* first 240 colours *)
         	   PutLine(w,x,y-1,x,y-3,pixel+16);
	 	   PutLine(w,x+1,y-1,x+1,y-3,pixel+16);
	 	   PutLine(w,x+2,y-1,x+2,y-3,pixel+16);
      	END;

      	INC(x,3);

      	IF x > x_size*3-1 THEN		(* reached end of a scan line *)
         	   x := 0;
         	   DEC(y,3);

	 	   IF (y = 0) THEN		(* reached last scan line *)
      	    	EXIT; (* LOOP 2 *)
   	 	   END; (* IF *)
      	END;

      	INC(i);
      	IF i > actual-1 THEN		(* reached end of array *)
	 	   EXIT; (* LOOP 2 *)
      	END;
   	   END; (* LOOP 2 *)
   	END; (* IF *)

   	IF (y = 0) THEN			(* reached last sacn line *)
         EXIT; (* LOOP 1 *)		(* terminates plotting process *)
   	END; (* IF *)

   END; (* LOOP 1 *)

   ReleaseScreen();

   ReleaseXMSMemory();

   END;

   CloseFile(f);

   zoom3x := TRUE;
   normal := FALSE;
   zoom2x := FALSE;

END ZoomTGA3x;



(******************************************************************)
(* This procedure loads and plots a 256-colour TGA image on the 	*)
(* specified window.								*)
(******************************************************************)
PROCEDURE Load256TGA(w : GraphicalWindow; f : File);
VAR
   ok : BOOLEAN;
   dummy : BYTE;
   error : ErrorCode;
   r, g, b, red, green, blue, count : SHORTCARD;
   rows, actual, x, y, pixel,block_count, i : CARDINAL;
   block : POINTER TO ARRAY [0..block_size] OF BYTE;
BEGIN
   error := SetPosition(f,18);

   (*********************************************************)
   (* sets up palettes by storing original 8-bit RGB values *)
   (* and adjusted 6-bit RGB values			    		*)
   (*********************************************************)
   FOR count := 0 TO 239 DO
	b := SHORTCARD(ReadByte(f));
	g := SHORTCARD(ReadByte(f));
	r := SHORTCARD(ReadByte(f));

	SetPaletteValue(CARDINAL(count+16), r, g, b);
	SetVgaValue(CARDINAL(count+16));

	red := AdjustSingleBytePalette(r);
	green := AdjustSingleBytePalette(g);
	blue := AdjustSingleBytePalette(b);

	IF map_entry_size = 32 THEN
	   dummy := ReadByte(f);
	END;

	(**************************)
	(* sets up colour palette *)
	(**************************)
	SetPaletteColour(count+16, red, green, blue);
   END;

   x := 0;
   y := y_size;

   error := SetPosition(f,18+768);

   rows := block_size DIV x_size;

   block := ObtainXMSMemory();
   block_count := 0;

   ok := ObtainScreen();

   (*******************************)
   (* plots TGA image onto screen *)
   (*******************************)
   LOOP (* 1 *)
   	error := ReadRecord(f,block,x_size*rows,actual);
   
   	INC(block_count);
   	i := 0;

   	IF actual > 0 THEN
   	   LOOP (* 2 *)
      	pixel := VAL(CARDINAL,block^[i]);

      	IF pixel > 239 THEN		(* last 16 colours *)
         	   PutDot(w,x,y-1,pixel);
      	ELSE					(* first 240 colours *)
         	   PutDot(w,x,y-1,pixel+16);
      	END;

      	INC(x);

      	IF x > x_size-1 THEN		(* reached end of a scan line *)
         	   x := 0;
         	   DEC(y);

	 	   IF (y = 0) THEN		(* reached last scan line *)
      	    	EXIT; (* LOOP 2 *)
   	 	   END; (* IF *)
      	END;

      	INC(i);
      	IF i > actual-1 THEN		(* reached end of array *)
	 	   EXIT; (* LOOP 2 *)
      	END;
   	   END; (* LOOP 2 *)
   	END; (* IF *)

   	IF (y = 0) THEN			(* reached last sacn line *)
         EXIT; (* LOOP 1 *)		(* terminates plotting process *)
   	END; (* IF *)

   END; (* LOOP 1 *)

   ReleaseScreen();

   ReleaseXMSMemory();

   normal := TRUE;
   zoom2x := FALSE;
   zoom3x := FALSE;

END Load256TGA;


(******************************************************************)
(* This procedure reads and decodes the TGA header, determines 	*)
(* its type and calles the corresponding decoding routines.		*)
(******************************************************************)
PROCEDURE LoadTGA(w : GraphicalWindow; tga_file : ARRAY OF CHAR) : CARDINAL;
VAR
   f : File;
   error : ErrorCode;
   colour_map_length, colour_map_origin,
   colour_map_type, ident_field : CARDINAL;
BEGIN
   IF (OpenFile(f, tga_file, FALSE) <> OK) THEN
	Beep();
	CantOpenFile();
      RETURN (1);
   END;

   file_size := FileSize(f);

   error := SetPosition(f,0);
   ident_field := VAL(CARDINAL, ReadByte(f));
   IF ident_field <> 0 THEN
	Beep();
	InvalidSubtype();
      RETURN (2);
   END;

   colour_map_type := VAL(CARDINAL, ReadByte(f));
   image_type_code := VAL(CARDINAL, ReadByte(f));
   IF (image_type_code <> 1) THEN
	Beep();
	InvalidSubtype();
	RETURN (3);
   END;

   colour_map_origin := ReadWord(f);
   colour_map_length := ReadWord(f);
   map_entry_size := VAL(CARDINAL, ReadByte(f));

   x_start := ReadWord(f);
   y_start := ReadWord(f);

   x_size := ReadWord(f);
   y_size := ReadWord(f);

   bits_per_pixel := VAL(CARDINAL, ReadByte(f));
   IF bits_per_pixel <> 8 THEN
	Beep();
	Not8Bit();		(* not an 256-colour image *)
	RETURN (4);
   END;

   IF bits_per_pixel = 8 THEN
      Load256TGA(w,f);
      max_colours := 256;
   END;

   CloseFile(f);

   RETURN (0);

END LoadTGA;


(******************************************************************)
(* This function returns the width of the image				*)
(******************************************************************)
PROCEDURE GetWidth() : CARDINAL;
BEGIN
   RETURN x_size;
END GetWidth;

(******************************************************************)
(* This function returns the height of the image			*)
(******************************************************************)
PROCEDURE GetHeight() : CARDINAL;
BEGIN
   RETURN y_size;
END GetHeight;

(******************************************************************)
(* This function returns the number of colour planes used for the	*)
(* image										*)
(******************************************************************)
PROCEDURE GetColourPlanes() : CARDINAL;
BEGIN
   RETURN 1;
END GetColourPlanes;

(******************************************************************)
(* This function returns the number of bits used to represent one	*)
(* pixel										*)
(******************************************************************)
PROCEDURE GetBitsPerPixel() : CARDINAL;
BEGIN
   RETURN bits_per_pixel;
END GetBitsPerPixel;

(******************************************************************)
(* This function returns the maximum number of colours the image	*)
(* can contain									*)
(******************************************************************)
PROCEDURE GetMaxColours() : LONGCARD;
VAR
   f : File;
   error : ErrorCode;
BEGIN
   RETURN max_colours;
END GetMaxColours;

(******************************************************************)
(* This function returns the number of unique colours used by the	*)
(* image										*)
(******************************************************************)
PROCEDURE GetColoursUsed() : LONGCARD;
BEGIN
   RETURN 256;
END GetColoursUsed;

(******************************************************************)
(* This function returns the sub-type of the image			*)
(******************************************************************)
PROCEDURE GetSubType() : CARDINAL;
BEGIN
   RETURN image_type_code;
END GetSubType;

(******************************************************************)
(* This function returns the image file size				*)
(******************************************************************)
PROCEDURE GetFileSize() : LONGCARD;
BEGIN
   RETURN file_size;
END GetFileSize;

(******************************************************************)
(* This function returns the unpacked image size			*)
(******************************************************************)
PROCEDURE GetUnpackedSize() : LONGCARD;
BEGIN
   RETURN LONGCARD(x_size)*LONGCARD(y_size)*LONGCARD(bits_per_pixel DIV 8);
END GetUnpackedSize;



END ReadTGA.