IMPLEMENTATION MODULE ReadTIF;

(******************************************************************)
(*											*)
(*			  	TIFF Decoding Module				*)
(*											*)
(*	Programmer :	Matthew Ma (c9315720)				*)
(*											*)
(*	Last edited :	22-10-97						*)
(*											*)
(*	Function : 		This module reads and plots a TIF image	*)
(*				from a file.					*)
(*											*)
(*	Status :		Working.						*)
(******************************************************************)


FROM InfoDialogs IMPORT
(* proc *)   CantOpenFile, NotTifFile, Not8Bit, InvalidSubtype,
             NoPalette, NotIntel, NotSingleStrip;

FROM ColourRoutines IMPORT
(* var *)    vga_pal,
(* proc *)   SetPaletteValue, SetVgaValue;

FROM Lib IMPORT
(* proc *)   IncAddr;

FROM XMS IMPORT
(* proc *)   ObtainXMSMemory, ReleaseXMSMemory;

FROM FIO IMPORT
(* type *)   File;

FROM FileSys IMPORT
(* proc *)   OpenFile, CloseFile, ReadByte, FileSize,
             SetPosition, ReadRecord, SavePosition;

FROM Graphics IMPORT
(* proc *)   SetPaletteColour;

FROM GraphicWindow IMPORT
(* var  *)   XMAX, YMAX, CharWidth, CharHeight,
(* proc *)   PutDot, PutLine, ObtainScreen, ReleaseScreen;

FROM IOErrorCodes IMPORT
(* type *)   ErrorCode;

FROM Support IMPORT
(* proc *)   ReadWord, ReadLongWord, AdjustSingleBytePalette, AdjustDoubleBytePalette;

FROM Str IMPORT
(* proc *)   Compare;

FROM Canvas IMPORT
(* var  *)   normal, zoom2x, zoom3x;

FROM SoundEffects IMPORT
(* proc *)   Beep;


CONST
   block_size = 65534;


VAR
   version : CARDINAL;
   width, height, bits_per_sample, compression, photometric,
   strip_offset, strips_per_image, samples_per_pixel, rows_per_strip,
   strip_byte_count, x_resolution, y_resolution, resolution_unit, colour_map,
   palette_offset, max_colours, file_size : LONGCARD;



(******************************************************************)
(* This procedure checks all of the TIFF 6.0 baseline fields and	*)
(* sets the corresponding variables.					*)
(******************************************************************)
PROCEDURE CheckField(f : File);
VAR
   error : ErrorCode;
   tag, type : CARDINAL;
   field_head, length, value_offset : LONGCARD;
BEGIN
   field_head := SavePosition(f);
   tag := ReadWord(f);
   CASE tag OF
	  315: (* Artist *)
		error := SetPosition(f,field_head+12);

	| 258: (* BitsPerSample *)
		error := SetPosition(f,field_head+8);
            bits_per_sample := ReadLongWord(f);
		IF bits_per_sample = 8 THEN
		   (* a 256 colour image *)
		   max_colours := 256;
		END;

	| 265: (* CellLength *)
		error := SetPosition(f,field_head+12);

	| 264: (* CellWidth *)
		error := SetPosition(f,field_head+12);

	| 320: (* ColorMap *)
		error := SetPosition(f,field_head+4);
            colour_map := ReadLongWord(f);
		IF colour_map <> 768 THEN
		   (* not 256 colour image *)
		   (*Not8Bit();
		   RETURN (2);*)
		END;
		error := SetPosition(f,field_head+8);
            palette_offset := ReadLongWord(f); (* start of palette *)

	| 259: (* Compression *)
		error := SetPosition(f,field_head+8);
            compression := ReadLongWord(f);
		IF compression <> 1 THEN
		   (* image is compressed *)
		   (*InvalidSubtype();
		   RETURN (3);*)
		END;

	| 33432: (* Copyright *)
		error := SetPosition(f,field_head+12);

	| 306: (* DateTime *)
		error := SetPosition(f,field_head+12);

	| 338: (* ExtraSamples *)
		error := SetPosition(f,field_head+12);

	| 266: (* FillOrder *)
		error := SetPosition(f,field_head+12);

	| 289: (* FreeByteCounts *)
		error := SetPosition(f,field_head+12);

	| 288: (* FreeOffsets *)
		error := SetPosition(f,field_head+12);

	| 291: (* GrayResponseCurve *)
		error := SetPosition(f,field_head+12);

	| 290: (* GrayResponseUnit *)
		error := SetPosition(f,field_head+12);

	| 316: (* HostComputer *)
		error := SetPosition(f,field_head+12);

	| 270: (* ImageDescription *)
		error := SetPosition(f,field_head+12);

	| 257: (* ImageLength *)
		error := SetPosition(f,field_head+8);
            height := ReadLongWord(f);

	| 256: (* ImageWidth *)
		error := SetPosition(f,field_head+8);
            width := ReadLongWord(f);

	| 271: (* Maker *)
		error := SetPosition(f,field_head+12);

	| 281: (* MaxSampleValue *)
		error := SetPosition(f,field_head+12);

	| 280: (* MinSampleValue *)
		error := SetPosition(f,field_head+12);

	| 272: (* Model *)
		error := SetPosition(f,field_head+12);

	| 254: (* NewSubfileType *)
		error := SetPosition(f,field_head+12);

	| 274: (* Orientation *)
		error := SetPosition(f,field_head+12);

	| 262: (* PhotometricInterpretation *)
		error := SetPosition(f,field_head+8);
            photometric := ReadLongWord(f);
		(*IF photometric = 3 THEN
		   (* palette colour image *)
		END;*)

	| 284: (* PlanarConfiguration *)
		error := SetPosition(f,field_head+12);

	| 296: (* ResolutionUnit *)
		error := SetPosition(f,field_head+8);
            resolution_unit := ReadLongWord(f);
		CASE CARDINAL(resolution_unit) OF
		       1: (* none *)
		     | 2: (* inch *)
		     | 3: (* cm *)
		END;

	| 278: (* RowsPerStrip *)
		error := SetPosition(f,field_head+8);
            rows_per_strip := ReadLongWord(f);

	| 277: (* SamplesPerPixel *)
		error := SetPosition(f,field_head+8);
            samples_per_pixel := ReadLongWord(f);
		(*IF samples_per_pixel = 1 THEN
		   (* palette colour image *)
		END;*)

	| 305: (* Software *)
		error := SetPosition(f,field_head+12);

	| 279: (* StripByteCounts *)
		error := SetPosition(f,field_head+8);
            strip_byte_count := ReadLongWord(f);

	| 273: (* StripOffsets *)
		error := SetPosition(f,field_head+4);
            strips_per_image := ReadLongWord(f); (* total number of strips *)
		(*IF strips_per_image > 1 THEN
		   (*NotSingleStrip();
		   RETURN (5);*)
		END;*)
		error := SetPosition(f,field_head+8);
            strip_offset := ReadLongWord(f); (* start of strips *)

	| 255: (* SubfileType *)
		error := SetPosition(f,field_head+12);

	| 263: (* Thresholding *)
		error := SetPosition(f,field_head+12);
	
	| 282: (* XResolution *)
		error := SetPosition(f,field_head+8);
            x_resolution := ReadLongWord(f);

	| 283: (* YResolution *)
		error := SetPosition(f,field_head+8);
            y_resolution := ReadLongWord(f);
	
	ELSE
		error := SetPosition(f,field_head+12);
   END;

END CheckField;


(******************************************************************)
(* This procedure displays an TIF image in normal ratio.		*)
(******************************************************************)
PROCEDURE NoZoomTIF(w : GraphicalWindow; file_name : ARRAY OF CHAR);
VAR
   f : File;
   dummy : BYTE;
   ok : BOOLEAN;
   error : ErrorCode;
   r, g, b, red, green, blue, count : SHORTCARD;
   colour_map_length, colour_map_origin,
   colour_map_type, image_type_code, ident_field,
   i, j, x, y, pixel, actual, IFD_entry_count, bits_per_pixel : CARDINAL;
   r1, g1, b1 : CARDINAL;
   byte_order : ARRAY [0..1] OF CHAR;
   block : POINTER TO ARRAY [0..block_size-1] OF BYTE;
   rows, IFD_offset, next_strip : LONGCARD;
BEGIN
   IF (OpenFile(f, file_name, FALSE) = OK) THEN

   x := 0;
   y := 0;

   error := SetPosition(f,strip_offset);

   rows := block_size DIV width;

   block := ObtainXMSMemory();

   ok := ObtainScreen();

   (*******************************)
   (* plots TIF image onto screen *)
   (*******************************)
   LOOP (* 1 *)
   	error := ReadRecord(f,block,CARDINAL(width*rows),actual);

   	i := 0;

   	IF actual > 0 THEN
   	   LOOP (* 2 *)
         	pixel := VAL(CARDINAL,block^[i]);

      	IF pixel > 239 THEN			(* last 16 colours *)
         	   PutDot(w,x,y,pixel);
      	ELSE						(* first 240 colours *)
         	   PutDot(w,x,y,pixel+16);
      	END;

      	INC(x);

      	IF LONGCARD(x) > width-1 THEN		(* reached end of a scan line *)
         	   x := 0;
         	   INC(y);					(* next scan line *)
      	END;

      	INC(i);
      	IF i > actual-1 THEN			(* reached end of array *)
	 	   EXIT; (* LOOP 2 *)
      	END;
   	   END; (* LOOP 2 *)
   	END; (* IF *)

   	IF (LONGCARD(y) > height-1) THEN		(* reached last scan line *)
         EXIT; (* LOOP 1 *)				(* terminates plotting process *)
   	END; (* IF *)

   END; (* LOOP 1 *)

   ReleaseScreen();

   ReleaseXMSMemory();

   CloseFile(f);

   normal := TRUE;
   zoom2x := FALSE;
   zoom3x := FALSE;

   END;

END NoZoomTIF;


(******************************************************************)
(* This procedure magnifies an TIF image 2 times.			*)
(******************************************************************)
PROCEDURE ZoomTIF2x(w : GraphicalWindow; file_name : ARRAY OF CHAR);
VAR
   f : File;
   dummy : BYTE;
   ok : BOOLEAN;
   error : ErrorCode;
   r, g, b, red, green, blue, count : SHORTCARD;
   colour_map_length, colour_map_origin,
   colour_map_type, image_type_code, ident_field,
   i, j, x, y, pixel, actual, IFD_entry_count, bits_per_pixel : CARDINAL;
   r1, g1, b1 : CARDINAL;
   byte_order : ARRAY [0..1] OF CHAR;
   block : POINTER TO ARRAY [0..block_size-1] OF BYTE;
   rows, IFD_offset, next_strip : LONGCARD;
BEGIN
   IF (OpenFile(f, file_name, FALSE) = OK) THEN

   x := 0;
   y := 0;

   error := SetPosition(f,strip_offset);

   rows := block_size DIV width;

   block := ObtainXMSMemory();

   ok := ObtainScreen();

   (*******************************)
   (* plots TIF image onto screen *)
   (*******************************)
   LOOP (* 1 *)
   	error := ReadRecord(f,block,CARDINAL(width*rows),actual);

   	i := 0;

   	IF actual > 0 THEN
   	   LOOP (* 2 *)
         	pixel := VAL(CARDINAL,block^[i]);

      	IF pixel > 239 THEN			(* last 16 colours *)
         	   PutLine(w,x,y,x,y+1,pixel);
	 	   PutLine(w,x+1,y,x+1,y+1,pixel);
      	ELSE						(* first 240 colours *)
         	   PutLine(w,x,y,x,y+1,pixel+16);
	 	   PutLine(w,x+1,y,x+1,y+1,pixel+16);
      	END;

      	INC(x,2);

      	IF LONGCARD(x) > width*2-1 THEN		(* reached end of a scan line *)
         	   x := 0;
         	   INC(y,2);					(* next scan line *)
      	END;

      	INC(i);
      	IF i > actual-1 THEN			(* reached end of array *)
	 	   EXIT; (* LOOP 2 *)
      	END;
   	   END; (* LOOP 2 *)
   	END; (* IF *)

   	IF (LONGCARD(y) > height*2-1) THEN		(* reached last scan line *)
         EXIT; (* LOOP 1 *)				(* terminates plotting process *)
   	END; (* IF *)

   END; (* LOOP 1 *)

   ReleaseScreen();

   ReleaseXMSMemory();

   CloseFile(f);

   zoom2x := TRUE;
   normal := FALSE;
   zoom3x := FALSE;

   END;

END ZoomTIF2x;


(******************************************************************)
(* This procedure magnifies an TIF image 3 times.			*)
(******************************************************************)
PROCEDURE ZoomTIF3x(w : GraphicalWindow; file_name : ARRAY OF CHAR);
VAR
   f : File;
   dummy : BYTE;
   ok : BOOLEAN;
   error : ErrorCode;
   r, g, b, red, green, blue, count : SHORTCARD;
   colour_map_length, colour_map_origin,
   colour_map_type, image_type_code, ident_field,
   i, j, x, y, pixel, actual, IFD_entry_count, bits_per_pixel : CARDINAL;
   r1, g1, b1 : CARDINAL;
   byte_order : ARRAY [0..1] OF CHAR;
   block : POINTER TO ARRAY [0..block_size-1] OF BYTE;
   rows, IFD_offset, next_strip : LONGCARD;
BEGIN
   IF (OpenFile(f, file_name, FALSE) = OK) THEN

   x := 0;
   y := 0;

   error := SetPosition(f,strip_offset);

   rows := block_size DIV width;

   block := ObtainXMSMemory();

   ok := ObtainScreen();

   (*******************************)
   (* plots TIF image onto screen *)
   (*******************************)
   LOOP (* 1 *)
   	error := ReadRecord(f,block,CARDINAL(width*rows),actual);

   	i := 0;

   	IF actual > 0 THEN
   	   LOOP (* 2 *)
         	pixel := VAL(CARDINAL,block^[i]);

      	IF pixel > 239 THEN			(* last 16 colours *)
         	   PutLine(w,x,y,x,y+2,pixel);
	 	   PutLine(w,x+1,y,x+1,y+2,pixel);
	 	   PutLine(w,x+2,y,x+2,y+2,pixel);
      	ELSE						(* first 240 colours *)
         	   PutLine(w,x,y,x,y+2,pixel+16);
	 	   PutLine(w,x+1,y,x+1,y+2,pixel+16);
	 	   PutLine(w,x+2,y,x+2,y+2,pixel+16);
      	END;

      	INC(x,3);

      	IF LONGCARD(x) > width*3-1 THEN		(* reached end of a scan line *)
         	   x := 0;
         	   INC(y,3);					(* next scan line *)
      	END;

      	INC(i);
      	IF i > actual-1 THEN			(* reached end of array *)
	 	   EXIT; (* LOOP 2 *)
      	END;
   	   END; (* LOOP 2 *)
   	END; (* IF *)

   	IF (LONGCARD(y) > height*3-1) THEN		(* reached last scan line *)
         EXIT; (* LOOP 1 *)				(* terminates plotting process *)
   	END; (* IF *)

   END; (* LOOP 1 *)

   ReleaseScreen();

   ReleaseXMSMemory();

   CloseFile(f);

   zoom3x := TRUE;
   normal := FALSE;
   zoom2x := FALSE;

   END;

END ZoomTIF3x;



(******************************************************************)
(* This procedure loads and plots a 256-colour TIF image on the 	*)
(* specified window.								*)
(******************************************************************)
PROCEDURE LoadTIF(w : GraphicalWindow; file_name : ARRAY OF CHAR) : CARDINAL;
VAR
   f : File;
   dummy : BYTE;
   ok : BOOLEAN;
   error : ErrorCode;
   r, g, b, red, green, blue, count : SHORTCARD;
   colour_map_length, colour_map_origin,
   colour_map_type, image_type_code, ident_field,
   i, j, x, y, pixel, actual, IFD_entry_count, bits_per_pixel : CARDINAL;
   r1, g1, b1 : CARDINAL;
   byte_order : ARRAY [0..1] OF CHAR;
   block : POINTER TO ARRAY [0..block_size-1] OF BYTE;
   rows, IFD_offset, next_strip : LONGCARD;
BEGIN
   IF (OpenFile(f, file_name, FALSE) <> OK) THEN
	Beep();
	CantOpenFile();
      RETURN (1);
   END;

   file_size := FileSize(f);

   error := SetPosition(f,0);
   byte_order[0] := VAL(CHAR, ReadByte(f));
   byte_order[1] := VAL(CHAR, ReadByte(f));

   
   IF Compare(byte_order,"II") <> 0 THEN
	Beep();
      NotIntel();		(* not Intel byte order *)
      RETURN (2);
   END;
   

   version := ReadWord(f);
   IF version <> 42 THEN
      (* not a .TIF file *)
	Beep();
	NotTifFile();	(* not a valid TIF file *)
	RETURN (3);
   END;

   IFD_offset := ReadLongWord(f);

   error := SetPosition(f,IFD_offset);

   IFD_entry_count := ReadWord(f);

   FOR i := 0 TO IFD_entry_count-1 DO
       CheckField(f);
   END;

   IF (bits_per_sample <> 8) OR (colour_map <> 768) OR (samples_per_pixel <> 1) THEN
	Beep();
	Not8Bit();			(* not a 256-colour image *)
	RETURN (4);
   ELSIF compression <> 1 THEN
	Beep();
	InvalidSubtype();		(* unsupported subtype *)
	RETURN (5);
   ELSIF photometric <> 3 THEN
	Beep();
	NoPalette();		(* no colour palette found *)
	RETURN (6);
   ELSIF strips_per_image > 1 THEN
	Beep();
	NotSingleStrip();		(* not a single-strip-per-image file *)
	RETURN (7);
   ELSE
   END;


   (**********************************************************)
   (* sets up palettes by storing original 16-bit RGB values *)
   (* and adjusted 6-bit RGB values		   	    		 *)
   (**********************************************************)
   FOR count := 0 TO 239 DO
	error := SetPosition(f,palette_offset+2*LONGCARD(count));
	r := SHORTCARD(ReadByte(f));

	error := SetPosition(f,palette_offset+256*2+2*LONGCARD(count));
	g := SHORTCARD(ReadByte(f));

	error := SetPosition(f,palette_offset+256*4+2*LONGCARD(count));
	b := SHORTCARD(ReadByte(f));

	SetPaletteValue(CARDINAL(count+16), r, g, b);
	SetVgaValue(CARDINAL(count+16));

	red := AdjustSingleBytePalette(r);
	green := AdjustSingleBytePalette(g);
	blue := AdjustSingleBytePalette(b);

	SetPaletteColour(count+16, red, green, blue);
   END;

   x := 0;
   y := 0;

   error := SetPosition(f,strip_offset);

   rows := block_size DIV width;

   block := ObtainXMSMemory();

   ok := ObtainScreen();

   (*******************************)
   (* plots TIF image onto screen *)
   (*******************************)
   LOOP (* 1 *)
   	error := ReadRecord(f,block,CARDINAL(width*rows),actual);

   	i := 0;

   	IF actual > 0 THEN
   	   LOOP (* 2 *)
         	pixel := VAL(CARDINAL,block^[i]);

      	IF pixel > 239 THEN			(* last 16 colours *)
         	   PutDot(w,x,y,pixel);
      	ELSE						(* first 240 colours *)
         	   PutDot(w,x,y,pixel+16);
      	END;

      	INC(x);

      	IF LONGCARD(x) > width-1 THEN		(* reached end of a scan line *)
         	   x := 0;
         	   INC(y);					(* next scan line *)
      	END;

      	INC(i);
      	IF i > actual-1 THEN			(* reached end of array *)
	 	   EXIT; (* LOOP 2 *)
      	END;
   	   END; (* LOOP 2 *)
   	END; (* IF *)

   	IF (LONGCARD(y) > height-1) THEN		(* reached last scan line *)
         EXIT; (* LOOP 1 *)				(* terminates plotting process *)
   	END; (* IF *)

   END; (* LOOP 1 *)

   ReleaseScreen();


   ReleaseXMSMemory();

   CloseFile(f);

   normal := TRUE;
   zoom2x := FALSE;
   zoom3x := FALSE;

   RETURN 0;

END LoadTIF;


(******************************************************************)
(* This function returns the width of the image.			*)
(******************************************************************)
PROCEDURE GetWidth() : LONGCARD;
BEGIN
   RETURN width;
END GetWidth;


(******************************************************************)
(* This function returns the height of the image.			*)
(******************************************************************)
PROCEDURE GetHeight() : LONGCARD;
BEGIN
   RETURN height;
END GetHeight;


(******************************************************************)
(* This function returns the number of colour planes used for the	*)
(* image.										*)
(******************************************************************)
PROCEDURE GetColourPlanes() : CARDINAL;
BEGIN
   RETURN 1;
END GetColourPlanes;


(******************************************************************)
(* This function returns the number of bits used to represent one	*)
(* pixel.										*)
(******************************************************************)
PROCEDURE GetBitsPerPixel() : LONGCARD;
BEGIN
   RETURN bits_per_sample;
END GetBitsPerPixel;


(******************************************************************)
(* This function returns the maximum number of colours the image	*)
(* can contain.									*)
(******************************************************************)
PROCEDURE GetMaxColours() : LONGCARD;
VAR
   f : File;
   error : ErrorCode;
BEGIN
   RETURN max_colours;
END GetMaxColours;


(******************************************************************)
(* This function returns the number of unique colours used by the	*)
(* image.										*)
(******************************************************************)
PROCEDURE GetColoursUsed() : LONGCARD;
BEGIN
   RETURN 256;
END GetColoursUsed;


(******************************************************************)
(* This function returns the sub-type of the image.			*)
(******************************************************************)
PROCEDURE GetSubType() : LONGCARD;
BEGIN
   RETURN compression;
END GetSubType;


(******************************************************************)
(* This function returns the image file size.				*)
(******************************************************************)
PROCEDURE GetFileSize() : LONGCARD;
BEGIN
   RETURN file_size;
END GetFileSize;


(******************************************************************)
(* This function returns the unpacked image size.			*)
(******************************************************************)
PROCEDURE GetUnpackedSize() : LONGCARD;
BEGIN
   RETURN LONGCARD(width)*LONGCARD(height)*LONGCARD(bits_per_sample DIV 8);
END GetUnpackedSize;



END ReadTIF.