IMPLEMENTATION MODULE SetModes;

(******************************************************************)
(*											*)
(*			  	Graphics Mode Module				*)
(*											*)
(*	Programmer :	Matthew Ma (c9315720)				*)
(*											*)
(*	Last edited :	22-10-97						*)
(*											*)
(*	Function : 		This module reads in the graphics number	*)
(*				from an initialisation file and sets up	*)
(*				the screen. It also writes the specified	*)
(*				graphics number to that file whenever a	*)
(*				new graphics mode has been selected.	*)
(*											*)
(*	Status :		Working.						*)
(******************************************************************)


FROM FIO IMPORT
(* type *)   File,
(* proc *)	 Erase;

FROM IOErrorCodes IMPORT
(* type *)   ErrorCode;

FROM FileSys IMPORT
(* proc *)   OpenFile, CloseFile, EOF, ReadByte, SetPosition, WriteByte, SavePosition;

FROM Str IMPORT
(* proc *)   Copy, Compare, StrToCard, StrToReal, Pos;



CONST
   NULL = CHR(0);
   EqualSign = "=";
   SemiColon = ";";


TYPE
   LongStr = ARRAY [0..255] OF CHAR;



(************************************************************)
(* This procedure writes the new graphics mode number to	*)
(* the painter.ini file.						*)
(************************************************************)
PROCEDURE ChangeMode(mode : CARDINAL);
VAR
   error : ErrorCode;
   f : File;
BEGIN

   CASE mode OF
	  0:
	| 1:	(* 640x480x256 *)
		(* if file does not exist *)
		IF (OpenFile(f, "painter.ini", FALSE) <> OK) THEN
		   error := OpenFile(f, "painter.ini", TRUE);
		   error := WriteByte(f,"V");
		   error := WriteByte(f,"i");
		   error := WriteByte(f,"d");
		   error := WriteByte(f,"e");
		   error := WriteByte(f,"o");
		   error := WriteByte(f,"M");
		   error := WriteByte(f,"o");
		   error := WriteByte(f,"d");
		   error := WriteByte(f,"e");
		   error := WriteByte(f,"=");
		   error := WriteByte(f,"2");
		   error := WriteByte(f,"5");
		   error := WriteByte(f,"7");
	  	   error := WriteByte(f,";");
		   error := WriteByte(f,0DH);
		   error := WriteByte(f,0AH);
		ELSE	(* file exists *)
		   error := SetPosition(f,10);
		   error := WriteByte(f,"2");
		   error := WriteByte(f,"5");
		   error := WriteByte(f,"7");
		   error := WriteByte(f,";");
		   error := WriteByte(f,0DH);
		   error := WriteByte(f,0AH);
		END;
		CloseFile(f);

	| 2:	(* 800x600x256 *)
		IF (OpenFile(f, "painter.ini", FALSE) <> OK) THEN
		   error := OpenFile(f, "painter.ini", TRUE);
		   error := WriteByte(f,"V");
		   error := WriteByte(f,"i");
		   error := WriteByte(f,"d");
		   error := WriteByte(f,"e");
		   error := WriteByte(f,"o");
		   error := WriteByte(f,"M");
		   error := WriteByte(f,"o");
		   error := WriteByte(f,"d");
		   error := WriteByte(f,"e");
		   error := WriteByte(f,"=");
		   error := WriteByte(f,"2");
		   error := WriteByte(f,"5");
		   error := WriteByte(f,"9");
	  	   error := WriteByte(f,";");
		   error := WriteByte(f,0DH);
		   error := WriteByte(f,0AH);
		ELSE
		   error := SetPosition(f,10);
		   error := WriteByte(f,"2");
		   error := WriteByte(f,"5");
		   error := WriteByte(f,"9");
		   error := WriteByte(f,";");
		   error := WriteByte(f,0DH);
		   error := WriteByte(f,0AH);
		END;
		CloseFile(f);

	| 3:	(* 1024x768x256 *)
		IF (OpenFile(f, "painter.ini", FALSE) <> OK) THEN
		   error := OpenFile(f, "painter.ini", TRUE);
		   error := WriteByte(f,"V");
		   error := WriteByte(f,"i");
		   error := WriteByte(f,"d");
		   error := WriteByte(f,"e");
		   error := WriteByte(f,"o");
		   error := WriteByte(f,"M");
		   error := WriteByte(f,"o");
		   error := WriteByte(f,"d");
		   error := WriteByte(f,"e");
		   error := WriteByte(f,"=");
		   error := WriteByte(f,"2");
		   error := WriteByte(f,"6");
		   error := WriteByte(f,"1");
	  	   error := WriteByte(f,";");
		   error := WriteByte(f,0DH);
		   error := WriteByte(f,0AH);
		ELSE
		   error := SetPosition(f,10);
		   error := WriteByte(f,"2");
		   error := WriteByte(f,"6");
		   error := WriteByte(f,"1");
		   error := WriteByte(f,";");
		   error := WriteByte(f,0DH);
		   error := WriteByte(f,0AH);
		END;
		CloseFile(f);

	| 4:	(* 1280x1024x256 *)
		IF (OpenFile(f, "painter.ini", FALSE) <> OK) THEN
		   error := OpenFile(f, "painter.ini", TRUE);
		   error := WriteByte(f,"V");
		   error := WriteByte(f,"i");
		   error := WriteByte(f,"d");
		   error := WriteByte(f,"e");
		   error := WriteByte(f,"o");
		   error := WriteByte(f,"M");
		   error := WriteByte(f,"o");
		   error := WriteByte(f,"d");
		   error := WriteByte(f,"e");
		   error := WriteByte(f,"=");
		   error := WriteByte(f,"2");
		   error := WriteByte(f,"6");
		   error := WriteByte(f,"3");
	  	   error := WriteByte(f,";");
		   error := WriteByte(f,0DH);
		   error := WriteByte(f,0AH);
		ELSE
		   error := SetPosition(f,10);
		   error := WriteByte(f,"2");
		   error := WriteByte(f,"6");
		   error := WriteByte(f,"3");
		   error := WriteByte(f,";");
		   error := WriteByte(f,0DH);
		   error := WriteByte(f,0AH);
		END;
		CloseFile(f);
	ELSE
   END; (* CASE *)

END ChangeMode;


(************************************************************)
(* This procedure reads in the graphics number written in	*)
(* the painter.ini file and sets the video mode accordingly *)
(* If painter.ini does not exist, it will set video mode to *)
(* the default 640x480x256.						*)
(************************************************************)
PROCEDURE ReadMode();
VAR
   error : ErrorCode;
   f : File;
   i : CARDINAL;
   string, temp_string : LongStr;
   ok : BOOLEAN;
BEGIN

   IF (OpenFile(f, "Painter.ini", FALSE) = OK) THEN
    	error := SetPosition(f, 0);
    	i := 0;

	WHILE (NOT(EOF(f))) DO
         string[i] := ReadByte(f);
         CASE string[i] OF
        	  EqualSign:
          		string[i] := NULL;
          		Copy(temp_string, string);
          		i := 0;

		| SemiColon:
          		string[i] := NULL;
	  		IF (Compare(temp_string, "VideoMode") = 0) THEN
            	   video_mode := VAL(CARDINAL, StrToCard(string,10,ok));
          		END;
         		i := 0;

      	| "A".."Z", "a".."z", "0".."9", "_", ".":
          		INC(i);

      	ELSE

         END;(* CASE *)
	END; (* WHILE *)

      CloseFile(f);

   ELSE	(* .ini file not fould *)
      ChangeMode(1);
   	video_mode := 257;
   END; (* IF *)

END ReadMode;


(*******************)
(* initialisations *)
(*******************)
BEGIN
    ChangeMode(0);
    ReadMode();
END SetModes.