IMPLEMENTATION MODULE Support;

(******************************************************************)
(*											*)
(*			  	 Supporting Module				*)
(*											*)
(*	Programmer :	Matthew Ma (c9315720)				*)
(*											*)
(*	Last edited :	22-10-97						*)
(*											*)
(*	Function : 		This is a supporting module containing	*)
(*				utilities for reading and writing SHORT	*)
(*				and LONG data types and other conversion	*)
(*				routines.						*)
(*											*)
(*	Status :		Working.						*)
(******************************************************************)


FROM FIO IMPORT
(* type *)   File;

FROM IOErrorCodes IMPORT
(* type *)   ErrorCode;

FROM FileSys IMPORT
(* proc *)   ReadByte, WriteByte;

FROM LowLevel IMPORT
(* proc *)   LowByte, HighByte, LowWord, HighWord, MakeWord, MakeLongword;



(************************************************************)
(* This procedure reads in a 16-bit WORD and converts it to *)
(* a CARDINAL.								*)
(************************************************************)
PROCEDURE ReadWord(f : File) : CARDINAL;
VAR
   word : WORD;
   high_byte, low_byte : BYTE;
BEGIN
   low_byte := ReadByte(f);
   high_byte := ReadByte(f);

   word := MakeWord(high_byte,low_byte);

   RETURN VAL(CARDINAL,word);
END ReadWord;


(************************************************************)
(* This procedure reads in a 32-bit LONGWORD and converts	*)
(* it to a LONGCARD.							*)
(************************************************************)
PROCEDURE ReadLongWord(f : File) : LONGCARD;
VAR
   long_word : LONGWORD;
   high_byte, low_byte : BYTE;
   high_word, low_word : WORD;
BEGIN
   low_byte := ReadByte(f);
   high_byte := ReadByte(f);
   low_word := MakeWord(high_byte,low_byte);

   low_byte := ReadByte(f);
   high_byte := ReadByte(f);
   high_word := MakeWord(high_byte,low_byte);

   long_word := MakeLongword(high_word,low_word);

   RETURN VAL(LONGCARD,long_word);
END ReadLongWord;


(************************************************************)
(* This procedure writes a 16-bit WORD to a file.		*)
(************************************************************)
PROCEDURE WriteWord(f : File; num : CARDINAL);
VAR
   error : ErrorCode;
   word : WORD;
   high_byte, low_byte : BYTE;
BEGIN
   word := VAL(WORD, num);
   low_byte := LowByte(word);
   high_byte := HighByte(word);

   error := WriteByte(f, low_byte);
   error := WriteByte(f, high_byte);
END WriteWord;
   

(************************************************************)
(* This procedure writes a 32-bit LONGWORD to a file.		*)
(************************************************************)
PROCEDURE WriteLongWord(f : File; num : LONGCARD);
VAR
   error : ErrorCode;
   long_word : LONGWORD;
   high_byte, low_byte : BYTE;
   high_word, low_word : WORD;
BEGIN
   long_word := VAL(LONGWORD, num);

   low_word := LowWord(long_word);
   low_byte := LowByte(low_word);
   high_byte := HighByte(low_word);

   error := WriteByte(f, low_byte);
   error := WriteByte(f, high_byte);

   high_word := HighWord(long_word);
   low_byte := LowByte(high_word);
   high_byte := HighByte(high_word);

   error := WriteByte(f, low_byte);
   error := WriteByte(f, high_byte);
END WriteLongWord;


(************************************************************)
(* This procedure rounds off a REAL number to a CARDINAL.	*)
(************************************************************)
PROCEDURE RoundOff(num : REAL) : CARDINAL;
BEGIN
   IF num - VAL(REAL,TRUNC(num)) >= 0.5 THEN
      RETURN (VAL(CARDINAL,TRUNC(num))+1);
   ELSE
      RETURN VAL(CARDINAL,TRUNC(num));
   END;
END RoundOff;


(*************************************************************)
(* This procedure rounds off a LONGREAL number to a LONGCARD *)
(*************************************************************)
PROCEDURE RoundOff2(num : LONGREAL) : LONGCARD;
BEGIN
   IF num - VAL(LONGREAL,TRUNC(num)) >= 0.5 THEN
      RETURN (VAL(LONGCARD,TRUNC(num))+1);
   ELSE
      RETURN VAL(LONGCARD,TRUNC(num));
   END;
END RoundOff2;


(************************************************************)
(* This procedure scales an 8-bit colour palette entry into	*)
(* a 6-bit entry for use with the VGA display.			*)
(************************************************************)
PROCEDURE AdjustSingleBytePalette(value : SHORTCARD) : SHORTCARD;
VAR
   num : REAL;
   new_value : CARDINAL;
BEGIN
   num := (VAL(REAL,value))*(63.0/255.0);
   new_value := RoundOff(num);
   RETURN VAL(SHORTCARD,new_value);
END AdjustSingleBytePalette;


(************************************************************)
(* This procedure scales an 16-bit colour palette entry	*)
(* into a 6-bit entry for use with the VGA display.		*)
(************************************************************)
PROCEDURE AdjustDoubleBytePalette(value : CARDINAL) : SHORTCARD;
VAR
   num : REAL;
   new_value : CARDINAL;
BEGIN
   num := (VAL(REAL,value))*(63.0/65535.0);
   new_value := RoundOff(num);
   RETURN VAL(SHORTCARD,new_value);
END AdjustDoubleBytePalette;



END Support.