IMPLEMENTATION MODULE TgaToBmp;

(******************************************************************)
(*											*)
(*			  TGA To BMP Conversion Module			*)
(*											*)
(*	Programmer :	Matthew Ma (c9315720)				*)
(*											*)
(*	Last edited :	22-10-97						*)
(*											*)
(*	Function : 		This module converts a TGA image to a BMP	*)
(*				image.						*)
(*											*)
(*	Status :		Working.						*)
(******************************************************************)


FROM InfoDialogs IMPORT
(* proc *)   CantOpenFile, NotTgaFile, Not8Bit, InvalidSubtype,
             NoPalette, CantCreateFile;

FROM Support IMPORT
(* proc *)   ReadWord, WriteWord, ReadLongWord, WriteLongWord;

FROM ColourRoutines IMPORT
(* proc *)   SetPaletteValue;

FROM XMS IMPORT
(* proc *)   ObtainXMSMemory, ReleaseXMSMemory;

FROM FIO IMPORT
(* type *)   File;

FROM FileSys IMPORT
(* proc *)   OpenFile, CloseFile, ReadByte, WriteByte,
             SetPosition, ReadRecord, FileSize, WriteRecord;

FROM Graphics IMPORT
(* proc *)   SetPaletteColour;

FROM GraphicWindow IMPORT
(* var *)    PutDot, XMAX, YMAX, CharWidth, CharHeight,
(* proc *)   ObtainScreen, ReleaseScreen;

FROM IOErrorCodes IMPORT
(* type *)   ErrorCode;

FROM SoundEffects IMPORT
(* proc *)   Beep;


CONST
   block_size = 65534;


(******************************************************************)
(* This procedure converts an 8-bit TGA image to an 8-bit BMP	*)
(* image.										*)
(******************************************************************)
PROCEDURE Tga256ToBmp256(tga_file, bmp_file : ARRAY OF CHAR) : CARDINAL;
VAR
   in_file, out_file : File;
   error : ErrorCode;
   file_size, max_colours : LONGCARD;
   colour_map_length, colour_map_origin, colour_map_type, ident_field : CARDINAL;
   x_start, y_start, x_size, y_size, x_size2, blanks, y_count : CARDINAL;
   map_entry_size, bits_per_pixel, image_type_code : CARDINAL;
   ok : BOOLEAN;
   dummy : BYTE;
   red, green, blue : BYTE;
   count : SHORTCARD;
   actual, i : CARDINAL;
   block : POINTER TO ARRAY [0..block_size-1] OF BYTE;
BEGIN
   IF (OpenFile(in_file, tga_file, FALSE) <> OK) THEN
	Beep();
	CantOpenFile();
      RETURN (1);
   END;

   IF (OpenFile(out_file, bmp_file, TRUE) <> OK) THEN
	Beep();
	CantCreateFile();
	RETURN (2);
   END;

   ident_field := VAL(CARDINAL, ReadByte(in_file));
   IF ident_field <> 0 THEN
	Beep();
	InvalidSubtype();		(* unsupported subtype *)
      RETURN (3);
   END;

   error := SetPosition(in_file,2);
   image_type_code := VAL(CARDINAL, ReadByte(in_file));
   IF (image_type_code <> 1) THEN
	Beep();
	InvalidSubtype();		(* unsupported subtype *)
      RETURN (4);
   END;

   map_entry_size := VAL(CARDINAL, ReadByte(in_file));

   error := SetPosition(in_file,12);
   x_size := ReadWord(in_file);
   y_size := ReadWord(in_file);

   x_size2 := ((x_size + 3) DIV 4) * 4;
   blanks := ABS(x_size2 - x_size);

   file_size := 54+256*4+LONGCARD(x_size2)*LONGCARD(y_size);
   

   error := SetPosition(in_file,16);
   bits_per_pixel := VAL(CARDINAL, ReadByte(in_file));
   IF bits_per_pixel <> 8 THEN
	Beep();
	Not8Bit();		(* not an 256-colour image *)
      RETURN (5);
   END;


   (*************************)
   (* writes the BMP header *)
   (*************************)
   error := WriteByte(out_file, BYTE("B"));
   error := WriteByte(out_file, BYTE("M"));
   
   WriteLongWord(out_file, file_size);

   WriteWord(out_file, 0);
   WriteWord(out_file, 0);

   WriteLongWord(out_file, 1078);

   WriteLongWord(out_file, 40);

   WriteLongWord(out_file, LONGCARD(x_size));
   WriteLongWord(out_file, LONGCARD(y_size));

   WriteWord(out_file, 1);		(* Planes *)

   WriteWord(out_file, 8);		(* BitCount *)

   WriteLongWord(out_file, 0);	(* Compression *);

   WriteLongWord(out_file, LONGCARD(x_size2)*LONGCARD(y_size));

   WriteLongWord(out_file, 0);	(* XPelsPerMeter *)

   WriteLongWord(out_file, 0);	(* YPelsPerMeter *)

   WriteLongWord(out_file, 256);

   WriteLongWord(out_file, 256);




   error := SetPosition(in_file,18);

   (**************************)
   (* writes palette entries *)
   (**************************)
   FOR count := 0 TO 255 DO
	 blue := ReadByte(in_file);
	 green := ReadByte(in_file);
	 red := ReadByte(in_file);

	 IF map_entry_size = 32 THEN
	    dummy := ReadByte(in_file);
	 END;

	 error := WriteByte(out_file, blue);
	 error := WriteByte(out_file, green);
	 error := WriteByte(out_file, red);
	 error := WriteByte(out_file, VAL(BYTE, 0));
   END;


   error := SetPosition(in_file,18+768);

   block := ObtainXMSMemory();

   y_count := 0;

   (************************)
   (* scan line conversion *)
   (************************)
   LOOP
   	error := ReadRecord(in_file, block, x_size, actual);

   	error := WriteRecord(out_file, block, x_size);

	(*****************************************)
	(* writes padding bytes to make sure BMP *)
	(* scan lines end on a LONG boundary.    *)
	(*****************************************)
   	CASE blanks OF
		  1:
			error := WriteByte(out_file, VAL(BYTE, 0));

		| 2:
			error := WriteByte(out_file, VAL(BYTE, 0));
			error := WriteByte(out_file, VAL(BYTE, 0));

		| 3:
			error := WriteByte(out_file, VAL(BYTE, 0));
			error := WriteByte(out_file, VAL(BYTE, 0));
			error := WriteByte(out_file, VAL(BYTE, 0));

		ELSE
	END;

   	INC(y_count);

   	IF y_count = y_size THEN	(* reached last scan line *)
	   EXIT;				(* terminates conversion process *)
   	END;

   END;

   ReleaseXMSMemory();

   CloseFile(in_file);
   CloseFile(out_file);

   RETURN (0);

END Tga256ToBmp256;



END TgaToBmp.