IMPLEMENTATION MODULE TgaToPcx;

(******************************************************************)
(*											*)
(*			  TGA To PCX Conversion Module			*)
(*											*)
(*	Programmer :	Matthew Ma (c9315720)				*)
(*											*)
(*	Last edited :	22-10-97						*)
(*											*)
(*	Function : 		This module converts a TGA image to a PCX	*)
(*				image.						*)
(*											*)
(*	Status :		Working.						*)
(******************************************************************)


FROM InfoDialogs IMPORT
(* proc *)   CantOpenFile, NotTgaFile, Not8Bit, InvalidSubtype,
             NoPalette, CantCreateFile;

FROM Lib IMPORT
(* proc *)   AddAddr, IncAddr;

FROM Support IMPORT
(* proc *)   ReadWord, WriteWord, ReadLongWord, WriteLongWord;

FROM XMS IMPORT
(* proc *)   ObtainXMSMemory, ReleaseXMSMemory;

FROM FIO IMPORT
(* type *)   File, Exists;

FROM FileSys IMPORT
(* proc *)   OpenFile, CloseFile, ReadByte, WriteByte,
             SetPosition, SavePosition, FileSize, ReadRecord, WriteRecord;

FROM IOErrorCodes IMPORT
(* type *)   ErrorCode;

FROM Graphics IMPORT
(* proc *)   Point;

FROM Str IMPORT
(* proc *)   Compare;

FROM SoundEffects IMPORT
(* proc *)   Beep;


CONST
   block_size = 65534;



(******************************************************************)
(* This procedure converts an 8-bit TGA image to an 8-bit PCX	*)
(* image.										*)
(******************************************************************)
PROCEDURE Tga256ToPcx256(tga_file, pcx_file : ARRAY OF CHAR) : CARDINAL;
VAR
   in_file, out_file : File;
   ok : BOOLEAN;
   i, j, x_size, y_size, x_count, y_count, actual, repeat : CARDINAL;
   red, green, blue, dummy : BYTE;
   bitmap_offset, header_offset : LONGCARD;
   n_planes, x_start, y_start, colour_map_origin, colour_map_length : CARDINAL;
   pixel, head, ident_field, bits_per_pixel,
   colour_map_type, image_type_code, map_entry_size : SHORTCARD;
   error : ErrorCode;
   signature : ARRAY [0..1] OF CHAR;
   row1 : POINTER TO ARRAY [0..block_size-1] OF BYTE;
BEGIN
   IF (OpenFile(in_file, tga_file, FALSE) <> OK) THEN
	Beep();
	CantOpenFile();
      RETURN (1);
   END;

   IF (OpenFile(out_file, pcx_file, TRUE) <> OK) THEN
	Beep();
	CantCreateFile();
      RETURN (2);
   END;

   error := SetPosition(in_file,0);
   ident_field := VAL(SHORTCARD, ReadByte(in_file));
   IF ident_field <> 0 THEN
	Beep();
	InvalidSubtype();
      RETURN (3);
   END;

   colour_map_type := VAL(SHORTCARD, ReadByte(in_file));
   image_type_code := VAL(SHORTCARD, ReadByte(in_file));
   IF image_type_code <> 1 THEN
	Beep();
	InvalidSubtype();
      RETURN (4);
   END;

   colour_map_origin := ReadWord(in_file);
   colour_map_length := ReadWord(in_file);
   map_entry_size := VAL(SHORTCARD, ReadByte(in_file));

   x_start := ReadWord(in_file);
   y_start := ReadWord(in_file);

   x_size := ReadWord(in_file);
   y_size := ReadWord(in_file);

   bits_per_pixel := VAL(SHORTCARD, ReadByte(in_file));

   IF bits_per_pixel <> 8 THEN
	Beep();
	Not8Bit();		(* not a 256-colour image *)
      RETURN (5);
   END;


   (**********************************)
   (* writes the 128-byte PCX header *)
   (**********************************)
   error := WriteByte(out_file, BYTE(10));	(* manufacturer *)
   error := WriteByte(out_file, BYTE(5));		(* version *)
   error := WriteByte(out_file, BYTE(1));		(* run length encoding *)
   error := WriteByte(out_file, BYTE(8));		(* bits per pixel *)

   WriteWord(out_file, 0);				(* xmin *)
   WriteWord(out_file, 0);				(* ymin *)
   WriteWord(out_file, CARDINAL(x_size-1));	(* xmax *)
   WriteWord(out_file, CARDINAL(y_size-1));	(* ymax *)

   WriteWord(out_file, 75);				(* H-dpi *)
   WriteWord(out_file, 75);				(* V-dpi *)

   FOR i := 0 TO 48-1 DO
       error := WriteByte(out_file, BYTE(0));
   END;

   error := WriteByte(out_file, BYTE(0));		(* reserverd - offset 64 *)
   error := WriteByte(out_file, BYTE(1));		(* planes *)

   IF ODD(x_size) THEN	(* bytes per scan line must be even *)
      WriteWord(out_file, CARDINAL(x_size+1));
   ELSE
      WriteWord(out_file, CARDINAL(x_size));
   END;

   WriteWord(out_file, 1);				(* palette info *)
   WriteWord(out_file, 640);				(* horizontal screen size *)
   WriteWord(out_file, 480);				(* vertical screen size *)

   (*******************************************)
   (* fills in the last 54 bytes with value 0 *)
   (* to complete the 128-byte file header    *)
   (*******************************************)
   FOR i := 0 TO 54-1 DO
       error := WriteByte(out_file, BYTE(0));
   END;
   (* end of 128-byte header *)


   y_count := 1;

   row1 := ObtainXMSMemory();

   (************************)
   (* scan line conversion *)
   (************************)
   LOOP
      IF y_count <= y_size THEN
         error := SetPosition(in_file, 18+768+LONGCARD(y_size-y_count)*LONGCARD(x_size));
      END;
      error := ReadRecord(in_file,row1,CARDINAL(x_size),actual);

      i := (block_size DIV 2);
      repeat := 0;
      x_count := 0;

      LOOP
	   pixel := row1^[x_count];
	   IF (repeat = 0) THEN
	    	head := pixel;
	   END;

	   INC(x_count);

	   IF (pixel = head) THEN
	    	INC(repeat);

		(****************************************************)
	    	(* consecutive pixels with same colours can only be *)
	    	(* repeated maximum 63 times or to the end of the   *)
	    	(* scan line 						    *)
	    	(****************************************************)
	    	IF (repeat = 63) OR (x_count = CARDINAL(x_size)) THEN
	         row1^[i] := BYTE(192+repeat);	(* repeat more than once *)
	         INC(i);
	         row1^[i] := BYTE(head);
	         INC(i);

	         repeat := 0;
	     	END;

	   ELSE (* pixel <> head *)

	    	IF repeat = 1 THEN
	         IF head >= 192 THEN
	          	row1^[i] := BYTE(192+1);	(* repaet once *)
	          	INC(i);
	           	row1^[i] := BYTE(head);
	          	INC(i);
	         ELSE
	          	row1^[i] := BYTE(head);
	          	INC(i);
	         END;
	    	ELSE (* repeat > 1 *)
	         row1^[i] := BYTE(192+repeat);	(* repaet more than once *)
	         INC(i);
	         row1^[i] := BYTE(head);
	         INC(i);
	    	END;

	    	IF x_count = CARDINAL(x_size) THEN	(* reached last pixel of scan line *)
	         IF pixel >= 192 THEN
	          	row1^[i] := BYTE(192+1);	(* repaet once *)
	          	INC(i);
	          	row1^[i] := BYTE(pixel);
	          	INC(i);
	         ELSE
	          	row1^[i] := BYTE(pixel);
	          	INC(i);
	         END;
	    	END;

	    	IF x_count < CARDINAL(x_size) THEN
	         repeat := 1;
	         head := pixel;
	    	END;
	    
	   END; (* IF *)


	   IF x_count = CARDINAL(x_size) THEN
	    	error := WriteRecord(out_file, AddAddr(row1,(block_size DIV 2)), i-(block_size DIV 2));

	    IF ODD(x_size) THEN
	       error := WriteByte(out_file, BYTE(0));
	    END;

	    INC(y_count);

            EXIT; (* LOOP 2 *)
	 END;

      END; (* LOOP 2 *)

      IF y_count > y_size THEN
	 EXIT; (* LOOP 1 *)
      END;
   END;


   ReleaseXMSMemory();


   error := WriteByte(out_file, BYTE(12));	(* indicate palette present *)

   error := SetPosition(in_file,18);

   (**************************)
   (* writes palette entries *)
   (**************************)
   FOR j := 0 TO 255 DO
       blue := ReadByte(in_file);
       green := ReadByte(in_file);
       red := ReadByte(in_file);

       IF map_entry_size = 32 THEN
	   dummy := ReadByte(in_file);
       END;

       error := WriteByte(out_file,red);
       error := WriteByte(out_file,green);
       error := WriteByte(out_file,blue);
   END;

   CloseFile(in_file);
   CloseFile(out_file);

   RETURN (0);

END Tga256ToPcx256;



END TgaToPcx.