IMPLEMENTATION MODULE TgaToRaw;

(******************************************************************)
(*											*)
(*			  TGA To RAW Conversion Module			*)
(*											*)
(*	Programmer :	Matthew Ma (c9315720)				*)
(*											*)
(*	Last edited :	22-10-97						*)
(*											*)
(*	Function : 		This module converts a TGA image to an	*)
(*				un-encoded RAW image.				*)
(*											*)
(*	Status :		Working.						*)
(******************************************************************)


FROM InfoDialogs IMPORT
(* proc *)   CantOpenFile, NotTgaFile, Not8Bit, InvalidSubtype,
             NoPalette, CantCreateFile;

FROM Support IMPORT
(* proc *)   ReadWord, WriteWord, ReadLongWord, WriteLongWord;

FROM ColourRoutines IMPORT
(* proc *)   SetPaletteValue;

FROM XMS IMPORT
(* proc *)   ObtainXMSMemory, ReleaseXMSMemory;

FROM FIO IMPORT
(* type *)   File;

FROM FileSys IMPORT
(* proc *)   OpenFile, CloseFile, ReadByte, WriteByte,
             SetPosition, ReadRecord, FileSize, WriteRecord;

FROM Graphics IMPORT
(* proc *)   SetPaletteColour;

FROM GraphicWindow IMPORT
(* var *)    PutDot, XMAX, YMAX, CharWidth, CharHeight,
(* proc *)   ObtainScreen, ReleaseScreen;

FROM IOErrorCodes IMPORT
(* type *)   ErrorCode;

FROM SoundEffects IMPORT
(* proc *)   Beep;


CONST
   block_size = 65534;



(******************************************************************)
(* This procedure converts an 8-bit TGA image to an un-encoded	*)
(* RAW file.									*)
(******************************************************************)
PROCEDURE Tga256ToRawData(tga_file, raw_file : ARRAY OF CHAR) : CARDINAL;
VAR
   in_file, out_file : File;
   error : ErrorCode;
   file_size, max_colours : LONGCARD;
   colour_map_length, colour_map_origin, colour_map_type, ident_field : CARDINAL;
   x_start, y_start, x_size, y_size, x_size2, blanks, y_count : CARDINAL;
   map_entry_size, bits_per_pixel, image_type_code : CARDINAL;
   ok : BOOLEAN;
   dummy : BYTE;
   red, green, blue : BYTE;
   count : SHORTCARD;
   actual, i : CARDINAL;
   block : POINTER TO ARRAY [0..block_size-1] OF BYTE;
BEGIN
   IF (OpenFile(in_file, tga_file, FALSE) <> OK) THEN
	Beep();
	CantOpenFile();
      RETURN (1);
   END;

   IF (OpenFile(out_file, raw_file, TRUE) <> OK) THEN
	Beep();
	CantCreateFile();
	RETURN (2);
   END;

   (************************)
   (* reads the TGA header *)
   (************************)
   ident_field := VAL(CARDINAL, ReadByte(in_file));
   IF ident_field <> 0 THEN
	Beep();
	InvalidSubtype();		(* unsupported subtype *)
      RETURN (3);
   END;

   error := SetPosition(in_file,2);
   image_type_code := VAL(CARDINAL, ReadByte(in_file));
   IF (image_type_code <> 1) THEN
	Beep();
	InvalidSubtype();		(* unsupported subtype *)
      RETURN (4);
   END;

   map_entry_size := VAL(CARDINAL, ReadByte(in_file));

   error := SetPosition(in_file,12);
   x_size := ReadWord(in_file);
   y_size := ReadWord(in_file);

   file_size := 54+256*4+LONGCARD(x_size2)*LONGCARD(y_size);

   error := SetPosition(in_file,16);
   bits_per_pixel := VAL(CARDINAL, ReadByte(in_file));
   IF bits_per_pixel <> 8 THEN
	Beep();
	Not8Bit();		(* not a 256-colour image *)
      RETURN (5);
   END;

   block := ObtainXMSMemory();

   y_count := 1;

   (************************)
   (* scan line conversion *)
   (************************)
   LOOP
      error := SetPosition(in_file,18+768+LONGCARD(y_size-y_count)*LONGCARD(x_size));
      error := ReadRecord(in_file, block, x_size, actual);

      error := WriteRecord(out_file, block, x_size);

      INC(y_count);

      IF y_count > y_size THEN	(* reached last scan line *)
	   EXIT;				(* terminates conversion process *)
      END;
   END; (* LOOP *)

   ReleaseXMSMemory();

   CloseFile(in_file);
   CloseFile(out_file);

RETURN (0);

END Tga256ToRawData;



END TgaToRaw.