IMPLEMENTATION MODULE TgaToTif;

(******************************************************************)
(*											*)
(*			  TGA To TIF Conversion Module			*)
(*											*)
(*	Programmer :	Matthew Ma (c9315720)				*)
(*											*)
(*	Last edited :	22-10-97						*)
(*											*)
(*	Function : 		This module converts a TGA image to a TIF	*)
(*				image.						*)
(*											*)
(*	Status :		Working.						*)
(******************************************************************)


FROM InfoDialogs IMPORT
(* proc *)   CantOpenFile, NotTgaFile, Not8Bit, InvalidSubtype,
             NoPalette, CantCreateFile;

FROM Lib IMPORT
(* proc *)   AddAddr, IncAddr;

FROM Support IMPORT
(* proc *)   ReadWord, WriteWord, ReadLongWord, WriteLongWord;

FROM XMS IMPORT
(* proc *)   ObtainXMSMemory, ReleaseXMSMemory;

FROM FIO IMPORT
(* type *)   File, Exists;

FROM FileSys IMPORT
(* proc *)   OpenFile, CloseFile, ReadByte, WriteByte,
             SetPosition, SavePosition, FileSize, ReadRecord, WriteRecord;

FROM IOErrorCodes IMPORT
(* type *)   ErrorCode;

FROM Graphics IMPORT
(* proc *)   Point;

FROM Str IMPORT
(* proc *)   Compare;

FROM SoundEffects IMPORT
(* proc *)   Beep;


CONST
   block_size = 65534;



(******************************************************************)
(* This procedure converts an 8-bit TGA image to an 8-bit TIF	*)
(* image.										*)
(******************************************************************)
PROCEDURE Tga256ToTif256(tga_file, tif_file : ARRAY OF CHAR) : CARDINAL;
VAR
   in_file, out_file : File;
   ok : BOOLEAN;
   i, j, x_size, y_size, x_count, y_count, actual, repeat, count : CARDINAL;
   red, green, blue, dummy : BYTE;
   bitmap_offset, header_offset : LONGCARD;
   n_planes, x_start, y_start, colour_map_origin, colour_map_length : CARDINAL;
   pixel, head, ident_field, bits_per_pixel,
   colour_map_type, image_type_code, map_entry_size : SHORTCARD;
   error : ErrorCode;
   signature : ARRAY [0..1] OF CHAR;
   row1 : POINTER TO ARRAY [0..block_size-1] OF BYTE;
BEGIN
   IF (OpenFile(in_file, tga_file, FALSE) <> OK) THEN
	Beep();
	CantOpenFile();
      RETURN (1);
   END;

   IF (OpenFile(out_file, tif_file, TRUE) <> OK) THEN
	Beep();
	CantCreateFile();
      RETURN (2);
   END;

   (************************)
   (* reads the TGA header *)
   (************************)
   error := SetPosition(in_file,0);
   ident_field := VAL(SHORTCARD, ReadByte(in_file));
   IF ident_field <> 0 THEN
	Beep();
	InvalidSubtype();		(* unsupported subtype *)
      RETURN (3);
   END;

   colour_map_type := VAL(SHORTCARD, ReadByte(in_file));
   image_type_code := VAL(SHORTCARD, ReadByte(in_file));
   IF image_type_code <> 1 THEN
	Beep();
	InvalidSubtype();		(* unsupported subtype *)
      RETURN (4);
   END;

   colour_map_origin := ReadWord(in_file);
   colour_map_length := ReadWord(in_file);
   map_entry_size := VAL(SHORTCARD, ReadByte(in_file));

   x_start := ReadWord(in_file);
   y_start := ReadWord(in_file);

   x_size := ReadWord(in_file);
   y_size := ReadWord(in_file);

   bits_per_pixel := VAL(SHORTCARD, ReadByte(in_file));

   IF bits_per_pixel <> 8 THEN
	Beep();
	Not8Bit();		(* not a 256-colour image *)
      RETURN (5);
   END;


   (*************************)
   (* writes the TIF header *)
   (*************************)
   error := WriteByte(out_file, BYTE("I"));
   error := WriteByte(out_file, BYTE("I"));

   (*********************)
   (* writes TIF fields *)
   (*********************)
   WriteWord(out_file, 42);
   WriteLongWord(out_file, 8);		(* first IFD *)

   WriteWord(out_file, 15-3);			(* IFD_entry_count *)

   WriteWord(out_file, 254);			(* NewSubfileType *)
   WriteWord(out_file, 4);
   WriteLongWord(out_file, 1);
   WriteLongWord(out_file, 0);

   WriteWord(out_file, 256);			(* image width *)
   WriteWord(out_file, 3);
   WriteLongWord(out_file, 1);
   WriteLongWord(out_file, LONGCARD(x_size));

   WriteWord(out_file, 257);			(* image height *)
   WriteWord(out_file, 3);
   WriteLongWord(out_file, 1);
   WriteLongWord(out_file, LONGCARD(y_size));

   WriteWord(out_file, 258);
   WriteWord(out_file, 3);
   WriteLongWord(out_file, 1);
   WriteLongWord(out_file, 8);		(* bits-per-pixel *)

   WriteWord(out_file, 259);
   WriteWord(out_file, 3);
   WriteLongWord(out_file, 1);
   WriteLongWord(out_file, 1);		(* non-compressed *)

   WriteWord(out_file, 262);
   WriteWord(out_file, 3);
   WriteLongWord(out_file, 1);
   WriteLongWord(out_file, 3);		(* palette colour *)

   WriteWord(out_file, 273);
   WriteWord(out_file, 4);
   WriteLongWord(out_file, 1);		(* StripsPerImage *)
   WriteLongWord(out_file, 1730-12*3);	(* strip offset *)

   WriteWord(out_file, 277);
   WriteWord(out_file, 3);
   WriteLongWord(out_file, 1);
   WriteLongWord(out_file, 1);		(* RGB *)

   WriteWord(out_file, 278);
   WriteWord(out_file, 3);
   WriteLongWord(out_file, 1);
   WriteLongWord(out_file, 8192);		(* RowsPerStrip *)

   WriteWord(out_file, 279);
   WriteWord(out_file, 4);
   WriteLongWord(out_file, 1);
   WriteLongWord(out_file, 1);		(* StripByteCounts *)

   WriteWord(out_file, 296);
   WriteWord(out_file, 3);
   WriteLongWord(out_file, 1);
   WriteLongWord(out_file, 1);		(* ResolutionUnit *)

   WriteWord(out_file, 320);
   WriteWord(out_file, 3);
   WriteLongWord(out_file, 768);
   WriteLongWord(out_file, 194-12*3);	(* ColorMap *)

   WriteLongWord(out_file, 0);		(* no more IFDs *)


   (*****************************)
   (* writes red colour entries *)
   (*****************************)
   FOR count := 0 TO 255 DO
	error := SetPosition(in_file,LONGCARD(18+2+3*count));
	red := ReadByte(in_file);

	error := WriteByte(out_file, red);
	error := WriteByte(out_file, red);
   END;

   (*******************************)
   (* writes green colour entries *)
   (*******************************)
   FOR count := 0 TO 255 DO
	error := SetPosition(in_file,LONGCARD(18+1+3*count));
	green := ReadByte(in_file);

	error := WriteByte(out_file, green);
	error := WriteByte(out_file, green);
   END;

   (******************************)
   (* writes blue colour entries *)
   (******************************)
   FOR count := 0 TO 255 DO
	error := SetPosition(in_file,LONGCARD(18+3*count));
	blue := ReadByte(in_file);

	error := WriteByte(out_file, blue);
	error := WriteByte(out_file, blue);
   END;

   y_count := 1;

   row1 := ObtainXMSMemory();

   (************************)
   (* scan line conversion *)
   (************************)
   LOOP
      IF y_count <= y_size THEN
         error := SetPosition(in_file, 18+768+LONGCARD(y_size-y_count)*LONGCARD(x_size));
      END;
      error := ReadRecord(in_file,row1,CARDINAL(x_size),actual);

      error := WriteRecord(out_file, row1, CARDINAL(x_size));

      INC(y_count);

      IF y_count > y_size THEN
	   EXIT; (* LOOP 1 *)
      END;
   END;

   ReleaseXMSMemory();

   CloseFile(in_file);
   CloseFile(out_file);

   RETURN (0);

END Tga256ToTif256;



END TgaToTif.