IMPLEMENTATION MODULE TifToPcx;

(******************************************************************)
(*											*)
(*			  TIF To PCX Conversion Module			*)
(*											*)
(*	Programmer :	Matthew Ma (c9315720)				*)
(*											*)
(*	Last edited :	22-10-97						*)
(*											*)
(*	Function : 		This module converts a TIF image to a PCX	*)
(*				image.						*)
(*											*)
(*	Status :		Working.						*)
(******************************************************************)


FROM InfoDialogs IMPORT
(* proc *)   CantOpenFile, NotTifFile, Not8Bit, InvalidSubtype,
             NoPalette, CantCreateFile, NotIntel, NotSingleStrip;

FROM ColourRoutines IMPORT
(* var  *)   vga_pal,
(* proc *)   SetPaletteValue, SetVgaValue;

FROM Lib IMPORT
(* proc *)   AddAddr, IncAddr;

FROM XMS IMPORT
(* proc *)   ObtainXMSMemory, ReleaseXMSMemory;

FROM FIO IMPORT
(* type *)   File;

FROM FileSys IMPORT
(* proc *)   OpenFile, CloseFile, ReadByte, WriteByte, FileSize,
             SetPosition, ReadRecord, SavePosition, WriteRecord;

FROM Graphics IMPORT
(* proc *)   SetPaletteColour;

FROM GraphicWindow IMPORT
(* var *)    PutDot, XMAX, YMAX, CharWidth, CharHeight,
(* proc *)   ObtainScreen, ReleaseScreen;

FROM IOErrorCodes IMPORT
(* type *)   ErrorCode;

FROM Support IMPORT
(* proc *)   ReadWord, ReadLongWord, AdjustSingleBytePalette,
             WriteWord, WriteLongWord;

FROM Str IMPORT
(* proc *)   Compare;

FROM SoundEffects IMPORT
(* proc *)   Beep;


CONST
   block_size = 65534;


VAR
   version : CARDINAL;
   width, width2, height, bits_per_sample, compression, photometric,
   strip_offset, strips_per_image, samples_per_pixel, rows_per_strip,
   strip_byte_count, x_resolution, y_resolution, resolution_unit, colour_map,
   palette_offset, max_colours, file_size : LONGCARD;



(******************************************************************)
(* This procedure checks all of the TIFF 6.0 baseline fields and	*)
(* sets the corresponding variables.					*)
(******************************************************************)
PROCEDURE CheckField(f : File);
VAR
   error : ErrorCode;
   tag, type : CARDINAL;
   field_head, length, value_offset : LONGCARD;
BEGIN
   field_head := SavePosition(f);
   tag := ReadWord(f);
   CASE tag OF
	  315: (* Artist *)
		error := SetPosition(f,field_head+12);

	| 258: (* BitsPerSample *)
		error := SetPosition(f,field_head+8);
            bits_per_sample := ReadLongWord(f);
		IF bits_per_sample = 8 THEN
		   (* a 256 colour image *)
		   max_colours := 256;
		END;

	| 265: (* CellLength *)
		error := SetPosition(f,field_head+12);

	| 264: (* CellWidth *)
		error := SetPosition(f,field_head+12);

	| 320: (* ColorMap *)
		error := SetPosition(f,field_head+4);
            colour_map := ReadLongWord(f);
		IF colour_map <> 768 THEN
		   (* not 256 colour image *)
		END;
		error := SetPosition(f,field_head+8);
            	palette_offset := ReadLongWord(f); (* start of palette *)

	| 259: (* Compression *)
		error := SetPosition(f,field_head+8);
            compression := ReadLongWord(f);
		IF compression <> 1 THEN
		   (* image is compressed *)
		END;

	| 33432: (* Copyright *)
		error := SetPosition(f,field_head+12);

	| 306: (* DateTime *)
		error := SetPosition(f,field_head+12);

	| 338: (* ExtraSamples *)
		error := SetPosition(f,field_head+12);

	| 266: (* FillOrder *)
		error := SetPosition(f,field_head+12);

	| 289: (* FreeByteCounts *)
		error := SetPosition(f,field_head+12);

	| 288: (* FreeOffsets *)
		error := SetPosition(f,field_head+12);

	| 291: (* GrayResponseCurve *)
		error := SetPosition(f,field_head+12);

	| 290: (* GrayResponseUnit *)
		error := SetPosition(f,field_head+12);

	| 316: (* HostComputer *)
		error := SetPosition(f,field_head+12);

	| 270: (* ImageDescription *)
		error := SetPosition(f,field_head+12);

	| 257: (* ImageLength *)
		error := SetPosition(f,field_head+8);
            height := ReadLongWord(f);

	| 256: (* ImageWidth *)
		error := SetPosition(f,field_head+8);
            width := ReadLongWord(f);

	| 271: (* Maker *)
		error := SetPosition(f,field_head+12);

	| 281: (* MaxSampleValue *)
		error := SetPosition(f,field_head+12);

	| 280: (* MinSampleValue *)
		error := SetPosition(f,field_head+12);

	| 272: (* Model *)
		error := SetPosition(f,field_head+12);

	| 254: (* NewSubfileType *)
		error := SetPosition(f,field_head+12);

	| 274: (* Orientation *)
		error := SetPosition(f,field_head+12);

	| 262: (* PhotometricInterpretation *)
		error := SetPosition(f,field_head+8);
            photometric := ReadLongWord(f);
		IF photometric = 1 THEN
		   (* colour palette *)
		END;

	| 284: (* PlanarConfiguration *)
		error := SetPosition(f,field_head+12);

	| 296: (* ResolutionUnit *)
		error := SetPosition(f,field_head+8);
            resolution_unit := ReadLongWord(f);
		CASE CARDINAL(resolution_unit) OF
		       1: (* none *)
		     | 2: (* inch *)
		     | 3: (* cm *)
		END;

	| 278: (* RowsPerStrip *)
		error := SetPosition(f,field_head+8);
            rows_per_strip := ReadLongWord(f);

	| 277: (* SamplesPerPixel *)
		error := SetPosition(f,field_head+8);
            samples_per_pixel := ReadLongWord(f);
		IF samples_per_pixel = 1 THEN
		   (* not a RGB file *)
		END;

	| 305: (* Software *)
		error := SetPosition(f,field_head+12);

	| 279: (* StripByteCounts *)
		error := SetPosition(f,field_head+8);
            strip_byte_count := ReadLongWord(f);

	| 273: (* StripOffsets *)
		error := SetPosition(f,field_head+4);
            strips_per_image := ReadLongWord(f); (* total number of strips *)
		error := SetPosition(f,field_head+8);
            strip_offset := ReadLongWord(f); (* start of strips *)

	| 255: (* SubfileType *)
		error := SetPosition(f,field_head+12);

	| 263: (* Thresholding *)
		error := SetPosition(f,field_head+12);
	
	| 282: (* XResolution *)
		error := SetPosition(f,field_head+8);
            x_resolution := ReadLongWord(f);

	| 283: (* YResolution *)
		error := SetPosition(f,field_head+8);
            y_resolution := ReadLongWord(f);
	
	ELSE
		error := SetPosition(f,field_head+12);
   END;
END CheckField;


(******************************************************************)
(* This procedure converts an 8-bit TIF image to an 8-bit PCX	*)
(* image.										*)
(******************************************************************)
PROCEDURE Tif256ToPcx256(tif_file, pcx_file : ARRAY OF CHAR) : CARDINAL;
VAR
   in_file, out_file : File;
   dummy : BYTE;
   ok : BOOLEAN;
   error : ErrorCode;
   r, g, b, red, green, blue, count : SHORTCARD;
   repeat, x_count, y_count, colour_map_length, colour_map_origin,
   colour_map_type, image_type_code, ident_field,
   i, j, x, y, actual, IFD_entry_count, bits_per_pixel : CARDINAL;
   r1, g1, b1 : CARDINAL;
   byte_order : ARRAY [0..1] OF CHAR;
   head, pixel : SHORTCARD;
   block : POINTER TO ARRAY [0..block_size-1] OF BYTE;
   rows, IFD_offset, next_strip : LONGCARD;
BEGIN
   IF (OpenFile(in_file, tif_file, FALSE) <> OK) THEN
	Beep();
	CantOpenFile();
      RETURN (1);
   END;

   IF (OpenFile(out_file, pcx_file, TRUE) <> OK) THEN
	Beep();
	CantCreateFile();
	RETURN (2);
   END;

   file_size := FileSize(in_file);

   error := SetPosition(in_file,0);
   byte_order[0] := VAL(CHAR, ReadByte(in_file));
   byte_order[1] := VAL(CHAR, ReadByte(in_file));

   IF Compare(byte_order,"II") <> 0 THEN
	Beep();
	NotIntel();
      RETURN (3);
   END;

   version := ReadWord(in_file);
   IF version <> 42 THEN
	Beep();
	NotTifFile();	(* not a valid .TIF file *)
	RETURN (4);
   END;

   IFD_offset := ReadLongWord(in_file);

   error := SetPosition(in_file,IFD_offset);

   IFD_entry_count := ReadWord(in_file);

   FOR i := 0 TO IFD_entry_count-1 DO
       CheckField(in_file);
   END;


   IF (bits_per_sample <> 8) OR (colour_map <> 768) OR (samples_per_pixel <> 1) THEN
	Beep();
	Not8Bit();
	RETURN (5);
   ELSIF compression <> 1 THEN
	Beep();
	InvalidSubtype();
	RETURN (6);
   ELSIF photometric <> 3 THEN
	Beep();
	NoPalette();
	RETURN (7);
   ELSIF strips_per_image > 1 THEN
	Beep();
	NotSingleStrip();
	RETURN (8);
   ELSE
   END;


   (**********************************)
   (* writes the 128-byte PCX header *)
   (**********************************)
   error := WriteByte(out_file, BYTE(10));	(* manufacturer *)
   error := WriteByte(out_file, BYTE(5));		(* version *)
   error := WriteByte(out_file, BYTE(1));		(* run length encoding *)
   error := WriteByte(out_file, BYTE(8));		(* bits per pixel *)

   WriteWord(out_file, 0);				(* xmin *)
   WriteWord(out_file, 0);				(* ymin *)
   WriteWord(out_file, CARDINAL(width-1));	(* xmax *)
   WriteWord(out_file, CARDINAL(height-1));	(* ymax *)

   WriteWord(out_file, 75);				(* H-dpi *)
   WriteWord(out_file, 75);				(* V-dpi *)

   FOR i := 0 TO 48-1 DO
       error := WriteByte(out_file, BYTE(0));
   END;

   error := WriteByte(out_file, BYTE(0));		(* reserverd - offset 64 *)
   error := WriteByte(out_file, BYTE(1));		(* planes *)

   IF ODD(width) THEN		(* bytes per scan line must be even *)
      WriteWord(out_file, CARDINAL(width+1));
   ELSE
      WriteWord(out_file, CARDINAL(width));
   END;

   WriteWord(out_file, 1);				(* palette info *)
   WriteWord(out_file, 640);				(* horizontal screen size *)
   WriteWord(out_file, 480);				(* vertical screen size *)

   (*******************************************)
   (* fills in the last 54 bytes with value 0 *)
   (* to complete the 128-byte file header    *)
   (*******************************************)
   FOR i := 0 TO 54-1 DO
       error := WriteByte(out_file, BYTE(0));
   END;
   (* end of 128-byte header *)


(*
   x := 0;
   y := 0;
*)

   error := SetPosition(in_file,strip_offset);

   y_count := 1;

   block := ObtainXMSMemory();

   (*ok := ObtainScreen();*)

   (************************)
   (* scan line conversion *)
   (************************)
   LOOP
      error := ReadRecord(in_file,block,CARDINAL(width),actual);

      i := (block_size DIV 2);
      repeat := 0;
      x_count := 0;

      LOOP
	   pixel := block^[x_count];
	   IF (repeat = 0) THEN
	    	head := pixel;
	   END;

	   INC(x_count);

	   IF (pixel = head) THEN
	    	INC(repeat);

		(****************************************************)
	    	(* consecutive pixels with same colours can only be *)
	    	(* repeated maximum 63 times or to the end of the   *)
	    	(* scan line 						    *)
	    	(****************************************************)
	    	IF (repeat = 63) OR (x_count = CARDINAL(width)) THEN
	         block^[i] := BYTE(192+repeat);	(* repeat more than once *)
	         INC(i);
	         block^[i] := BYTE(head);
	         INC(i);

	         repeat := 0;
	     	END;

	   ELSE (* pixel <> head *)

	    	IF repeat = 1 THEN
	         IF head >= 192 THEN
	          	block^[i] := BYTE(192+1);	(* repaet once *)
	          	INC(i);
	           	block^[i] := BYTE(head);
	          	INC(i);
	         ELSE
	          	block^[i] := BYTE(head);
	          	INC(i);
	         END;
	    	ELSE (* repeat > 1 *)
	         block^[i] := BYTE(192+repeat);	(* repaet more than once *)
	         INC(i);
	         block^[i] := BYTE(head);
	         INC(i);
	    	END;

	    	IF x_count = CARDINAL(width) THEN	(* reached last pixel of scan line *)
	         IF pixel >= 192 THEN
	          	block^[i] := BYTE(192+1);	(* repaet once *)
	          	INC(i);
	          	block^[i] := BYTE(pixel);
	          	INC(i);
	         ELSE
	          	block^[i] := BYTE(pixel);
	          	INC(i);
	         END;
	    	END;

	    	IF x_count < CARDINAL(width) THEN
	         repeat := 1;
	         head := pixel;
	    	END;
	    
	   END; (* IF *)


	   IF x_count = CARDINAL(width) THEN
	    	error := WriteRecord(out_file, AddAddr(block,(block_size DIV 2)), i-(block_size DIV 2));

	    	IF ODD(width) THEN
	         error := WriteByte(out_file, BYTE(0));
	      END;

	    	INC(y_count);

            EXIT; (* LOOP 2 *)
	   END;

      END; (* LOOP 2 *)

      IF LONGCARD(y_count) > height THEN
	   EXIT; (* LOOP 1 *)
      END;

   END;

   ReleaseXMSMemory();


   error := WriteByte(out_file, BYTE(12));	(* indicate palette present *)

   (**************************)
   (* writes palette entries *)
   (**************************)
   FOR count := 0 TO 255 DO
	 error := SetPosition(in_file,palette_offset+2*LONGCARD(count));
	 r := SHORTCARD(ReadByte(in_file));

	 error := SetPosition(in_file,palette_offset+256*2+2*LONGCARD(count));
	 g := SHORTCARD(ReadByte(in_file));

	 error := SetPosition(in_file,palette_offset+256*4+2*LONGCARD(count));
	 b := SHORTCARD(ReadByte(in_file));

	 error := WriteByte(out_file, r);
	 error := WriteByte(out_file, g);
	 error := WriteByte(out_file, b);
   END;

   CloseFile(in_file);
   CloseFile(out_file);

   RETURN 0;

END Tif256ToPcx256;


END TifToPcx.