IMPLEMENTATION MODULE Toolbox;

(******************************************************************)
(*											*)
(*				   Toolbox module					*)
(*											*)
(*	Programmer :	Matthew Ma (c9315720)				*)
(*											*)
(*	Last edited :	22-10-97						*)
(*											*)
(*	Function : 		This module contains all of the drawing 	*)
(*				functions that are controlled by mouse 	*)
(*				operations.						*)
(* 											*)
(*	Status :		Working.						*)
(******************************************************************)


FROM Support IMPORT
(* proc *)   RoundOff;

FROM Graphics IMPORT
(* proc *)   Point;

FROM Lib IMPORT
(* proc *)   RANDOMIZE, RANDOM;

FROM Bars IMPORT
(* proc *)   UpdateStatusBar, UpdateForegroundColour, UpdateBackgroundColour, ClearOptionsWindow;

FROM Dialogs IMPORT
(* type *)   Dialog, DialogType, DialogStringPointer, OtherInputType,
(* proc *)	 CreateDialog, AddABox, DisplayDialog, DialogWindow,
        	 SetOtherInput, EndCreateDialog, HideDialog, DialogSelection;

FROM ColourRoutines IMPORT
(* var  *)   ForegroundColour;

FROM MATHLIB IMPORT
(* proc *)   Sqrt, Pow;

FROM Canvas IMPORT
(* var  *)   CanvasLeft, CanvasTop, CanvasRight, CanvasBottom,
(* proc *)   MsgCanvas;

FROM GraphicWindow IMPORT
(* type *)   _GPSET, GraphicalWindow, ColourType,
(* var  *)   XMAX, YMAX, CharWidth, CharHeight, MouseParatemer_xoffset, MouseParatemer_yoffset,
(* proc *)   ReleaseScreen,ObtainScreen,PutChar,PutCircle,PutLine,
        	 PutRectangle, PutString, PutDot, PutTitleStrip, PutSquare, PutHVLine, ImageSize;


CONST
   NULL = CHR(0);
   MaxStringLength = 80;
   block_size = 65534;

TYPE
   LongStr = ARRAY [0..255] OF CHAR;
   LongStrPtr = POINTER TO LongStr;

   FileNameTYPE = ARRAY [0..255] OF CHAR;

   TextTYPE = ARRAY [0..60] OF CHAR;

   (* used to store co-ordinates *)
   DotType = RECORD
			x : INTEGER;
			y : INTEGER;
	       END;

   frameTYPE = RECORD
			x : CARDINAL;
			y : CARDINAL;
			colour : CARDINAL;
		   END;

VAR
   FileOpenned,OpenFileSaved:BOOLEAN;
   TextDialog : Dialog;
   InputString : TextTYPE;

   x1, y1, x2,y2, poly_x1, poly_y1, poly_x, poly_y, poly_x2, poly_y2 : INTEGER;

   started, stop, first_time, first_poly : BOOLEAN;

   prev_x, prev_y, prev_x2, prev_y2, temp_x, temp_y : INTEGER;

   curr_small_block, prev_small_block : ARRAY [0..3] OF ARRAY [0..3] OF DotType;
   curr_medium_block, prev_medium_block : ARRAY [0..5] OF ARRAY [0..5] OF DotType;
   curr_large_block, prev_large_block : ARRAY [0..7] OF ARRAY [0..7] OF DotType;

   curr_small_brush, prev_small_brush : ARRAY [0..3] OF ARRAY [0..3] OF DotType;
   curr_medium_brush, prev_medium_brush : ARRAY [0..6] OF ARRAY [0..6] OF DotType;
   curr_large_brush, prev_large_brush : ARRAY [0..7] OF ARRAY [0..7] OF DotType;



(******************************************************************)
(* This procedure draws a transparent polygon on the specified 	*)
(* window with the foreground colour.					*)
(******************************************************************)
PROCEDURE PolygonRoutine(window : GraphicalWindow; done : BOOLEAN);
VAR
   ok : BOOLEAN;
   x3, y3 : INTEGER;
BEGIN

   (************************************)
   (* gets relative mouse co-ordinates *)
   (************************************)
   x3 := MouseParatemer_xoffset;
   y3 := MouseParatemer_yoffset;

   ok := ObtainScreen();

   IF done THEN
	IF first_time THEN
	   PutLine(window, poly_x, poly_y, prev_x2, prev_y2, 256);
	END;
	   PutLine(window, poly_x, poly_y, x3, y3, ForegroundColour);
   ELSE
   	IF started THEN
	   IF first_time THEN
	   	PutLine(window, poly_x, poly_y, x3, y3, 256);
	   	prev_x2 := x3;
	   	prev_y2 := y3;
	   	first_time := FALSE;
	   ELSE
		(*************************)
	   	(* eliminates flickering *)
		(*************************)
	   	IF (x3 <> prev_x2) OR (y3 <> prev_y2) THEN
	   	   PutLine(window, poly_x, poly_y, prev_x2, prev_y2, 256);
	   	   PutLine(window, poly_x, poly_y, x3, y3, 256);
	   	END;
	   	temp_x := prev_x2;
	   	temp_y := prev_y2;
	   	prev_x2 := x3;
	   	prev_y2 := y3;
	   END;
   	END;
   END;

   ReleaseScreen();

END PolygonRoutine;


(******************************************************************)
(* This procedure records the co-ordinates x1, y1 of the starting	*)
(* point.										*)
(******************************************************************)
PROCEDURE Getx1y1();
BEGIN
   x1 := MouseParatemer_xoffset;
   y1 := MouseParatemer_yoffset;
   first_time := TRUE;
END Getx1y1;


(******************************************************************)
(* This procedure records the co-ordinates x2, y2 of the end 	*)
(* point.										*)
(******************************************************************)
PROCEDURE Getx2y2();
BEGIN
   x2 := MouseParatemer_xoffset;
   y2 := MouseParatemer_yoffset;
END Getx2y2;


(******************************************************************)
(* This procedure records the co-ordinates x1, y1 of the next	*)
(* vertex of the polygon.							*)
(******************************************************************)
PROCEDURE GetPolyx1y1(window : GraphicalWindow);
VAR
   temp_x, temp_y : INTEGER;
BEGIN
   temp_x := MouseParatemer_xoffset;
   temp_y := MouseParatemer_yoffset;

   IF NOT(first_time) AND NOT(started) THEN
   	poly_x1 := temp_x;
   	poly_y1 := temp_y;
	poly_x := temp_x;
   	poly_y := temp_y;
   END;

   PolygonRoutine(window,TRUE);
   poly_x := temp_x;
   poly_y := temp_y;

   first_time := TRUE;
   started := TRUE;
END GetPolyx1y1;


(******************************************************************)
(* This procedure closes the polygon by joining the first and the	*)
(* last vertices with a line.							*)
(******************************************************************)
PROCEDURE ClosePolygon(window : GraphicalWindow);
VAR
   ok : BOOLEAN;
BEGIN
   ok := ObtainScreen();
   PutLine(window,poly_x1,poly_y1,poly_x,poly_y,ForegroundColour);
   ReleaseScreen();
END ClosePolygon;


(******************************************************************)
(* This procedure calculates the radius given the centre and a 	*)
(* point on	the circumference.						*)
(******************************************************************)
PROCEDURE GetRadius(x_start, y_start, x_end, y_end : INTEGER) : CARDINAL;
VAR
   radius : CARDINAL;
BEGIN
   radius := RoundOff(REAL(Sqrt(LONGREAL(Pow(LONGREAL(ABS(x_end - x_start)),2.0) + Pow(LONGREAL(ABS(y_end - y_start)),2.0)))));

   RETURN radius;
END GetRadius;


(******************************************************************)
(* This procedure starts a drawing routine by setting the		*)
(* first_time variable to TURE.						*)
(******************************************************************)
PROCEDURE StartRoutine();
BEGIN
   first_time := TRUE;
END StartRoutine;


(******************************************************************)
(* This procedure starts a drawing routine by setting the		*)
(* first_time and started variable to FALSE.				*)
(******************************************************************)
PROCEDURE StopRoutine();
BEGIN
   started := FALSE;
   first_time := FALSE;
END StopRoutine;


(******************************************************************)
(* This procedure draws pixels continously on the specified 	*)
(* window with the foreground colour.					*)
(******************************************************************)
PROCEDURE FreehandRoutine(window : GraphicalWindow);
VAR
   ok : BOOLEAN;
   x,y : INTEGER;
BEGIN

   (************************************)
   (* gets relative mouse co-ordinates *)
   (************************************)
   x := MouseParatemer_xoffset;
   y := MouseParatemer_yoffset;

   ok := ObtainScreen();

   IF (first_time) THEN
	prev_x := x;
      prev_y := y;

	(***********************)
	(* draws initial pixel *)
	(***********************)
	PutLine(window,prev_x,prev_y,x,y,ForegroundColour);
	first_time := FALSE;
   ELSE

	(***************************************************)
	(* approximates mouse trail by joining the current *)
	(* cursor position with the previous position	   *)
	(***************************************************)
	PutLine(window,prev_x,prev_y,x,y,ForegroundColour);
	prev_x := x;
      prev_y := y;
   END;

   ReleaseScreen();

END FreehandRoutine;


(******************************************************************)
(* This procedure draws a straight line at any angle on the 	*)
(* specified window with the foreground colour				*)
(******************************************************************)
PROCEDURE LineRoutine(window : GraphicalWindow; done : BOOLEAN);
VAR
   ok : BOOLEAN;
   x3, y3 : INTEGER;
BEGIN

   (************************************)
   (* gets relative mouse co-ordinates *)
   (************************************)
   x3 := MouseParatemer_xoffset;
   y3 := MouseParatemer_yoffset;

   ok := ObtainScreen();

   IF done THEN
	IF NOT(first_time) THEN

	   (************************************************)
	   (* re-subtracts the colour values of the pixels *)
	   (* over which the line is to be drawn by 255	   *)
	   (* thus restoring the previous colours		   *)
	   (************************************************)
	   PutLine(window, x1, y1, prev_x2, prev_y2, 256);

	   Getx2y2();

	   (************************************************)
	   (* plots the final line with current foreground *)
	   (* colour after left mouse button is released   *)	
	   (************************************************)
	   PutLine(window, x1, y1, x2, y2, ForegroundColour);
	END;
	first_time := TRUE;
   ELSE
	IF first_time THEN

	   (**********************************************)
	   (* subtracts the colour values of the pixels	 *)
	   (* over which the line is to be drawn by 255  *)
	   (**********************************************)
	   PutLine(window, x1, y1, x3, y3, 256);
	   prev_x2 := x3;
	   prev_y2 := y3;
	   first_time := FALSE;
	ELSE

	   (*******************************************)
	   (* eliminates flickering by comparing the  *)
	   (* current position with previous position *)
	   (*******************************************)
	   IF (x3 <> prev_x2) OR (y3 <> prev_y2) THEN

		(**********************************************)
	   	(* subtracts the colour values of the pixels  *)
	   	(* over which the line is to be drawn by 255  *)
	   	(**********************************************)
	   	PutLine(window, x1, y1, prev_x2, prev_y2, 256);

		(**************************************************)
	   	(* re-subtracts the colour values of the pixels	  *)
	   	(* over which the line is to be drawn by 255	  *)
		(* thus restoring the previous colours		  *)
	   	(**************************************************)
	   	PutLine(window, x1, y1, x3, y3, 256);
	   END;

	   temp_x := prev_x2;
	   temp_y := prev_y2;
	   prev_x2 := x3;
	   prev_y2 := y3;
	END;
   END;

   ReleaseScreen();

END LineRoutine;


(******************************************************************)
(* This procedure draws a horizontal, vertical or 45 degree line 	*)
(* on the specified window with the foreground colour.		*)
(******************************************************************)
PROCEDURE HVLineRoutine(window : GraphicalWindow; done : BOOLEAN);
VAR
   ok : BOOLEAN;
   x3,y3 : INTEGER;
BEGIN

   (************************************)
   (* gets relative mouse co-ordinates *)
   (************************************)
   x3 := MouseParatemer_xoffset;
   y3 := MouseParatemer_yoffset;

   ok := ObtainScreen();

   IF done THEN
	IF NOT(first_time) THEN

	   (************************************************)
	   (* re-subtracts the colour values of the pixels *)
	   (* over which the line is to be drawn by 255	   *)
	   (* thus restoring the previous colours		   *)
	   (************************************************)
	   PutHVLine(window, x1, y1, prev_x2, prev_y2, 256);

	   Getx2y2();

	   (************************************************)
	   (* plots the final line with current foreground *)
	   (* colour after left mouse button is released   *)	
	   (************************************************)
	   PutHVLine(window, x1, y1, x2, y2, ForegroundColour);
	END;
	first_time := TRUE;
   ELSE
	IF first_time THEN

	   (**********************************************)
	   (* subtracts the colour values of the pixels	 *)
	   (* over which the line is to be drawn by 255  *)
	   (**********************************************)
	   PutHVLine(window, x1, y1, x3, y3, 256);
	   prev_x2 := x3;
	   prev_y2 := y3;
	   first_time := FALSE;
	ELSE

	   (*******************************************)
	   (* eliminates flickering by comparing the  *)
	   (* current position with previous position *)
	   (*******************************************)
	   IF (x3 <> prev_x2) OR (y3 <> prev_y2) THEN

	  	(**********************************************)
	   	(* subtracts the colour values of the pixels  *)
	   	(* over which the line is to be drawn by 255  *)
	   	(**********************************************)
	   	PutHVLine(window, x1, y1, prev_x2, prev_y2, 256);

		(************************************************)
	   	(* re-subtracts the colour values of the pixels *)
	   	(* over which the line is to be drawn by 255	*)
	   	(* thus restoring the previous colours		*)
	   	(************************************************)
	   	PutHVLine(window, x1, y1, x3, y3, 256);
	   END;

	   temp_x := prev_x2;
	   temp_y := prev_y2;
	   prev_x2 := x3;
	   prev_y2 := y3;
	END;
   END;

   ReleaseScreen();

END HVLineRoutine;


(******************************************************************)
(* This procedure draws a circle on the specified window with the	*)
(* foreground colour.								*)
(******************************************************************)
PROCEDURE CircleRoutine(window : GraphicalWindow; done : BOOLEAN);
VAR
   ok : BOOLEAN;
   x3, y3 : INTEGER;
   radius : INTEGER;
BEGIN

   (************************************)
   (* gets relative mouse co-ordinates *)
   (************************************)
   x3 := MouseParatemer_xoffset;
   y3 := MouseParatemer_yoffset;

   ok := ObtainScreen();

   IF done THEN
	IF NOT(first_time) THEN
	   radius := INTEGER(GetRadius(x1,y1,prev_x2,prev_y2));

	   (************************************************)
	   (* re-subtracts the colour values of the pixels *)
	   (* over which the circle is to be drawn, by 255 *)
	   (* thus restoring the previous colours		   *)
	   (************************************************)
	   PutCircle(window, x1, y1, radius, 256, FALSE);
	   Getx2y2();
	   radius := INTEGER(GetRadius(x1,y1,x2,y2));

	   (**************************************************)
	   (* plots the final circle with current foreground *)
	   (* colour after left mouse button is released     *)	
	   (**************************************************)
	   PutCircle(window, x1, y1, radius, ForegroundColour, FALSE);
	END;
	first_time := TRUE;
   ELSE
	IF first_time THEN
	   radius := INTEGER(GetRadius(x1,y1,x3,y3));

	   (*************************************************)
	   (* subtracts the colour values of the pixels	    *)
	   (* over which the circle is to be drawn, by 255  *)
	   (*************************************************)
	   PutCircle(window, x1, y1, radius, 256, FALSE);

	   prev_x2 := x3;
	   prev_y2 := y3;
	   first_time := FALSE;
	ELSE

	   (*******************************************)
	   (* eliminates flickering by comparing the  *)
	   (* current position with previous position *)
	   (*******************************************)
	   IF (x3 <> prev_x2) OR (y3 <> prev_y2) THEN
		radius := INTEGER(GetRadius(x1,y1,prev_x2,prev_y2));

		(************************************************)
	   	(* re-subtracts the colour values of the pixels *)
	   	(* over which the circle is to be drawn, by 255	*)
	   	(* thus restoring the previous colours		*)
	   	(************************************************)
	   	PutCircle(window, x1, y1, radius, 256, FALSE);

		radius := INTEGER(GetRadius(x1,y1,x3,y3));

		(*************************************************)
	   	(* subtracts the colour values of the pixels	 *)
	   	(* over which the circle is to be drawn, by 255  *)
	   	(*************************************************)
	   	PutCircle(window, x1, y1, radius, 256, FALSE);
	   END;

	   temp_x := prev_x2;
	   temp_y := prev_y2;
	   prev_x2 := x3;
	   prev_y2 := y3;
	END;
   END;

   ReleaseScreen();

END CircleRoutine;


(******************************************************************)
(* This procedure draws a solid circle on the specified window 	*)
(* with the	foreground colour.						*)
(******************************************************************)
PROCEDURE SolidCircleRoutine(window : GraphicalWindow; done : BOOLEAN);
VAR
   ok : BOOLEAN;
   x3, y3 : INTEGER;
   radius : INTEGER;
BEGIN

   (************************************)
   (* gets relative mouse co-ordinates *)
   (************************************)
   x3 := MouseParatemer_xoffset;
   y3 := MouseParatemer_yoffset;

   ok := ObtainScreen();

   IF done THEN
	IF NOT(first_time) THEN
	   radius := INTEGER(GetRadius(x1,y1,prev_x2,prev_y2));

	   (************************************************)
	   (* re-subtracts the colour values of the pixels *)
	   (* over which the circle is to be drawn, by 255 *)
	   (* thus restoring the previous colours		   *)
	   (************************************************)
	   PutCircle(window, x1, y1, radius, 256, FALSE);

	   Getx2y2();

	   (**************************************************)
	   (* plots the final circle with current foreground *)
	   (* colour after left mouse button is released     *)	
	   (**************************************************)
	   radius := INTEGER(GetRadius(x1,y1,x2,y2));
	   PutCircle(window, x1, y1, radius, ForegroundColour, TRUE);
	END;
	first_time := TRUE;
   ELSE
	IF first_time THEN
	   radius := INTEGER(GetRadius(x1,y1,x3,y3));

	   (*************************************************)
	   (* subtracts the colour values of the pixels	    *)
	   (* over which the circle is to be drawn, by 255  *)
	   (*************************************************)
	   PutCircle(window, x1, y1, radius, 256, FALSE);

	   prev_x2 := x3;
	   prev_y2 := y3;
	   first_time := FALSE;
	ELSE

	   (*******************************************)
	   (* eliminates flickering by comparing the  *)
	   (* current position with previous position *)
	   (*******************************************)
	   IF (x3 <> prev_x2) OR (y3 <> prev_y2) THEN
		radius := INTEGER(GetRadius(x1,y1,prev_x2,prev_y2));

		(************************************************)
	   	(* re-subtracts the colour values of the pixels *)
	   	(* over which the circle is to be drawn, by 255	*)
	   	(* thus restoring the previous colours		*)
	   	(************************************************)
	   	PutCircle(window, x1, y1, radius, 256, FALSE);

		radius := INTEGER(GetRadius(x1,y1,x3,y3));

		(*************************************************)
	   	(* subtracts the colour values of the pixels	 *)
	   	(* over which the circle is to be drawn, by 255  *)
	   	(*************************************************)
	   	PutCircle(window, x1, y1, radius, 256, FALSE);
	   END;

	   temp_x := prev_x2;
	   temp_y := prev_y2;
	   prev_x2 := x3;
	   prev_y2 := y3;
	END;
   END;

   ReleaseScreen();

END SolidCircleRoutine;


(******************************************************************)
(* This procedure draws a rectangle on the specified window with 	*)
(* the foreground colour							*)
(******************************************************************)
PROCEDURE RectangleRoutine(window : GraphicalWindow; done : BOOLEAN);
VAR
   ok : BOOLEAN;
   x3, y3 : INTEGER;
BEGIN

   (************************************)
   (* gets relative mouse co-ordinates *)
   (************************************)
   x3 := MouseParatemer_xoffset;
   y3 := MouseParatemer_yoffset;

   ok := ObtainScreen();

   IF done THEN
	IF NOT(first_time) THEN

	   (************************************************)
	   (* re-subtracts the colour values of the pixels *)
	   (* over which the rectangle is to be drawn, by  *)
	   (* 255 thus restoring the previous colours	   *)
	   (************************************************)
	   PutRectangle(window, x1, y1, prev_x2, prev_y2, 256, FALSE);

	   Getx2y2();

	   (*****************************************************)
	   (* plots the final rectangle with current foreground *)
	   (* colour after left mouse button is released        *)	
	   (*****************************************************)
	   PutRectangle(window, x1, y1, x2, y2, ForegroundColour, FALSE);
	END;
	first_time := TRUE;
   ELSE
	IF first_time THEN

	   (**************************************************)
	   (* subtracts the colour values of the pixels	over *)
	   (* which the rectangle is to be drawn, by 255     *)
	   (**************************************************)
	   PutRectangle(window, x1, y1, x3, y3, 256, FALSE);

	   prev_x2 := x3;
	   prev_y2 := y3;
	   first_time := FALSE;
	ELSE

	   (*******************************************)
	   (* eliminates flickering by comparing the  *)
	   (* current position with previous position *)
	   (*******************************************)
	   IF (x3 <> prev_x2) OR (y3 <> prev_y2) THEN

	 	(************************************************)
	   	(* re-subtracts the colour values of the pixels *)
	   	(* over which the rectangle is to be drawn, by  *)
	   	(* 255 thus restoring the previous colours	*)
	   	(************************************************)
	   	PutRectangle(window, x1, y1, prev_x2, prev_y2, 256, FALSE);

		(**************************************************)
	   	(* subtracts the colour values of the pixels over *)
	   	(* which the rectangle is to be drawn, by 255  	  *)
	   	(**************************************************)
	   	PutRectangle(window, x1, y1, x3, y3, 256, FALSE);
	   END;

	   temp_x := prev_x2;
	   temp_y := prev_y2;
	   prev_x2 := x3;
	   prev_y2 := y3;
	END;
   END;

   ReleaseScreen();

END RectangleRoutine;


(******************************************************************)
(* This procedure draws a solid rectangle on the specified window	*)
(* with the foreground colour.						*)
(******************************************************************)
PROCEDURE SolidRectangleRoutine(window : GraphicalWindow; done : BOOLEAN);
VAR
   ok : BOOLEAN;
   x3, y3 : INTEGER;
BEGIN

   (************************************)
   (* gets relative mouse co-ordinates *)
   (************************************)
   x3 := MouseParatemer_xoffset;
   y3 := MouseParatemer_yoffset;

   ok := ObtainScreen();

   IF done THEN
	IF NOT(first_time) THEN

	   (************************************************)
	   (* re-subtracts the colour values of the pixels *)
	   (* over which the rectangle is to be drawn, by  *)
	   (* 255 thus restoring the previous colours	   *)
	   (************************************************)
	   PutRectangle(window, x1, y1, prev_x2, prev_y2, 256, FALSE);

	   Getx2y2();

	   (*********************************************************)
	   (* plots the final solid rectangle filled with current   *)
	   (* foreground colour after left mouse button is released *)	
	   (*********************************************************)
	   PutRectangle(window, x1, y1, x2, y2, ForegroundColour, TRUE);
	END;
	first_time := TRUE;
   ELSE
	IF first_time THEN

	   (**************************************************)
	   (* subtracts the colour values of the pixels	over *)
	   (* which the rectangle is to be drawn, by 255     *)
	   (**************************************************)
	   PutRectangle(window, x1, y1, x3, y3, 256, FALSE);

	   prev_x2 := x3;
	   prev_y2 := y3;
	   first_time := FALSE;
	ELSE

	   (*******************************************)
	   (* eliminates flickering by comparing the  *)
	   (* current position with previous position *)
	   (*******************************************)
	   IF (x3 <> prev_x2) OR (y3 <> prev_y2) THEN

		(************************************************)
	   	(* re-subtracts the colour values of the pixels *)
	   	(* over which the rectangle is to be drawn, by  *)
	   	(* 255 thus restoring the previous colours	*)
	   	(************************************************)
	   	PutRectangle(window, x1, y1, prev_x2, prev_y2, 256, FALSE);

		(**************************************************)
	   	(* subtracts the colour values of the pixels over *)
	   	(* which the rectangle is to be drawn, by 255     *)
	   	(**************************************************)
	   	PutRectangle(window, x1, y1, x3, y3, 256, FALSE);
	   END;

	   temp_x := prev_x2;
	   temp_y := prev_y2;
	   prev_x2 := x3;
	   prev_y2 := y3;
	END;
   END;

   ReleaseScreen();

END SolidRectangleRoutine;


(******************************************************************)
(* This procedure draws a square on the specified window with the	*)
(* foreground colour.								*)
(******************************************************************)
PROCEDURE SquareRoutine(window : GraphicalWindow; done : BOOLEAN);
VAR
   ok : BOOLEAN;
   x3, y3 : INTEGER;
BEGIN

   (************************************)
   (* gets relative mouse co-ordinates *)
   (************************************)
   x3 := MouseParatemer_xoffset;
   y3 := MouseParatemer_yoffset;

   ok := ObtainScreen();

   IF done THEN
	IF NOT(first_time) THEN

	   (************************************************)
	   (* re-subtracts the colour values of the pixels *)
	   (* over which the square is to be drawn, by 255 *)
	   (* thus restoring the previous colours    	   *)
	   (************************************************)
	   PutSquare(window, x1, y1, prev_x2, prev_y2, 256, FALSE);

	   Getx2y2();

	   (**************************************************)
	   (* plots the final square with current foreground *)
	   (* colour after left mouse button is released     *)	
	   (**************************************************)
	   PutSquare(window, x1, y1, x2, y2, ForegroundColour, FALSE);
	END;
	first_time := TRUE;
   ELSE
	IF first_time THEN

	   (**************************************************)
	   (* subtracts the colour values of the pixels over *)
	   (* which the square is to be drawn, by 255        *)
	   (**************************************************)
	   PutSquare(window, x1, y1, x3, y3, 256, FALSE);

	   prev_x2 := x3;
	   prev_y2 := y3;
	   first_time := FALSE;
	ELSE

	   (*******************************************)
	   (* eliminates flickering by comparing the  *)
	   (* current position with previous position *)
	   (*******************************************)
	   IF (x3 <> prev_x2) OR (y3 <> prev_y2) THEN

		(************************************************)
	   	(* re-subtracts the colour values of the pixels *)
	   	(* over which the square is to be drawn, by 255 *)
	   	(* thus restoring the previous colours    	*)
	   	(************************************************)
	   	PutSquare(window, x1, y1, prev_x2, prev_y2, 256, FALSE);

		(**************************************************)
	   	(* subtracts the colour values of the pixels over *)
	   	(* which the rectangle is to be drawn, by 255     *)
	   	(**************************************************)
	   	PutSquare(window, x1, y1, x3, y3, 256, FALSE);
	   END;

	   temp_x := prev_x2;
	   temp_y := prev_y2;
	   prev_x2 := x3;
	   prev_y2 := y3;
	END;
   END;

   ReleaseScreen();

END SquareRoutine;


(******************************************************************)
(* This procedure draws a solid square on the specified window 	*)
(* with the	foreground colour							*)
(******************************************************************)
PROCEDURE SolidSquareRoutine(window : GraphicalWindow; done : BOOLEAN);
VAR
   ok : BOOLEAN;
   x3, y3 : INTEGER;
BEGIN

   (************************************)
   (* gets relative mouse co-ordinates *)
   (************************************)
   x3 := MouseParatemer_xoffset;
   y3 := MouseParatemer_yoffset;

   ok := ObtainScreen();
   IF done THEN
	IF NOT(first_time) THEN

	   (************************************************)
	   (* re-subtracts the colour values of the pixels *)
	   (* over which the square is to be drawn, by 255 *)
	   (* thus restoring the previous colours      	   *)
	   (************************************************)
	   PutSquare(window, x1, y1, prev_x2, prev_y2, 256, FALSE);

	   Getx2y2();

	   (*********************************************************)
	   (* plots the final solid square filled with the current  *)
	   (*	foreground colour after left mouse button is released *)	
	   (*********************************************************)
	   PutSquare(window, x1, y1, x2, y2, ForegroundColour, TRUE);
	END;
	first_time := TRUE;
   ELSE
	IF first_time THEN

	   (**************************************************)
	   (* subtracts the colour values of the pixels over *)
	   (* which the square is to be drawn, by 255        *)
	   (**************************************************)
	   PutSquare(window, x1, y1, x3, y3, 256, FALSE);

	   prev_x2 := x3;
	   prev_y2 := y3;
	   first_time := FALSE;
	ELSE

	   (*******************************************)
	   (* eliminates flickering by comparing the  *)
	   (* current position with previous position *)
	   (*******************************************)
	   IF (x3 <> prev_x2) OR (y3 <> prev_y2) THEN

	  	(************************************************)
	   	(* re-subtracts the colour values of the pixels *)
	   	(* over which the square is to be drawn, by 255 *)
	   	(* thus restoring the previous colours    	*)
	   	(************************************************)
	   	PutSquare(window, x1, y1, prev_x2, prev_y2, 256, FALSE);

		(**************************************************)
	   	(* subtracts the colour values of the pixels over *)
	   	(* which the square is to be drawn, by 255        *)
	   	(**************************************************)
	   	PutSquare(window, x1, y1, x3, y3, 256, FALSE);
	   END;

	   temp_x := prev_x2;
	   temp_y := prev_y2;
	   prev_x2 := x3;
	   prev_y2 := y3;
	END;
   END;

   ReleaseScreen();

END SolidSquareRoutine;


(******************************************************************)
(* This procedure erases any unwanted parts of an image using the	*)
(* small size eraser with the current background colour.		*)
(******************************************************************)
PROCEDURE SmallEraserRoutine(window : GraphicalWindow);
VAR
   ok : BOOLEAN;
   i, j, x, y : INTEGER;
BEGIN

   (************************************)
   (* gets relative mouse co-ordinates *)
   (************************************)
   x := MouseParatemer_xoffset - 2;
   y := MouseParatemer_yoffset - 1;

   FOR j := 0 TO 3 DO
       FOR i := 0 TO 3 DO
	     curr_small_block[i,j].x := x + i;
	     curr_small_block[i,j].y := y + j;
       END;
   END;

   ok := ObtainScreen();

   IF first_time THEN

	(*****************************************************)
	(* draws initial eraser block with foreground colour *)
	(*****************************************************)
      PutRectangle(window, x, y, x+3, y+3, 15+16, TRUE);
      first_time := FALSE;
   ELSE

	(************************************************)
	(* approximates mouse trail by joinning current *)
	(* eraser block with the previous eraser block  *)
	(************************************************)
      FOR j := 0 TO 3 DO
          FOR i := 0 TO 3 DO
	        PutLine(window, prev_small_block[i,j].x, prev_small_block[i,j].y,
		  curr_small_block[i,j].x, curr_small_block[i,j].y, 15+16);
          END;
      END;
   END;

   ReleaseScreen();
   
   FOR j := 0 TO 3 DO
       FOR i := 0 TO 3 DO
	     prev_small_block[i,j].x := curr_small_block[i,j].x;
	     prev_small_block[i,j].y := curr_small_block[i,j].y;
       END;
   END;

END SmallEraserRoutine;


(******************************************************************)
(* This procedure erases any unwanted parts of an image using the	*)
(* medium size eraser with the current background colour.		*)
(******************************************************************)
PROCEDURE MediumEraserRoutine(window : GraphicalWindow);
VAR
   ok : BOOLEAN;
   i, j, x, y : INTEGER;
BEGIN

   (************************************)
   (* gets relative mouse co-ordinates *)
   (************************************)
   x := MouseParatemer_xoffset - 3;
   y := MouseParatemer_yoffset - 2;

   FOR j := 0 TO 5 DO
       FOR i := 0 TO 5 DO
	     curr_medium_block[i,j].x := x + i;
	     curr_medium_block[i,j].y := y + j;
       END;
   END;

   ok := ObtainScreen();

   IF first_time THEN

	(*****************************************************)
	(* draws initial eraser block with foreground colour *)
	(*****************************************************)
      PutRectangle(window, x, y, x+5, y+5, 15+16, TRUE);
      first_time := FALSE;
   ELSE

	(************************************************)
	(* approximates mouse trail by joinning current *)
	(* eraser block with the previous eraser block  *)
	(************************************************)
      FOR j := 0 TO 5 DO
          FOR i := 0 TO 5 DO
	        PutLine(window, prev_medium_block[i,j].x, prev_medium_block[i,j].y,
		  curr_medium_block[i,j].x, curr_medium_block[i,j].y, 15+16);
          END;
      END;
   END;

   ReleaseScreen();

   FOR j := 0 TO 5 DO
       FOR i := 0 TO 5 DO
	     prev_medium_block[i,j].x := curr_medium_block[i,j].x;
	     prev_medium_block[i,j].y := curr_medium_block[i,j].y;
       END;
   END;

END MediumEraserRoutine;


(******************************************************************)
(* This procedure erases any unwanted parts of an image using the	*)
(* large size eraser with the current background colour.		*)
(******************************************************************)
PROCEDURE LargeEraserRoutine(window : GraphicalWindow);
VAR
   ok : BOOLEAN;
   i, j, x, y : INTEGER;
BEGIN

   (************************************)
   (* gets relative mouse co-ordinates *)
   (************************************)
   x := MouseParatemer_xoffset - 4;
   y := MouseParatemer_yoffset - 3;

   FOR j := 0 TO 7 DO
       FOR i := 0 TO 7 DO
	     curr_large_block[i,j].x := x + i;
	     curr_large_block[i,j].y := y + j;
       END;
   END;

   ok := ObtainScreen();

   IF first_time THEN

	(*****************************************************)
	(* draws initial eraser block with foreground colour *)
	(*****************************************************)
      PutRectangle(window, x, y, x+7, y+7, 15+16, TRUE);
      first_time := FALSE;
   ELSE

	(************************************************)
	(* approximates mouse trail by joinning current *)
	(* eraser block with the previous eraser block  *)
	(************************************************)
      FOR j := 0 TO 7 DO
          FOR i := 0 TO 7 DO
	        PutLine(window, prev_large_block[i,j].x, prev_large_block[i,j].y,
		  curr_large_block[i,j].x, curr_large_block[i,j].y, 15+16);
          END;
      END;
   END;

   ReleaseScreen();

   FOR j := 0 TO 7 DO
       FOR i := 0 TO 7 DO
	     prev_large_block[i,j].x := curr_large_block[i,j].x;
	     prev_large_block[i,j].y := curr_large_block[i,j].y;
       END;
   END;

END LargeEraserRoutine;


(******************************************************************)
(* This procedure draws small size round dots continuously on the *)
(* specified window with the foreground colour.				*)
(******************************************************************)
PROCEDURE SmallBrushRoutine(window : GraphicalWindow);
VAR
   ok : BOOLEAN;
   i, j, x, y : INTEGER;
BEGIN

   (************************************)
   (* gets relative mouse co-ordinates *)
   (************************************)
   x := MouseParatemer_xoffset - 2;
   y := MouseParatemer_yoffset - 1;

   FOR j := 0 TO 3 DO
       FOR i := 0 TO 3 DO
	     IF NOT((i = 0) AND (j = 0)) AND NOT((i = 3) AND (j = 0)) AND
	        NOT((i = 0) AND (j = 3)) AND NOT((i = 3) AND (j = 3)) THEN
	   	  curr_medium_brush[i,j].x := x + i;
	   	  curr_medium_brush[i,j].y := y + j;
	     END;
       END;
   END;

   ok := ObtainScreen();

   IF first_time THEN

	(**************************************************)
	(* draws initial brush dot with foreground colour *)
	(**************************************************)
	FOR j := 0 TO 3 DO
          FOR i := 0 TO 3 DO
		  IF NOT((i = 0) AND (j = 0)) AND NOT((i = 3) AND (j = 0)) AND
		     NOT((i = 0) AND (j = 3)) AND NOT((i = 3) AND (j = 3)) THEN
		     PutDot(window, x+i, y+j, ForegroundColour);
		  END;
	    END;
	END;
      first_time := FALSE;
   ELSE

	(************************************************)
	(* approximates mouse trail by joinning current *)
	(* brush dot with the previous brush dot	      *)
	(************************************************)
      FOR j := 0 TO 3 DO
          FOR i := 0 TO 3 DO
		  IF NOT((i = 0) AND (j = 0)) AND NOT((i = 3) AND (j = 0)) AND
		     NOT((i = 0) AND (j = 3)) AND NOT((i = 3) AND (j = 3)) THEN
	           PutLine(window, prev_medium_brush[i,j].x, prev_medium_brush[i,j].y,
		             curr_medium_brush[i,j].x, curr_medium_brush[i,j].y, ForegroundColour);
		  END;
          END;
      END;
   END;

   ReleaseScreen();

   FOR j := 0 TO 3 DO
       FOR i := 0 TO 3 DO
	     IF NOT((i = 0) AND (j = 0)) AND NOT((i = 3) AND (j = 0)) AND
	        NOT((i = 0) AND (j = 3)) AND NOT((i = 3) AND (j = 3)) THEN
	   	  prev_medium_brush[i,j].x := curr_medium_brush[i,j].x;
	   	  prev_medium_brush[i,j].y := curr_medium_brush[i,j].y;
	     END;
       END;
   END;

END SmallBrushRoutine;


(******************************************************************)
(* This procedure draws medium size round dots continuously on 	*)
(* the specified window with the foreground colour.			*)
(******************************************************************)
PROCEDURE MediumBrushRoutine(window : GraphicalWindow);
VAR
   ok : BOOLEAN;
   i, j, x, y : INTEGER;
BEGIN

   (************************************)
   (* gets relative mouse co-ordinates *)
   (************************************)
   x := MouseParatemer_xoffset - 3;
   y := MouseParatemer_yoffset - 3;

   FOR j := 0 TO 6 DO
       FOR i := 0 TO 6 DO
	     IF NOT((i = 0) AND (j = 0)) AND NOT((i = 1) AND (j = 0)) AND NOT((i = 0) AND (j = 1)) AND
		  NOT((i = 5) AND (j = 0)) AND NOT((i = 6) AND (j = 0)) AND NOT((i = 6) AND (j = 1)) AND
	 	  NOT((i = 0) AND (j = 5)) AND NOT((i = 0) AND (j = 6)) AND NOT((i = 1) AND (j = 6)) AND
	        NOT((i = 6) AND (j = 5)) AND NOT((i = 6) AND (j = 6)) AND NOT((i = 5) AND (j = 6)) THEN
	   	  curr_large_brush[i,j].x := x + i;
	   	  curr_large_brush[i,j].y := y + j;
	     END;
       END;
   END;

   ok := ObtainScreen();

   IF first_time THEN

	(**************************************************)
	(* draws initial brush dot with foreground colour *)
	(**************************************************)
	FOR j := 0 TO 6 DO
          FOR i := 0 TO 6 DO
		  IF NOT((i = 0) AND (j = 0)) AND NOT((i = 1) AND (j = 0)) AND NOT((i = 0) AND (j = 1)) AND
		     NOT((i = 5) AND (j = 0)) AND NOT((i = 6) AND (j = 0)) AND NOT((i = 6) AND (j = 1)) AND
		     NOT((i = 0) AND (j = 5)) AND NOT((i = 0) AND (j = 6)) AND NOT((i = 1) AND (j = 6)) AND
	           NOT((i = 6) AND (j = 5)) AND NOT((i = 6) AND (j = 6)) AND NOT((i = 5) AND (j = 6)) THEN
		    PutDot(window, x+i, y+j, ForegroundColour);
		  END;
	    END;
	END;
      first_time := FALSE;
   ELSE

	(************************************************)
	(* approximates mouse trail by joinning current *)
	(* brush dot with the previous brush dot	      *)
	(************************************************)
      FOR j := 0 TO 6 DO
          FOR i := 0 TO 6 DO
		  IF NOT((i = 0) AND (j = 0)) AND NOT((i = 1) AND (j = 0)) AND NOT((i = 0) AND (j = 1)) AND
		     NOT((i = 5) AND (j = 0)) AND NOT((i = 6) AND (j = 0)) AND NOT((i = 6) AND (j = 1)) AND
		     NOT((i = 0) AND (j = 5)) AND NOT((i = 0) AND (j = 6)) AND NOT((i = 1) AND (j = 6)) AND
	           NOT((i = 6) AND (j = 5)) AND NOT((i = 6) AND (j = 6)) AND NOT((i = 5) AND (j = 6)) THEN
	           PutLine(window, prev_large_brush[i,j].x, prev_large_brush[i,j].y,
		             curr_large_brush[i,j].x, curr_large_brush[i,j].y, ForegroundColour);
		  END;
          END;
      END;
   END;

   ReleaseScreen();

   FOR j := 0 TO 6 DO
       FOR i := 0 TO 6 DO
	     IF NOT((i = 0) AND (j = 0)) AND NOT((i = 1) AND (j = 0)) AND NOT((i = 0) AND (j = 1)) AND
		  NOT((i = 5) AND (j = 0)) AND NOT((i = 6) AND (j = 0)) AND NOT((i = 6) AND (j = 1)) AND
		  NOT((i = 0) AND (j = 5)) AND NOT((i = 0) AND (j = 6)) AND NOT((i = 1) AND (j = 6)) AND
	        NOT((i = 6) AND (j = 5)) AND NOT((i = 6) AND (j = 6)) AND NOT((i = 5) AND (j = 6)) THEN
	   	  prev_large_brush[i,j].x := curr_large_brush[i,j].x;
	   	  prev_large_brush[i,j].y := curr_large_brush[i,j].y;
	     END;
       END;
   END;

END MediumBrushRoutine;


(******************************************************************)
(* This procedure draws large size round dots continuously on 	*)
(* the specified window with the foreground colour.			*)
(******************************************************************)
PROCEDURE LargeBrushRoutine(window : GraphicalWindow);
VAR
   ok : BOOLEAN;
   i, j, x, y : INTEGER;
BEGIN

   (************************************)
   (* gets relative mouse co-ordinates *)
   (************************************)
   x := MouseParatemer_xoffset - 4;
   y := MouseParatemer_yoffset - 3;

   FOR j := 0 TO 7 DO
       FOR i := 0 TO 7 DO
	     IF NOT((i = 0) AND (j = 0)) AND NOT((i = 1) AND (j = 0)) AND NOT((i = 0) AND (j = 1)) AND
		  NOT((i = 6) AND (j = 0)) AND NOT((i = 7) AND (j = 0)) AND NOT((i = 7) AND (j = 1)) AND
		  NOT((i = 0) AND (j = 6)) AND NOT((i = 0) AND (j = 7)) AND NOT((i = 1) AND (j = 7)) AND
	        NOT((i = 7) AND (j = 6)) AND NOT((i = 7) AND (j = 7)) AND NOT((i = 6) AND (j = 7)) THEN
	   	  curr_large_brush[i,j].x := x + i;
	   	  curr_large_brush[i,j].y := y + j;
	     END;
       END;
   END;

   ok := ObtainScreen();

   IF first_time THEN

	(**************************************************)
	(* draws initial brush dot with foreground colour *)
	(**************************************************)
	FOR j := 0 TO 7 DO
          FOR i := 0 TO 7 DO
		  IF NOT((i = 0) AND (j = 0)) AND NOT((i = 1) AND (j = 0)) AND NOT((i = 0) AND (j = 1)) AND
		     NOT((i = 6) AND (j = 0)) AND NOT((i = 7) AND (j = 0)) AND NOT((i = 7) AND (j = 1)) AND
		     NOT((i = 0) AND (j = 6)) AND NOT((i = 0) AND (j = 7)) AND NOT((i = 1) AND (j = 7)) AND
	           NOT((i = 7) AND (j = 6)) AND NOT((i = 7) AND (j = 7)) AND NOT((i = 6) AND (j = 7)) THEN
		     PutDot(window, x+i, y+j, ForegroundColour);
		  END;
	    END;
	END;
      first_time := FALSE;
   ELSE

	(************************************************)
	(* approximates mouse trail by joinning current *)
	(* brush dot with the previous brush dot	      *)
	(************************************************)
      FOR j := 0 TO 7 DO
          FOR i := 0 TO 7 DO
		  IF NOT((i = 0) AND (j = 0)) AND NOT((i = 1) AND (j = 0)) AND NOT((i = 0) AND (j = 1)) AND
		     NOT((i = 6) AND (j = 0)) AND NOT((i = 7) AND (j = 0)) AND NOT((i = 7) AND (j = 1)) AND
		     NOT((i = 0) AND (j = 6)) AND NOT((i = 0) AND (j = 7)) AND NOT((i = 1) AND (j = 7)) AND
	           NOT((i = 7) AND (j = 6)) AND NOT((i = 7) AND (j = 7)) AND NOT((i = 6) AND (j = 7)) THEN
	           PutLine(window, prev_large_brush[i,j].x, prev_large_brush[i,j].y,
		             curr_large_brush[i,j].x, curr_large_brush[i,j].y, ForegroundColour);
		  END;
          END;
      END;
   END;

   ReleaseScreen();

   FOR j := 0 TO 7 DO
       FOR i := 0 TO 7 DO
	     IF NOT((i = 0) AND (j = 0)) AND NOT((i = 1) AND (j = 0)) AND NOT((i = 0) AND (j = 1)) AND
	 	  NOT((i = 6) AND (j = 0)) AND NOT((i = 7) AND (j = 0)) AND NOT((i = 7) AND (j = 1)) AND
		  NOT((i = 0) AND (j = 6)) AND NOT((i = 0) AND (j = 7)) AND NOT((i = 1) AND (j = 7)) AND
	        NOT((i = 7) AND (j = 6)) AND NOT((i = 7) AND (j = 7)) AND NOT((i = 6) AND (j = 7)) THEN
	   	  prev_large_brush[i,j].x := curr_large_brush[i,j].x;
	   	  prev_large_brush[i,j].y := curr_large_brush[i,j].y;
	     END;
       END;
   END;

END LargeBrushRoutine;


(******************************************************************)
(* This procedure "sprays" groups of random pixels continuously	*)
(* on the specified window with the current foreground colour.	*)
(******************************************************************)
PROCEDURE AirBrushRoutine(window : GraphicalWindow);
VAR
   ok : BOOLEAN;
   i, j, x, y, temp_x, temp_y : INTEGER;
BEGIN

   (************************************)
   (* gets relative mouse co-ordinates *)
   (************************************)
   x := MouseParatemer_xoffset - 15;
   y := MouseParatemer_yoffset - 4;

   ok := ObtainScreen();

   (******************************)
   (* generates 25 random pixels *)
   (******************************)
   FOR i := 1 TO 25 DO
	 temp_x := RANDOM(14);
	 temp_y := RANDOM(14);

	 PutDot(window,x+temp_x,y+temp_y,ForegroundColour);
   END;

   ReleaseScreen();

END AirBrushRoutine;


(******************************************************************)
(* This procedure probes a pixel on the screen and returns its	*)
(* colour number.									*)
(******************************************************************)
PROCEDURE DropperRoutine();
VAR
   x, y : CARDINAL;
   idx : SHORTCARD;
BEGIN

   (************************************)
   (* gets absolute mouse co-ordinates *)
   (************************************)
   x := CARDINAL(MouseParatemer_xoffset) + CanvasLeft;
   y := CARDINAL(MouseParatemer_yoffset) + CanvasTop;

   idx := SHORTCARD(Point(x,y));
   UpdateForegroundColour(idx);

   ForegroundColour := CARDINAL(idx);

END DropperRoutine;


(*
PROCEDURE GetOldColour(x,y : INTEGER) : SHORTCARD;
BEGIN
   RETURN (SHORTCARD(Point(CARDINAL(x)+CanvasLeft,CARDINAL(y)+CanvasTop)));
END GetOldColour;


PROCEDURE FloodFill(window : GraphicalWindow; x,y : INTEGER; old_colour,new_colour : SHORTCARD);
VAR
   x1, y1 : INTEGER;
BEGIN

   IF SHORTCARD(Point(CARDINAL(x)+CanvasLeft,CARDINAL(y)+CanvasTop)) = old_colour THEN
	PutDot(window,x,y,CARDINAL(new_colour));
	FloodFill(window,x-1,y,old_colour,new_colour);
	FloodFill(window,x+1,y,old_colour,new_colour);
	FloodFill(window,x,y-1,old_colour,new_colour);
	FloodFill(window,x,y+1,old_colour,new_colour);
   END;

END FloodFill;
*)


(******************************************************************)
(* This procedure retuns the absolute co-ordinates of the		*)
(* upper-left corner of the selection rectangle.			*)
(******************************************************************)
PROCEDURE GetMarqueex1y1();
BEGIN
   (************************************)
   (* gets relative mouse co-ordinates *)
   (************************************)
   x1 := MouseParatemer_xoffset;
   y1 := MouseParatemer_yoffset;
   marquee_x1 := x1+INTEGER(CanvasLeft);
   marquee_y1 := y1+INTEGER(CanvasTop);
   first_time := TRUE;
END GetMarqueex1y1;


(******************************************************************)
(* This procedure retuns the absolute co-ordinates of the		*)
(* lower-right corner of the selection rectangle.			*)
(******************************************************************)
PROCEDURE GetMarqueex2y2();
BEGIN
   (************************************)
   (* gets relative mouse co-ordinates *)
   (************************************)
   x2 := MouseParatemer_xoffset;
   y2 := MouseParatemer_yoffset;
   marquee_x2 := x2+INTEGER(CanvasLeft);
   marquee_y2 := y2+INTEGER(CanvasTop);
END GetMarqueex2y2;


(******************************************************************)
(* This procedure selects a rectangular area.				*)
(******************************************************************)
PROCEDURE MarqueeRoutine(window : GraphicalWindow; done : BOOLEAN);
VAR
   ok : BOOLEAN;
   x3, y3 : INTEGER;
BEGIN

   (************************************)
   (* gets relative mouse co-ordinates *)
   (************************************)
   x3 := MouseParatemer_xoffset;
   y3 := MouseParatemer_yoffset;

   ok := ObtainScreen();
   IF done THEN
	IF NOT(first_time) THEN
	   PutRectangle(window, x1, y1, prev_x2, prev_y2, 256, FALSE);
	   GetMarqueex2y2();
	   PutRectangle(window, x1, y1, x2, y2, 256, FALSE);
	END;
	first_time := TRUE;
   ELSE
	IF first_time THEN
	   PutRectangle(window, x1, y1, x3, y3, 256, FALSE);
	   prev_x2 := x3;
	   prev_y2 := y3;
	   first_time := FALSE;
	ELSE

	   (*******************************************)
	   (* eliminates flickering by comparing the  *)
	   (* current position with previous position *)
	   (*******************************************)
	   IF (x3 <> prev_x2) OR (y3 <> prev_y2) THEN
	   	PutRectangle(window, x1, y1, prev_x2, prev_y2, 256, FALSE);
	   	PutRectangle(window, x1, y1, x3, y3, 256, FALSE);
	   END;

	   temp_x := prev_x2;
	   temp_y := prev_y2;
	   prev_x2 := x3;
	   prev_y2 := y3;
	END;
   END;

   ReleaseScreen();

END MarqueeRoutine;


(******************************************************************)
(* This procedure reads keyboard inputs to the Text tool input	*)
(* dialog.										*)
(******************************************************************)
PROCEDURE ReadText(str_ptr : DialogStringPointer; width : INTEGER; str_leng : INTEGER;
                   string : LongStrPtr; x_offset,y_offset : INTEGER) : BOOLEAN;
VAR
   ok : BOOLEAN;
BEGIN

   (**********************)
   (* initilaises buffer *)
   (**********************)
   ClearTextBuffer();

   SetOtherInput(TextDialog,1,String,x_offset,y_offset+CharHeight,0,0,width, str_leng,1+7);

   DisplayDialog(TextDialog, TRUE);

   IF (ObtainScreen()) THEN
      PutTitleStrip(DialogWindow(TextDialog),"Add Text");
    	PutString(DialogWindow(TextDialog),string^,3,y_offset+CharHeight,1+7,FALSE);
    	ReleaseScreen();
   END; (* IF *)

   ok := (DialogSelection(TextDialog,str_ptr) = 1);

   HideDialog(TextDialog);

   RETURN (ok);
END ReadText;


(******************************************************************)
(* This procedure stores the Text tool input string to a text	*)
(* array.										*)
(******************************************************************)
PROCEDURE InputText();
VAR
   ok : BOOLEAN;
BEGIN
   ok := ReadText(ADR(InputString),MaxStringLength,HIGH(InputString),ADR("Text: "),
		     	   11*CharWidth-40,10);
END InputText;


(******************************************************************)
(* This procedure places the Text tool string onto the specified	*)
(* window with the current foreground colour.				*)
(******************************************************************)
PROCEDURE TextRoutine(window : GraphicalWindow);
VAR
   ok : BOOLEAN;
   x,y : INTEGER;
BEGIN

   (************************************)
   (* gets relative mouse co-ordinates *)
   (************************************)
   x := MouseParatemer_xoffset + 2;
   y := MouseParatemer_yoffset + 1;

   ok := ObtainScreen();

   (*******************************************)
   (* places string onto the specified window *)
   (*******************************************)
   PutString(window, InputString, x, y, ForegroundColour, FALSE);
   ReleaseScreen();
END TextRoutine;


(******************************************************************)
(* This procedure clears the text array by assigning the first	*)
(* element with the NULL character.						*)
(******************************************************************)
PROCEDURE ClearTextBuffer();
BEGIN
   InputString[0] := NULL;
END ClearTextBuffer;


(*******************)
(* Initialisations *)
(*******************)
BEGIN

   (**********************************************)
   (* initialises pseudo-random number generator *)
   (**********************************************)
   RANDOMIZE();

   started := FALSE;
   stop := FALSE;

   (*first_time := TRUE;*)
   first_poly := FALSE;
   first_time := FALSE;

   (**************************************)
   (* creates the Text tool input dialog *)
   (**************************************)
   CreateDialog(TextDialog, NIL,WithOtherInput,1,100-7,100+23,7,1+7,1+7,FALSE,NIL,
                220+340-24+1,60-20+10,6*CharWidth+1,CharHeight,5,5);
   AddABox(TextDialog,ADR("  Ok"),1+2,15-7,TRUE);
   AddABox(TextDialog,ADR("Cancel"),1,4+4,TRUE);
   EndCreateDialog(TextDialog);

   marquee_x1 := -1;
   marquee_y1 := -1;
   marquee_x2 := -1;
   marquee_y2 := -1;
   
END Toolbox.