IMPLEMENTATION MODULE WriteBMP;

(******************************************************************)
(*											*)
(*			  	 BMP Encoding Module				*)
(*											*)
(*	Programmer :	Matthew Ma (c9315720)				*)
(*											*)
(*	Last edited :	22-10-97						*)
(*											*)
(*	Function : 		This module saves a screen image to a BMP	*)
(*				file.							*)
(*											*)
(*	Status :		Working.						*)
(******************************************************************)


FROM XMS IMPORT
(* proc *)   ObtainXMSMemory, ReleaseXMSMemory;

FROM Canvas IMPORT
(* var  *)   CanvasLeft, CanvasTop, CanvasRight, CanvasBottom;

FROM FIO IMPORT
(* type *)   File, Exists;

FROM FileSys IMPORT
(* proc *)   OpenFile, CloseFile, ReadByte, WriteByte,
             SetPosition, SavePosition, FileSize, ReadRecord, WriteRecord;

FROM GraphicWindow IMPORT
(* var *)    PutDot, XMAX, YMAX, CharWidth, CharHeight,
(* proc *)   ObtainScreen, ReleaseScreen, PutRectangle, PutLine;

FROM IOErrorCodes IMPORT
(* type *)   ErrorCode;

FROM Support IMPORT
(* proc *)   WriteWord, WriteLongWord;

FROM ColourRoutines IMPORT
(* type *)   palette;

FROM Graphics IMPORT
(* proc *)   Point;

FROM InfoDialogs IMPORT
(* proc *)   CantCreateFile;

FROM SoundEffects IMPORT
(* proc *)   Beep;



(******************************************************************)
(* This procedure saves the screen area specified by x1, y1, x2 	*)
(* and y2 to a 256-colour BMP file with the given file name; x1,	*)
(* y1 are the co-ordinates of the upper-left corner while x2, y2 	*)
(* are the co-ordinates of the lower-right corner.			*)
(******************************************************************)
PROCEDURE Write256BMP(x1, y1, x2, y2 : CARDINAL; name : ARRAY OF CHAR);
VAR
   f : File;
   ok : BOOLEAN;
   i, j, count, x_count, y_count : CARDINAL;
   x_size, y_size, file_size, x_size2 : LONGCARD;
   dummy : BYTE;
   blanks : CARDINAL;
   error : ErrorCode;
   row : POINTER TO ARRAY [0..1279] OF BYTE;
BEGIN
   IF (OpenFile(f, name, TRUE) = OK) THEN

	(*************************)
   	(* writes the BMP header *)
   	(*************************)
      error := WriteByte(f, BYTE("B"));
      error := WriteByte(f, BYTE("M"));

	(*******************************)
   	(* calculates image dimensions *)
   	(*******************************)
      x_size := LONGCARD(x2-x1+1);
      y_size := LONGCARD(y2-y1+1);

	(**********************************************)
	(* BMP scan lines must end on a LONG boubdary *)
	(**********************************************)
      x_size2 := ((x_size + 3) DIV 4) * 4;
      blanks := CARDINAL(x_size2 - x_size);

      file_size := LONGCARD(54+256*4+x_size2*y_size);
      WriteLongWord(f, file_size);

      WriteWord(f, 0);
      WriteWord(f, 0);

      WriteLongWord(f, 1078);

      WriteLongWord(f, 40);

      WriteLongWord(f, x_size);
      WriteLongWord(f, y_size);

      WriteWord(f, 1);			(* Planes *)

      WriteWord(f, 8);			(* BitCount *)

      WriteLongWord(f, 0);		(* Compression *);

      (*WriteLongWord(f, LONGCARD(x_size2*y_size));*)
      WriteLongWord(f, x_size2*y_size);

      WriteLongWord(f, 0);		(* XPelsPerMeter *)

      WriteLongWord(f, 0);

      WriteLongWord(f, 256);

      WriteLongWord(f, 256);


	(**************************)
   	(* writes palette entries *)
   	(**************************)
      FOR count := 0 TO 255 DO
	    error := WriteByte(f, VAL(BYTE, palette[count].blue));
	    error := WriteByte(f, VAL(BYTE, palette[count].green));
	    error := WriteByte(f, VAL(BYTE, palette[count].red));
	    error := WriteByte(f, VAL(BYTE, 0));
      END; (* FOR *)


      x_count := 0;
      y_count := CARDINAL(y_size);

	row := ObtainXMSMemory();

      ok := ObtainScreen();

      LOOP
	   (****************************************)
	   (* reads colour number of current pixel *)
	   (****************************************)
         row^[x_count] := VAL(BYTE, SHORTCARD(Point(CARDINAL(x_count+x1), CARDINAL(y_count+y1-1))));
         INC(x_count);
         IF LONGCARD(x_count) = x_size THEN

		(*****************************************)
		(* writes padding bytes to make sure BMP *)
		(* scan lines end on a LONG boundary.    *)
		(*****************************************)
	   	CASE blanks OF
		  	  1:
				row^[x_count] := VAL(BYTE, 0);

			| 2:
				row^[x_count] := VAL(BYTE, 0);
				INC(x_count);
				row^[x_count] := VAL(BYTE, 0);

			| 3:
				row^[x_count] := VAL(BYTE, 0);
				INC(x_count);
				row^[x_count] := VAL(BYTE, 0);
				INC(x_count);
				row^[x_count] := VAL(BYTE, 0);

			ELSE
		END;

	   	error := WriteRecord(f, row, CARDINAL(x_size2));

	   	x_count := 0;

	   	DEC(y_count);
	   	IF LONGCARD(y_count) = 0 THEN		(* reached last scan line *)
	         EXIT;					(* terminates encoding process *)
	   	END;

	   END;
   	END; (* LOOP *)

      ReleaseScreen();

   	ReleaseXMSMemory();

   ELSE

	Beep();
	CantCreateFile();

   END; (* IF *)

   CloseFile(f);

END Write256BMP;



END WriteBMP.