IMPLEMENTATION MODULE WritePCX;

(******************************************************************)
(*											*)
(*			  	 PCX Encoding Module				*)
(*											*)
(*	Programmer :	Matthew Ma (c9315720)				*)
(*											*)
(*	Last edited :	22-10-97						*)
(*											*)
(*	Function : 		This module saves a screen image to a PCX	*)
(*				file.							*)
(*											*)
(*	Status :		Working.						*)
(******************************************************************)


FROM XMS IMPORT
(* proc *)   ObtainXMSMemory, ReleaseXMSMemory;

FROM Canvas IMPORT
(* var  *)   CanvasLeft, CanvasTop, CanvasRight, CanvasBottom;

FROM FIO IMPORT
(* type *)   File, Exists;

FROM FileSys IMPORT
(* proc *)   OpenFile, CloseFile, ReadByte, WriteByte,
             SetPosition, SavePosition, FileSize, ReadRecord, WriteRecord;

FROM GraphicWindow IMPORT
(* var  *)   PutDot, XMAX, YMAX, CharWidth, CharHeight,
(* proc *)   ObtainScreen, ReleaseScreen, PutRectangle, PutLine;

FROM IOErrorCodes IMPORT
(* type *)   ErrorCode;

FROM Support IMPORT
(* proc *)   WriteWord, WriteLongWord;

FROM ColourRoutines IMPORT
(* type *)   palette;

FROM Graphics IMPORT
(* proc *)   Point;

FROM InfoDialogs IMPORT
(* proc *)   CantCreateFile;

FROM SoundEffects IMPORT
(* proc *)   Beep;


CONST
   block_size = 65534;



(******************************************************************)
(* This procedure saves the screen area specified by x1, y1, x2 	*)
(* and y2 to a 256-colour compressed PCX file with the given file *)
(* name; x1, y1 are the co-ordinates of the upper-left corner	*)
(* while x2, y2 are the co-ordinates of the lower-right corner.	*)
(******************************************************************)
PROCEDURE Write256PCX(x1, y1, x2, y2 : CARDINAL; name : ARRAY OF CHAR; show_cursor : BOOLEAN);
VAR
   f : File;
   ok : BOOLEAN;
   i, j, count, x_count, y_count, repeat : CARDINAL;
   x_size, y_size, x_size2 : CARDINAL;
   pixel, head : SHORTCARD;
   dummy : BYTE;
   blanks : CARDINAL;
   error : ErrorCode;
   row : POINTER TO ARRAY [0..block_size-1] OF BYTE;
BEGIN
   IF (OpenFile(f, name, TRUE) = OK) THEN
      error := WriteByte(f, BYTE(10));	(* manufacturer *)
      error := WriteByte(f, BYTE(5));	(* version 5 *)
      error := WriteByte(f, BYTE(1));	(* run length encoding *)
      error := WriteByte(f, BYTE(8));	(* bits per pixel *)

      x_size := x2-x1+1;
      y_size := y2-y1+1;

      WriteWord(f, 0);		(* xmin *)
      WriteWord(f, 0);		(* ymin *)
      WriteWord(f, x_size-1);	(* xmax *)
      WriteWord(f, y_size-1);	(* ymax *)

      WriteWord(f, 75);		(* Horizontal dpi *)
      WriteWord(f, 75);		(* Vertical dpi *)

      FOR i := 0 TO 48-1 DO
	  error := WriteByte(f, BYTE(0));
      END;

      error := WriteByte(f, BYTE(0));	(* reserverd - offset 64 *)
      error := WriteByte(f, BYTE(1));	(* planes *)

      IF ODD(x_size) THEN
	   WriteWord(f, x_size+1);		(* scan line length must be even *)
      ELSE
	   WriteWord(f, x_size);
      END;

      WriteWord(f, 1);		(* palette info *)
      WriteWord(f, 640);	(* horizontal screen size *)
      WriteWord(f, 480);	(* vertical screen size *)

	(*******************************************)
	(* fills in the last 54 bytes with value 0 *)
	(* to complete the 128-byte file header	 *)
	(*******************************************)
      FOR i := 0 TO 54-1 DO
	    error := WriteByte(f, BYTE(0));
      END;
      (* end of 128-byte header *)


      i := 0;
      repeat := 0;
      x_count := 0;
      y_count := 0;

      row := ObtainXMSMemory();

      IF NOT(show_cursor) THEN
         ok := ObtainScreen();
      END;

	(******************************)
	(* saving image in PCX format *)
	(******************************)
      LOOP (* 1 *)
	   pixel := SHORTCARD(Point(CARDINAL(x_count+x1), CARDINAL(y_count+y1)));

	   IF (repeat = 0) THEN
	    	head := pixel;
	   END;

	   INC(x_count);

	   IF (pixel = head) THEN
	    	INC(repeat);

	     	(****************************************************)
	     	(* consecutive pixels with same colours can only be *)
	     	(* repeated maximum 63 times or to the end of the   *)
	    	(* scan line 						    *)
	    	(****************************************************)
	     	IF (repeat = 63) OR (x_count = x_size) THEN
	         row^[i] := BYTE(192+repeat);	(* repeat more than once *)
	         INC(i);
	         row^[i] := BYTE(head);
	         INC(i);

	        repeat := 0;
	    	END;

	   ELSE (* pixel <> head *)

	    	IF repeat = 1 THEN
	         IF head >= 192 THEN
	          	row^[i] := BYTE(192+1);	(* repaet once *)
	          	INC(i);
	          	row^[i] := BYTE(head);
	          	INC(i);
	         ELSE
	          	row^[i] := BYTE(head);
	          	INC(i);
	         END;
	    	ELSE (* repeat > 1 *)
	         row^[i] := BYTE(192+repeat);	(* repaet more than once *)
	         INC(i);
	         row^[i] := BYTE(head);
	         INC(i);
	    	END;

	    	IF x_count = x_size THEN	(* reached last pixel of scan line *)
	         IF pixel >= 192 THEN
	          	row^[i] := BYTE(192+1);	(* repaet once *)
	          	INC(i);
	          	row^[i] := BYTE(pixel);
	          	INC(i);
	         ELSE
	          	row^[i] := BYTE(pixel);
	          	INC(i);
	         END;
	    	END;

	    	IF x_count < x_size THEN
	         repeat := 1;
	         head := pixel;
	    	END;
	    
	   END; (* IF *)


	   IF (x_count = x_size) THEN

	    	error := WriteRecord(f, row, i);
	    	IF ODD(x_size) THEN
	         error := WriteByte(f, BYTE(0));
	    	END;
	      
	    	i := 0;
	    	repeat := 0;

	    	x_count := 0;

	    	INC(y_count);
	    	IF y_count = y_size THEN
	         EXIT; (* LOOP 1 *)
	    	END;
	   END;

      END; (* LOOP 1 *)

      IF NOT(show_cursor) THEN
         ReleaseScreen();
      END;

      ReleaseXMSMemory();


      error := WriteByte(f, BYTE(12));	(* indicate palette present *)

	(**************************)
	(* writes palette entries *)
	(**************************)
      FOR i := 0 TO 255 DO
	  error := WriteByte(f, VAL(BYTE, palette[i].red));
	  error := WriteByte(f, VAL(BYTE, palette[i].green));
	  error := WriteByte(f, VAL(BYTE, palette[i].blue));
      END;

   ELSE

	Beep();
	CantCreateFile();

   END; (* IF *)

   CloseFile(f);

END Write256PCX;


END WritePCX.