IMPLEMENTATION MODULE WriteTGA;

(******************************************************************)
(*											*)
(*			  	 TGA Encoding Module				*)
(*											*)
(*	Programmer :	Matthew Ma (c9315720)				*)
(*											*)
(*	Last edited :	22-10-97						*)
(*											*)
(*	Function : 		This module saves a screen image to a TGA	*)
(*				file.							*)
(*											*)
(*	Status :		Working.						*)
(******************************************************************)


FROM XMS IMPORT
(* proc *)   ObtainXMSMemory, ReleaseXMSMemory;

FROM Canvas IMPORT
(* var  *)   CanvasLeft, CanvasTop, CanvasRight, CanvasBottom;

FROM FIO IMPORT
(* type *)   File, Exists;

FROM FileSys IMPORT
(* proc *)   OpenFile, CloseFile, ReadByte, WriteByte,
             SetPosition, SavePosition, FileSize, ReadRecord, WriteRecord;

FROM GraphicWindow IMPORT
(* var *)    PutDot, XMAX, YMAX, CharWidth, CharHeight,
(* proc *)   ObtainScreen, ReleaseScreen, PutRectangle, PutLine;

FROM IOErrorCodes IMPORT
(* type *)   ErrorCode;

FROM Support IMPORT
(* proc *)   WriteWord, WriteLongWord;

FROM ColourRoutines IMPORT
(* type *)   palette;

FROM Graphics IMPORT
(* proc *)   Point;

FROM InfoDialogs IMPORT
(* proc *)   CantCreateFile;

FROM SoundEffects IMPORT
(* proc *)   Beep;



(******************************************************************)
(* This procedure saves the screen area specified by x1, y1, x2 	*)
(* and y2 to a 256-colour TGA file with the given file name; x1,	*)
(* y1 are the co-ordinates of the upper-left corner while x2, y2 	*)
(* are the co-ordinates of the lower-right corner.			*)
(******************************************************************)
PROCEDURE Write256TGA(x1, y1, x2, y2 : CARDINAL; name : ARRAY OF CHAR);
VAR
   f : File;
   ok : BOOLEAN;
   i, j, count, x_count, y_count : CARDINAL;
   x_size, y_size, file_size : CARDINAL;
   dummy : BYTE;
   error : ErrorCode;
   row : POINTER TO ARRAY [0..1279] OF BYTE;
BEGIN
   IF (OpenFile(f, name, TRUE) = OK) THEN

	(*************************)
   	(* writes the TGA header *)
   	(*************************)
      error := WriteByte(f, BYTE(0));
      error := WriteByte(f, BYTE(1));
      error := WriteByte(f, BYTE(1));

      WriteWord(f, 0);
      WriteWord(f, 256);			(* colourmap length *)
      error := WriteByte(f, BYTE(24));	(* colourmap entry *)

      WriteWord(f, 0);				(* x origin *)
      WriteWord(f, 0);				(* y origin *)

	(*******************************)
   	(* calculates image dimensions *)
   	(*******************************)
      x_size := x2-x1+1;
      y_size := y2-y1+1;

      WriteWord(f, x_size);
      WriteWord(f, y_size);

      error := WriteByte(f, 8);		(* bits per pixel *)
      error := WriteByte(f, 0);


	(**************************)
   	(* writes palette entries *)
   	(**************************)
      FOR count := 0 TO 255 DO
	    error := WriteByte(f, VAL(BYTE, palette[count].blue));
	    error := WriteByte(f, VAL(BYTE, palette[count].green));
	    error := WriteByte(f, VAL(BYTE, palette[count].red));
      END;


      x_count := 0;
      y_count := y_size;

      row := ObtainXMSMemory();

      ok := ObtainScreen();

      LOOP
	   (****************************************)
	   (* reads colour number of current pixel *)
	   (****************************************)
         row^[x_count] := VAL(BYTE, SHORTCARD(Point(CARDINAL(x_count+x1), CARDINAL(y_count+y1-1))));
         INC(x_count);
         IF x_count = x_size THEN
	    	error := WriteRecord(f, row, x_size);

	    	x_count := 0;

	    	DEC(y_count);

	    	IF y_count = 0 THEN	(* reached last scan line *)
	         EXIT;			(* terminates encoding process *)
	    	END;
	   END;
      END; (* LOOP *)

      ReleaseScreen();

	ReleaseXMSMemory();

   ELSE

	Beep();
	CantCreateFile();

   END;
  
   CloseFile(f);

END Write256TGA;



END WriteTGA.