IMPLEMENTATION MODULE WriteTIF;

(******************************************************************)
(*											*)
(*			  	 TIF Encoding Module				*)
(*											*)
(*	Programmer :	Matthew Ma (c9315720)				*)
(*											*)
(*	Last edited :	22-10-97						*)
(*											*)
(*	Function : 		This module saves a screen image to a TIF	*)
(*				file.							*)
(*											*)
(*	Status :		Working.						*)
(******************************************************************)


FROM XMS IMPORT
(* proc *)   ObtainXMSMemory, ReleaseXMSMemory;

FROM Canvas IMPORT
(* var  *)   CanvasLeft, CanvasTop, CanvasRight, CanvasBottom;

FROM FIO IMPORT
(* type *)   File, Exists;

FROM FileSys IMPORT
(* proc *)   OpenFile, CloseFile, ReadByte, WriteByte,
             SetPosition, SavePosition, FileSize, ReadRecord, WriteRecord;

FROM GraphicWindow IMPORT
(* var *)    PutDot, XMAX, YMAX, CharWidth, CharHeight,
(* proc *)   ObtainScreen, ReleaseScreen, PutRectangle, PutLine;

FROM IOErrorCodes IMPORT
(* type *)   ErrorCode;

FROM Support IMPORT
(* proc *)   WriteWord, WriteLongWord;

FROM ColourRoutines IMPORT
(* type *)   palette;

FROM Graphics IMPORT
(* proc *)   Point;

FROM InfoDialogs IMPORT
(* proc *)   CantCreateFile;

FROM SoundEffects IMPORT
(* proc *)   Beep;



(******************************************************************)
(* This procedure saves the screen area specified by x1, y1, x2 	*)
(* and y2 to a 256-colour TIF file with the given file name; x1,	*)
(* y1 are the co-ordinates of the upper-left corner while x2, y2 	*)
(* are the co-ordinates of the lower-right corner.			*)
(******************************************************************)
PROCEDURE Write256TIF(x1, y1, x2, y2 : CARDINAL; name : ARRAY OF CHAR);
VAR
   f : File;
   ok : BOOLEAN;
   i, j, count, x_count, y_count : CARDINAL;
   x_size, y_size, file_size : CARDINAL;
   dummy : BYTE;
   error : ErrorCode;
   row : POINTER TO ARRAY [0..1279] OF BYTE;
BEGIN
   IF (OpenFile(f, name, TRUE) = OK) THEN

	(*************************)
   	(* writes the TIF header *)
   	(*************************)
      error := WriteByte(f, BYTE("I"));
      error := WriteByte(f, BYTE("I"));

      WriteWord(f, 42);
      WriteLongWord(f, 8);	(* first IFD *)

      WriteWord(f, 15-3);	(* IFD_entry_count *)

      x_size := x2-x1+1;
      y_size := y2-y1+1;


	(*********************)
   	(* writes TIF fields *)
   	(*********************)
      WriteWord(f, 254);	(* NewSubfileType *)
      WriteWord(f, 4);
      WriteLongWord(f, 1);
      WriteLongWord(f, 0);

      WriteWord(f, 256);	(* image width *)
      WriteWord(f, 3);
      WriteLongWord(f, 1);
      WriteLongWord(f, LONGCARD(x_size));

      WriteWord(f, 257);	(* image height *)
      WriteWord(f, 3);
      WriteLongWord(f, 1);
      WriteLongWord(f, LONGCARD(y_size));

      WriteWord(f, 258);
      WriteWord(f, 3);
      WriteLongWord(f, 1);
      WriteLongWord(f, 8);	(* bits-per-pixel *)

      WriteWord(f, 259);
      WriteWord(f, 3);
      WriteLongWord(f, 1);
      WriteLongWord(f, 1);	(* non-compressed *)

      WriteWord(f, 262);
      WriteWord(f, 3);
      WriteLongWord(f, 1);
      WriteLongWord(f, 3);	(* palette colour *)

      WriteWord(f, 273);
      WriteWord(f, 4);
      WriteLongWord(f, 1);		(* strips per image *)
      WriteLongWord(f, 1730-12*3);	(* strip offset *)

      WriteWord(f, 277);
      WriteWord(f, 3);
      WriteLongWord(f, 1);
      WriteLongWord(f, 1);	(* RGB *)

      WriteWord(f, 278);
      WriteWord(f, 3);
      WriteLongWord(f, 1);
      (*WriteLongWord(f, 1);	(* RowsPerStrip *)*)
      WriteLongWord(f, 8192);	(* RowsPerStrip *)

      WriteWord(f, 279);
      WriteWord(f, 4);
      WriteLongWord(f, 1);
      WriteLongWord(f, 1);	(* StripByteCounts *)

      (*WriteWord(f, 282);
      WriteWord(f, 5);
      WriteLongWord(f, 1);
      WriteLongWord(f, 194);	(* XResolution *)

      WriteWord(f, 283);
      WriteWord(f, 5);
      WriteLongWord(f, 1);
      WriteLongWord(f, 202);	(* YResolution *)*)

      WriteWord(f, 296);
      WriteWord(f, 3);
      WriteLongWord(f, 1);
      WriteLongWord(f, 1);	(* ResolutionUnit *)

      WriteWord(f, 320);
      WriteWord(f, 3);
      WriteLongWord(f, 768);
      WriteLongWord(f, 194-12*3);	(* ColorMap *)

      (*WriteWord(f, 255);
      WriteWord(f, 3);
      WriteLongWord(f, 1);
      WriteLongWord(f, 1);	(* SubFileType *)*)

      WriteLongWord(f, 0);	(* no more IFDs *)


	(*****************************)
   	(* writes red colour entries *)
   	(*****************************)
      FOR count := 0 TO 255 DO
	  error := WriteByte(f, VAL(BYTE, palette[count].red));
	  error := WriteByte(f, VAL(BYTE, palette[count].red));
      END;

	(*******************************)
   	(* writes green colour entries *)
   	(*******************************)
      FOR count := 0 TO 255 DO
	  error := WriteByte(f, VAL(BYTE, palette[count].green));
	  error := WriteByte(f, VAL(BYTE, palette[count].green));
      END;

	(******************************)
   	(* writes blue colour entries *)
   	(******************************)
      FOR count := 0 TO 255 DO
	  error := WriteByte(f, VAL(BYTE, palette[count].blue));
	  error := WriteByte(f, VAL(BYTE, palette[count].blue));
      END;

      x_count := 0;
      y_count := 0;

      row := ObtainXMSMemory();

      ok := ObtainScreen();

      LOOP
	   (****************************************)
	   (* reads colour number of current pixel *)
	   (****************************************)
         row^[x_count] := VAL(BYTE, SHORTCARD(Point(CARDINAL(x_count+x1), CARDINAL(y_count+y1-1))));
         INC(x_count);
         IF x_count = x_size THEN
	    	error := WriteRecord(f, row, x_size);

	    	x_count := 0;

	    	INC(y_count);
	    	IF y_count = y_size THEN	(* reached last scan line *)
	         EXIT;				(* terminates encoding process *)
	    	END;
	   END;
      END; (* LOOP *)

      ReleaseScreen();

      ReleaseXMSMemory();

   ELSE

	Beep();
	CantCreateFile();

   END;

   CloseFile(f);

END Write256TIF;



END WriteTIF.