IMPLEMENTATION MODULE XMS;

(****************************************************************************)
(*      FILE NAME   : XMS.MOD                                               *)
(*                                                                          *)
(*      WRITTEN BY  : Chee hong, KOK                                        *)
(*      STUD. NO.   : 9328853                                               *)
(*      COURSE      : BACHELOR OF ELECTRICAL ENGINEERING                    *)
(*                                                                          *)
(*      DATE START  : 24 APRIL 1996                                         *)
(*      LAST EDITED : 9 MAY 1996                                            *)
(*                                                                          *)
(*      DESCRIPTION :                                                       *)
(*             This module provides access to Extended Memory any computer  *)
(*        has.  Onle XMS functions are implemented.  UMB functions will     *)
(*        be implemented in due time.                                       *)
(*                                                                          *)
(*             The term XMS is supposed to be abbreviation for              *)
(*                  eXtended Memory Specification (XMS)                     *)
(*        but in this module, it means extended memory block (EMB); because *)
(*        XMS is easier to understand compared to EMB which could be        *)
(*        mistaken by EMS(Expended Memory Specification).                   *)
(*                                                                          *)
(*              All the XMS functions DO NOT require ObtainXMSMemory and    *)
(*        ReleaseXMSMemory protection except the followings:                *)
(*              1.  SaveToXMS                                               *)
(*              2.  RestoreFromXMS                                          *)
(*              3.  XMSError                                                *)
(*                                                                          *)
(*              AllocateXMSMemory returns a private record type storing     *)
(*        information about the XMS memory allocated.  End user must use    *)
(*        this block to do all XMS related work such as                     *)
(*              1.  SaveToXMS                                               *)
(*              2.  RestoreFromXMS                                          *)
(*              3.  FreeXMSMemory                                           *)
(*                                                                          *)
(*              Once FreeXMSMemory is executed, the information in XMSBlock *)
(*        passed as parameter will nolonger be valid.                       *)
(*                                                                          *)
(*              SITUATION:                                                  *)
(*              At to this time, I have not found a way for allowing        *)
(*        Modula-2 to access the XMS memory area directly.  A fudge is used *)
(*        where any program that needs to store information to XMS region   *)
(*        will have to store information in a temporary memory area 64K in  *)
(*        size that is accessible by Modula-2 routines, then transfer the   *)
(*        information from this working space to XMS using SaveToXMS.       *)
(*              Conversly, to use information saved in XMS region, first    *)
(*        thing to do is RestoreFromXMS and then work on the temporary space*)
(*        provided.                                                         *)
(*              The temporary memory space for Modula-2 access is returned  *)
(*        via ObtainXMSMemory.  When operation has completed, the temporary *)
(*        working space has to be released for others to use via            *)
(*        ReleaseXMSMemory.                                                 *)
(*                                                                          *)
(*              To use XMS memory the following must be done:               *)
(*              1. AllocateXMSMemory - this will allocate proper XMS memory *)
(*              2. ObtainXMSMemory - provide temporary working space        *)
(*              3. do any thing to the working space                        *)
(*              4. ReleaseXMSMemory - release temporary working space       *)
(*                                                                          *)
(*      STATUS      : ok.  working.                                         *)
(*                                                                          *)
(****************************************************************************)

IMPORT XMSMemory;

FROM Lib IMPORT
(*proc*)IncAddr;

FROM LowLevel IMPORT
(*proc*)SEGMENT, OFFSET;

FROM TaskControl IMPORT
(*type*)Lock,
(*proc*)CreateLock, Obtain, Release;

FROM TerminationControl IMPORT
(*proc*)SetTerminationProcedure, Crash;

FROM Str IMPORT
(*proc*)Copy, Append, CardToStr;

FROM IO IMPORT
(*proc*)WrStr, WrLngCard;

FROM Storage1 IMPORT
(*proc*)ALLOCATE, DEALLOCATE;

FROM LinkList IMPORT
(*type*)List, ListElement,
(*proc*)CreateLinkList, CreateListElement, DestroyListElement, MoveToTop,
        MoveToBottom, PutAfter, PutBefore, RemoveFromList, ObtainList,
        ReleaseList, GetFirstElementInList, GetNextElementInList,
        GetPreviousElementInList,ReadObjectFromElement, ElementValid;

TYPE
  XMSBlock = POINTER TO RECORD
               offset:LONGCARD;
               size:LONGCARD;
             END;(*RECORD*)

  XMSMoveDataRecord = POINTER TO RECORD
                        length:LONGCARD;
                        SourceHandle:CARDINAL;
                        SourceOffset:LONGCARD;
                        DestinationHandle:CARDINAL;
                        DestinationOffset:LONGCARD;
                      END;(*RECORD*)

VAR
  XMSFreeBlockList:List;
  XMSLock:Lock;
  XMSMaxMemory:LONGCARD;
  XMSVersionNumber:CARDINAL;
  XMSMAINHandler:CARDINAL;
  MoveDataRecord:XMSMoveDataRecord;
  image:POINTER TO ARRAY [0..MAX(CARDINAL)-1] OF BYTE;
  ImageInLongCard:LONGCARD;

(****************************************************************************)
(* RoundUptoNearestMultipleOf4                                              *)
(****************************************************************************)
(* rounds a cardinal upto nearest multiple of 4                             *)
(****************************************************************************)
PROCEDURE RoundUptoNearestMultipleOf4(VAR (*INOUT*) number:LONGCARD);
BEGIN
  IF (number MOD 4 <> 0) THEN
    number := (number DIV 4) * 4 + 4;
  END;(*IF*)
END RoundUptoNearestMultipleOf4;

(****************************************************************************)
(* XMSAvailable                                                             *)
(****************************************************************************)
(* to check if XMS is available.  TRUE IF AVAILABLE                         *)
(* RETURN:                                                                  *)
(*    TRUE if available                                                     *)
(****************************************************************************)
PROCEDURE XMSAvailable():BOOLEAN;
VAR
  dummy:BOOLEAN;
BEGIN
  Obtain(XMSLock);
  dummy := XMSMemory.XMSAvailable();
  Release(XMSLock);
  RETURN (dummy);
END XMSAvailable;

(****************************************************************************)
(* XMSVersion                                                               *)
(****************************************************************************)
(* to find out the version of XMS used.                                     *)
(* returned value is expressed in BCD form:                                 *)
(* eg.     if number = 300H (or 768 in decimal) this means                  *)
(*             that version is 3.00                                         *)
(****************************************************************************)
PROCEDURE XMSVersion():CARDINAL;
BEGIN
  RETURN (XMSVersionNumber);
END XMSVersion;

(****************************************************************************)
(* XMSMemoryAvailable                                                       *)
(****************************************************************************)
(* returns total number of free extendeed memory available in bytes         *)
(****************************************************************************)
PROCEDURE XMSMemoryAvailable():LONGCARD;
VAR
  dummy:LONGCARD;
BEGIN
  Obtain(XMSLock);
  dummy := XMSMaxMemory;
  Release(XMSLock);
  RETURN (dummy);
END XMSMemoryAvailable;

(****************************************************************************)
(* XMSMemoryBlockAvailable                                                  *)
(****************************************************************************)
(* the largest extended memory block that can be allocated in bytes         *)
(****************************************************************************)
PROCEDURE XMSMemoryBlockAvailable():LONGCARD;
VAR
  TempFreeBlock:ListElement;
  LargestBlock:LONGCARD;
  TempXMSBlock:XMSBlock;
BEGIN
  Obtain(XMSLock);

  GetFirstElementInList(TempFreeBlock, XMSFreeBlockList);

  LargestBlock := 0;

  WHILE (ElementValid(TempFreeBlock)) DO
    IF (ElementValid(TempFreeBlock)) THEN
      ReadObjectFromElement(TempXMSBlock,TempFreeBlock);
      IF (LargestBlock < TempXMSBlock^.size) THEN
        LargestBlock := TempXMSBlock^.size;
      END;(*IF*)
    END;(*IF*)
    GetNextElementInList(TempFreeBlock, TempFreeBlock);
  END;(*WHILE*)

  Release(XMSLock);
  RETURN (LargestBlock);
END XMSMemoryBlockAvailable;

(****************************************************************************)
(* AllocateXMSMemory                                                        *)
(****************************************************************************)
(* allocates a number of memory in XMS for any use.                         *)
(*    size should be multiple of 4; if not AllocateXMSMemory will change the*)
(*    size automatically and return the new value.                          *)
(* PARAMETER PASSED:                                                        *)
(*  size - actual the size of memory to be allocated in bytes.              *)
(*         required XMS to be allocated is calculated internally            *)
(* RETURN:                                                                  *)
(*  a block of records having information about the XMS memory allocated    *)
(****************************************************************************)
PROCEDURE AllocateXMSMemory(size:LONGCARD):XMSBlock;
VAR
  TempFreeBlock:ListElement;
  TempXMSBlock,TempLocalXMSBlock:XMSBlock;
  blFound:BOOLEAN;
  msg:ARRAY [0..20] OF CHAR;
BEGIN
  RoundUptoNearestMultipleOf4(size);

  blFound := FALSE;

  Obtain(XMSLock);

  GetFirstElementInList(TempFreeBlock, XMSFreeBlockList);

  WHILE (ElementValid(TempFreeBlock) AND (NOT(blFound))) DO
    ReadObjectFromElement(TempLocalXMSBlock, TempFreeBlock);
    IF (TempLocalXMSBlock^.size >= size) THEN
      blFound := TRUE;
      NEW(TempXMSBlock);
      TempXMSBlock^.offset := TempLocalXMSBlock^.offset;
      TempXMSBlock^.size := size;
      TempLocalXMSBlock^.offset := TempLocalXMSBlock^.offset + size;
      TempLocalXMSBlock^.size := TempLocalXMSBlock^.size - size;

      XMSMaxMemory := XMSMaxMemory - size;

      IF (TempLocalXMSBlock^.size = 0) THEN
        DestroyListElement(TempFreeBlock);
        DISPOSE(TempLocalXMSBlock);
      END;(*IF*)
    ELSE
      GetNextElementInList(TempFreeBlock, TempFreeBlock);
    END;(*IF*)
  END;(*WHILE*)

  IF NOT(blFound) THEN
    Crash("Out Of extended memory.  Program stopped");
  END;(*IF*)

  Release(XMSLock);
  RETURN (TempXMSBlock);
END AllocateXMSMemory;

(****************************************************************************)
(* MergeXLSBlock                                                            *)
(****************************************************************************)
(* this procedure is used try to merge 2 XMS memory block together to reduce*)
(*      memory fragmentation problems.                                      *)
(* Order of TempFreeBlock1 and TempFreeBlock2 is NOT important, as this is  *)
(*      resolved internally in this procedure                               *)
(****************************************************************************)
PROCEDURE MergeXMSBlock(VAR (*INOUT*) TempFreeBlock1, TempFreeBlock2:
                          ListElement);
VAR
  TempLocalXMSBlock1,TempLocalXMSBlock2:XMSBlock;
  NextOffset1, NextOffset2:LONGCARD;
BEGIN
  IF (ElementValid(TempFreeBlock1) AND ElementValid(TempFreeBlock2)) THEN
    ReadObjectFromElement(TempLocalXMSBlock1,TempFreeBlock1);
    ReadObjectFromElement(TempLocalXMSBlock2,TempFreeBlock2);

    NextOffset1 := TempLocalXMSBlock1^.offset + TempLocalXMSBlock1^.size;
    NextOffset2 := TempLocalXMSBlock2^.offset + TempLocalXMSBlock2^.size;

    IF (NextOffset1 = TempLocalXMSBlock2^.offset) THEN
      TempLocalXMSBlock1^.size := TempLocalXMSBlock1^.size + TempLocalXMSBlock2^.size;
      DISPOSE(TempLocalXMSBlock2);
      DestroyListElement(TempFreeBlock2);
    ELSIF (NextOffset2 = TempLocalXMSBlock1^.offset) THEN
      TempLocalXMSBlock2^.size := TempLocalXMSBlock2^.size + TempLocalXMSBlock1^.size;
      DISPOSE(TempLocalXMSBlock1);
      DestroyListElement(TempFreeBlock1);
    END;(*IF*)
  END;(*IF*)
END MergeXMSBlock;


(*
(****************************************************************************)
(* FragmentedFreeBlock                                                      *)
(****************************************************************************)
(* this procedure is used check the number of free XMS blocks.              *)
(*      only needed for checking deallocation routines.                     *)
(****************************************************************************)
(*THIS IS USED FOR CHECKING THE NUMBER OF FRAGMENTED FREE XMS MEMORY BLOCK*)
PROCEDURE FragmentedFreeBlocks();
VAR
  TempFreeBlock:ListElement;
  counter:LONGCARD;
  string:ARRAY [0..20] OF CHAR;
  ok:BOOLEAN;
BEGIN
  GetFirstElementInList(TempFreeBlock, XMSFreeBlockList);

  counter := 0;

  WHILE (ElementValid(TempFreeBlock)) DO
    GetNextElementInList(TempFreeBlock, TempFreeBlock);
    INC(counter);
  END;(*WHILE*)

  CardToStr(counter,string,10,ok);
  IF (GraphicWindow.ObtainScreen()) THEN
    GraphicWindow.Rectangle(20,20,200,36,0,TRUE);
    GraphicWindow.DisplayString(string,20,20,15,FALSE);
    GraphicWindow.ReleaseScreen();
  END;(*IF*)
END FragmentedFreeBlocks;
*)

(****************************************************************************)
(* FreeXMSMemory                                                            *)
(****************************************************************************)
(* acts just like DEALLOCATE.  Information in ThisXMSBlock is nolonger valid*)
(*      after execution of this procedure.                                  *)
(* PARAMETER PASSED:                                                        *)
(*  ThisXMSBlock - the block of XMS memory allocated with AllocateXMSMemory *)
(****************************************************************************)
PROCEDURE FreeXMSMemory(ThisXMSBlock:XMSBlock);
VAR
  TempFreeBlock,TempFreeBlock2:ListElement;
  TempLocalXMSBlock:XMSBlock;
  NextOffset:LONGCARD;
  blFound:BOOLEAN;
  counter:CARDINAL;
BEGIN
  NextOffset := ThisXMSBlock^.offset + ThisXMSBlock^.size;

  blFound := FALSE;

  Obtain(XMSLock);

  GetFirstElementInList(TempFreeBlock, XMSFreeBlockList);

  WHILE (ElementValid(TempFreeBlock) AND (NOT(blFound))) DO
    ReadObjectFromElement(TempLocalXMSBlock, TempFreeBlock);

    IF (NextOffset > TempLocalXMSBlock^.offset) THEN
      GetNextElementInList(TempFreeBlock, TempFreeBlock);
    ELSIF (NextOffset = TempLocalXMSBlock^.offset) THEN
      blFound := TRUE;
      TempLocalXMSBlock^.offset := ThisXMSBlock^.offset;
      TempLocalXMSBlock^.size := TempLocalXMSBlock^.size + ThisXMSBlock^.size;
      XMSMaxMemory := XMSMaxMemory + ThisXMSBlock^.size;
      DISPOSE(ThisXMSBlock);
    ELSE
      blFound := TRUE;
      XMSMaxMemory := XMSMaxMemory + ThisXMSBlock^.size;
      CreateListElement(XMSFreeBlockList, TempFreeBlock2, ThisXMSBlock);
      PutBefore(TempFreeBlock2, TempFreeBlock);
      TempFreeBlock := TempFreeBlock2;
    END;(*IF*)
  END;(*WHILE*)

  IF (NOT(blFound)) THEN
    CreateListElement(XMSFreeBlockList, TempFreeBlock, ThisXMSBlock);
  END;(*IF*)

  GetPreviousElementInList(TempFreeBlock, TempFreeBlock2);
  MergeXMSBlock(TempFreeBlock, TempFreeBlock2);

  (*debugging only to check if deallocation is correct*)
  (*FragmentedFreeBlocks();*)

  Release(XMSLock);

  ThisXMSBlock := NIL;

END FreeXMSMemory;


(****************************************************************************)
(* ObtainXMSMemory                                                          *)
(****************************************************************************)
(* to grab hold onto XMS memory allocated and use it.  XMS memory MUST be   *)
(* obtained before using it and once finished, MUST ReleaseXMSMemory        *)
(* RETURN:                                                                  *)
(*  address pointing to memory of size 64k for use by other routines        *)
(****************************************************************************)
PROCEDURE ObtainXMSMemory():FarADDRESS;
BEGIN
  Obtain(XMSLock);
  RETURN (image);
END ObtainXMSMemory;

(****************************************************************************)
(* ReleaseXMSMemory                                                         *)
(****************************************************************************)
(* to release a hold on XMS access.  Must Release after XMS to a block of   *)
(* has completed, just as any Obtain and Release Locks                      *)
(****************************************************************************)
PROCEDURE ReleaseXMSMemory();
BEGIN
  Release(XMSLock);
END ReleaseXMSMemory;

(****************************************************************************)
(* SaveToXMS                                                                *)
(****************************************************************************)
(* to transfer information in temporary memory accessable by program to     *)
(*     to XMS data region allocated for XMSBlock                            *)
(****************************************************************************)
PROCEDURE SaveToXMS(XMSBlock:XMSBlock);
VAR
  msg, msg2:ARRAY [0..80] OF CHAR;
  ok:BOOLEAN;
BEGIN
  MoveDataRecord^.length := XMSBlock^.size;
  MoveDataRecord^.SourceHandle := 0;
  MoveDataRecord^.SourceOffset := ImageInLongCard;
  MoveDataRecord^.DestinationHandle := XMSMAINHandler;
  MoveDataRecord^.DestinationOffset := XMSBlock^.offset;
  XMSMemory.MoveData(MoveDataRecord);
  IF (XMSError(msg)) THEN
    Copy(msg2,"Storing XMS data. ");
    Append(msg2, msg);
    CardToStr(XMSBlock^.size, msg,10,ok);
    Append(msg2, msg);
    CardToStr(XMSBlock^.offset, msg,10,ok);
    Append(msg2, msg);
    Crash(msg2);
  END;(*IF*)
END SaveToXMS;

(****************************************************************************)
(* RestoreFromXMS                                                           *)
(****************************************************************************)
(* to transfer information from XMS data region allocated for XMSBlock to   *)
(*     temporary memory area accessable by program.                         *)
(****************************************************************************)
PROCEDURE RestoreFromXMS(XMSBlock:XMSBlock);
VAR
  msg, msg2:ARRAY [0..80] OF CHAR;
  ok:BOOLEAN;
BEGIN
  MoveDataRecord^.length := XMSBlock^.size;
  MoveDataRecord^.SourceHandle := XMSMAINHandler;
  MoveDataRecord^.SourceOffset := XMSBlock^.offset;
  MoveDataRecord^.DestinationHandle := 0;
  MoveDataRecord^.DestinationOffset := ImageInLongCard;
  XMSMemory.MoveData(MoveDataRecord);
  IF (XMSError(msg)) THEN
    Copy(msg2,"Restoring XMS data. ");
    Append(msg2, msg);
    CardToStr(XMSBlock^.size, msg,10,ok);
    Append(msg2, msg);
    CardToStr(XMSBlock^.offset, msg,10,ok);
    Append(msg2, msg);
    Crash(msg2);
  END;(*IF*)
END RestoreFromXMS;


(****************************************************************************)
(* XMSError                                                                 *)
(****************************************************************************)
(* to return the error for last XMS operation, if any.                      *)
(* RETURN:                                                                  *)
(*  TRUE if error occured and the message is passed into ErrMsg             *)
(****************************************************************************)
PROCEDURE XMSError(VAR (*OUT*) ErrMsg:ARRAY OF CHAR):BOOLEAN;
VAR
  ErrorCode:WORD;
BEGIN
  ErrorCode := XMSMemory.ReturnXMSErrorCode();
  CASE ErrorCode OF
    0H:
      RETURN (FALSE);
  | 080H:
      Copy(ErrMsg,"function not implemented");
  | 081H:
      Copy(ErrMsg,"VDISK device detected");
  | 082H:
      Copy(ErrMsg,"A20 error");
  | 090H:
      Copy(ErrMsg,"HMA does not exist");
  | 091H:
      Copy(ErrMsg,"HMA in use");
  | 092H:
      Copy(ErrMsg,"DX less than /HMAMIN parameter");
  | 093H:
      Copy(ErrMsg,"HMA not allocated");
  | 094H:
      Copy(ErrMsg,"A20 line is still enabled");
  | 0A0H:
      Copy(ErrMsg,"all available extended memory allocated");
  | 0A1H:
      Copy(ErrMsg,"all available extended memory handler in use");
  | 0A2H:
      Copy(ErrMsg,"extended memory handler invalid");
  | 0A3H:
      Copy(ErrMsg,"source handler invalid");
  | 0A4H:
      Copy(ErrMsg,"source offset invalid");
  | 0A5H:
      Copy(ErrMsg,"destination handler invalid");
  | 0A6H:
      Copy(ErrMsg,"destination offset invalid");
  | 0A7H:
      Copy(ErrMsg,"length invalid");
  | 0A8H:
      Copy(ErrMsg,"move has invalid overlap");
  | 0A9H:
      Copy(ErrMsg,"parity error");
  | 0AAH:
      Copy(ErrMsg,"block not locked");
  | 0ABH:
      Copy(ErrMsg,"extended memory handler locked");
  | 0ACH:
      Copy(ErrMsg,"block lock count overflow");
  | 0ADH:
      Copy(ErrMsg,"lock fails");
  | 0B0H:
      Copy(ErrMsg,"smaller UMB available");
  | 0B1H:
      Copy(ErrMsg,"no UMB available");
  | 0B2H:
      Copy(ErrMsg,"UMB segment number invalid");
  | ELSE
      Copy(ErrMsg,"Unknown XMS error");
  END;(*CASE*)
  RETURN (TRUE);
END XMSError;


(****************************************************************************)
(* AddressToLongCard                                                        *)
(****************************************************************************)
(* converts 4 byte address variable type into long cardinal type with same  *)
(*        byte for byte assignment.                                         *)
(* RETURN:                                                                  *)
(*  the address expressed in long cardinal                                  *)
(****************************************************************************)
PROCEDURE AddressToLongCard(address:ADDRESS):LONGCARD;
VAR
  dummy:LONGCARD;
  WordPtr:POINTER TO WORD;
BEGIN
  WordPtr := ADR(dummy);
  WordPtr^ := OFFSET(address);
  IncAddr(WordPtr, 2);
  WordPtr^ := SEGMENT(address);
  RETURN (dummy);
END AddressToLongCard;


(****************************************************************************)
(* InitialiseXMS                                                            *)
(****************************************************************************)
(* this procedure is used for initialising XMS.                             *)
(****************************************************************************)
PROCEDURE InitialiseXMS;
VAR
  TempElement:ListElement;
  TempXMSBlock:XMSBlock;
  TempMaxXMS:CARDINAL;
BEGIN
  IF (NOT(XMSMemory.XMSAvailable())) THEN
    Crash("XMS NOT PROPERLY INSTALLED");
  END;(*IF*)

  TempMaxXMS := XMSMemory.XMSMemoryBlockAvailable() DIV 2;

  XMSMaxMemory := VAL(LONGCARD, TempMaxXMS) * 1000;

  XMSMAINHandler := XMSMemory.AllocateXMSMemory(TempMaxXMS);

  XMSVersionNumber := XMSMemory.XMSVersion();

  CreateLinkList(XMSFreeBlockList);

  NEW(TempXMSBlock);

  TempXMSBlock^.offset := 0;
  TempXMSBlock^.size := XMSMaxMemory;

  CreateListElement(XMSFreeBlockList, TempElement, TempXMSBlock);

  NEW (MoveDataRecord);

  CreateLock(XMSLock,"XMSLock");

  IF (XMSMAINHandler < 1) THEN
    Crash("Error Allocating XMS");
  END;(*IF*)

  ALLOCATE(image, 65535);

  ImageInLongCard := AddressToLongCard(image);

END InitialiseXMS;

(****************************************************************************)
(* ShutDown                                                                 *)
(****************************************************************************)
(* This procedure is called upon ending of program                          *)
(****************************************************************************)
PROCEDURE ShutDown;
BEGIN
  XMSMemory.FreeXMSMemory(XMSMAINHandler);
END ShutDown;

BEGIN
  InitialiseXMS;

  SetTerminationProcedure(ShutDown);

END XMS.
